package prompt.contexthints;

import java.io.File;
import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;

import org.apache.log4j.Logger;

/**
 * Engines, Inputs und Visualisations koennen mittels
 * <code>addHint</code> beliebig viele Hints hinzufuegen.<br>
 * Jeder Hint-text ist dabei in einer Datei gespeichert die im Package
 * <code>prompt.contexthints</code> liegen muss. Damit jedes Mal nur ein Hint angezeigt wird
 * muss jedes Hintfile mit einer Nummer anfangen. Diese Nummer spiegelt die
 * Wichtigkeit (Lernkurve) des Hints wieder. Von allen Hints die in einer HintListe
 * gestellt wurden wird anschliessend diejenige angezeigt die die kleinste Nummer hat.
 * 
 * @author tschmidt
 *
 */
public class HintList implements Serializable{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	protected static ArrayList<String> list = new ArrayList<String>();

	
	
	public static URL getURL(String name) {
//		 check if html file found
		Logger logger = Logger.getLogger(HintList.class);
		URL url = null;
		String name2 = "/prompt/contexthints/" + name + ".html";
		try {
			File f2 = new File(name2);
			//logger.info(name2 + " Check in working directory: '" + f2.toURL().toExternalForm() + "'");
			if (f2.exists()) {
				logger.info(name2 + " Found. Loading from working directory: '" + f2.toURL().toExternalForm() + "'");
				url = f2.toURL();
			}

			if (url == null) {
				url = HintList.class.getResource(name2);
				//logger.info(name2 + " Check in resources root (inside jar or classpath)");
				if (url != null) {
					logger.debug(name2 + " Found. Loading from resources: '" + url.toExternalForm() + "'");
				}
			}
		} catch (MalformedURLException e) {
			logger.error(e); return null;
		}
		return url;
	}
	
	
	/**
	 * Adds this hint to the list of hints that may pop up
	 * 
	 * @param name
	 *            String without (.html suffix) and without path start (/prompt/contexthints)
	 * @return true if the html file in this package was found
	 */
	public static boolean addHint(String name) {
		Logger logger = Logger.getLogger(HintList.class);
		
		URL url = getURL(name);
		if (url==null) {
			logger.debug("hint '"+name+"'.html file could not be found");
			return false;
		}
		list.add(name); 
		return true;
	}

	/**
	 * Runs through the queue and returns the hit that a) is not yet deactived
	 * by the user and b) has the lowest number (=least complex)
	 * 
	 * @return String (filename without pre- and suffix) or null (if none)
	 */
	public static String getBestHint() {
		Logger logger = Logger.getLogger(HintList.class);
		int best = Integer.MAX_VALUE;
		String bestName = null;
		//work through list, extract starting number, return the smallest one
		if (list.size()==0) return null;
		for (String current : list) {
			int first = current.indexOf("_");
			if (first==-1) continue; //not found 
			if (first==0) continue;
			String nr = current.substring(0,first);
			//logger.debug(nr);
			try {
				int curInt = Integer.parseInt(nr);
				if (curInt<best)  { 
					//ist die bessere Nummer noch nicht deactivated
					boolean bool = HintStates.getState(current);
					if (bool==true) {
						best=curInt;
						bestName = current;
					}
				}
			} catch (Exception e) {
				logger.debug("can not convert number of string '"+nr+"' of '"+current+"'",e);
			}
		}
		if (best==Integer.MAX_VALUE) return null;
		logger.debug("best nr="+best+" fname='"+bestName+"'");
	
		return bestName;
	}

	public static void clearList() {
		list.clear();
	}
	
	public static void main(String argv[]) throws Exception {
		
		System.out.println(addHint("2_added_result"));
		System.out.println(addHint("0_loaded_demo_workspace"));
		System.out.println(addHint("1_added_input"));
		System.out.println(getBestHint());
	}

}
