/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: Counter.java,v 1.5 2006/03/19 11:58:10 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;

import org.apache.log4j.Logger;

import prompt.input.IPedant;
import prompt.input.classes.Pedant;
import tschmidt.utils.Db;
import tschmidt.utils.SimpleHashInteger;



public class Counter {

	/**
	 * @param data
	 * @return HashMap, value format= features getrennt mit strichpunkten
	 * key=feature; value=Object[]{ Double rel.frq, String frac }
	 */
	public static HashMap<String,CountRes> countRelFrq(HashMap<String,String> data) {
		if (data==null) return null;
		
		int totalNoGenesWithAnnotations = 0; 
		SimpleHashInteger featCountHash = new SimpleHashInteger(); // anzahl wie haeufig jedes feature vorkommt
		
		for (String key : data.keySet()) {
			String value = data.get(key);
			if (value != null) {
				totalNoGenesWithAnnotations++;
				String[] feats = value.split(";"); //Annotation ids are seperated with ;

				for (String feat : feats) {
					feat = feat.trim();
					featCountHash.inc(feat);
				}
			}
		}
		
		HashMap<String,CountRes> result = new HashMap<String,CountRes>();
		Iterator i = featCountHash.keySet().iterator();
		while (i.hasNext()) {
			String currentKey = (String) i.next();
			double currentValue = featCountHash.get(currentKey);
			double newValue = currentValue / ((double) totalNoGenesWithAnnotations);
			CountRes currentRes = new CountRes();
			currentRes.freq=newValue;
			currentRes.frac=featCountHash.get(currentKey)+"/"+totalNoGenesWithAnnotations;
			currentRes.oben= currentValue;
			currentRes.unten = totalNoGenesWithAnnotations;
			result.put(currentKey,currentRes);
		}
		return result;
	}
	
	public static HashMap<String,CountRes> countAbsFrq(HashMap<String,String> data, int total) {
		if (data==null) return null;
		
		//int totalNoGenesWithAnnotations = 0; 
		SimpleHashInteger featCountHash = new SimpleHashInteger(); // anzahl wie haeufig jedes feature vorkommt
		
		for (String key : data.keySet()) {
			String value = data.get(key);
			if (value != null) {
				
				String[] feats = value.split(";"); //Annotation ids are seperated with ;

				for (String feat : feats) {
					feat = feat.trim();
					featCountHash.inc(feat);
				}
			}
		}
		
		HashMap<String,CountRes> result = new HashMap<String,CountRes>();
		Iterator i = featCountHash.keySet().iterator();
		while (i.hasNext()) {
			String currentKey = (String) i.next();
			double currentValue = featCountHash.get(currentKey);
			double newValue = currentValue / ((double) total);
			CountRes currentRes = new CountRes();
			currentRes.freq=newValue;
			currentRes.frac=featCountHash.get(currentKey)+"/"+total;
			currentRes.oben= currentValue;
			currentRes.unten = total;
			result.put(currentKey,currentRes);
		}
		return result;
	}
	
    
    
    
    public static LinkedList<String> keysOnlyInHashA(HashMap<String, ?> hashA, HashMap<String,?> hashB) {
        Logger logger = Logger.getLogger(Counter.class.getName());
        logger.debug("property keys only in setA");
        //alle symbols die nur in property1 vorkommen
        HashMap<String,String> onlyIn = new HashMap<String,String>();
        for (String obj1 : hashA.keySet()) {
           
            boolean foundIn2 = false;
            for (String obj2 : hashB.keySet()) {
               
                if (obj1.equals(obj2)) {
                    foundIn2 = true;
                    break;
                }
            }
            if (!foundIn2) onlyIn.put(obj1,"");
        }
        LinkedList<String> list = new LinkedList<String>();
        list.addAll(onlyIn.keySet());
        return list;
    }
    
    
    public static LinkedList valuesOnlyInHashA(HashMap<String, ?> hashA, HashMap<String,?> hashB) {
        Logger logger = Logger.getLogger(Counter.class.getName());
        logger.debug("property keys only in setA");
        //alle symbols die nur in property1 vorkommen
        HashMap<Object,String> onlyIn = new HashMap<Object,String>();
        for (Object obj1 : hashA.values()) {
           
            boolean foundIn2 = false;
            for (Object obj2 : hashB.values()) {
               //TODO: values splitten
                if (obj1.equals(obj2)) {
                    foundIn2 = true;
                    break;
                }
            }
            if (!foundIn2) onlyIn.put(obj1,"");
        }
        LinkedList list = new LinkedList();
        list.addAll(onlyIn.keySet());
        return list;
    }
    
     
    public static LinkedList keysOnlyInHashB(HashMap<String, ?> hashA, HashMap<String, ?> hashB) {
        Logger logger = Logger.getLogger(Counter.class.getName());
        logger.debug("property keys only in setA");
        //alle symbols die nur in property1 vorkommen
        HashMap<String,String> onlyIn = new HashMap<String,String>();
        for (String obj2 : hashB.keySet()) {
           
            boolean foundIn1 = false;
            for (String obj1 : hashA.keySet()) {
               
                if (obj1.equals(obj2)) {
                    foundIn1 = true;
                    break;
                }
            }
            if (!foundIn1) onlyIn.put(obj2,"");
        }
        LinkedList<String> list = new LinkedList<String>();
        list.addAll(onlyIn.keySet());
        return list;
    }


    public static LinkedList valuesOnlyInHashB(HashMap<String, ?> hashA, HashMap<String, ?> hashB) {
        Logger logger = Logger.getLogger(Counter.class.getName());
        logger.debug("property keys only in setA");
        //alle symbols die nur in property1 vorkommen
        HashMap<Object,String> onlyIn = new HashMap<Object,String>();
        for (Object obj2 : hashB.values()) {
            //TODO: values splitten
            boolean foundIn1 = false;
            for (Object obj1 : hashA.values()) {
               
                if (obj1.equals(obj2)) {
                    foundIn1 = true;
                    break;
                }
            }
            if (!foundIn1) onlyIn.put(obj2,"");
        }
        LinkedList list = new LinkedList();
        list.addAll(onlyIn.keySet());
        return list;
    }
    
    
    /**
     * feature 
     * @param list
     * @return
     */
    public static String buildStringFromList(LinkedList list) {
        StringBuilder buf = new StringBuilder();
        buf.append("Feature\n");
        for (Object object : list) {
            buf.append(object);
            buf.append("\n");
        }
        return buf.toString();
    }
    
          
    
    
    

    
    
   /* 
//  Ausgabe aller symbols die nur in property2 vorkommen        
    StringBuffer sb2 = new StringBuffer("Property\n");
    for (Iterator iter = onlyIn2.keySet().iterator(); iter.hasNext();) {
        Object element = (Object) iter.next();
        sb2.append(element+"\tsetB\t0\n");
    }
    IEngineResults res2 = new EngineResults();
    try {
        res2.setResult(sb2.toString(),property1.getId()+" | "+setA.getDescription(),setB.getDescription(),false,"Compare:symbolic:onlyInSetB");
        allResults.add(res2);
        logger.debug(res2);
    } catch (Exception e) {
        logger.error("setting result of symbolic comparison failed ",e);
    }       
*/
    
    
    
    
    /**Result<br>
     * anno id | frq_rel | frac_rel 
     * */
        public static String buildResStringSimple(HashMap<String,String> ids) {
            

            HashMap<String,CountRes> frqRel = Counter.countRelFrq(ids);
           
            
            StringBuilder buf = new StringBuilder();
            buf.append("feature_id\tfrq_rel\tfrac_rel\n");
            for (String key : frqRel.keySet()) {
                buf.append(key); buf.append("\t");
                
                CountRes valueRel = frqRel.get(key);
                String currentFrqRel = "NaN";
                String currentFracRel = "NaN";
                if (valueRel != null) {
                    currentFrqRel = valueRel.freq+"";
                    currentFracRel = valueRel.frac;
                }
                
                buf.append(currentFrqRel+"\t"+currentFracRel);
           
                buf.append("\n");
            }
            String resultStr = buf.toString();
            return resultStr;
        }
    
    
	/**Result<br>
	 * anno id | frq_rel | frac_rel | frq_abs | frac_abs | anno_descr
	 * 
	 * 
	 */
	public static String buildResString(HashMap<String,String> ids,
			HashMap<String,String> desc, int size) {
		

		HashMap<String,CountRes> frqRel = Counter.countRelFrq(ids);
		HashMap<String,CountRes> frqAbs = Counter.countAbsFrq(ids,size);
		
		StringBuilder buf = new StringBuilder();
		buf.append("feature_id\tfrq_rel\tfrac_rel\tfrq_abs\tfrac_abs\tfeature_description\n");
		for (String key : frqRel.keySet()) {
			buf.append(key); buf.append("\t");
			
			CountRes valueRel = frqRel.get(key);
			String currentFrqRel = "NaN";
			String currentFracRel = "NaN";
			if (valueRel != null) {
				currentFrqRel = valueRel.freq+"";
				currentFracRel = valueRel.frac;
			}
			CountRes valueAbs = frqAbs.get(key);
			String currentFrqAbs = "NaN";
			String currentFracAbs = "NaN";
			if (valueAbs != null) {
				currentFrqAbs = valueAbs.freq+"";
				currentFracAbs = valueAbs.frac;
			}	
			buf.append(currentFrqRel+"\t"+currentFracRel+"\t");
			buf.append(currentFrqAbs+"\t"+currentFracAbs+"\t");
			String descr = "not available";
			if (desc!=null) { 
			 descr = desc.get(key);
			}
		    buf.append(descr+"\n");
		}
		String resultStr = buf.toString();
		return resultStr;
	}
	
	
	public static void main(String[] args) throws Exception {
		Db.checkDBSettings();
		
		String[] dbs = new String[]{ 
		"Escherichia_coli_K12_updated",
		"Onion_yellows_phytoplasma",
		"Mesoplasma_florum",
		"Mycoplasma_mycoides",
		"Mycoplasma_synoviae_53",
		"Mycoplasma_mobile_163K",
		"Mycoplasma_hyopneumoniae_232",
		"Mycoplasma_hyopneumoniae_7448",
		"Mycoplasma_hyopneumoniae_J",
		"Mycoplasma_penetrans",
		"Ureaplasma_urealyticum_serovar3",
		"Mycoplasma_gallisepticum_R",
		"Mycoplasma_pneumoniae_M129",
		"Mycoplasma_genitalium_G37",
		"Mpulmonis",
		"Mycoplasma_capricolum"};
		
         Logger logger = Logger.getLogger(Counter.class);
		
		for (String string : dbs) {
			IPedant full = new Pedant();
			full.setDatabaseName(string);
			full.setDescription(full.getDatabaseName());
			HashMap hash = full.getScopFoldIds("1e-20");
			
			
			HashMap<String,CountRes> frq = countRelFrq(hash);
			logger.debug(string+"\t"+frq.get("c.1"));
		}
		
		
	}
	
}




//-Ddef.dbUrl=jdbc:mysql://localhost/ -Ddef.dbDriver=com.mysql.jdbc.Driver -Ddef.dbUser=pedant -Ddef.dbPassword= -Dsimap.dbhost=jdbc:mysql://localhost/ -Dsimap.password=SimapIsFun
