/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: Engine.java,v 1.6 2006/03/19 11:58:09 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.net.URL;
import java.util.List;

import prompt.IDescription;
import prompt.input.IInput;
import prompt.input.filter.IFilterValue;
import prompt.util.ListTools;
import tschmidt.utils.ValidLocation;
import de.tum.binfo.jdrm.JobTemplate;



public abstract class Engine extends JobTemplate implements IEngine,IDescription {
  private final static long serialVersionUID = -6849794470754667700L;
	protected boolean rownames = false;
	protected String desc1 = null;
	protected String desc2 = null;

	protected IFilterValue valFilter = null;
	
	
	public  Engine() {
	}

	public String getClassDescription(Object sender) {
//    TODO: oberklassen durchsuchen, falls aktuelle klasse keine beschreibung hat
        URL url = getClass().getResource(this.getClass().getSimpleName() + ".html");
        if (url == null) {
            return "No description found :-(";
        }
        return ValidLocation.readFromUrl(url);
    }
	

	/* (non-Javadoc)
	 * @see de.tum.binfo.proj.engine.IEngine#writeResultsToFile()
	 */
	public String writeResultsToTmpFile() throws Exception {
	
		File file = null;
		try {
			file = File.createTempFile("ASdat", null);
			BufferedWriter bw = new BufferedWriter(new FileWriter(file));
			bw.write(this.toString());
			bw.close();
		} catch (IOException z) {
			throw new Exception(
				"Can not write to file '" + file.toString() + "' ");
		}
		return file.toString();
	}




	/* (non-Javadoc)
	 * @see de.tum.binfo.proj.engine.IEngine#setRownames()
	 */
	public void setRownames() {
		rownames = true;
	}

	
	/* (non-Javadoc)
	 * @see de.tum.binfo.proj.engine.IEngine#getResultsAll()
	 */
	public IEngineResults[] getResultsAll() throws Exception {
	  IEngineResults[] results = new EngineResults[1];
	  results[0]= new EngineResults();
	  results[0].setResult(
			this.toString(),
			desc1,
			desc2,
			rownames,
			this.getClass().getName());
	  //Default result description = engine name and "_res.html";
	  String desRes = "Error while reading default result description. More info in the log.";
		URL url = getClass().getResource(this.getClass().getSimpleName() + "_res.html");
		if (url == null) {
			desRes = "No default result description found :-(";
		} else {
			desRes = ValidLocation.readFromUrl(url);
			if ((desRes!=null) && (desRes.trim().length()>4)) {
				if (desc1!=null) desRes = desRes.replaceAll("\\$SETA", desc1);
				if (desc2!=null) desRes = desRes.replaceAll("\\$SETB", desc2);
			}
			
		}
		results[0].setResultDescription(desRes);
	  
	  return results;
	}
	
	public void applyValFilterToList(List list)
	{
	  ListTools.applyValFilterToList(list,valFilter);
	}
	
	/* (non-Javadoc)
	 * @see de.tum.binfo.proj.engine.IEngine#getResults()
	 */
	public IEngineResults getResults() throws Exception {

		return getResultsAll()[0];
	}
	
	public IFilterValue getFilterValue()
	{
		return valFilter;
	}


	public IInput[] getInputsAll() throws Exception {
		return null;
	}

	public void setFilterValue(IFilterValue filter){
		valFilter=filter;
	}

    
    
    private int hashcode = (int) Math.abs(Math.random()*1000000);
    /* (non-Javadoc)
     * @see de.tum.binfo.jdrm.JobTemplate#hashCode()
     */
    @Override
    public int hashCode() {
       return hashcode;
    }
	

}
