/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: EngineResults.java,v 1.5 2006/03/02 21:13:36 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine;

import java.awt.Component;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import tschmidt.utils.StringUtils;


public class EngineResults implements java.io.Serializable, IEngineResults {

  private final static long serialVersionUID = -6849794470754667700L;

    protected Date createdTime = new Date();

    protected Date lastChangedTime = new Date();

    protected String result = null;

    protected String desc1 = null;

    protected String desc2 = null;

    protected boolean rownames = false;

    /**
     * 
     * @uml.property name="resultType" multiplicity="(0 1)"
     */
    protected String resultType = null;

    /**
     * 
     * @uml.property name="filename" multiplicity="(0 1)"
     */
    protected String filename = null;

    /**
     * @return the raw data
     */
    public String getData() {
        return result;
    }

    public static String header(String str){
        String ret = StringUtils.replaceAll(str, ",","\t");
        String[] values = ret.split("\t");
        ret = "";
        for (int i = 0; i < values.length; i++) {
            String string = values[i];
            string = string.trim();
            ret = ret + "\""+string.toUpperCase()+"\"";
            if (i<values.length-1) ret= ret+ "\t";
        }
        ret=ret+"\n";
        return ret;
    }
    
    public void saveDataToFile(String destFilename) throws IOException {
        if (destFilename != null) {
            if (!destFilename.endsWith(".txt"))
                destFilename = destFilename + ".txt";

            BufferedWriter out = new BufferedWriter(
                    new FileWriter(destFilename));
            out.write(this.getMetaData());
            out.write(this.toString());
            out.flush();
            out.close();
           // logger.debug("write engine results data to file '"+filename+"'");
        }
    }

    /* (non-Javadoc)
     * @see de.tum.binfo.proj.engine.IEngineResults#loadDataFromFile(java.lang.String)
     */
    public void loadDataFromFile(String filename) throws IOException {
//FIXME: daten create, last changed, saved werden aus der datei nicht uebernommen
      StringBuffer strBuff = new StringBuffer();

      BufferedReader br = new BufferedReader(new FileReader(filename));
      String thisLine;
      while ((thisLine = br.readLine()) != null) {
          strBuff.append(thisLine + "\n");
      }

      String s = strBuff.toString();
      if (!s.startsWith("#resultType")) {
          Logger logger = Logger.getLogger(this.getClass());
          logger.error(filename + " does not seem to be a data file");
          JOptionPane.showMessageDialog(null, filename
                  + " does not seem to be a data file!", "Error",
                  JOptionPane.ERROR_MESSAGE);

      } else {
          boolean rownames = false;
          String t = s.substring(0, s.indexOf("\n"));
          String resultType = t.substring(t.indexOf(": ") + 2, t.length());
          s = s.substring(s.indexOf("\n") + 1, s.length());
          t = s.substring(0, s.indexOf("\n"));
          String desc1 = t.substring(t.indexOf(": ") + 2, t.length());
          s = s.substring(s.indexOf("\n") + 1, s.length());
          t = s.substring(0, s.indexOf("\n"));
          String desc2 = t.substring(t.indexOf(": ") + 2, t.length());
          s = s.substring(s.indexOf("#Rownames"), s.length());
          t = s.substring(0, s.indexOf("\n"));
          String rows = t.substring(t.indexOf(": ") + 2, t.length());
          if (rows.equals("true"))
              rownames = true;

          s = s.substring(s.indexOf("\n") + 1, s.length());

         
          try {
              this.setResult(s, desc1, desc2, rownames, resultType);
          } catch (Exception e) {
              Logger logger = Logger.getLogger(this.getClass());
              logger.error("result could not be set",e);
          }
      }
      br.close();
  }
    
    
    public void setResult(String data, String desc1, String desc2,
            boolean rownames, String resultType) throws Exception {
        if (result == null) {
            result = data;
            this.lastChangedTime = new Date();
        } else {
            throw new Exception("Can NOT override first results");
        }
        this.desc1 = desc1;
        this.desc2 = desc2;
        setDescription(resultType+": "+desc1+" "+desc2);
        this.rownames = rownames;
        this.resultType = resultType;
    }

    public void overwriteResultStr(String str) {
        result = str;
        lastChangedTime = new Date();
    }

    public String toString() {
        return result;
    }

    /**
     * @return the description of the first parameter
     */
    public String getDesc1() {
        return desc1;
    }

    /**
     * @return the description of the second parameter
     */
    public String getDesc2() {
        return desc2;
    }

    public String getDescAll() {
        return desc1 + " <-> " + desc2;
    }

    public boolean hasRownames() {
        return rownames;
    }

    /**
     * @return createdTime
     */
    public Date getCreatedTime() {
        return createdTime;
    }

    /**
     * @return a <code>Date</code> when the data was changed for the last time
     */
    public Date getLastChangedTime() {
        return lastChangedTime;
    }

    /**
     * @return Returns the resultType.
     * 
     * @uml.property name="resultType"
     */
    public String getResultType() {
        return resultType;
    }

    /**
     * @param resultType
     *            The resultType to set.
     * 
     * @uml.property name="resultType"
     */
    public void setResultType(String resultType) {
        this.resultType = resultType;
    }

    /**
     * @return Returns the filename.
     * 
     * @uml.property name="filename"
     */
    public String getFilename() {
        return filename;
    }

    /**
     * @param filename
     *            The filename to set.
     * 
     * @uml.property name="filename"
     */
    public void setFilename(String filename) {
        this.filename = filename;
    }

  /**
   * 
   * @uml.property name="description" multiplicity="(0 1)"
   */
  public String description;

    /**
     * 
     * @uml.property name="description"
     */
    /* (non-Javadoc)
     * @see de.tum.binfo.proj.input.IInput#getDescription()
     */
    public String getDescription() {
      return description;
    }

    /**
     * 
     * @uml.property name="description"
     */
    /* (non-Javadoc)
     * @see de.tum.binfo.proj.input.IInput#setDescription(java.lang.String)
     */
    public void setDescription(String string) {
      this.description = string;

    }

    /* (non-Javadoc)
     * @see de.tum.binfo.proj.input.IInput#getMetaData()
     */
    public String getMetaData() {
      StringBuffer out = new StringBuffer();
      String datefmt = "dd.MM.yy HH:mm:ss";
      out.append("#resultType: " + this.resultType + "\n");
      out.append("#Desc1: " + this.getDesc1() + "\n");
      out.append("#Desc2: " + this.getDesc2() + "\n");
      out.append("#Results created: "
              + (new SimpleDateFormat(datefmt)).format(this
                      .getCreatedTime()) + "\n");
      out.append("#Results last changed: "
              + (new SimpleDateFormat(datefmt)).format(this
                      .getLastChangedTime()) + "\n");
      out
              .append("#Date saved: "
                      + (new SimpleDateFormat(datefmt))
                              .format(new Date()) + "\n");
      out.append("#Saved by user: " + System.getProperty("user.name")
              + "\n");
      out.append("#Rownames: " + this.rownames + "\n");
      //out.append("Result description: "+this.getResultDescription()+"\n");
      return out.toString();
    }

    /* (non-Javadoc)
     * @see de.tum.binfo.proj.input.IJobTemplate#run()
     */
    public void run() throws Exception {
     //empty since the EngineResult will never "run" anything itself
      
    }

   

    /**
     * @param desc1 The desc1 to set.
     */
    public void setDesc1(String desc1) {
      this.desc1 = desc1;
    }
    /**
     * @param desc2 The desc2 to set.
     */
    public void setDesc2(String desc2) {
      this.desc2 = desc2;
    }

    public String resultDescription = null;
    
    public void setResultDescription(String str) {
       resultDescription= str;
        
    }

    public String getResultDescription() {
        return resultDescription;
    }
    
    public Component parentWin = null; 
	public void setParentWindow(Component win) {
		this.parentWin = win;
	}
}
