/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: IEngine.java,v 1.3 2006/02/14 16:20:12 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine;


import prompt.input.IInput;
import prompt.input.IJobTemplate;
import prompt.input.filter.IFilterValueAble;

/**
 * @author Schmidt
 */
public interface IEngine extends IJobTemplate, IFilterValueAble{
	
  final long serialVersionUID = -6849794470754667700L;
    /**
	 * Writes the calculated results to a temporary file (tab delimited table format = readable by R, Excel and others)
	 * @return the random path and file name where the data was saved
	 */
	public abstract String writeResultsToTmpFile() throws Exception;

	/**
	 * MUST BE OVERRIDDEN BY THE ACTUAL ENGINES WHICH RETURN ONLY ONE ENGINERESULT<br>
	 * Here the actual result string has to be returned.<br>
	 * @return complete result data as string.<br><br>
	 * Note: The meta data header is added automatically by the EngineResults.setResult method which is
	 * called in the getResultsAll() method.
	 */
	public abstract String toString();

	/**
	 * must be called if your result data table has rownames
	 */
	public abstract void setRownames();

	
	/**
	 * @return an IEngineResult Object, if more than one EngineResults are created, the first one is returned
	 * @throws Exception<br>
	 * Note: if more than one EngineResult is produced the toString Method may not be used to set up the result string.
	 */
	public abstract IEngineResults getResults() throws Exception;
	
	
	/**
	 * Returns EngineResult-Objects that are placed in the result-list of the GUI
	 * @return an array of IEngineResults instances, contains at least one Object
	 * @throws Exception
	 * <br>
	 * Note: if more than one EngineResult is produced the toString Method may not be used to set up the result string.
	 */
	public abstract IEngineResults[] getResultsAll() throws Exception;
	
	
	/**
	 * Returns Input-Objects that are placed in the input-list within the GUI
	 * @return an array of IInput instances, may be empty (null)
	 * @throws Exception
	 */
	public abstract IInput[] getInputsAll() throws Exception;
}
