/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: IEngineResults.java,v 1.2 2006/01/05 02:19:15 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine;

import java.io.IOException;
import java.util.Date;

import prompt.input.IInput;



/**
 * <b>Interface of the objects which are returned by the engines</b><br>
 * An engine results consists of a Meta data part and the actual data part.<br>
 * This can be set up and read with the help of getter and setter methods.<br>
 * Additionally the methods saveDataToFile and loadDataToFile allow a persistencent storage of the results.
 * @author Schmidt
 *
 */
public interface IEngineResults extends IInput{
  final long serialVersionUID = -6849794470754667700L; 
  /**
     * @return the raw data
     */
    public String getData();

    public void saveDataToFile(String destFilename) throws IOException;
	public void loadDataFromFile(String filename)throws IOException;
    
    public void setResult(String data, String desc1, String desc2,
            boolean rownames, String resultType) throws Exception;

    
    /**
     * Beschreibung des Inhalt des EngineResults, also z.B.
     * was die Variablen/Spalten beinhalten wie/was die
     * einzelnen Enrichmentwerte entstanden sind/bedeuten.
     * 
     * @param str String (can be plain text or html)
     */
    public void setResultDescription(String str);
    
    
    /**
     * @return String, or null if no result description found
     */
    public String getResultDescription();
    
    
    public void overwriteResultStr(String str);

    public String toString();

    /**
     * @return	the description of the first parameter
     */
    public String getDesc1();

    /**
     * @return	the description of the second parameter
     */
    public String getDesc2();

    public String getDescAll();

    public boolean hasRownames();

    /**
     * @return createdTime
     */
    public Date getCreatedTime();

    /**
     * @return a <code>Date</code> when the data was changed
     * 			for the last time
     */
    public Date getLastChangedTime();

    /**
     * @return Returns the resultType.
     * 
     * @uml.property name="resultType"
     */
    public String getResultType();

    /**
     * @param resultType The resultType to set.
     * 
     * @uml.property name="resultType"
     */
    public void setResultType(String resultType);

    /**
     * @return Returns the filename.
     * 
     * @uml.property name="filename"
     */
    public String getFilename();

    /**
     * @param filename The filename to set.
     * 
     * @uml.property name="filename"
     */
    public void setFilename(String filename);
    
    public void setDesc1(String desc1);
   
    public void setDesc2(String desc2);

}
