package prompt.engine;

import gr.zeus.ui.JMessage;

import java.io.FileWriter;

import javax.swing.JOptionPane;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamWriter;

import org.apache.log4j.Logger;

import prompt.input.IInput;
import prompt.ui.SaveUI;
import tschmidt.utils.NumberTest;





public class SaveResultsAsXML extends Engine1 {
	
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -1550340983580265202L;
	
	private IEngineResults results;
	//private String imageDirectory;
	
	public void init(IInput input) throws Exception {
		this.results = (IEngineResults) input;
	}
	
	
	String desFilename; 
	boolean worked = false;
	
	
	
	/**
	 * Tries to detect the column types, 
	 * xxx in the field if no type could be detected
	 * <b>Note</b>: This method DOES NOT performs any error checking
	 * @param dataLines
	 * @return
	 */
	String[] detectColTypes(String[] dataLines) {
		String[] columns = dataLines[0].split("\\t");
		String[] columns_type = new String[columns.length];
		
		
		for (int i = 1; i < dataLines.length; i++) {
			String row = dataLines[i];
			if ((row==null) || (row.length()<2)) continue;
			
			String[] vals = row.split("\\t"); 
			if ((vals==null) || (vals.length<columns.length)) continue; //leere oder bugous lines
			
			boolean allTyped = true;
			for (int j = 0; j < columns.length; j++) {
				if (columns_type[j]==null) {
					allTyped = false;
					String curVal = vals[j];
					if ((curVal==null) || (curVal.length()==0)) continue;
					
					if(NumberTest.isNumber(curVal)){
						columns_type[j] = "numeric";
					}else{
						columns_type[j] = "symbolic";
					}
					
				} //no kein typisierung von dieser Spalte
			
			} //foreach column
			
			if (allTyped) return columns_type;
		} //foreach line
		
		return columns_type;
	}
	
	
	@Override
    public void run() throws Exception {

		Logger logger = Logger.getLogger(this.getClass().getName());
		if (desFilename==null) desFilename = SaveUI.save("Export Results as XML",".xml");
	    
		if (desFilename != null) {
					if (this.results==null)  {
						String errmsg ="EngineResult is null, please provide data first";
				    	logger.debug(errmsg);
				    	JMessage.showMessageDialog(null,errmsg,JOptionPane.ERROR_MESSAGE,JOptionPane.DEFAULT_OPTION);
				    	return;
					}
					String resultStr = this.results.getData();
					if ((resultStr==null) || (resultStr.length()<=0)) {
						String errmsg ="EngineResult DATA is null or empty, please provide data first";
				    	logger.debug(errmsg);
				    	JMessage.showMessageDialog(null,errmsg,JOptionPane.ERROR_MESSAGE,JOptionPane.DEFAULT_OPTION);
				    	return;
					}
					String[] dataLines = resultStr.split("\\n");
					if (dataLines==null) {
						String errmsg ="<html>No data Lines found." +
								"<br> The lines must be delimited by linebreaks (ascii code #13)";
				    	logger.debug(errmsg);
				    	JMessage.showMessageDialog(null,errmsg,JOptionPane.ERROR_MESSAGE,JOptionPane.DEFAULT_OPTION);
				    	return;
					}
					if (dataLines.length<2) {
						String errmsg ="<html>Less than 2 lines lines found." +
								"<br> We need at least one column name line and row with data" +
								"<br> The lines must be delimited by linebreaks (ascii code #13)";
				    	logger.debug(errmsg);
				    	JMessage.showMessageDialog(null,errmsg,JOptionPane.ERROR_MESSAGE,JOptionPane.DEFAULT_OPTION);
				    	return;
					}
					
					
					String[] columns = dataLines[0].split("\\t");
					if ((columns==null) || (columns.length<2)) {
						String errmsg ="<html>Less than 2 COLUMNS found." +
								"<br>We need at least one column as ID and one column providing the VALUES." +
								"<br> The lines must be delimited by linebreaks (ascii code #13)";
				    	logger.debug(errmsg);
				    	JMessage.showMessageDialog(null,errmsg,JOptionPane.ERROR_MESSAGE,JOptionPane.DEFAULT_OPTION);
				    	return;
					}
					
					
					String[] columns_type = detectColTypes(dataLines);
						
					
					
					
					
					//
					//logger.debug(this.results.getData());
					XMLOutputFactory xof =  XMLOutputFactory.newInstance(); 
					XMLStreamWriter xtw = null; 
					FileWriter fw = new FileWriter(desFilename);
					xtw = xof.createXMLStreamWriter(fw);
					xtw.writeStartDocument("utf-8","1.0");
					xtw.writeStartElement("dataset");
						
				
					
					for (int curCol = 1; curCol < columns.length; curCol++) {
						
						if (columns_type[curCol]==null) {
							logger.debug("Skip column #"+curCol+" as the column type could not been discovered");
							continue;
						}
						
						xtw.writeStartElement("property");
						xtw.writeAttribute("id",columns[curCol]);
						xtw.writeAttribute("type",columns_type[curCol]); // ARRY_O_B
						
						for (int curRow = 1; curRow < dataLines.length; curRow++) {	
							String[] row_values = dataLines[curRow].split("\\t");
							if ((row_values.length>1) && (row_values.length>curCol)) {
								xtw.writeStartElement("input");
								xtw.writeAttribute("id",row_values[0]);
						
								xtw.writeAttribute("value",row_values[curCol]);
								xtw.writeEndElement();
							} else {
								logger.info("No info in column '"+columns[curCol]+"' in row "+curRow); //kein +1 da erste Zeile bereits der Header
							}
						}
						xtw.writeEndElement();
					}
					xtw.writeEndElement();
					xtw.writeEndDocument();
					xtw.flush();
					xtw.close(); 
				
				

			logger.debug("File export successful: "+desFilename);
		}
	    if (desFilename==null) {
	    	String errmsg ="Saving aborted by user";
	    	logger.debug(errmsg);
	    	JMessage.showMessageDialog(null,errmsg,JOptionPane.ERROR_MESSAGE,JOptionPane.DEFAULT_OPTION);
	    	return;
	    }
		worked =true;
    }



    /* (non-Javadoc)
     * @see prompt.engine.Engine#getResults()
     */
    @Override
    public IEngineResults getResults() throws Exception { 
        return null;
    }
    
    
    
    
	@Override
	public String toString() {
		return null;
	}

	@Override
	public IEngineResults[] getResultsAll() throws Exception {
		// TODO Auto-generated method stub
		return super.getResultsAll();
	}

	/*public static void main(String[] args) {
		IEngineResults results = new EngineResults();
		
		try {
			results.loadDataFromFile("engineResults.txt");
		} catch (Exception e) {
			logger.debug("Error:"+e);
		}
		
		SaveResultsAsXML xml = new SaveResultsAsXML();
		try {
			xml.init(results);
			xml.run();
		} catch (Exception e) {
			logger.debug("Error:"+e);
		}
	} */



	public String getFilename() {
		return desFilename;
	}



	/**
	 * @param filename der XML Datei in das die Resultdaten gespeichert werden
	 * sollen. Wird in der RUN Methode abgefragt. Ist noch kein Filename
	 * gesetzt dann wird der Benutzer mittels GUI Dialog gefragt.
	 */
	public void setFilename(String filename) {
		this.desFilename = filename;
	}
	
	/**
	 * @return true, if the last run() finished successfully
	 */
	public boolean hasWorked() {
		return worked;
	}
}
