package prompt.engine;

import java.util.HashMap;
import java.util.StringTokenizer;
import java.util.Vector;

public class Tabelle {

	private Vector<Spalte> columns;

	private String name = null;
	
	public Tabelle(String name) {
		this.name = name;
		columns = new Vector<Spalte>();
	}

	public Tabelle(String name, String tabbedString) {
		this.name = name;
		columns = new Vector<Spalte>();
		TabbedString2Tabelle(tabbedString);
	}

	private void TabbedString2Tabelle(String tabbedString) {
		StringTokenizer tok = null;
		String[] resultLines = tabbedString.split("\n");
		String[] types = getColumnTypes(resultLines);
		tok = new StringTokenizer(resultLines[0]);
		// add colums to Tabelle
		int i = 0;
		while (tok.hasMoreTokens())
			try {
				this.addColumn(tok.nextToken(), types[i]);
				i++;
			} catch (Exception e) {
				e.printStackTrace();
			}
		// fill Tabelle
		for (int j = 1; j < resultLines.length; j++){
			String[] aux = resultLines[j].split("\t");
			int count = 0;
			for (String input : aux) {
				if (types[count].equals("double"))
					this.insertData2Column(count, Double.parseDouble(input));
				else
					this.insertData2Column(count, input);
				count++;
				
			}
			
		}
	}

	private String[] getColumnTypes(String[] resultLines) {
		// get amount of cols
		int amountCols = resultLines[0].split("\t").length;
		String[] cols = null;
		String[] ret = new String[amountCols];
		int counter = amountCols;
		HashMap<Integer, Integer> doubleCols = new HashMap<Integer, Integer>();
		for (int j = 0; j < amountCols; j++)
			doubleCols.put(j, 0);
		for (int i = 1; i < resultLines.length; i++){
			cols = resultLines[i].split("\t");
			int k = 0;
			for (String s: cols){
				if (s.equals("") || ret[k] != null && ret[k].equals("string")) {k++; continue;}
				try {
					Double.parseDouble(s);
					ret[k] = "double";
					doubleCols.put(k, doubleCols.get(k)+1);
				}
				catch (Exception e){
					ret[k] = "string";
					counter--;
				}
			k++;
			}
			if (counter == 0 || checkHashMap(doubleCols))
				return ret;
		}
		return null;		
	}

	private boolean checkHashMap(HashMap<Integer, Integer> h){
		int aux = 0;
		for (Integer i : h.keySet())
			aux += i;
		if (aux >= h.keySet().size()*3)
			return true;
		return false;
	}
	
	public String getName() {
		return name;
	}

	public Spalte getSpalte(String name) {
		for (Spalte s : getColumns()) {
			if (s.getName().equals(name))
				return s;
		}
		return null;
	}

	public void addColumn(String name, String type) throws Exception {
		addColumn(name, type, columns.size());
	}

	public void addColumn(String name, String type, int pos) throws Exception {
		// if (columns.elementAt(pos) == null)
		columns.add(pos, createSpalte(name, type, pos));
		// else
		// throw new Exception("Column already in use");
	}

	public void deleteColumn(String name) {
		for (Spalte s : columns) {
			if (s.getName().equals(name))
				s = null;
		}
	}

	public void deleteColumn(int pos) {
		columns.add(pos, null);
	}

	private Spalte createSpalte(String name, String type, int pos)
			throws Exception {
		if (type.equals("int"))
			return new Spalte<Integer>(name, pos, type);
		else if (type.equals("double"))
			return new Spalte<Double>(name, pos, type);
		else if (type.equals("float"))
			return new Spalte<Float>(name, pos, type);
		else if (type.equals("string"))
			return new Spalte<String>(name, pos, type);
		else if (type.equals("boolean"))
			return new Spalte<Boolean>(name, pos, type);
		else if (type.equals("char"))
			return new Spalte<Character>(name, pos, type);
		else
			throw new Exception("Type not supported: " + type);
	}

	@SuppressWarnings("unused")
	public Vector<Spalte> getColumns() {
		return columns;
	}

	public int getLength() {
		int ret = 0;
		for (Spalte s : columns) {
			if (s.size() > ret)
				ret = s.size();
		}
		return ret;
	}

	// *************************************************************************
	// all Insert Commands
	// *************************************************************************
	public void insertData2Column(int colNum, Object data) {
		Object[] aux = new Object[1];
		aux[0] = data;
		insertData2Column(colNum, aux);
	}

	public void insertData2Column(int colNum, Object[] data) {
		int i = 0;
		for (Object o : data) {
			insertData2Column(colNum, i, o);
			i++;
		}
	}

	public void insertData2Column(String colName, Object[] data) {
		int i = 0;
		for (Object o : data) {
			insertData2Column(colName, i, o);
			i++;
		}
	}

	public void insertData2Column(int colNum, int pos, Object data) {
		insertData2Column(null, colNum, pos, data);
	}

	public void insertData2Column(String colName, int pos, Object data) {
		insertData2Column(colName, -1, pos, data);
	}

	@SuppressWarnings("unchecked")
	private void insertData2Column(String colName, int colNum, int pos,
			Object data) {
		Spalte aux = null;
		if (colName != null) {
			for (Spalte s : columns) {
				if (s.getName().equals(colName)) {
					aux = s;
					break;
				}
			}
		} else if (colNum != -1) {
			aux = columns.get(colNum);
		}
		aux.add(pos, data);
	}

	// *******************************************************************
	// all return commands
	// *******************************************************************

	public Vector<Object> getAllDataFromColumn(int colNum) {
		Vector<Object> ret = new Vector<Object>();
		Spalte aux = columns.get(colNum);
		while (aux.hasMoreElements()) {
			ret.add(aux.delete());
		}
		return ret;
	}

	public Vector<Object> getAllDataFromColumn(String colName) {
		Vector<Object> ret = new Vector<Object>();
		Spalte aux = null;
		for (Spalte s : columns) {
			if (s.getName().equals(colName)) {
				aux = s;
				break;
			}
		}
		while (aux.hasMoreElements()) {
			ret.add(aux.delete());
		}
		return ret;
	}

	public Object getDataFromColumn(int colNum, int pos) {
		return getDataFromColumn(null, colNum, pos);
	}

	public Object getDataFromColumn(String colName, int pos) {
		return getDataFromColumn(colName, -1, pos);
	}

	private Object getDataFromColumn(String colName, int colNum, int pos) {
		Object ret = null;
		try {
			if (colName != null) {
				for (Spalte s : columns) {
					if (s.getName().equals(colName)) {
						ret = s.getElement(pos);
						break;
					}
				}
			} else if (colNum != -1) {
				ret = columns.get(colNum).getElement(pos);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		return ret;
	}

	@SuppressWarnings("unused")
	public Object[] getRow(int i) {
		Object[] aux = new Object[this.getColumns().size()];
		int j = 0;
		for (Spalte s : getColumns()) {
			if (s.size() <= i)
				aux[j] = null;
			else
				aux[j] = getDataFromColumn(s.getName(), i);
			j++;
		}
		return aux;
	}

}
