package prompt.engine;

import java.io.File;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;

public class VirtualDB2 {
	 Logger logger = Logger.getLogger(VirtualDB2.class.getName());
	private Connection conn = null;

	private String dbName = null;

	private Vector<Tabelle> allTables = null;
	private Vector<String> allNames = null;
	
	public VirtualDB2(String dbName) {
		this.dbName = dbName;
		allTables = new Vector<Tabelle>();
		allNames = new Vector<String>();
		createNewDB();
	}

	private void createNewDB() {
		try {
			Class.forName("org.hsqldb.jdbcDriver");
			conn = DriverManager.getConnection("jdbc:hsqldb:"
					+ System.getProperty("java.io.tmpdir") + "/" + dbName,
					"sa", "");

		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	@SuppressWarnings("unchecked")
	public synchronized String query(String expression) {
		// first array slot is table name
		String[] ret = prepareExp(expression);
		Tabelle result = new Tabelle("filtered_" + getTable(ret[0]));
		try {
			for (int i = 1; i < ret.length; i++) {
				if (getTable(ret[0]).getSpalte(ret[i]) != null)
					result.addColumn(ret[i], getTable(ret[0]).getSpalte(ret[i])
							.getType());
				else
					result.addColumn(ret[i], "string");
			}

			Statement st = conn.createStatement();
			ResultSet rs = st.executeQuery(expression);
			while (rs.next()) {
				for (Spalte s : result.getColumns()) {
					if (s.getType().equals("string"))
						s.add((rs.getString(s.getName())));
					else if (s.getType().equals("int"))
						s.add((rs.getInt(s.getName())));
					else if (s.getType().equals("double"))
						s.add((rs.getDouble(s.getName())));
					else if (s.getType().equals("float"))
						s.add((rs.getFloat(s.getName())));
					else if (s.getType().equals("boolean"))
						s.add((rs.getBoolean(s.getName())));
					else if (s.getType().equals("char"))
						s.add((rs.getCharacterStream(s.getName())));
				}
				// logger.debug("adding filtered data");
			}
			st.close();

		} catch (Exception e) {
			result = null;
			return "Incorrect sql Statement" + e.getMessage();
		}
		return Tabelle2TabbedString(result);
	}

	public synchronized void update(String expression) {
		try {
			Statement st = conn.createStatement();
			int i = st.executeUpdate(expression);

			if (i == -1) {
				logger.debug("db error : " + expression);
			}
			st.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void shutdown() {
		try {
			Statement st = conn.createStatement();
			st.execute("SHUTDOWN");
			conn.close();
			// delete DB Files
			File f = new File(System.getProperty("java.io.tmpdir"));
			File[] filePaths = f.listFiles();
			for (File s : filePaths) {
				if (s.getName().equals(dbName + ".properties")
						|| s.getName().equals(dbName + ".script"))
					s.delete();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void createTable(Tabelle t) {
		if (!allNames.contains(t.getName())) {
			allTables.add(t);
			allNames.add(t.getName());
			String cols = "";
			for (int i = 0; i < t.getColumns().size(); i++) {
				cols += "," + t.getColumns().get(i).getName() + " "
						+ type(t.getColumns().get(i));
			}
			// Create Table
			this.update("CREATE TABLE " + t.getName() + " ("
					+ cols.substring(1) + ")");
			try {
				fillTable(t);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	private void fillTable(Tabelle t) throws Exception {
		if (allNames.contains(t.getName())) {
			String cols = "";
			for (int i = 0; i < t.getColumns().size(); i++) {
				cols += "," + t.getColumns().get(i).getName();
			}
			for (int j = 0; j < t.getLength(); j++) {
				Object[] aux = t.getRow(j);
				String data = "";
				for (Object o : aux) {
					if (o != null) {
						if (o.getClass() == String.class
								|| o.equals(Double.NaN))
							data += ",'" + o.toString() + "'";
						else
							data += "," + o.toString();
					} else
						data += ", NULL";
				}
				this.update("INSERT INTO " + t.getName() + " ("
						+ cols.substring(1) + ") VALUES (" + data.substring(1)
						+ ")");
			}
		} else {
			throw new Exception("Table not found: " + t.getName());
		}
	}

	private String type(Spalte spalte) {
		if (spalte.getType().equals("int"))
			return "INTEGER";
		else if (spalte.getType().equals("double"))
			return "DOUBLE";
		else if (spalte.getType().equals("float"))
			return "FLOAT";
		else if (spalte.getType().equals("string"))
			return "VARCHAR(255)";
		else if (spalte.getType().equals("boolean"))
			return "BOOL";// "TINYINT(1)"
		else if (spalte.getType().equals("char"))
			return "VARCHAR(1)";
		return null;
	}

	private String[] prepareExp(String expression) {
		Pattern pattern = Pattern.compile("SELECT\\s(.+)\\sFROM\\s(\\w+)",
				Pattern.CASE_INSENSITIVE);
		Matcher matcher = pattern.matcher(expression);
		matcher.find();
		String returns = matcher.group(1);
		Tabelle t = getTable(matcher.group(2));
		int count = 1;
		String[] aux = null;
		if (returns.equals("*")) {
			aux = new String[t.getColumns().size() + 1];
			for (Spalte s : t.getColumns()) {
				aux[count] = s.getName();
				count++;
			}
		} else {
			StringTokenizer tok = new StringTokenizer(returns, ",");
			aux = new String[tok.countTokens() + 1];
			while (tok.hasMoreTokens()) {
				aux[count] = tok.nextToken().trim();
				count++;
			}
		}
		aux[0] = t.getName();

		return aux;
	}

	private Tabelle getTable(String tab) {
		for (Tabelle tabelle : allTables) {
			if (tabelle.getName().equals(tab)) {
				return tabelle;
			}
		}
		return null;
	}

	private String Tabelle2TabbedString(Tabelle t2) {
		StringBuilder ret = new StringBuilder();
		// add all columns to String
		for (int i = 0; i < t2.getColumns().size(); i++) {
			if (i == t2.getColumns().size() - 1)
				ret.append(t2.getColumns().get(i).getName() + "\n");
			else
				ret.append(t2.getColumns().get(i).getName() + "\t");
		}
		// add all Lines
		for (int i = 0; i < t2.getLength(); i++) {
			for (int j = 0; j < t2.getColumns().size(); j++) {
				if (j == t2.getColumns().size() - 1)
					ret.append(t2.getDataFromColumn(j, i) + "\n");
				else
					ret.append(t2.getDataFromColumn(j, i) + "\t");
			}
		}
		return ret.toString();
	}

	public String getName(){
		return dbName;
	}
	
	// public void go(VirtualDB2 db) {
	// Tabelle a = new Tabelle("tab");
	// try {
	// a.addColumn("id", "int");
	// a.addColumn("name", "string");
	// a.addColumn("alter", "double");
	// a.insertData2Column("id", new Object[] { 1, 2, 3 });
	// a.insertData2Column("name",
	// new Object[] { "hans", "doffer", "biba" });
	// a.insertData2Column("alter", new Object[] { 100.0, 92.0, 32.0 });
	// } catch (Exception e1) {
	// e1.printStackTrace();
	// }
	//
	// createTable(a);
	// try {
	// fillTable(a);
	// } catch (Exception e) {
	// e.printStackTrace();
	// }
	//
	// Tabelle aux = db.query("SELECT * FROM tab");
	// logger.debug(Tabelle2TabbedString(aux));
	// // must be tested!
	// Tabelle aux2 = db.query("SELECT count(name) as bla FROM tab");
	// logger.debug(Tabelle2TabbedString(aux2));
	//
	// db.shutdown();
	//
	// }
	//
	// public static void main(String[] args) {
	// try {
	//
	// VirtualDB2 db = new VirtualDB2("1");
	// db.go(db);
	// } catch (Exception e) {
	// e.printStackTrace();
	// }
	//
	// }

}
