package prompt.engine.applymapping;

import java.io.File;

import prompt.engine.IEngineResults;
import prompt.engine.SaveResultsAsXML;
import prompt.input.IGenericXML;
import prompt.input.IGenericXMLFile;
import prompt.input.IInput;
import prompt.input.classes.GenericXMLFile;

public class ApplyMappingFromResultObject extends ApplyMappingFromXMLObject {


	private static final long serialVersionUID = 1L;
	IGenericXML xmlInputObj;
	IGenericXMLFile xmlResultObj;
	IEngineResults resObj;
	
	boolean initFinished =false;
	
	public ApplyMappingFromResultObject() {
		
	}
	
	@Override
	public void init(IInput setA, IInput setB) throws Exception {
		//Check if we got a XML x Result input combination
		boolean correctInput = false;
		logger.debug("setA="+setA.getClass().getName());
		logger.debug("setB="+setB.getClass().getName());
		try {
			xmlInputObj = (IGenericXML) setA;
			resObj = (IEngineResults) setB;
			correctInput=true;
		} catch (Exception e) {
		 logger.debug("No (XML x Result) input :-(");
		}
		if (!correctInput) {
			
			try {
				resObj = (IEngineResults) setA;
			}
			catch (Exception e){
				  logger.debug("No (Result x XML) input :-(; cast setA to RESULT failed");
				  throw new ClassCastException("This method needs a Mapping Result and an Input object");
			}
			try {
				xmlInputObj = (IGenericXML) setB;
			} catch (Exception e) {
				  logger.debug("No (Result x XML) input :-(; cast setB to XML failed");
				  throw new ClassCastException("This method needs a Mapping Result and an Input object");
			}	
			correctInput=true;
			
		}
		logger.debug("Correct Input = "+correctInput);
		SaveResultsAsXML saver = new SaveResultsAsXML();
		saver.init(resObj);
		
		File f = File.createTempFile("Choosen_"+resObj.getFilename()+"_tmpNr","");
		saver.setFilename(f.getAbsolutePath());
		saver.run();
		//saving kann abgebrochen werden falls der Benutzer z.B. keine Properties auswaehlt
		if (!saver.hasWorked()) {
			logger.info("aborted");
			return;
		}
		
		xmlResultObj = new GenericXMLFile();
		//avoids that the user is asked about a temp. filename
		xmlResultObj.init(f.getAbsolutePath());
		//avoid that we are asked for a description of the temporary result xml
		xmlResultObj.setDescription(resObj.getFilename());
		xmlResultObj.run();
		
		//xmlResultObj haelt die Mapping daten
		//xmlInputObj enthaelt die Daten die gemappt werden sollen 
		
		super.init(xmlInputObj, xmlResultObj);
		/*falls es sich bei dem Engine Result um ein Blast result handelt wissen wir bereits das die SUBJECT_ID die
		 entsprechende Property ist.
		*/
		if (resObj.getResultType().contains("Blast")) {
			logger.info("We are using a Blast result for mapping --> automatically choose the mapping set and SUBJECT_ID Field for mapping");
			this.set = xmlInputObj; //this should be the data object...
			this.setMappingProperty(xmlResultObj.getProperty("SUBJECT_ID"));
	
		} else {
			logger.debug("Result does not seems to fit to a Blast output -> let the user choose the property that contains the mapping info");	
		}
		initFinished=true;
	}
	
	public IEngineResults getResults() throws Exception {
		if (!initFinished) return null;
		return super.getResults();
	}
	
	@Override
	public IEngineResults[] getResultsAll() throws Exception {
		if (!initFinished) return null;
		return super.getResultsAll();
	}

	@Override
	public String toString() {
		if (!initFinished) return null;
		return super.toString();
	}

	@Override
	public void run() throws Exception {
		if (!initFinished) { 
			logger.debug("init was not called or finished correctly -> we do not even start the run method");
			return;
		}
		super.run();
	}



	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub

	}

}
