/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: CmpSymb.java,v 1.8 2006/03/19 11:58:08 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.genericxml;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;

import org.apache.commons.math.MathException;
import org.apache.commons.math.stat.inference.TestUtils;
import org.apache.log4j.Logger;

import prompt.engine.CountRes;
import prompt.engine.Counter;
import prompt.engine.Engine2;
import prompt.engine.EngineResults;
import prompt.engine.IEngineResults;
import prompt.input.IGenericProperty;
import prompt.input.IGenericXML;
import prompt.input.IInput;


/**
 * Noch die Hash-Value Split Funktion mit rein!!
 * 
 * @author poweruser2
 *
 */
public class CmpSymb extends Engine2 {

    protected IGenericXML setA, setB;
    private Logger logger = Logger.getLogger(this.getClass());
 
    ArrayList allResults = new ArrayList();
    
    public CmpSymb() {
        super();
    }

    public void init(IInput setA, IInput setB) throws Exception {
        super.init(setA, setB);
        this.setA = (IGenericXML) setA;
        this.setB = (IGenericXML) setB;
    }
    
    
    
    
    public void cmpSymbolic(IGenericProperty property1, IGenericProperty property2) {
        
        HashMap data1 = property1.getValues();
        
        HashMap data2 = property2.getValues();
        
        HashMap<String,CountRes> frqRel1 = Counter.countRelFrq(data1);
        HashMap<String,CountRes> frqRel2 = Counter.countRelFrq(data2);
        
        LinkedList listOnly1 = Counter.keysOnlyInHashA(frqRel1,frqRel2);
        LinkedList listOnly2 = Counter.keysOnlyInHashB(frqRel1,frqRel2);
        
        if ((listOnly1!=null) && (listOnly1.size()>0)) {
        IEngineResults res2 = new EngineResults();
        try {
            res2.setResult(Counter.buildStringFromList(listOnly1),property1.getId()+" | "+setA.getDescription(),setB.getDescription(),false,"Compare:symbolic:onlyInSetA");
            String str1 = "<html>Shows all features that occur only in '"+setA.getDescription()+"' and " +
                    "not in '"+setB.getDescription()+"'.<br> " +
                            "The annotation feature analysed:<br>" +
                            "'"+property1.getId()+"' in '"+setA.getDescription()+"' and <br>" +
                            "'"+property2.getId()+"' in '"+setB.getDescription()+"' and <br>" +
                                    "</html>";
            res2.setResultDescription(str1);
            allResults.add(res2);
           // logger.debug(res2);
        } catch (Exception e) {
            logger.error("setting result ONLY A of symbolic comparison failed ",e);
        }       
        }
        
        if ((listOnly2!=null) && (listOnly2.size()>0)) {
        IEngineResults res3 = new EngineResults();
        try {
            res3.setResult(Counter.buildStringFromList(listOnly2),property2.getId()+" | "+setA.getDescription(),setB.getDescription(),false,"Compare:symbolic:onlyInSetB");
            String str3 = "<html>Shows all features that occur only in '"+setB.getDescription()+"' and " +
            "not in '"+setA.getDescription()+"'.<br> " +
                    "The annotation feature analysed:<br>" +
                    "'"+property1.getId()+"' in '"+setA.getDescription()+"' and <br>" +
                    "'"+property2.getId()+"' in '"+setB.getDescription()+"' and <br>" +
                            "</html>";
            res3.setResultDescription(str3);
            allResults.add(res3);
         //   logger.debug(res3);
        } catch (Exception e) {
            logger.error("setting result ONLY B of symbolic comparison failed ",e);
        }     
        }
        
        double absAll = -1;
        double frq1 = -1;
        double frq2 = -1;
        
        //both: gehe ueber alle in frqRel1
        StringBuilder buf = new StringBuilder();
        buf.append("feature\tdiff_frqA_minus_B\tpval\tfrqA\tfracA\tfrqB\tfracB\n");
        for (String key : frqRel1.keySet()) {
            CountRes res1 = frqRel1.get(key);
            CountRes res2 = frqRel2.get(key);
            if ((res2!=null) && (res1!=null)) {
                buf.append(key+"\t");
                //smaller div bigger
               /* enrichment
                * double resValue;
				resValue = Double.NaN;
                try {
                if (res1.freq<res2.freq) resValue=res1.freq /res2.freq; 
                else resValue=res2.freq/res1.freq;
                }catch (Exception e) {}
                //buf.append(resValue+"\t");
                 * */
                 
                
                double diff = res1.freq-res2.freq;
                
                buf.append(diff+"\t");
                
              
                double obsTotal = res1.oben + res2.oben;
                //buf.append(obsTotal+"\t");
                if (absAll==-1) { 
                	absAll= res1.unten+ res2.unten; //relative bezugsgroesse, z.B. alle Proteine mit scop fold
                	logger.debug("absAll = "+absAll);
                	frq1 = res1.unten / absAll; //Verhaeltnis der Mengen 
                	logger.debug("frq1 = "+frq1);
                	frq2 = res2.unten / absAll;
                	logger.debug("frq2 = "+frq2);
                }
                long obs1 = Math.round(res1.oben); //sollte eh geradzahlig sein
                long obs2 = Math.round(res2.oben);
                double exp1 = frq1 * obsTotal;
                double exp2 = frq2 * obsTotal;
                //buf.append(exp1+"\t");
                //buf.append(exp2+"\t");
                double pval = Double.NaN;
                
                double[] expected = { exp1, exp2 };
                long[] observed = { obs1, obs2 };
                try {
					pval = TestUtils.chiSquareTest(expected, observed);
				} catch (IllegalArgumentException e) {
					logger.warn("can not test as input is invalid ",e);
				} catch (MathException e) {
					logger.warn("Chi Square failed",e);
				}
				if (pval<0) pval = pval * -1.0d;
                
                buf.append(pval+"\t");
                buf.append(res1.freq+"\t");
                buf.append(res1.frac+"\t");
                buf.append(res2.freq+"\t");
                buf.append(res2.frac+"\n");
            }
            
        }
        String enri = buf.toString();
        IEngineResults res1 = new EngineResults();
        try {
            res1.setResult(enri,property2.getId()+" | "+setA.getDescription(),setB.getDescription(),false,"Compare:symbolic:enrichment");
            String str3 = "<html>Enrichment of features that occur BOTH in '"+setB.getDescription()+"' and " +
            " in '"+setA.getDescription()+"'. <br>" +
                    "The frequencies are relative to the number of proteins that have at least a annotation feature of the choosen type.<br> " +
                    "The annotation feature analysed:<br>" +
                    "'"+property1.getId()+"' in '"+setA.getDescription()+"' and <br>" +
                    "'"+property2.getId()+"' in '"+setB.getDescription()+"'. <br>" +
                            "</html>";
            res1.setResultDescription(str3);
            allResults.add(res1);
         //   logger.debug(res3);
        } catch (Exception e) {
            logger.error("setting result BOTH of symbolic comparison failed ",e);
        }     
        
        
    }
    
    
    
   /* private void cmpSymbolicOLD(IGenericProperty property1, IGenericProperty property2) {
        //vergleiche unabhaengig von der prot_id
        
        Object[] val1 = property1.getValues().values().toArray();
        Object[] val2 = property2.getValues().values().toArray();
        
        HashMap onlyIn1 = new HashMap(); 
        HashMap onlyIn2 = new HashMap();
        HashMap inBoth = new HashMap();
        
        logger.debug("symbolic compare of property with id '"+property1.getId()+"'");

        
        logger.debug("property id '"+property1.getId()+"' values in both sets");
        //alle symbols die in beiden vorkommen
        for (int i = 0; i < val1.length; i++) {
            Object obj1 = val1[i];
            for (int j = 0; j < val2.length; j++) {
                Object obj2 = val2[j];
                if (obj1.equals(obj2)) inBoth.put(obj1,"");                   
            }
        }
        
        
        //relative hauefigenkeiten der symbole in beiden datensaetzen
        int anz1 = val1.length;
        int anz2 = val2.length;
        HashMap frq1 = new HashMap();  //symbol -> abs number
        HashMap frq2 = new HashMap();
        
        
        for (Object object : val1) {
            String symbol = (String) object;
            Integer anz = (Integer) frq1.get(symbol);
            if (anz==null) {
                frq1.put(symbol,new Integer(1));
            } else {
                frq1.put(symbol,new Integer(anz.intValue()+1));
            }
        }
        HashTools.divHashMap(frq1,anz1);
        //logger.debug("FRQ1\n"+HashTools.showHash(frq1));
        
        for (Object object : val2) {
            String symbol = (String) object;
            Integer anz = (Integer) frq2.get(symbol);
            if (anz==null) {
                frq2.put(symbol,new Integer(1));
            } else {
                frq2.put(symbol,new Integer(anz.intValue()+1));
            }
        }
        HashTools.divHashMap(frq2,anz2);        
        //logger.debug("FRQ2\n"+HashTools.showHash(frq2));
        
        HashMap dif = new HashMap();
        for (Object object : inBoth.keySet()) {
            String symbol = (String) object;
            double f1 = ((Double) frq1.get(symbol)).doubleValue();
            double f2 = ((Double) frq2.get(symbol)).doubleValue();
            double enrichment = f1-f2;
            dif.put(symbol,new Double(enrichment));
        }
        //logger.debug("DIFF\n"+HashTools.showHash(dif));
        
        StringBuffer sbBoth = new StringBuffer("Property\tEnrichment\n");
        for (Object object : dif.keySet()) {
            Double enr = (Double) dif.get(object);
            sbBoth.append(object+"\t"+enr+"\n");
        }
        
        IEngineResults res = new EngineResults();
        try {
            res.setResult(sbBoth.toString(),property1.getId()+" | "+setA.getDescription(),setB.getDescription(),false,"Compare:symbolic:inBoth");
            allResults.add(res);
            logger.debug(res);
        } catch (Exception e) {
            logger.error("setting result of symbolic comparison failed ",e);
        }
    }
    
    */
    
    /**
     * 
     */
    private static final long serialVersionUID = 1L;





    @Override
    public void run() throws Exception {
        IGenericProperty  propA = GenericHelper.getProperty(setA,IGenericProperty.SYMBOLIC);
        if (propA==null) return;
        IGenericProperty  propB = GenericHelper.getProperty(setB,IGenericProperty.SYMBOLIC);
        if (propB==null) return;
        allResults = new ArrayList();
        cmpSymbolic(propA,propB);
        
    }
    
    
    public IEngineResults[] getResultsAll() throws Exception {
        logger.debug("allResults.size():"+allResults.size());
        //nur results die nicht null sind sollen zurueck
         ArrayList tmp = new ArrayList();
        
         for (int i = 0; i < allResults.size(); i++) {
            IEngineResults curRes = (IEngineResults) allResults.get(i);
            if (!((curRes+"").equals("null")))  { //oh nein open source argl,
            //logger.debug("BB"+curRes.getData());
            tmp.add(curRes);
         
            }
         }
            logger.debug("tmp.size():"+tmp.size());
         IEngineResults[] results = new IEngineResults[tmp.size()];
         for (int j = 0; j < tmp.size(); j++) {
            results[j] = (IEngineResults) tmp.get(j);
        }
         
         return results;
    }

}
