/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: CorNumeric2.java,v 1.7 2006/03/19 11:58:08 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.genericxml;

import gr.zeus.ui.JMessage;

import java.net.URL;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JOptionPane;

import org.apache.commons.math.stat.descriptive.DescriptiveStatistics;
import org.apache.log4j.Logger;

import prompt.engine.Engine2;
import prompt.engine.EngineResults;
import prompt.engine.IEngineResults;
import prompt.input.IGenericProperty;
import prompt.input.IGenericXML;
import prompt.input.IInput;
import prompt.statistic.PearsonCorTest;
import tschmidt.utils.ValidLocation;



public class CorNumeric2 extends Engine2 {

    protected IGenericXML setA, setB;
   
    private static final long serialVersionUID = 1L;  
    
    
    public CorNumeric2() {
        super();
    }

    public void init(IInput setA, IInput setB) throws Exception {
        super.init(setA, setB);
        this.setA = (IGenericXML) setA;
        this.setB = (IGenericXML) setB;
    }
    

    

    

    
    public static IEngineResults corr(List valA, List valB, String descA,String descB,String comparisonName) throws Exception {
         Logger logger = Logger.getLogger(CorNumeric2.class.getName());
        StringBuffer sb = new StringBuffer("FIELD\tVALUE\n");

        double ksp = Double.NaN;
        String estimate = "";
        try {
            PearsonCorTest ks = new PearsonCorTest();
            ks.init(valA, valB);
            ks.run();
            ksp = ks.getPValue();
            estimate = (String) ks.getTestResults().get("$estimate");
        } catch (Exception e) {
            logger.debug("error testing whole distribution with KS ", e);
        }
        
        sb.append("Pearson_correlation\t" + estimate + "\n");
        sb.append("Pearson_correlation_test_pvalue\t" + ksp + "\n");
       
        sb.append("setA_Description\t\"" + descA + "\"\n");
        sb.append("setB_Description\t\"" + descB + "\"\n");

        try {
            DescriptiveStatistics stats = DescriptiveStatistics.newInstance();
            //      Add the data from the array
            for (int i = 0; i < valA.size(); i++) {
                double va = Double.parseDouble(valA.get(i) + "");
                if (va == Double.NaN) {
                    logger.warn(i + "-th entry is Not a Number (NaN)");
                    continue;
                }
                if (va == Double.POSITIVE_INFINITY) {
                    logger.warn(i + "-th entry is POSITIVE_INFINITY)");
                    continue;
                }
                if (va == Double.NEGATIVE_INFINITY) {
                    logger.warn(i + "-th entry is NEGATIVE_INFINITY)");
                    continue;
                }
                stats.addValue(va);
            }
            //      Compute some statistics 
            double mean = stats.getMean();
            double min = stats.getMin();
            double max = stats.getMax();
            double std = stats.getStandardDeviation();
            double median = stats.getPercentile(50);
            sb.append("setA_mean\t" + mean + "\n");
            sb.append("setA_std\t" + std + "\n");
            sb.append("setA_median\t" + median + "\n");
            sb.append("setA_min\t" + min + "\n");
            sb.append("setA_max\t" + max + "\n");
            

        } catch (Exception e) {
            logger.debug("error calculation descriptive stats for set A", e);
        }

        try {
            DescriptiveStatistics stats = DescriptiveStatistics.newInstance();
            stats = DescriptiveStatistics.newInstance();
            //   Add the data from the array
            for (int i = 0; i < valB.size(); i++) {

                double va = Double.parseDouble(valB.get(i) + "");
                if (va == Double.NaN) {
                    logger.warn(i + "-th entry is Not a Number (NaN)");
                    continue;
                }
                if (va == Double.POSITIVE_INFINITY) {
                    logger.warn(i + "-th entry is POSITIVE_INFINITY)");
                    continue;
                }
                if (va == Double.NEGATIVE_INFINITY) {
                    logger.warn(i + "-th entry is NEGATIVE_INFINITY)");
                    continue;
                }
                stats.addValue(va);

            }

            //   Compute some statistics 
            double mean = stats.getMean();
            double min = stats.getMin();
            double max = stats.getMax();
            double std = stats.getStandardDeviation();
            double median = stats.getPercentile(50);
            sb.append("setB_mean\t" + mean + "\n");
            sb.append("setB_std\t" + std + "\n");
            sb.append("setB_median\t" + median + "\n");
            sb.append("setB_min\t" + min + "\n");
            sb.append("setB_max\t" + max + "\n");
        } catch (Exception e) {
            logger.debug("error calculation descriptive stats for set B", e);
        }

       
        IEngineResults res = new EngineResults();
        res.setResult(sb.toString(), "", "", true, comparisonName + ":statvalues");
        //res.setDesc1(comparisonName + " : " + descA + " vs. " + descB);
        res.setDesc1(descA + " vs. " + descB);
        res.setDesc2("Pearson correlation and test");

        String desRes = null;
        URL url = CorNumeric2.class.getResource(CorNumeric2.class.getSimpleName() + "_whole.html");
        if (url == null) {
            desRes = "No description found :-(";
        }
        else desRes = ValidLocation.readFromUrl(url);
        res.setResultDescription(desRes);
        return res;
    }
    
    
    /**
     * @param propA
     * @param propB
     * @return IEngineResults[] 
     * 0: correlation tests and mean etc.
     * 1: list of all value pairs e.g. for plotting
     * @throws Exception
     */
    public static IEngineResults[] corNumeric(IGenericProperty propA, IGenericProperty propB,String descA,String descB, String comparisonName) throws Exception {
      
        Logger logger = Logger.getLogger(CorNumeric2.class.getName());
        
        HashMap hashA = propA.getValues();
        HashMap hashB = propB.getValues();
        
       // LinkedList listKeys = new LinkedList();
        LinkedList<Number> listA = new LinkedList();
        LinkedList<Number> listB = new LinkedList();

        StringBuilder sb = new StringBuilder();
        sb.append("entry\t"+propA.getId()+"\t"+propB.getId()+"\n");
        int nrFound = 0;
        
        for (Object key : hashA.keySet()) {
            if (key!=null) {
                Object valAObj = hashA.get(key);
                Object valBObj = hashB.get(key);
                if ((valAObj!=null) && (valBObj!=null)) {
                    Number numA = null;
                    try {
                        numA    = Double.parseDouble(valAObj+"");
                    } catch (NumberFormatException e) {
                        logger.warn(descA+": value of key='"+key+"' IS NOT A NUMBER");
                        continue;  
                    }
                    Number numB = null;
                    try {
                        numB    = Double.parseDouble(valBObj+"");
                    } catch (NumberFormatException e) {
                        logger.warn(descB+": value of key='"+key+"' IS NOT A NUMBER");
                        continue;  
                    }
                    //listKeys.add(key);
                    listA.add(numA);
                    listB.add(numB);
                    nrFound++;
                    sb.append(key+"\t"+numA+"\t"+numB+"\n");
                    
                    /*
                    boolean isNumA = Number.class.isAssignableFrom(valAObj.getClass());
                    boolean isNumB = Number.class.isAssignableFrom(valBObj.getClass());
                    if (!isNumA) {
                        logger.warn(setA.getDescription()+": value of key='"+key+"' IS NOT A NUMBER");
                        continue;
                    }
                    if (!isNumB) {
                        logger.warn(setB.getDescription()+": value of key='"+key+"' IS NOT A NUMBER");
                        continue;
                    }                    
                    */
                }
            }
        } // end for
        if (nrFound==0)  {
            String errmsg = "No protein codes found that occur in both properties<br>" +
                    "Only values with equal protein codes/identifiers can be correlated.";
            logger.fatal(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
            return null;
        }
        IEngineResults[] retData = new EngineResults[2];
        
        retData[0]=corr(listA,listB,descA,descB,comparisonName);
        URL url1 = CorNumeric2.class.getResource(CorNumeric2.class.getSimpleName() + "_statvalues.html");
        logger.debug("looking for result description: "+CorNumeric2.class.getSimpleName() + "_statvalues.html");
        
        String desRes1 = null;
        if (url1 == null) {
            desRes1 = "No description found :-(";
        }
        else { 
        	desRes1 = ValidLocation.readFromUrl(url1);
        	if ((desRes1!=null) && (desRes1.trim().length()>4)) {
				if (descA!=null) desRes1 = desRes1.replaceAll("\\$SETA", descA);
				if (descB!=null) desRes1 = desRes1.replaceAll("\\$SETB", descB);
			}
        }
        if (retData[0]!=null) retData[0].setResultDescription(desRes1);
        
        
        
        
        //here value pairs listen
        
        IEngineResults res = new EngineResults();
        res.setResult(sb.toString(), "", "", true, comparisonName + ":datapairs");
        //res.setDesc1(comparisonName + " : " + descA + " vs. " + descB);
        res.setDesc1(descA);
        res.setDesc2(descB);
        
        //res.setDesc2("Numeric correlation, list of pairs");

        String desRes = null;
      
        URL url = CorNumeric2.class.getResource(CorNumeric2.class.getSimpleName() + "_pairs.html");
        logger.debug("looking for result description: "+CorNumeric2.class.getSimpleName() + "_pairs.html");
        if (url == null) {
            desRes = "No description found :-(";
        }
        else { 
        	desRes = ValidLocation.readFromUrl(url);
        	if ((desRes!=null) && (desRes.trim().length()>4)) {
				if (descA!=null) desRes = desRes.replaceAll("\\$SETA", descA);
				if (descB!=null) desRes = desRes.replaceAll("\\$SETB", descB);
			}
        }
        res.setResultDescription(desRes);
        
        retData[1]= res;
        
        return retData;
        
    }
    
    IEngineResults[] res; 
    
    @Override
    public void run() throws Exception {
        //choose properties of A and B
        
        IGenericProperty  propA = GenericHelper.getProperty(setA,IGenericProperty.NUMERIC);
        if (propA==null) return;
        IGenericProperty  propB = GenericHelper.getProperty(setB,IGenericProperty.NUMERIC);
        if (propB==null) return;
        String comparisonName = this.getClass().getName();
        String descA = setA.getDescription()+":"+propA.getId();
        String descB = setB.getDescription()+":"+propB.getId();
        IEngineResults[] ret = corNumeric(propA,propB,descA,descB,comparisonName);
        if (ret==null) return;
        
        res= ret;
       
    }



    /* (non-Javadoc)
     * @see bea.engine.Engine#getResultsAll()
     */
    @Override
    public IEngineResults[] getResultsAll() throws Exception {
        return res;
    }

}
