/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: ExportFeatureToTxtFile.java,v 1.7 2006/03/19 11:58:09 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.genericxml;

import gr.zeus.ui.JMessage;

import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.HashMap;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import prompt.engine.Engine1;
import prompt.engine.IEngineResults;
import prompt.input.IGenericProperty;
import prompt.input.IGenericXML;
import prompt.input.IInput;
import prompt.ui.SaveUI;




/**
 * 
 * 
 * @author poweruser2
 *
 */
public class ExportFeatureToTxtFile extends Engine1 {
    protected IGenericXML set;

    private static final long serialVersionUID = 1L;

    String resultStr = null;

    public void init(IInput set) throws Exception {
        super.init(set);
        this.set = (IGenericXML) set;

    }



    @Override
    public void run() throws Exception {
        Logger logger = Logger.getLogger(ExportFeatureToTxtFile.class.getName());


        IGenericProperty propArray[] = GenericHelper.getProperties(set, IGenericProperty.ANY);
        if (propArray == null) {
            String errmsg = "Saving canceled (no feature(s) choosen)";
            logger.fatal(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
            return;
        }
        String destFile = SaveUI.save("Save properties to tab-delimited text file", ".txt");
        if (destFile == null) {
            String errmsg = "Saving canceled (no destination file choosen)";
            logger.fatal(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
            return;
        }
        FileOutputStream out; // declare a file output object
        PrintStream p; // declare a print stream object
        out = new FileOutputStream(destFile);
        // Connect print stream to the output stream
        p = new PrintStream(out);
        p.println("#dataset\t" + set.getDescription());
        p.println("");
        for (IGenericProperty prop : propArray) {
            this.desc2 = prop.getId();
            String typeLabel = "";
            if (prop.getType() == IGenericProperty.NUMERIC)
                typeLabel = "numeric";
            if (prop.getType() == IGenericProperty.SYMBOLIC)
                typeLabel = "symbolic";
            if (prop.getType() == IGenericProperty.SETDEF)
                typeLabel = "setdef";

            try {

                p.println(">property");
                p.println("id\t" + prop.getId());
                p.println("type\t" + typeLabel);
                
                HashMap<Object, Object> hash = prop.getValues();
                logger.debug("number of items in this property: " + hash.keySet().size());
                for (Object key : hash.keySet()) {
                    Object value = hash.get(key);
                    if (value != null) {
                        if (value instanceof String) {
                            String value2 = (String) value;
                            value2 = value2.replaceAll("\\\n", "");
                            //logger.debug("replaced");
                            p.println(key + "\t" + value2);
                        } else {
                            p.println(key + "\t" + value.toString());
                        }
                    }
                }
                p.println();

            } catch (Exception e) {
                String errmsg = "Error saving to file";
                logger.fatal(errmsg, e);
                JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION, e);
                return;
            }

        }
        p.close();
        out.close();
        String errmsg = "Exported " + propArray.length + " properties sucessfully to file '" + destFile + "'";
        logger.info(errmsg);
        JMessage.showMessageDialog(null, errmsg, JOptionPane.INFORMATION_MESSAGE, JOptionPane.DEFAULT_OPTION);
       

    }

    @Override
    public String toString() {
        return null;
    }

    @Override
    public IEngineResults getResults() throws Exception {
        return null;
    }

}
