/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id$
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.genericxml;

import gr.zeus.ui.JMessage;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import prompt.engine.Engine1;
import prompt.engine.IEngineResults;
import prompt.input.IGenericProperty;
import prompt.input.IGenericXML;
import prompt.input.IInput;
import prompt.ui.SaveUI;




/**
 * 
 * 
 * @author poweruser2
 *
 */
public class ExportFeatureToXMLFileOLDWITHJDOM extends Engine1 {
    protected IGenericXML set;

    private static final long serialVersionUID = 1L;

    String resultStr = null;

    public void init(IInput set) throws Exception {
        super.init(set);
        this.set = (IGenericXML) set;

    }



    @Override
    public void run() throws Exception {
        Logger logger = Logger.getLogger(ExportFeatureToXMLFileOLDWITHJDOM.class.getName());
        IGenericProperty propArray[] = GenericHelper.getProperties(set, IGenericProperty.ANY);
        if (propArray == null) {
            String errmsg = "Saving canceled (no feature(s) choosen)";
            logger.fatal(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
            return;
        }
        String destFile = SaveUI.saveDefault("Save properties to a Generic XML file", ".xml",set.getDescription());
        if (destFile == null) {
            String errmsg = "Saving canceled (no destination file choosen)";
            logger.fatal(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
            return;
        }
        Element[] vec = new Element[propArray.length];
        for (int i = 0; i < propArray.length; i++) {
            IGenericProperty prop = propArray[i];
            Element el = new Element("property");

            el.setAttribute("id", prop.getId());
            String typeLabel = "unkown";
            if (prop.getType() == IGenericProperty.NUMERIC)
                typeLabel = "numeric";
            if (prop.getType() == IGenericProperty.SYMBOLIC)
                typeLabel = "symbolic";
            if (prop.getType() == IGenericProperty.SETDEF)
                typeLabel = "setdef";
            el.setAttribute("type", typeLabel);
            vec[i] = el;

            //add data items
            //item id="gi_1786188" value="
            HashMap hash = prop.getValues();
            if (hash!=null) {
            for (Object key : hash.keySet()) {
                if (key == null)
                    continue;
                Object val = hash.get(key);
                String valS = val.toString();
                if (val == null)
                    valS = "";
                Element item = new Element("input");
                item.setAttribute("id", key + "");
                item.setAttribute("value", valS);
                el.addContent(item);
            }
            } else {
                String errmsg = "The property  '"+ prop.getId()+"' has no elements";
                logger.warn(errmsg);
                JMessage.showMessageDialog(null, errmsg, JOptionPane.WARNING_MESSAGE, JOptionPane.DEFAULT_OPTION);
            }
        }

        //1. erzeuge XML Documents
        Document doc = new Document();
        Element xmlRoot = new Element("dataset");
        String label = "";
        if (set.getLabel()!=null) label=set.getLabel();
        xmlRoot.setAttribute("label", label);
        doc.addContent(xmlRoot);

        //baue alle properties zusammen
        for (int j = 0; j < vec.length; j++) {
            Element element = vec[j];
            if (element != null)
                xmlRoot.addContent(element);
        }

        //save to file
        XMLOutputter outputter = new XMLOutputter();
        Format fmt = outputter.getFormat();
        fmt.setIndent("  ");
        fmt.setLineSeparator("\n");
        outputter.setFormat(fmt);
        FileOutputStream fs = null;
        try {
            logger.debug("save Generic XML file '" + destFile + "'");
            fs = new FileOutputStream(destFile);
        } catch (FileNotFoundException e) {
            String errmsg = "Output filename is invalid";
            logger.fatal(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION, e);
            return;
        }
        try {
            outputter.output(doc, fs);
        } catch (IOException e1) {
            String errmsg = "I/O Error saving Generic XML file '" + destFile + "'";
            logger.fatal(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION, e1);
            return;
        }
        if (fs!=null) fs.close();
        
        
        String errmsg = "Exported " + propArray.length + " properties sucessfully to file '" + destFile + "'";
        logger.info(errmsg);
        JMessage.showMessageDialog(null, errmsg, JOptionPane.INFORMATION_MESSAGE, JOptionPane.DEFAULT_OPTION);
        
    }

    @Override
    public String toString() {
        return null;
    }

    @Override
    public IEngineResults getResults() throws Exception {
        return null;
    }

}
