/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: ExportFeatureToXmlFile.java,v 1.4 2006/03/20 18:04:53 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.genericxml;

import gr.zeus.ui.JMessage;

import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;

import javax.swing.JOptionPane;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamWriter;

import org.apache.log4j.Logger;

import prompt.engine.Engine1;
import prompt.engine.IEngineResults;
import prompt.input.IGenericProperty;
import prompt.input.IGenericXML;
import prompt.input.IInput;
import prompt.ui.SaveUI;




/**
 * 
 * 
 * @author poweruser2
 *
 */
public class ExportFeatureToXmlFile extends Engine1 {
    protected IGenericXML set;

    private static final long serialVersionUID = 1L;

    String resultStr = null;
    boolean worked = false;

    public void init(IInput set) throws Exception {
        super.init(set);
        this.set = (IGenericXML) set;

    }

    String desFilename;
    IGenericProperty propArray[];

    
    public int IF_NOT_MAPPED_DO_SKIP = 1;
    public int IF_NOT_MAPPED_KEEP_OLD_ID =2;
    public int IF_NOT_MAPPED_ABORT = 3;
    public int IF_NOT_MAPPED_ASK_USER = 0;
    
    public int actionIfNotMapped = IF_NOT_MAPPED_ASK_USER;
    
    public String runResult ="PROPERTY\tACTION\tVALUE\n";
    
   
    
    @Override
    public void run() throws Exception {
        Logger logger = Logger.getLogger(ExportFeatureToXmlFile.class.getName());
        
        
        if ((propArray==null) || (propArray.length<1))  {
          propArray = GenericHelper.getProperties(set, IGenericProperty.ANY);
        }
        
        if ((propArray == null) || (propArray.length==0)) {
            String errmsg = "Canceled because no feature(s) was/were choosen)";
            logger.fatal(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
            return;
        }
        if (desFilename==null) {
        	 String vorschlag = "data";
        	 if (set!=null) vorschlag = set.getDescription();
        	 desFilename = SaveUI.saveDefault("Save properties to a Generic XML file", ".xml",vorschlag);
        }
        if (desFilename == null) {
            String errmsg = "Saving canceled (no destination file choosen)";
            logger.fatal(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
            return;
        }

        XMLOutputFactory xof =  XMLOutputFactory.newInstance(); 
		XMLStreamWriter xtw = null; 
		FileWriter fw;
		try {
			fw = new FileWriter(desFilename);
		} catch (IOException e) {
			String errmsg = "I/O Error saving Generic XML file '" + desFilename + "'";
            logger.fatal(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION, e);
            return;
		}
		
		
		xtw = xof.createXMLStreamWriter(fw);
		xtw.writeStartDocument("utf-8","1.0");
		xtw.writeStartElement("dataset");
		
		String label = "data";
        if (set!=null) {
        	if (set.getLabel()!=null) label=set.getLabel();
        }
		xtw.writeAttribute("label", label);
        
		String lastID = null;
        for (int i = 0; i < propArray.length; i++) {
            IGenericProperty prop = propArray[i];
            if (prop==null) continue;
            
            /*FIXME: Falls die aktuelle ID genauso ist wie die letzte dann nicht
             * nochmal eine neues Property tag aufmachen
             */
            
            String currentID = prop.getId();
            if (lastID!=null) { //not the first property
            	if (!lastID.equals(currentID)) { //letzte property ID != aktueller ID
            		xtw.writeEndElement();            
            	}
            }
            
            if ((lastID==null) || (!lastID.equals(currentID))) { 
            	//erste property oder neue ID
            	xtw.writeStartElement("property");

            	String typeLabel = "unkown";
            	if (prop.getType() == IGenericProperty.NUMERIC)
            		typeLabel = "numeric";
            	if (prop.getType() == IGenericProperty.SYMBOLIC)
            		typeLabel = "symbolic";
            	if (prop.getType() == IGenericProperty.SETDEF)
            		typeLabel = "setdef";
            
            	xtw.writeAttribute("id",prop.getId());
            	xtw.writeAttribute("type",typeLabel);
            }
			
			int anz_processed = 0;
			int anz_skipped = 0;
			int anz_mapped = 0;
            //add data items
            //item id="gi_1786188" value="
            HashMap hash = prop.getValues();
            if (hash!=null) {
            for (Object key : hash.keySet()) {
                if (key == null)
                    continue;
                Object val = hash.get(key);
                String valS = val.toString();
                if (val == null)
                    valS = "";
                anz_processed++;
               
				
                Object key2 = replaceIds(key);
          
                if (key2==null) {
                	int actionNow = actionIfNotMapped;
                	if (actionNow==IF_NOT_MAPPED_ASK_USER) {
                		String msg = "No mapping assignment for the id '"+key+" was found. How do you want to proceed?";
                		logger.debug(msg);
                		//dlg: skip, skipAlways, keepOldId, keepOldIdAlways, abort
                		
                		Object[] options = { "Skip", "Skip Always", "Keep old Id", "Keep old Id always", "Abort"};
                		int answ =  JOptionPane.showOptionDialog(null,
                	                msg, "Mapping",
                	                JOptionPane.YES_NO_CANCEL_OPTION,
                	                JOptionPane.QUESTION_MESSAGE, null, options,
                	                options[1]);
                		switch (answ) {
						case 0: actionNow=IF_NOT_MAPPED_DO_SKIP;
							break;
						case 1: actionNow=IF_NOT_MAPPED_DO_SKIP; actionIfNotMapped=IF_NOT_MAPPED_DO_SKIP;
							break;
						case 2: actionNow=IF_NOT_MAPPED_KEEP_OLD_ID; 
							break;	
						case 3: actionNow=IF_NOT_MAPPED_KEEP_OLD_ID; actionIfNotMapped=IF_NOT_MAPPED_KEEP_OLD_ID;
							break;
						case 4: actionNow=IF_NOT_MAPPED_ABORT;	
						default:
							break;
						}
                	}
                	if (actionNow==IF_NOT_MAPPED_ABORT) {
                		String errmsg = prop.getId()+" - ABORTED as no id replacement of element '"+key+"'='"+valS+"' is possible";
                		logger.warn(errmsg);
                		 JMessage.showMessageDialog(null, errmsg, JOptionPane.WARNING_MESSAGE, JOptionPane.DEFAULT_OPTION);
                		return;
                	}
                	if (actionNow==IF_NOT_MAPPED_DO_SKIP) {
                		anz_skipped++;
                		logger.debug(prop.getId()+" - SKIPPED id replacement of element '"+key+"'='"+valS+"' ");
                		continue;
                	}
                	if (actionNow==IF_NOT_MAPPED_KEEP_OLD_ID) {
                		logger.debug(prop.getId()+" - KEPT old id of element '"+key+"'='"+valS+"' ");
                		key2=key;
                	}
                }
                xtw.writeStartElement("input");
                if (key2!=key) anz_mapped++; //da ansonsten sind die zeiger gleich
                xtw.writeAttribute("id",key2 + "");
				xtw.writeAttribute("value",valS);
				xtw.writeEndElement();
				
				
            }
            
            } else {
                String errmsg = "The property  '"+ prop.getId()+"' has no elements";
                logger.warn(errmsg);
                JMessage.showMessageDialog(null, errmsg, JOptionPane.WARNING_MESSAGE, JOptionPane.DEFAULT_OPTION);
            }
            runResult= runResult+prop.getId()+"\t"+"Processed\t"+anz_processed+"\n";
            runResult= runResult+prop.getId()+"\t"+"Mapped\t"+anz_mapped+"\n";
			runResult= runResult+prop.getId()+"\t"+"Skipped\t"+anz_skipped+"\n";
			
			lastID=currentID;
			
			
        }
        //close last open property
        xtw.writeEndElement();
        
    	xtw.writeEndElement();
		xtw.writeEndDocument();
		xtw.flush();
		xtw.close(); 
		fw.close();
		
		
		if (showSuccessMsg) {
		  //HMM diese msg wollen wir nur wenn wir es direkt aus dem men aufrufen
          String errmsg = "Exported " + propArray.length + " properties sucessfully to file '" + desFilename + "'";
          logger.info(errmsg);
          JMessage.showMessageDialog(null, errmsg, JOptionPane.INFORMATION_MESSAGE, JOptionPane.DEFAULT_OPTION);
		}
        worked = true;
      
    }

    public boolean showSuccessMsg = true;
    
    
    public Object replaceIds(Object id){
    	return id;
    }
    
    @Override
    public String toString() {
    	if (worked)
        return runResult;
    	else return null;
    }

    @Override
    public IEngineResults getResults() throws Exception {
        return null;
    }

    
	public String getFilename() {
		return desFilename;
	}



	/**
	 * @param filename der XML Datei in das die Resultdaten gespeichert werden
	 * sollen. Wird in der RUN Methode abgefragt. Ist noch kein Filename
	 * gesetzt dann wird der Benutzer mittels GUI Dialog gefragt.
	 */
	public void setFilename(String filename) {
		this.desFilename = filename;
	}

	public IGenericProperty[] getPropArray() {
		return propArray;
	}

	/**
	 * Gibt die XML Properties bekannt die exportiert werden sollen. <br>
	 * Falls diese Setter Methode nicht aufgerufen wird dann wird der
	 * Benutzer gefragt welche Properties aus dem Dataset das er bei 
	 * der Init Methode angegeben hat exportieren will.<br>
	 * <b>Beispiel um ALLE Properties zu exportieren:</b><br>
	 * setPropArray(prompt.engine.genericxml.GenericHelper.getProperties(set,IGenericProperty.ANY))
	 * @param propArray IGenericProperty[]
	 */
	public void setPropArray(IGenericProperty[] propArray) {
		this.propArray = propArray;
	}



	public int getActionIfNotMapped() {
		return actionIfNotMapped;
	}



	public void setActionIfNotMapped(int actionIfNotMapped) {
		this.actionIfNotMapped = actionIfNotMapped;
	}
    
	/**
	 * @return true, if the last run() finished successfully
	 */
	public boolean hasWorked() {
		return worked;
	}
}
