/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id$
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.genericxml;

import gr.zeus.ui.JMessage;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import prompt.input.IGenericProperty;
import prompt.input.IGenericXML;
import prompt.input.ISeqProperties;
import prompt.ui.CheckBoxDlg;
import prompt.ui.DialogFactory;
import tschmidt.iter.TupelIter;


public class GenericHelper implements Serializable {
    private static final long serialVersionUID = 1L;

    public static HashMap makeHash(ISeqProperties backref, List listValues) {
        Logger logger = Logger.getLogger(GenericHelper.class);
        HashMap hash = new HashMap();
        TupelIter inputA = new TupelIter(backref.getSequencesNames(), listValues);
        int i = 0;
        while (inputA.hasNext()) {
            i++;
            Object[] entry = (Object[]) inputA.next();
            if (entry==null) {
                logger.warn("entry "+i+" is null");
                continue;
            }
            if (entry.length<2) {
                logger.warn("entry "+i+" is needs at least two elements> id, and sequence");
                continue;
            }
            Object id = entry[0];
            if (id==null) {
                logger.warn("entry "+i+" has a NULL id,  this annotation will be ignored");
                continue;
            }
            Object seq = entry[1];
            if (seq==null) {
              //  logger.warn("entry "+i+" has a NULL value, this annotation will be ignored");
                continue;
            }      
            hash.put(id,seq);
        }
        return hash;
    }
    public static IGenericProperty[] mergePropertyArrays(IGenericProperty[] arrayA, IGenericProperty[] arrayB) {
        if (arrayA==null) return arrayB;
        if (arrayB==null) return arrayA;
        
        int sizeA = arrayA.length;
        int sizeB = arrayB.length;
        if ((sizeA==0) && (sizeB>0)) return arrayB;
        if ((sizeB==0) && (sizeA>0)) return arrayA;
        if ((sizeA==0) && (sizeB==0)) return arrayA;
        
        //merge
        IGenericProperty[] both = new IGenericProperty[sizeA+sizeB];
        for (int i = 0; i < sizeA; i++) {
            both[i]=arrayA[i];
        }
        for (int i = 0; i < sizeB; i++) {
            both[sizeA+i]=arrayB[i];
        }        
        return both;
    }
    
    static public Ret getPropsAvailable(IGenericXML set, int type) {
        Logger logger = Logger.getLogger(GenericHelper.class.getName());
        String typeLabel = "";
        if (type == IGenericProperty.NUMERIC)
            typeLabel = "numeric";
        if (type == IGenericProperty.SYMBOLIC)
            typeLabel = "symbolic";

        if (set == null) {
            String errmsg = "dataset is null";
            logger.error(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.WARNING_MESSAGE, JOptionPane.DEFAULT_OPTION);
            return null;
        }
        IGenericProperty[] properties = set.getProperties();
        if (properties.length == 0) {
            String errmsg = "No properties at all in the set '" + set.getDescription() + "' found";
            logger.error(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.WARNING_MESSAGE, JOptionPane.DEFAULT_OPTION);
            return null;
        }

        Ret ret = new Ret();

        for (IGenericProperty property : properties) {
            logger.debug("found property: '" + property.getId() + " type=" + property.getType());
            //ANY
            //ANY_NUM_OR_SYMB
            //SETDEF
            if (type == IGenericProperty.ANY) {
                ret.props.put(property.getId(), property);
                ret.pNamesA.add(property.getId());
                continue;
            }

            if ((type == IGenericProperty.SETDEF) || (type == IGenericProperty.NUMERIC) || (type == IGenericProperty.SYMBOLIC)) {
                if (property.getType() == type) {
                    ret.props.put(property.getId(), property);
                    ret.pNamesA.add(property.getId());
                }
                continue;
            }

            if (type == IGenericProperty.ANY_NUM_OR_SYM) {
                if ((property.getType() == IGenericProperty.NUMERIC) || (property.getType() == IGenericProperty.SYMBOLIC)) {
                    ret.props.put(property.getId(), property);
                    ret.pNamesA.add(property.getId());
                }
            }

        }
        if (ret.pNamesA.size() == 0) {
            String errmsg = "No " + typeLabel + " properties in the set '" + set.getDescription() + "' found";
            logger.error(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.WARNING_MESSAGE, JOptionPane.DEFAULT_OPTION);
            return null;
        }
        return ret;

    }

    public static IGenericProperty[] getProperties(IGenericXML set, int type) {
        Logger logger = Logger.getLogger(GenericHelper.class.getName());
        Ret ret = getPropsAvailable(set, type);
        
        boolean[] choosen = CheckBoxDlg.showDlg(ret.pNamesA.toArray());
        if (choosen==null ){
            String errmsg = "Canceled by User";
            logger.error(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.WARNING_MESSAGE, JOptionPane.DEFAULT_OPTION);
            return null;
        }
        
        //baue property return vector
        ArrayList<IGenericProperty> al = new ArrayList<IGenericProperty>();
        for (int i = 0; i < choosen.length; i++) {
            boolean takeit = choosen[i];
            if (takeit) {
                IGenericProperty p = ret.props.get(ret.pNamesA.getObjectAt(i));
                if (p!=null) al.add(p);
            }
        }
        IGenericProperty[] res = new IGenericProperty[al.size()];
        for (int i = 0; i < al.size(); i++) {
            res[i] = al.get(i);
        }
        return res;
    }

    public static IGenericProperty getProperty(IGenericXML set, int type) {
        Logger logger = Logger.getLogger(GenericHelper.class.getName());
        Ret ret = getPropsAvailable(set, type);
        if (ret==null) return null;
        String typeLabel = "";
        if (type == IGenericProperty.NUMERIC)
            typeLabel = "numeric";
        if (type == IGenericProperty.SYMBOLIC)
            typeLabel = "symbolic";

        if (ret.pNamesA.size()==1) {
        	logger.debug("only one choice, that will be returned automatically");
        	 return ret.props.get(  ret.pNamesA.getAt(0)  );
        	
        }
        
        String choosen = DialogFactory.chooseDlg(set.getDescription(), "Choose a  " + typeLabel + "  property in '" + set.getDescription() + "'",
                ret.pNamesA.toArray(), ret.pNamesA.getAt(0));
        if (choosen == null) {
            String errmsg = "Canceled by User";
            logger.error(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.WARNING_MESSAGE, JOptionPane.DEFAULT_OPTION);
            return null;
        }

        return ret.props.get(choosen);
    }
}


