/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: SubsetVsPop.java,v 1.6 2006/03/19 11:58:08 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.genericxml;

import gr.zeus.ui.JMessage;

import java.util.HashMap;
import java.util.List;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import prompt.engine.Engine2;
import prompt.engine.IEngineResults;
import prompt.engine.genemerge.GeneMerge;
import prompt.input.IGenericProperty;
import prompt.input.IGenericXML;
import prompt.input.IInput;
import prompt.ui.DialogFactory;
import tschmidt.utils.HashTools;



public class SubsetVsPop extends Engine2 {

    private Logger logger = Logger.getLogger(this.getClass());

    /**
     * Input instances
     */
    protected IGenericXML study, pop;
    
    /* (non-Javadoc)
     * @see bea.engine.IEngine2#init(prompt.input.IInput, prompt.input.IInput)
     */
    public void init(IInput setA, IInput setB) throws Exception {
        super.init(setA, setB);
        this.study = (IGenericXML) setA;
        this.pop = (IGenericXML) setB;
        //Better: check if ICath and ISeqProperties Interfaces are implemented
    }
    
    /**
     * 
     */
    private static final long serialVersionUID = 1L;

    String choosenSet;



    
    GeneMerge gm = new GeneMerge();
    
    @Override
    public void run() throws Exception {
        // TODO check init values
        
        List<String> listPop = pop.getSequencesNames();
        List<String> listStudy = study.getSequencesNames();
        if ((listPop==null) || (listPop.size()==0)) {
            String errmsg = "XML Set '"+pop.getDescription()+"' does NOT has any set definition<br>No setdef property found";
            logger.info(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.WARNING_MESSAGE, JOptionPane.DEFAULT_OPTION);
            return;
        }
        if ((listStudy==null) || (listStudy.size()==0)) {
            String errmsg = "XML Set '"+study.getDescription()+"' does NOT has any set definition<br>No setdef property found";
            logger.info(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.WARNING_MESSAGE, JOptionPane.DEFAULT_OPTION);
            return;
        }
        if (choosenSet==null) {
         choosenSet = DialogFactory.chooseDlg("Subset/Population","Choose the smaller sub set that will be analysed",new String[]{study.getDescription(),pop.getDescription()},study.getDescription());
        }
        if (choosenSet==null) {
        	String errmsg = "Canceled by user: Not choosen which set is the sub set";
            logger.info(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.WARNING_MESSAGE, JOptionPane.DEFAULT_OPTION);
            return;
        }
         if (!choosenSet.equals(study.getDescription())) {
            List<String> listTmp = listStudy; 
            IGenericXML tmp = study;
            
            study = pop;
            listStudy = listPop;
            
            pop = tmp;
            listPop= listTmp;
        }
       
        IGenericProperty  propB = GenericHelper.getProperty(pop,IGenericProperty.SYMBOLIC);
        if (propB==null) return;
        
        HashMap ids = propB.getValues();
        String popAnnoIds= HashTools.showHashPlain(ids);
        logger.debug("Number of described proteins in the pop set: "+ids.keySet().size());
        
        StringBuilder buf1 = new StringBuilder();
        
        for (String string : listPop) {
            buf1.append(string+"\n");
        }
        String popIds = buf1.toString();
        buf1=null;
        logger.debug("Number of POP-IDS: "+listPop.size());
        
        StringBuilder buf2 = new StringBuilder();
        
        for (String string : listStudy) {
            buf2.append(string+"\n");
        }
        String studyIds = buf2.toString();
        logger.debug("Number of STUDY-IDS: "+listStudy.size());
        buf2=null;
        
        
        gm.initInMem(popAnnoIds,null,popIds,studyIds);
        gm.setDesc1(study.getDescription());
        gm.setDesc2(pop.getDescription());
        gm.label_association=propB.getId();
        gm.label_resulttype=this.getClass().getSimpleName();
        gm.label_description="";
        gm.label_pop=pop.getDescription();
        gm.label_study=study.getDescription();
        gm.run();
    }
    
    @Override
    public IEngineResults[] getResultsAll() throws Exception {
        return gm.getResultsAll();
    }

	public String getChoosenSet() {
		return choosenSet;
	}

	/**
	 * Damit legen wir fest welches Input Objekt die Subset Menge
	 * ist.
	 * @param choosenSet String, getDescription() of the subset object
	 */
	public void setChoosenSet(String choosenSet) {
		this.choosenSet = choosenSet;
	}


}
