/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: Txt2Xml.java,v 1.10 2006/04/05 11:07:46 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.genericxml;

import gr.zeus.ui.JMessage;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import javax.swing.JOptionPane;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamWriter;

import org.apache.log4j.Logger;

import prompt.engine.Engine;
import prompt.ui.SaveUI;
import tschmidt.iter.FileLineIter;
import tschmidt.utils.SimpleHashInteger;


public class Txt2Xml extends Engine {

    private static final long serialVersionUID = 1L;

    boolean worked = false;
    
    boolean foundProperty(String line) {
        if (line.trim().equalsIgnoreCase(">property")) return true;
        return false;
    }
    
    boolean foundId(String line) {
        if (line.toLowerCase().trim().startsWith("id")) return true;
        return false;
    }
    
    boolean foundType(String line) {
        if (line.toLowerCase().trim().startsWith("type")) return true;
        return false;
    }
    
    String[] split(String line, long lineNr) {
        if (line==null){ // should not happen
            fatal("NULL line",lineNr);
            return null;
        }
        if (line.trim().length()==0){
            warn("empty line found",lineNr);
            return null;
        }
        String[]ret = line.trim().split("\\\t");
        if (ret==null) {
            fatal("No tab char found!<br>The line must contain two colums seperated by a tabulator character",lineNr);
            return null;
        }
        if (ret.length<2) {
            fatal("The line must contain two colums seperated by a tabulator character",lineNr);
            return null;
        }
        if (ret.length>2) { 
            warn("More than two columns found, the others will be ignored and only the first 2 cols processed",lineNr);
        }
        String col1 = ret[0];
        if ((col1==null) || (col1.trim().length()==0)) {
            fatal("No characters before the tabulator character",lineNr);
            return null;
        }
        String col2 = ret[1];
        if ((col2==null) || (col2.trim().length()==0)){
            fatal("No characters in the second colum found",lineNr);
            return null;
        }
        ret[0]=ret[0].trim();
        ret[1]=ret[1].trim();
        return ret;
    }
    
    boolean foundBlank(String line) {
        if (line==null) return true;
        if (line.trim().startsWith("#")) return true;  
        if (line.trim().length()==0) return true;
        return false;
    }
    
    void warn(String str,long lineNr) {
        Logger logger = Logger.getLogger(Txt2Xml.class.getName());
        String errmsg = "WARNING AT LINE "+lineNr+": <br>"+str;
        logger.warn(errmsg);
        JMessage.showMessageDialog(null, errmsg, JOptionPane.WARNING_MESSAGE, JOptionPane.DEFAULT_OPTION);
        //TODO: change to dlg were the user can cancel
    }
    
    void fatal(String str,long lineNr) {
        Logger logger = Logger.getLogger(Txt2Xml.class.getName());
        String errmsg = "FATAL ERROR AT LINE "+lineNr+":<br> "+str;
        logger.fatal(errmsg);
        JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
    }
    
    String destFileName = null;
    String srcFileName = null;
    
    @Override
    public void run() throws Exception {
        Logger logger = Logger.getLogger(Txt2Xml.class.getName());
        worked = false;
        

        
        if (srcFileName==null) {
        srcFileName = SaveUI.load("Load a TEXT file", ".txt");
        }
        if (srcFileName == null) {
            String errmsg = "Converting canceled (no src file choosen)";
            logger.fatal(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
            return;
        }
        File srcFile = new File(srcFileName);
        if (!srcFile.canRead()) {
            String errmsg = "Can not read file. Check filename,access and permissions";
            logger.fatal(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
            return; 
        }
        
        if (destFileName==null) {
            destFileName = SaveUI.save("Save data to GenericXML file",".xml");
           }
           if (destFileName == null) {
               String errmsg = "Converting canceled (destination file not choosen)";
               logger.fatal(errmsg);
               JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
               return;
           }
        XMLOutputFactory xof =  XMLOutputFactory.newInstance(); 
	

        XMLStreamWriter xtw = null; 
		FileWriter fw;
		try {
			fw = new FileWriter(destFileName);
		} catch (IOException e) {
			String errmsg = "I/O Error saving Generic XML file '" + destFileName + "'";
			logger.fatal(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION, e);
            return;
		}
		xtw = xof.createXMLStreamWriter(fw);
		xtw.writeStartDocument("utf-8","1.0");
		xtw.writeStartElement("dataset");
		xtw.writeAttribute("label", srcFile.toString());
		
        SimpleHashInteger inputsFound = new SimpleHashInteger();
       
        byte state = -1;
        long linenr = 0;
        boolean someDataFound = false;
        boolean propertyOpen = false;
        String curPropertyName = null;
        long lastPoint = System.currentTimeMillis();
        long linesToPoint = 0;
       
        try {
            FileLineIter fiter = new FileLineIter(srcFileName);
            //Element curProp = null;
            while (fiter.hasNext()) {
                String line = (String) fiter.next();
                linenr++;
                linesToPoint++;
                if (linesToPoint>200) { //alle 200 Zeilen schauen wir ob es Zeit ist fuer eine Info msg
                	
                	if ((System.currentTimeMillis()-lastPoint)>3000)  {
                		logger.info("read until line number "+linenr); 
                		lastPoint = System.currentTimeMillis();
                	}
                	linesToPoint=0;
                }
                
                
                if ((state==-1) || (state==0)) {
                    if (foundProperty(line)) {
                        if (propertyOpen) {
                        	//vorheriges offenes schliessen
                        	xtw.writeEndElement();
                        }
                        xtw.writeStartElement("property");
                        propertyOpen = true;
                         
                        state=1;
                        continue;
                    }
                    if (foundId(line)) {
                        fatal("ID line found, but no preceding 'PROPERTY' line found yet!<br>" +
                                "Each property description must start with a PROPERTY line first, afterwards the ID and TYPE line must follow.<br>" +
                                "The PROPERTY line must contain simply '>property' alone in one line",linenr);
                        return;
                    }
                    if (foundType(line)) {
                        fatal("TYPE line found, but no preceding 'PROPERTY' line found yet!<br>" +
                                "Each property description must start with a PROPERTY line first, afterwards the ID and TYPE line must follow.<br>" +
                                "The PROPERTY line must contain simply '>property' alone in one line",linenr);
                        return;
                    }
                    if (foundBlank(line)) continue;
                    if (!propertyOpen) {
                        warn("Some data found, but no preceding 'PROPERTY' line found before!<br>" +
                            "<b>This input line will be ignored!</b><br>"+
                            "Each property description must start with a PROPERTY line first, afterwards the ID and TYPE line must follow.<br>" +
                            "The PROPERTY line must contain simply '>property' alone in one line<br>" +
                            "Comments must start with an # at the first position",linenr);
                        continue;
                    }
                    //property wurde zuvor initialisiert, jetzt fuegen wir Werte hinzu
                    String[] cols = split(line,linenr);
                    if (cols==null) continue;
                    
                    /*TODO: check if we have this id within this property block already,
                     * if yes then add it as value (seperated by strichpunkt)
                     */
                
                    xtw.writeStartElement("input");
	                
                    xtw.writeAttribute("id",cols[0]);
					xtw.writeAttribute("value",cols[1]);
					xtw.writeEndElement();	
					inputsFound.inc(curPropertyName);
                    someDataFound=true;
//                    Object old = ids.put(cols[0],Long.valueOf(linenr));
//                    if (old!=null) {
//                    	logger.debug("found id '"+cols[0]+"' already at line "+linenr);
//                    }

                }  //end state 0
                
                if (state==1) {
                    if (foundId(line)) {
                        //split
                        String[] cols = split(line,linenr);
                        if (cols==null) continue;
                        xtw.writeAttribute("id",cols[1]);
                        curPropertyName = cols[1];
                        state = 2;
                        continue;
                    }
                    if (foundProperty(line)) {
                        fatal("Another PROPERTY line found, but the preceding PROPERTY description is not complete yet<br>" +
                                "After the last PROPERTY line at least an ID and TYPE line must follow.<br>" 
                                ,linenr);
                        return;
                    }
                    if (foundType(line)) {
                        fatal("TYPE line found, but ID line expected first<br>" +
                                "After the last PROPERTY first the ID line must follow.<br>" 
                                ,linenr);
                        return;
                    }                   
                    if (foundBlank(line)) continue;
                    warn("Some data found, but the ID line is expected<br>" +
                            "<b>This input line will be ignored!</b><br>"+
                            "After the PROPERTY line an ID line must follow<br>" +
                            "The ID line must simply start with 'ID' followed by the identifier description,<br> seperated with a tabulator character<br>",linenr);
                }
                
                if (state==2) {
                    if (foundType(line)) {
                        //split
                        String[] cols = split(line,linenr);
                        if (cols==null) continue;
                        String type = cols[1];
                        String type2 = null;
                        if (type.equalsIgnoreCase("numeric")) type2="numeric";
                        if (type.equalsIgnoreCase("symbolic")) type2="symbolic";
                        if (type.equalsIgnoreCase("setdef")) type2="setdef";
                        if (type2==null) {
                            fatal("Unknown type '"+type+"' found.<br>" +
                                    "Only 'setdef', 'numeric' or 'symbolic' is allowed as type definition",linenr);
                            return;
                        }                   
                        xtw.writeAttribute("type",type2);
                        state = 0;
                        continue;
                    }
                    if (foundProperty(line)) {
                        fatal("Another PROPERTY line found, but the preceding PROPERTY description is not complete yet<br>" +
                                "After the last PROPERTY and ID line a TYPE must still follow.<br>" 
                                ,linenr);
                        return;
                    }
                    if (foundId(line)) {
                        fatal("Another ID line found, but a TYPE line is expected<br>" +
                                "After an ID line a TYPE must always follow<br>" 
                                ,linenr);
                        return;
                    }
                    if (foundBlank(line)) continue;
                    warn("Some data found, but the TYPE line is expected<br>" +
                            "<b>This input line will be ignored!</b><br>"+
                            "After the ID line a TYPE line must follow<br>" +
                            "The TYPE line must simply start with 'TYPE' followed by 'setdef', 'numeric' or 'symbolic',<br> seperated with a tabulator character<br>",linenr);

                }

                
                
            } 
            fiter.close();
            logger.info("Finished parsing the TEXT file");
        } catch (Exception ex) {
            String errmsg = "Error parsing the TEXT file '" + srcFileName + "'";
            logger.error(errmsg, ex);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION, ex);
        }
        if (state==-1) {
            String errmsg = "No PROPERTY line at all found.<br>At least one property is needed in the text file." ;
            logger.error(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);  
            return;  
        }
        if (state==1) {
            String errmsg = "INCOMPLETE last property block. <BR>" +
                    "Although a PROPERTY line was found the following ID line is missing" ;
            logger.error(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);  
            return;  
        }        
        if (state==2) {
            String errmsg = "INCOMPLETE last property block. <BR>" +
                    "Although a PROPERTY and ID line was found the following TYPE line is missing" ;
            logger.error(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);  
            return;  
        }    
        if (!someDataFound) {
            String errmsg = "Sorry, although the syntax is of the file '" + srcFileName + "' is correct no annotation data was found";
            logger.error(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);  
            return; 
        }
        
        
       
        if (propertyOpen) {
        	//vorheriges offenes property schliessen
        	xtw.writeEndElement();
        }
    	xtw.writeEndDocument();
		xtw.flush();
		xtw.close(); 
		logger.info("finished output "+destFileName);
		
        worked = true;
        for (Object str : inputsFound.keySet()) {
			logger.info("found in property '"+str+"': "+inputsFound.get(str)+" input elements");
		}
        
        
        String errmsg = "Reading from text file finished successfully";
        logger.info(errmsg);
        JMessage.showMessageDialog(null, errmsg, JOptionPane.INFORMATION_MESSAGE, JOptionPane.DEFAULT_OPTION);
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return null;
    }

	public String getDestFileName() {
		return destFileName;
	}

	public void setDestFileName(String destFileName) {
		this.destFileName = destFileName;
	}

	public String getSrcFileName() {
		return srcFileName;
	}

	public void setSrcFileName(String srcFileName) {
		this.srcFileName = srcFileName;
	}

	/**
	 * @return true, if the last run() finished successfully
	 */
	public boolean hasWorked() {
		return worked;
	}

}
