/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: Weka2Xml.java,v 1.8 2006/04/05 11:07:46 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.genericxml;

import gr.zeus.ui.JMessage;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;

import javax.swing.JOptionPane;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamWriter;

import org.apache.log4j.Logger;

import prompt.engine.Engine;
import prompt.ui.SaveUI;
import tschmidt.iter.FileLineIter;
import weka.core.Attribute;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.converters.ArffLoader;

public class Weka2Xml extends Engine {

	private static final long serialVersionUID = 1L;

	String srcFileName = null;

	String destFileNamePrefix = null;

	String[] resultFiles = null;

	boolean worked = false;

	
	void buildXML(File vec[],HashMap<String, String> attrTypes, String desFilename) throws Exception {  
		Logger logger = Logger.getLogger(Weka2Xml.class.getName());
	  //lese alle files von dieser classe
	        XMLOutputFactory xof =  XMLOutputFactory.newInstance(); 
			XMLStreamWriter xtw = null; 
			FileWriter fw;
			try {
				fw = new FileWriter(desFilename);
			} catch (IOException e) {
				String errmsg = "I/O Error saving Generic XML file '" + desFilename + "'";
	            logger.fatal(errmsg);
	            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION, e);
	            return;
			}
			
			xtw = xof.createXMLStreamWriter(fw);
			xtw.writeStartDocument("utf-8","1.0");
			xtw.writeStartElement("dataset");
			String label = "weka2xml";
			xtw.writeAttribute("label", label);
			
			//lese alle attribut files hintereinander und schreibe sie in die XML datei
			for (int i = 0; i < vec.length; i++) {
				File file = vec[i];
				logger.debug("read now "+file.getName());
				//lese lines and add values
				FileLineIter iter = new FileLineIter(file);
				int lineNr = 0;
				while (iter.hasNext()) {
					String line = (String) iter.next();
					if (lineNr==0) {
						xtw.writeStartElement("property");
						String id = line.trim();
		            	String typeLabel = attrTypes.get(line);
		            	logger.debug("property id='"+id+"' type='"+typeLabel+"' ");
		            
		            	xtw.writeAttribute("id",id);
		            	xtw.writeAttribute("type",typeLabel);	
					}
					else {
						xtw.writeStartElement("input");
		                xtw.writeAttribute("id",lineNr + "");
						xtw.writeAttribute("value",line);
						xtw.writeEndElement();	
					}
					lineNr++;
				} //while
				xtw.writeEndElement(); //end property
				iter.close();
			} //for over all attributes/properties
			
			//close file
			
			xtw.writeEndDocument();
			xtw.flush();
			xtw.close(); 
			fw.flush();
			fw.close();
			logger.info("finished output "+desFilename);
	}
		
	@Override
	public void run() throws Exception {
		worked = false;
		Logger logger = Logger.getLogger(Weka2Xml.class.getName());
		if (srcFileName == null) {
			srcFileName = SaveUI.load("Load a WEKA arff file", ".arff");
		}
		if (srcFileName == null) {
			String errmsg = "Converting canceled (no src file choosen)";
			logger.fatal(errmsg);
			JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
			return;
		}

		File srcFile = new File(srcFileName);
		File[][] vec = null;
		FileWriter[][] vecWriter = null;

		try {
			ArffLoader atf = new ArffLoader();
			atf.setSource(srcFile);

			/*
			 * 1. get class attributes, ask filename for each class attrib if
			 * class attrib is numeric -> abort
			 * 
			 */
			Instances struc = atf.getStructure();
			logger.debug(struc);
			int numAttr = struc.numAttributes();
			if (numAttr <= 1) {
				String errmsg = "Not enough attributes in the WEKA file. <br>You need a class attribute and at least one feature attribute.";
				logger.fatal(errmsg);
				JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
				return;
			}
			int clIndex = struc.classIndex();
			if (clIndex == -1) {
				// use weka default the last one
				clIndex = numAttr - 1; // erstes Attribut hat den Index 0
				struc.setClassIndex(clIndex);
				logger.debug("set class attribute to index nr " + clIndex);

			}
			if (struc.checkForStringAttributes()) {
				logger.warn("Attention STRING attributes found. They will not converted");
			}
			Attribute clAttr = struc.classAttribute();
			if (!clAttr.isNominal()) {
				String errmsg = "The last class attribute '" + clAttr.name() + "' must be of the type nominal";
				logger.fatal(errmsg);
				JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
				return;
			}
			int clNumValues = clAttr.numValues();
			if (clNumValues == 0) {
				String errmsg = "The last class attribute '" + clAttr.name() + "' needs at least one value";
				logger.fatal(errmsg);
				JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
				return;
			}

			vec = new File[clNumValues][numAttr - 1]; // ohne class attribut
			// [classes][attributes] //
			vecWriter = new FileWriter[clNumValues][numAttr - 1];

			HashMap<String, String> attrTypes = new HashMap<String, String>();
			//attrName -> Type

			logger.info("Number of different classes declared " + clNumValues);
			logger.info("Number of attributes declared " + numAttr);

			//set up the output file names
			resultFiles = new String[clNumValues];
		    for (int i = 0; i < resultFiles.length; i++) {
		     	String destFileName = null;
            	if (destFileNamePrefix==null) {
            		//ask user about the output file
            		destFileName= SaveUI.save("Save data of '" + clAttr.name() + "'='" + clAttr.value(i) + "'", ".xml");
            	} else {
            		destFileName=destFileNamePrefix+"_"+clAttr.name() + "_" + clAttr.value(i);
            	}
                
                if (destFileName == null) {
                    String errmsg = "Converting canceled (destination file not choosen)";
                    logger.fatal(errmsg);
                    JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
                    return;
                }

                resultFiles[i] = destFileName;
                logger.debug("file " + i + " is '" + destFileName + "'");	
			}
                
           
            
			
			
			// Create tmp files (for large data), each row one value
			for (int i = 0; i < clNumValues; i++) {
				for (int j = 0; j < numAttr - 1; j++) {

					Attribute wekaAttr = struc.attribute(j);
					String attrName = wekaAttr.name();
					String attrType = "unknown";

					if (wekaAttr.isNumeric())
						attrType = "numeric";
					if (wekaAttr.isNominal())
						attrType = "symbolic";
					if (wekaAttr.isString())
						attrType = "string"; // ignore
					if (wekaAttr.isDate())
						attrType = "date"; // ignore
					attrTypes.put(attrName,attrType);
					File file = File.createTempFile("class" + i + "_attr" + j + "_tmp", "weka2xml.tmp");
					vec[i][j] = file;
					// logger.debug(file.getName());
				}
			}
			
			int[] protCount = new int[clNumValues]; 
			// anzahl der "proteine" pro Class

			// iterate over lines in the arff file = rausschreiben
			Instance temp;
			int instanceIndex = 0;
			long lastmsg = System.currentTimeMillis();
			do {
				temp = atf.getNextInstance();

				if (temp == null)
					continue;

				instanceIndex++;
				// logger.debug(temp);
				temp.setDataset(struc);
				// logger.debug(temp.classIndex());
				if (temp.classIsMissing()) {
					logger.warn("Class attribute is missing at instance number " + instanceIndex);
					continue;
				}
				int currentIndex = (int) (temp.classValue());
				protCount[currentIndex]++;
				int currentId = protCount[currentIndex];

				// gehe ueber alle attribute -> properties und baue ueber einen
				// protein record dazu
				for (int za2 = 0; za2 < numAttr - 1; za2++) {
					Attribute atr = temp.attribute(za2);
					double d = temp.value(za2);
					String value = d + ""; 
					// if nomial, then replace double with string from struc
					if (atr.isNominal())
						value = atr.value((int) d);
					FileWriter fw = vecWriter[currentIndex][za2]; 
					// von der Classe und dem aktuellen Attribut
					if (fw == null) {
						File curFile = vec[currentIndex][za2];
						fw = new FileWriter(curFile, true);
						vecWriter[currentIndex][za2] = fw;
						fw.write(atr.name()+"\n");
					}
					//fw.write(currentId + "\t" + value + "\n");
					fw.write(value + "\n");
				}

				if ((System.currentTimeMillis() - lastmsg) > 4000) {
					logger.info("finished until record " + instanceIndex);
					lastmsg = System.currentTimeMillis();
				}
			} while (temp != null);
			
			logger.debug("flush filewriter");
			for (int n = 0; n < vecWriter.length; n++) {
				for (int m = 0; m < vecWriter[n].length; m++) {
					FileWriter fw = vecWriter[n][m];
					if (fw!=null) { fw.flush(); fw.close(); }
				}
			}

			for (int i = 0; i < clNumValues; i++) {
				logger.info("read data of class "+i);
				
				buildXML(vec[i],attrTypes,resultFiles[i]); 
				
				
			}
			
			
			// einlesen und als XML wieder rausschreiben

			worked = true;
			String msg = "Finished successfully (records " + instanceIndex + ")";
			logger.info(msg);
			JMessage.showMessageDialog(null, msg, JOptionPane.INFORMATION_MESSAGE, JOptionPane.DEFAULT_OPTION);

		} catch (Exception ex) {
			String errmsg = "Error converting the WEKA arff file '" + srcFileName + "'";
			logger.error(errmsg, ex);
			JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION, ex);
		} finally {
			logger.debug("remove tmp files");
			if ((vec != null) && (vec.length > 0)) {
				for (int i = 0; i < vec.length; i++) {
					if ((vec[i] != null) && (vec[i].length > 0)) {
						for (int j = 0; j < vec[i].length; j++) {
							FileWriter fw = vecWriter[i][j];
							if (fw != null) {
								
								fw.close();
								
							}
							File f = vec[i][j];
							String name = f.getName();
							logger.debug(f.delete()+" "+name);
						} // zweite for
					} // vec[i] inited
				} // erste for
			} // vec inited

		} // end finally

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return null;
	}

	public String[] getResultFiles() {
		return resultFiles;
	}

	public String getDestFileNamePrefix() {
		return destFileNamePrefix;
	}

	public void setDestFileNamePrefix(String destFileNamePrefix) {
		this.destFileNamePrefix = destFileNamePrefix;
	}

	public String getSrcFileName() {
		return srcFileName;
	}

	public void setSrcFileName(String srcFileName) {
		this.srcFileName = srcFileName;
	}

	/**
	 * @return true, if the last run() finished successfully
	 */
	public boolean hasWorked() {
		return worked;
	}

}
