/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: AaDistribution.java,v 1.5 2006/03/19 11:57:58 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.seqproperties;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import org.biojava.bio.seq.SequenceIterator;

import prompt.input.IInput;
import tschmidt.utils.SimpleHashFloat;


/**
 * calculates the frequency(relative) of each aa<br>
 * in all sequences of each input object.
 * 
 */
public class AaDistribution extends SeqCheck2 {
	private final static long serialVersionUID = -6849794470754667700L;

	//Logger logger = Logger.getLogger(AaDistribution.class);

	//private String[] alphabet;

	protected Hashtable vecHashA = new Hashtable();

	protected Hashtable vecHashB = new Hashtable();

	

	protected SequenceIterator iter1, iter2;

	

	public AaDistribution() {
	}

	/**
	 * @param setA
	 * @param setB
	 * @throws Exception
	 */
	public void init(IInput setA, IInput setB) throws Exception {
		super.init(setA, setB);
	}

	public void countAminoAcids() {

		//	int actValue;

		Iterator iter1 = seqA.iterator();
		while (iter1.hasNext()) {
			String ssq = (String) iter1.next();
			if (ssq!=null) {
			SimpleHashFloat tmpHash = new SimpleHashFloat();

			for (int i = 1; i < ssq.length(); i++) {
				String sym = String.valueOf(ssq.charAt(i));
				if (!sym.equals("\n"))
					tmpHash.inc(sym);
			}

			Enumeration enu = tmpHash.keys();
			float len = (new Integer(ssq.length())).floatValue();
			while (enu.hasMoreElements()) {
				String thisKey = (String) enu.nextElement();

				float thisValue = tmpHash.get(thisKey);
				tmpHash.remove(thisKey);
				float frequency = thisValue / len;
				tmpHash.put(thisKey, frequency);

				if (vecHashA.containsKey(thisKey)) {
					((Vector) vecHashA.get(thisKey)).addElement(new Float(tmpHash.get(thisKey)));
				} else {
					vecHashA.put(thisKey, new Vector());
					((Vector) vecHashA.get(thisKey)).addElement(new Float(tmpHash.get(thisKey)));
				}
			}
			tmpHash.clear();
			}
		}

		Iterator iter2 = seqB.iterator();
		while (iter2.hasNext()) {
			String ssq = (String) iter2.next();
			if (ssq!=null) {
			SimpleHashFloat tmpHash = new SimpleHashFloat();

			for (int i = 1; i < ssq.length(); i++) {
				String sym = String.valueOf(ssq.charAt(i));
				if (!sym.equals("\n"))
					tmpHash.inc(sym);
			}

			Enumeration enu = tmpHash.keys();
			float len = (new Integer(ssq.length())).floatValue();
			while (enu.hasMoreElements()) {
				String thisKey = (String) enu.nextElement();
				float thisValue = tmpHash.get(thisKey);
				tmpHash.remove(thisKey);
				float frequency = thisValue / len;
				tmpHash.put(thisKey, frequency);

				if (vecHashB.containsKey(thisKey)) {
					((Vector) vecHashB.get(thisKey)).addElement(new Float(tmpHash.get(thisKey)));
				} else {
					vecHashB.put(thisKey, new Vector());
					((Vector) vecHashB.get(thisKey)).addElement(new Float(tmpHash.get(thisKey)));
				}
			}
			tmpHash.clear();
			}
		}

	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuffer strBuff = new StringBuffer();
		strBuff.append("DATASET\tAA\tFREQ\n");

		// fehlende Elemente im jeweils anderen Hashtable ersetzen
		/*	for (Enumeration e = vecHashA.keys(); e.hasMoreElements();) {
		 Object tmp = e.nextElement();
		 Float f1 = (Float) vecHashA.get(tmp);
		 Float f2 = (Float) vecHashB.get(tmp);
		 if (f2 == null) {
		 logger.warn("Element '" + tmp + " not found in setB");
		 f2 = new Float(0);
		 vecHashB.put(tmp, f2);
		 }
		 }
		 for (Enumeration e = vecHashB.keys(); e.hasMoreElements();) {
		 Object tmp = e.nextElement();
		 Float f1 = (Float) vecHashA.get(tmp);
		 Float f2 = (Float) vecHashB.get(tmp);
		 if (f1 == null) {
		 logger.warn("Element '" + tmp + " not found in setA");
		 f1 = new Float(0);
		 vecHashA.put(tmp, f1);
		 }
		 }
		 */
		//Ausgabe
		Enumeration enu = vecHashA.keys();
		while (enu.hasMoreElements()) {
			String thisKey = (String) enu.nextElement();

			Vector actAA = (Vector) vecHashA.get(thisKey);
			for (int j = 0; j < actAA.size(); j++) {
				strBuff.append("setA\t" + thisKey + "\t" + actAA.elementAt(j) + "\n");
			}
		}

		enu = vecHashB.keys();
		while (enu.hasMoreElements()) {
			String thisKey = (String) enu.nextElement();

			Vector actAA = (Vector) vecHashB.get(thisKey);
			for (int j = 0; j < actAA.size(); j++) {
				strBuff.append("setB\t" + thisKey + "\t" + actAA.elementAt(j) + "\n");
			}
		}
		
		return strBuff.toString();
	}



	/* (non-Javadoc)
	 * @see de.tum.binfo.jdrm.JobTemplate#run()
	 */
	public void run() throws Exception {
		countAminoAcids();
	}

}
