/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: AaFrequences.java,v 1.6 2006/03/19 11:57:57 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.seqproperties;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;

import prompt.input.IInput;


/**
 * calculates the frequency of each aa (absolute and relative) and the differences
 * between 2 sequences of the input object.
 * 
 *	
 */
public class AaFrequences extends SeqCheck2 {
  private final static long serialVersionUID = -6849794470754667700L;

  Logger logger = Logger.getLogger(AaFrequences.class);



  /**
   * @param setA
   * @param setB
   */
  public void init(IInput setA, IInput setB) throws Exception {
    super.init(setA, setB);
   
  }

  public Hashtable aHash1 = new Hashtable();

  public Hashtable aHash2 = new Hashtable();

  public Hashtable frequencyHash1 = new Hashtable();

  public Hashtable frequencyHash2 = new Hashtable();

  public Hashtable diffHash = new Hashtable();

  int countAll1 = 0;

  int countAll2 = 0;

  public void countEachAminoAcid(List seqA, List seqB) {
    int tmp = 0;
    Iterator iter1 = seqA.iterator();
    while (iter1.hasNext()) {
      String ssq = (String) iter1.next();
      if (ssq!=null) {
      for (int i = 0; i < ssq.length(); i++) {

        char sym = ssq.charAt(i);
        String s = String.valueOf(sym);
        if (!s.equals("\n")) {
          countAll1++; //moved countall++ from the outer block to here Aug. 05
          if (!aHash1.containsKey(s)) {
            aHash1.put(s, new Integer(1));
          } else {
            tmp = ((Integer) aHash1.get(s)).intValue();
            aHash1.remove(s);
            tmp++;
            aHash1.put(s, new Integer(tmp));
          }
        }
      }
      }
    }

    Iterator iter2 = seqB.iterator();
    while (iter2.hasNext()) {
      String ssq = (String) iter2.next();
      if (ssq!=null) {
      for (int i = 0; i < ssq.length(); i++) {

        String s = String.valueOf(ssq.charAt(i));
        if (!s.equals("\n")) {
          countAll2++; //moved countall++ from the outer block to here Aug. 05
          if (!aHash2.containsKey(s)) {
            aHash2.put(s, new Integer(1));
          } else {
            tmp = ((Integer) aHash2.get(s)).intValue();
            aHash2.remove(s);
            tmp++;
            aHash2.put(s, new Integer(tmp));
          }
        }
      }
      }
    }

  }

  public void compare(Hashtable localHash1, Hashtable localHash2, int countAll1, int countAll2) {

  //  float diff = 0;
    for (Enumeration e = localHash1.keys(); e.hasMoreElements();) {
      Object tmp = e.nextElement();
      frequencyHash1.put(tmp, new Float(((Integer) localHash1.get(tmp)).floatValue() / new Integer(countAll1).floatValue() * 100));
    }
    for (Enumeration e = localHash2.keys(); e.hasMoreElements();) {
      Object tmp = e.nextElement();
      frequencyHash2.put(tmp, new Float(((Integer) localHash2.get(tmp)).floatValue() / new Integer(countAll2).floatValue() * 100));
    }

    for (Enumeration e = frequencyHash1.keys(); e.hasMoreElements();) {
      Object tmp = e.nextElement();
//      Float f1 = (Float) frequencyHash1.get(tmp);
      Float f2 = (Float) frequencyHash2.get(tmp);
      if (f2 == null) {
        logger.warn("Element '" + tmp + " not found in setB");
        f2 = new Float(0);
        localHash2.put(tmp, f2);
        frequencyHash2.put(tmp, f2);
      }
    }
    for (Enumeration e = localHash2.keys(); e.hasMoreElements();) {
      Object tmp = e.nextElement();
      Float f1 = (Float) frequencyHash1.get(tmp);
//      Float f2 = (Float) frequencyHash2.get(tmp);
      if (f1 == null) {
        logger.warn("Element '" + tmp + " not found in setA");
        f1 = new Float(0);
        localHash1.put(tmp, f1);
        frequencyHash1.put(tmp, f1);
      }
    }

    for (Enumeration e = localHash1.keys(); e.hasMoreElements();) {
      Object tmp = e.nextElement();
      Float f1 = (Float) frequencyHash1.get(tmp);
      Float f2 = (Float) frequencyHash2.get(tmp);
      diffHash.put(tmp, new Float(f1.floatValue() - f2.floatValue()));
    }
  }



  public void run() {

    countEachAminoAcid(seqA, seqB);
    compare(aHash1, aHash2, countAll1, countAll2);
  }



public String toString() {
    StringBuffer strBuff = new StringBuffer();
    Enumeration a = aHash1.keys();
    strBuff.append("AS");
    while (a.hasMoreElements()) {
      Object tmp = a.nextElement();
      strBuff.append("\t" + tmp);
    }
    strBuff.append("\n");

    Enumeration g = aHash1.keys();
    strBuff.append("abs1");
    while (g.hasMoreElements()) {
      Object tmp = g.nextElement();
      strBuff.append("\t" + aHash1.get(tmp));
    }
    strBuff.append("\n");

    Enumeration b = aHash2.keys();
    strBuff.append("abs2");
    while (b.hasMoreElements()) {
      Object tmp = b.nextElement();
      strBuff.append("\t" + aHash2.get(tmp));
    }
    strBuff.append("\n");

    Enumeration c = frequencyHash1.keys();
    strBuff.append("rel1");
    while (c.hasMoreElements()) {
      Object tmp = c.nextElement();
      strBuff.append("\t" + frequencyHash1.get(tmp));
    }
    strBuff.append("\n");

    Enumeration d = frequencyHash2.keys();
    strBuff.append("rel2");
    while (d.hasMoreElements()) {
      Object tmp = d.nextElement();
      strBuff.append("\t" + frequencyHash2.get(tmp));
    }
    strBuff.append("\n");

    Enumeration e = diffHash.keys();
    strBuff.append("diff");
    while (e.hasMoreElements()) {
      Object tmp = e.nextElement();
      strBuff.append("\t" + diffHash.get(tmp));
    }
    setRownames();
    return strBuff.toString();
  }

}
