/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: DigestTryptic.java,v 1.6 2006/03/19 11:57:57 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.seqproperties;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.biojava.bio.proteomics.Digest;
import org.biojava.bio.proteomics.Protease;
import org.biojava.bio.proteomics.ProteaseManager;
import org.biojava.bio.seq.Feature;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.symbol.RangeLocation;

import prompt.input.IInput;
import prompt.input.classes.SeqProperties;



/**
 * @author schmidt
 *
 */
public class DigestTryptic extends  SeqCheck1{
  private final static long serialVersionUID = -6849794470754667700L;
	
	protected List digestedSeq;

	protected Digest digester = new Digest();
	protected Protease protease = ProteaseManager.getTrypsin();
	protected List ret;

	public DigestTryptic() {
		super();
		setMaxMissedCleavages(0);
	}

	/**
	 * @param setA
	 * @throws Exception
	 */
	public void init(IInput setA) throws Exception {
		super.init(setA);
		seqA=null;
		setMaxMissedCleavages(0);
	}

	/**
	 * Sets the maximum number of partial digest products to be annotated.
	 * @param maxMissedCleavages the max number of partial digest products
	 */
	public void setMaxMissedCleavages(int maxMissedCleavages) {
		digester.setMaxMissedCleavages(maxMissedCleavages);
	}

	/* (non-Javadoc)
	 * @see de.tum.binfo.jdrm.JobTemplate#run()
	 */
	public void run() throws Exception {
		
		//To tryptic digest
		digester.setProtease(protease);
		ret = new LinkedList();

		digestedSeq = new LinkedList();
		List list = setA.getSequences();
		
		
		for (Iterator iter = list.iterator(); iter.hasNext();) {
			String seq = (String) iter.next();

			//1) string -> biojava Seq
			seq = seq.replaceAll("\\\n", "");

			Sequence proteinSym =
				ProteinTools.createProteinSequence(seq, "name");
			//2) apply digester
			digester.setSequence(proteinSym);
			digester.addDigestFeatures();
			proteinSym = digester.getSequence();
			//3) hole die peptide aus der biojava seq heraus

			for (Iterator fi = proteinSym.features(); fi.hasNext();) {
				Feature f = (Feature) fi.next();
				RangeLocation loc = (RangeLocation) f.getLocation();
				//logger.debug(loc.getMin()+":"+loc.getMax());

				String peptid = proteinSym.subStr(loc.getMin(), loc.getMax());
				double mw = SeqProperties.mass(peptid);
				if ((mw > 799) && (mw < 2800)) {
					//logger.debug(f.getType() + "\t" + mw + "\t" + peptid);
					ret.add(peptid);
				}

			}
			
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
	   StringBuffer str = new StringBuffer();
	   for (Iterator iter = ret.iterator(); iter.hasNext();) {
		String element = (String) iter.next();
		str.append(element);
		str.append("\n");
	}
		return str.toString();
	}



}
