/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: FrequencyPlotAndMannWhitneyTest.java,v 1.6 2006/03/19 11:57:56 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.seqproperties;

import java.util.HashMap;

import org.apache.log4j.Logger;

import prompt.engine.IEngineResults;
import prompt.input.IInput;
import prompt.statistic.ext.MannWhitneyApp;
import prompt.vis.seqproperties.AaFrequencesDiffPlot;



public class FrequencyPlotAndMannWhitneyTest extends SeqCheck2 {
  private final static long serialVersionUID = -6849794470754667700L;
	Logger logger = Logger.getLogger(AaFrequences.class);
	String result;
	

	/**
	 * @param setA
	 * @param setB
	 */
	public void init(IInput setA, IInput setB)
			throws Exception {
		super.init(setA, setB);
		this.seqA=null;
        this.seqB=null;

	}

	public String toString() {

		return result;
	}



	protected String[][] splitTable(String result) {
		//Aptraum code ohne err-checking urgl argl
		String[] resultLines = result.split("\n", result.length());

		String firstLineStr = resultLines[0];
		String[] firstLine = firstLineStr.split("\t");

		String[][] ret = new String[resultLines.length][firstLine.length];

		//logger.debug("+"+resultLines[0]+"+");
		for (int i = 0; i < resultLines.length; i++) {
			String currentLineStr = resultLines[i];
			String[] currentLine = currentLineStr.split("\t", currentLineStr
					.length());
			ret[i] = currentLine;
		}
		return ret;
	}

	public float alpha = 0.05f;
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see de.tum.binfo.jdrm.JobTemplate#run()
	 */
	
	public void run() throws Exception {

		//Element distributions
		AaDistribution ad = new AaDistribution();
		ad.init(setA, setB);
		ad.run();
		IEngineResults res = ad.getResults();

		//Test all elements whether they are differently distributed
		MannWhitneyApp mwApp = new MannWhitneyApp();
		mwApp.init(res);
		mwApp.run();
		String resultTest = mwApp.toString();
		//logger.debug(resultTest);
		//this we are going to return in the .toString method 
		result=resultTest;
		String[][] resTest = splitTable(resultTest);
		/*Build significance hash for all elements since we do not know the 
		 * order in the frequency plot
		 */ 
		HashMap hash = new HashMap();
		for (int i = 1; i < resTest.length; i++) {
			String[] line = resTest[i];
			if (line.length==2) {

			String actpos = line[0];
			String color;
			Float f = new Float(line[1].trim());
			//logger.debug(actpos + " " + f);
			if (f.floatValue() < alpha) color="red"; else color="green";
			hash.put(actpos,color);
			}
		}
		
		
		//calculate the delta percentage of occurence of all elements
		AaFrequences af = new AaFrequences();
		af.init(setA, setB);
		af.run();
		IEngineResults resultFreq = af.getResults();
		
		//generate color list depending on the test results
		StringBuilder colList = new StringBuilder();
		colList.append("c(");
		String[][] resFreq = splitTable(resultFreq.getData());
		for (int i = 1; i < resFreq[0].length; i++) {
			String col = resFreq[0][i];
			logger.debug(col);
			String currentColor = (String) hash.get(col);
			if (currentColor==null) {
				currentColor="blue";
			}
			colList.append("\""+currentColor+"\"");
			if (i!=resFreq[0].length-1) {
				colList.append(",");
			}
		}
		colList.append(")");

		

		
		//logger.debug("COLIST:" + colList.toString());
	

		AaFrequencesDiffPlot afd = new AaFrequencesDiffPlot();
		afd.init(resultFreq, colList
				.toString());
		afd.titleWindow=setA.getDescription()+" - "+setB.getDescription();
		afd.run();
		afd.showGraphic();
		
	}

}
