/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: LengthAgainstHyphob.java,v 1.8 2006/03/19 11:57:58 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.seqproperties;

import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;

import prompt.input.IInput;
import tschmidt.iter.TupelIter;
import tschmidt.scales.HphobKyteDoolite;
import tschmidt.scales.Scale;



public class LengthAgainstHyphob extends SeqCheck1 {
  private final static long serialVersionUID = -6849794470754667700L;
   List list1;

    private StringBuffer tabA = new StringBuffer();

   

     Scale scale = new HphobKyteDoolite();

    /**
     * @param setA
     * @param setB
     * @throws Exception
     */
    public void init(IInput setA) throws Exception {
        super.init(setA);

        list1 = seqA;


    }

    public void calcHyphobValue(Iterator iter1) {
        Logger logger = Logger.getLogger(this.getClass());
        float hyphob;
      
        int i=0;
        while (iter1.hasNext()) {
        	i++;
        	Object[] entry = (Object[]) iter1.next();
			if (entry==null) {
				logger.warn("entry "+i+" is null");
				continue;
			}
			if (entry.length<2) {
				logger.warn("entry "+i+" is needs at least two elements> id, and sequence");
				continue;
			}
			Object id = entry[0];
			if (id==null) {
				logger.warn("entry "+i+" has a NULL id, set description to 'UNKNOWN_"+i+"'");
				continue;
			}
			Object seq = entry[1];
			if (seq==null) {
				logger.warn("entry "+i+" has a NULL seq, set sequence to 'XXX_"+i+"'");
				continue;
			}			
        	String aminos = (String) seq;
            hyphob = 0;
            try {
                
                if (aminos!=null) {
                hyphob = scale.avgValue(aminos);
                if (hyphob==Float.NaN) {
                    logger.debug(aminos);
                    continue;
                }
                tabA.append(id+"\t"+aminos.length() + "\t" + new Float(hyphob).toString() + "\n");
                }
                } catch (Exception e) {
                    
                    logger.error(e);
            }
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        StringBuffer strBuff = new StringBuffer();
        strBuff.append("ID\tLENGTH\tHYPHOB\n");
        strBuff.append(tabA);
        //Logger logger = Logger.getLogger(this.getClass());
        //logger.debug("#"+strBuff.toString()+"#");
        return strBuff.toString();
    }



    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.jdrm.JobTemplate#run()
     */
    public void run() throws Exception {
    	TupelIter inputA = new TupelIter(setA.getSequencesNames(),  this.seqA);
        calcHyphobValue(inputA);
    }

	public Scale getScale() {
		return scale;
	}

	public void setScale(Scale scale) {
		this.scale = scale;
	}
}
