/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: LengthCompare.java,v 1.5 2006/03/19 11:57:56 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.seqproperties;

import org.apache.log4j.Logger;

import prompt.engine.CompareNumericDistributions;
import prompt.engine.IEngineResults;
import prompt.input.IInput;
import tschmidt.utils.Binning;


/**
 * @author schmidt
 *
 */
public class LengthCompare extends SeqCheck2 {
  private final static long serialVersionUID = -6849794470754667700L;
  private Logger logger = Logger.getLogger(LengthCompare.class);
  

  
  private Binning binning; 

  CompareNumericDistributions cp = new CompareNumericDistributions();
  
  public void init(IInput setA, IInput setB) throws Exception {
    super.init(setA, setB);
    
    this.seqA=null;
    this.seqB=null;
  }
  
  
  /**
   * Generic constructor, must exist
   */
  public LengthCompare() {
    super();
  }
  


  /* (non-Javadoc)
   * @see de.tum.binfo.jdrm.JobTemplate#run()
   */
  public void run() throws Exception {
    cp.init(LengthCompare.class.getName(),setA.getDescription(), setB.getDescription(),setA.getLength(),setB.getLength(), this.valFilter);
    cp.setBinning(binning);
    cp.setTestWholeDistribution(testWholeDistribution);
    cp.setTestEachBin(testEachBin);
    cp.run();
  }

  
  /* (non-Javadoc)
   * @see de.tum.binfo.proj.engine.IEngine#getResultsAll()
   */
  public IEngineResults[] getResultsAll() throws Exception {
    return cp.getResultsAll();
  }
  
  /**
   * @return Returns the binning object
   */
  public Binning getBinning() {
    if (binning == null) {
      binning = new Binning();
    }
    return binning;
  }

  /**
   * @param binning
   *          The binning to set.
   */
  public void setBinning(Binning binning) {
    this.binning = binning;
  }
  
  private boolean testWholeDistribution = true;
  private boolean testEachBin = true;
  /**
   * @return whether a test of distribution differences of all bins should be performed
   */
  public boolean isTestEachBin() {
    return testEachBin;
  }

  /**
   * @param boolean,
   *          default is true
   */
  public void setTestEachBin(boolean testEachBin) {
    this.testEachBin = testEachBin;
  }

  /**
   * @return returns whether a test of distribution differences of the whole distribution should be performed
   */
  public boolean isTestWholeDistribution() {
    return testWholeDistribution;
  }

  /**
   * @param boolean,
   *          default is true
   */
  public void setTestWholeDistribution(boolean testWholeDistribution) {
    this.testWholeDistribution = testWholeDistribution;
  }
}
