/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: SequencePropertiesExec.java,v 1.3 2006/01/06 23:23:11 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.seqproperties;

import org.apache.log4j.Logger;

import prompt.input.ISeqProperties;
import prompt.vis.seqproperties.HydrophobicityComparePlot;
import prompt.vis.seqproperties.LengthComparePlot;
import prompt.vis.seqproperties.MassComparePlot;
import prompt.vis.seqproperties.PIComparePlot;
import tschmidt.scales.HphobKyteDoolite;
import tschmidt.scales.Scale;

/**
 * @author schmidt
 *
 */
public class SequencePropertiesExec {

  public static void doPI(ISeqProperties setA, ISeqProperties setB) throws Exception {
      Logger logger = Logger.getLogger(SequencePropertiesExec.class);
    PICompare pic = new PICompare();
    pic.init(setA, setB);
    pic.run();
    logger.debug(pic.getResults());

    //im PIcomparePlot noch die Y-Achsenbezeichnung dazu
    PIComparePlot plot = new PIComparePlot();
    plot.init(pic.getResults());
    plot.run();
    plot.showGraphic();

  /*  
    //jetzt schauen wir speziell den Bereich von pI 3 bis 4 an
    Binning myBinning = new Binning();
    myBinning.initNumberOfBins(10,3.0d,4.0d);
    logger.debug(myBinning);
    pic.setBinning(myBinning);
    pic.run();
    logger.debug(pic.getResults());

    //im PIcomparePlot noch die Y-Achsenbezeichnung dazu
    PIComparePlot plot2 = new PIComparePlot();
    plot2.init(pic.getResults());
    plot2.getTitle().setXlab("pI");
    plot2.run();
    plot2.showGraphic();
    */
    
    //		AVG, STD, MEAN,.. von pI, mw, mass
    //piCompare, massCompare, LcRentCompare, LengthComp, AA 1,2,3 Compare
  }

  public static void doMass(ISeqProperties setA, ISeqProperties setB) throws Exception {
    MassCompare engine = new MassCompare();
    engine.init(setA, setB);
   // Binning myBins = new Binning();
  //  myBins.initBinWidth(200, 700, 2700);
    //engine.setBinning(myBins);

    //engine.setFilterValue(new MyRange());
    engine.run();
    Logger logger = Logger.getLogger(SequencePropertiesExec.class);
    logger.debug("\nMass Compare");
    logger.debug(engine.getResults());

    MassComparePlot plot = new MassComparePlot();
    plot.init(engine.getResults());
    //plot.getPar().setFont(Par.FONT_BOLD_ITALIC);

    plot.run();
    plot.showGraphic();
  }
  
  public static void doLength(ISeqProperties setA, ISeqProperties setB) throws Exception {
    LengthCompare engine = new LengthCompare();
    engine.init(setA, setB);
//    Binning myBins = new Binning();
//    myBins.initTrunc(5,29);
//    engine.setBinning(myBins);
//    engine.setTestEachBin(false);
    
    //engine.setFilterValue(new MyRange());
    engine.run();
    Logger logger = Logger.getLogger(SequencePropertiesExec.class);
    logger.debug("\nLength Compare");
    logger.debug(engine.getResults());

    LengthComparePlot plot = new LengthComparePlot();
    plot.init(engine.getResults());
    plot.run();
    plot.showGraphic();
  }

  public static void doHphob(ISeqProperties setA, ISeqProperties setB) throws Exception {
      Scale scale = new HphobKyteDoolite();
      HydrophobicityCompare engine = new HydrophobicityCompare();
    engine.init(setA, setB);
    engine.scale=scale;
    engine.setTestEachBin(false);
    engine.setTestWholeDistribution(false);
    
    //engine.setFilterValue(new MyRange());
    engine.run();
    Logger logger = Logger.getLogger(SequencePropertiesExec.class);
    logger.debug("\nHydrophobicity Compare");
    logger.debug(engine.getResults());

    HydrophobicityComparePlot plot = new HydrophobicityComparePlot();
    plot.init(engine.getResults());
    plot.run();
    plot.showGraphic();
    
//    ShapiroTest shapiro = new ShapiroTest();
//    shapiro.init(setA.getHydrophobicityAvg(scale));
//    shapiro.run();
//    logger.debug(shapiro);
  }

}
