/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: XMLSaver.java,v 1.5 2006/03/19 11:57:57 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.seqproperties;

import gr.zeus.ui.JMessage;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import prompt.input.IInput;
import prompt.ui.SaveUI;
import tschmidt.iter.TupelIter;


public class XMLSaver extends SeqCheck1 {

	

	/**
	 * 
	 */
	private static final long serialVersionUID = -6035870984838370411L;
	private String filename = null;

	@Override
	public void init(IInput setA) throws Exception {
		
		super.init(setA);
        seqA = null;
	}

	public void init(IInput setA, String filename) throws Exception {
		
		this.filename = filename;
		super.init(setA);
        seqA = null;
	}


	@Override
	public void run() throws Exception {
		Logger logger = Logger.getLogger(XMLSaver.class);
		if (filename == null) {
			filename = SaveUI.save("Save sequences to a Generic XML file",".xml");
		}
		if (filename == null) {
			logger.info("canceled");
			return;
		}
		File f = new File(filename);
		logger.info("Start saving to '" + f.toURL() + "'");
		TupelIter inputA = new TupelIter(setA.getSequencesNames(), setA.getSequences());

		BufferedWriter out = new BufferedWriter(new FileWriter(filename));
		int i = 0;
        out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
        out.write("<dataset label=\""+setA.getDescription()+"\">\n");
        
        out.write("<property type=\"setdef\" id=\"setdef\">\n");
        while (inputA.hasNext()) {
			i++;
			Object[] entry = (Object[]) inputA.next();
			if (entry==null) {
				logger.warn("entry "+i+" is null");
				break;
			}
			if (entry.length<2) {
				logger.warn("entry "+i+" is needs at least two elements> id, and sequence");
				break;
			}
			Object id = entry[0];
			if (id==null) {
				logger.warn("entry "+i+" has a NULL id, set description to 'UNKNOWN_"+i+"'");
			}
			Object seq = entry[1];
			if (seq==null) {
				logger.warn("entry "+i+" has a NULL seq, set sequence to 'XXX_"+i+"'");
			}	
            
            out.write("\t<input id=\""+id+"\" value=\""+seq+"\" />\n");
//			out.write(">" + id + "\n");
//			out.write(seq + "\n");
		}
        out.write("</property>\n");
        out.write("</dataset>\n");
		out.flush();
		out.close();
		String msg = "finished saving to '"+filename+"'";
		logger.info(msg);
        JMessage.showMessageDialog(null, msg,
                JOptionPane.INFORMATION_MESSAGE, JOptionPane.DEFAULT_OPTION);

	}

	public String getFilename() {
		return filename;
	}

	public void setFilename(String filename) {
		this.filename = filename;
	}

	@Override
	public String toString() {
		return null;
	}

}
