/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: BlastVsDb.java,v 1.9 2006/04/12 12:24:57 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.seqproperties.mapper;

import gr.zeus.ui.JMessage;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import prompt.engine.Engine1;
import prompt.input.IGenericXMLFile;
import prompt.input.IInput;
import prompt.input.ISeqProperties;
import tschmidt.iter.TupelIter;
import tschmidt.mapper.MapBlast;


/**
 * Blast qry against database
 * @author tschmidt
 * 
 *
 */
public class BlastVsDb extends Engine1 {

	private static final long serialVersionUID = 8078791463371642170L;

	private ISeqProperties seqA;

	String resultStr;
	
	String blastParam =null;
	
	int sequenceType =-1;
	
	public static final int UNKNOWN = -1;
	public static final int PROTEIN = 1;
	public static final int NUCLEOTID = 2;

	/* (non-Javadoc)
	 * @see bea.engine.IEngine2#init(prompt.input.IInput, prompt.input.IInput)
	 */
	@Override
	public void init(IInput setA) throws Exception {
		super.init(setA);
        try {
            this.seqA = ((ISeqProperties) setA);
        } catch (ClassCastException e) {
           String errmsg = "The input '"+setA.getDescription()+" must be able to provide sequences";
           throw new ClassCastException(errmsg);
        }    
	}

	@Override
	public String toString() {
		return resultStr;
	}



    
    public static boolean testBlastRun()  {
        Logger logger = Logger.getLogger(BlastVsDb.class);
        Process p = null;
        try {
            p = Runtime.getRuntime().exec("blastall");
        } catch (IOException e) {
            logger.debug("exec blastall",e);
          if (e.getMessage().contains("blastall")) return false; 
        }

       /* BufferedReader bf = new BufferedReader(new InputStreamReader(p.getInputStream()));
        
        String pre_blastOut = bf.readLine();
      
        while ((pre_blastOut!=null) && (pre_blastOut.charAt(0)=='#')) {
             pre_blastOut = bf.readLine();
           
        }
        
     
        if (pre_blastOut != null) {

            String post_blastOut[] = pre_blastOut.split("\\t");
            
            
            for (int i = 1; i < post_blastOut.length; i++) {
                String string = post_blastOut[i];
          
            }

        }
        
        bf.close(); */
        p.destroy();
        return true;
    }
    
	@Override
	public void run() throws Exception {
		Logger logger = Logger.getLogger(BlastVsDb.class);
		
		TestBlastInstallation testBlast = new TestBlastInstallation();
		testBlast.setSupressSuccess(true);
		testBlast.setSilent(false);
		testBlast.run();
		if (! testBlast.hasWorked()) {
			String errmsg = "The Blast Installation Test returned errors. Please correct this and try again.";
            logger.fatal(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.INFORMATION_MESSAGE, JOptionPane.DEFAULT_OPTION);
            return;
		}
		
		//hier noch auswahl ob aminosr oder nucleotid sqs
        List listA = this.seqA.getSequences();
        if ((listA==null) || (listA.size()==0)) {
            String errmsg="<b>The input '"+seqA.getDescription()+"' does not contain any sequences</b>";
            if (IGenericXMLFile.class.isAssignableFrom(seqA.getClass())) {
                errmsg = errmsg +"<br>The Generic XML file needs a <b>setdef</b> property that defines " +
                        "the proteins within the set,<br> and must contain sequences for this analysis";
            }
            throw new ClassCastException(errmsg);
        }
        
        
		MapBlast mapper = null;
		boolean type = true;
		FastaVsDbDialogReturn ret= null;
		if (sequenceType==UNKNOWN) {
			//ask the user
			 ret = BlastVsDbDialog.showDialog(seqA.getDescription());
			if (ret.canceled) {
                String errmsg = "Canceled by the user";
                logger.fatal(errmsg);
                JMessage.showMessageDialog(null, errmsg, JOptionPane.INFORMATION_MESSAGE, JOptionPane.DEFAULT_OPTION);
                return;
			}
			if (ret.sequenceType==FastaVsDbDialogReturn.NUCLEOTID) type=false;
			
		} 
		
        if (!testBlastRun()) {
            String errmsg = "The program <b>blastall</b> could not executed<br>Check that it is in your PATH and has permissions to run";
            logger.fatal(errmsg);
            JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
            return;
        }
		mapper = new MapBlast(type);
		
		logger.debug("start data retrieval");
          //FIXME: check if sequences are available!!
		desc1 = seqA.getDescription();
		desc2 = ret.dbFile.getAbsolutePath();
		TupelIter inputA = new TupelIter(seqA.getSequencesNames(), listA);
		mapper.setSequenceIteratorQuery(inputA);

		
		mapper.setDatabase(ret.dbFile);
		
		
		logger.debug("start mapping");

		
		
		StringBuffer sb = new StringBuffer("QUERY_ID\tSUBJECT_ID\tIDENTITY_OF_HIT\t");
		sb.append("ALIGNMENT_LENGTH\tMISMATCHES\tGAP_OPENINGS\t");
		sb.append("Q_START\tQ_END\tS_START\tS_END\t");
		sb.append("E_VALUE\tBIT_SCORE");
		sb.append("\n");
		Iterator resIter = mapper.getResultIterator();
		int i = 0;
		while (resIter.hasNext()) {
			i++;
			try {
				ArrayList res = (ArrayList) resIter.next();
				if (res == null) {
					logger.warn("result gleich null, input error");
					sb.append(i + "#null\n");
				} else {
					sb.append("" + res.get(0));
					if (res.size()>1) {
					for (int j = 1; j < res.size(); j++) {
						String field = (String) res.get(j);
						sb.append("\t" + field);
					}}
					sb.append("\n");
				}
				if (i%50==0) {
					logger.info("BLAST finished until query protein "+i+" of "+seqA.getDescription()+" vs. "+ret.dbFile.getName());
				}
			} catch (Exception e) {
				logger.error("Error at query number "+i,e);
				
			}
			//logger.debug(res.get(0)+"\t"+res.get(1));
		}
		resultStr = sb.toString();
		logger.debug("finished mapping");
	}




	public String getBlastParam() {
		return blastParam;
	}




	/**
	 * if null (no blast params) are set, then default params are used
	 * @param blastParam
	 */
	public void setBlastParam(String blastParam) {
		this.blastParam = blastParam;
	}




	public int getSequenceType() {
		return sequenceType;
	}




	/**If the sequence type is not set, the a dialog will pop up 
	 * in the run-method
	 * @param sequenceType
	 */
	public void setSequenceType(int sequenceType) {
		this.sequenceType = sequenceType;
	}

}
