/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: EqualTotal.java,v 1.5 2006/03/02 13:24:05 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.seqproperties.mapper;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;

import prompt.engine.Engine2;
import prompt.input.IGenericXMLFile;
import prompt.input.IInput;
import prompt.input.ISeqProperties;
import tschmidt.iter.TupelIter;
import tschmidt.mapper.IMapper;
import tschmidt.mapper.MapEqualTotal;


public class EqualTotal extends Engine2 {
	

	private static final long serialVersionUID = 5044535467456884830L;

	Logger logger = Logger.getLogger(EqualTotal.class);
	  
	private ISeqProperties seqA;
	private ISeqProperties seqB;
	String resultStr;

	/* (non-Javadoc)
	 * @see bea.engine.IEngine2#init(prompt.input.IInput, prompt.input.IInput)
	 */
	@Override
	public void init(IInput setA, IInput setB) throws Exception {
		super.init(setA, setB);
        try {
            this.seqA = ((ISeqProperties) setA);
        } catch (ClassCastException e) {
           String errmsg = "The input '"+setA.getDescription()+" must be able to provide sequences";
           throw new ClassCastException(errmsg);
        }        
        try {
            this.seqB = ((ISeqProperties) setB);
        } catch (ClassCastException e) {
           String errmsg = "The input '"+setB.getDescription()+" must be able to provide sequence information";
           throw new ClassCastException(errmsg);
        }
	}

	@Override
	public String toString() {
		return resultStr;
	}



	
	
	@Override
	public void run() throws Exception {
		IMapper mapper = new MapEqualTotal();
		
         List listA = this.seqA.getSequences();
            if ((listA==null) || (listA.size()==0)) {
                String errmsg="<b>The input '"+seqA.getDescription()+"' does not contain any sequences</b>";
                if (IGenericXMLFile.class.isAssignableFrom(seqA.getClass())) {
                    errmsg = errmsg +"<br>The Generic XML file needs a <b>setdef</b> property that defines " +
                            "the proteins within the set,<br> and must contain sequences for this analysis";
                }
                throw new ClassCastException(errmsg);
            }
            
            List listB = this.seqB.getSequences();
            if ((listB==null) || (listB.size()==0)) {
                String errmsg="<b>The input '"+seqB.getDescription()+"' does not contain any sequences</b>";
                if (IGenericXMLFile.class.isAssignableFrom(seqA.getClass())) {
                    errmsg = errmsg +"<br>The Generic XML file needs a <b>setdef</b> property that defines " +
                            "the proteins within the set,<br> and must contain sequences for this analysis";
                }
                throw new ClassCastException(errmsg);
            }
        
		TupelIter inputA = new TupelIter(seqA.getSequencesNames(),listA);
	/*	logger.debug("inputA");
		while (inputA.hasNext()) {
			Object[] objs = (Object[]) inputA.next();
			logger.debug("'"+objs[0]+"'");
			logger.debug("'"+objs[1]+"'");
			logger.debug();
		}
		inputA.rewind(); */
		mapper.setSequenceIteratorQuery(inputA);
		
		TupelIter inputB = new TupelIter(seqB.getSequencesNames(),seqB.getSequences());
	/*	logger.debug("\n\ninputB");
		while (inputB.hasNext()) {
			Object[] objs = (Object[]) inputB.next();
			logger.debug("'"+objs[0]+"'");
			logger.debug("'"+objs[1]+"'");
			logger.debug();
		}
		inputB.rewind(); */
		mapper.setSequenceIteratorDatabase(inputB);
		
		logger.debug("start mapping");
		
		StringBuffer sb = new StringBuffer("ID1\tID2\n");
		Iterator resIter = mapper.getResultIterator();
        while (resIter.hasNext()) {
            ArrayList res = (ArrayList) resIter.next();
            if (res==null) {
            	logger.warn("result gleich null");
            }
            sb.append(res.get(0)+"\t"+res.get(1)+"\n");
            //logger.debug(res.get(0)+"\t"+res.get(1));
        }
        resultStr=sb.toString();
        logger.debug("finished mapping");
	}

}
