/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: SimapMd5.java,v 1.5 2006/01/06 23:48:49 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.seqproperties.mapper;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;

import prompt.engine.Engine1;
import prompt.input.IGenericXMLFile;
import prompt.input.IInput;
import prompt.input.ISeqProperties;
import tschmidt.iter.TupelIter;
import tschmidt.mapper.IMapper;
import tschmidt.mapper.MapToSimapIdsByMd5;


public class SimapMd5 extends Engine1 {

	private static final long serialVersionUID = -2954664105409004903L;
	private ISeqProperties setA;
	private String resultStr;
	
	@Override
	public void init(IInput setA) throws Exception {
        try {
            this.setA = ((ISeqProperties) setA);
        } catch (ClassCastException e) {
           String errmsg = "The input '"+setA.getDescription()+" must be able to provide sequences";
           throw new ClassCastException(errmsg);
        } 
		super.init(setA);
	}



	@Override
	public void run() throws Exception {
		Logger logger = Logger.getLogger(SimapMd5.class);
		logger.debug("start data retrieval");
        List listA = this.setA.getSequences();
        if ((listA==null) || (listA.size()==0)) {
            String errmsg="<b>The input '"+setA.getDescription()+"' does not contain any sequences</b>";
            if (IGenericXMLFile.class.isAssignableFrom(setA.getClass())) {
                errmsg = errmsg +"<br>The Generic XML file needs a <b>setdef</b> property that defines " +
                        "the proteins within the set,<br> and must contain sequences for this analysis";
            }
            throw new ClassCastException(errmsg);
        }
        
		IMapper mapper = new MapToSimapIdsByMd5();
		TupelIter inputA = new TupelIter(setA.getSequencesNames(), listA);
		
		mapper.setSequenceIteratorQuery(inputA);
		logger.debug("start mapping");
		
		StringBuffer sb = new StringBuffer("QUERY_ID\tSIMAP_ID\n");
		Iterator resIter = mapper.getResultIterator();
		int i = 0;
		while (resIter.hasNext()) {
			i++;
			try {
				ArrayList res = (ArrayList) resIter.next();
				if (res == null) {
					logger.warn("result gleich null, input error");
					sb.append(i + "#null\n");
				} else {
					sb.append("" + res.get(0));
					if (res.size()>1) {
					for (int j = 1; j < res.size(); j++) {
						Object field = (Object) res.get(j);
						sb.append("\t" + field);
					}}
					sb.append("\n");
				}
			} catch (Exception e) {
				logger.error("Error at query number "+i,e);
				
			}
			//logger.debug(res.get(0)+"\t"+res.get(1));
		}
		
		resultStr = sb.toString();
		logger.debug("finished mapping");
	}
	
	@Override
	public String toString() {
		return resultStr;
	}

}
