package prompt.engine.seqproperties.mapper;

import gr.zeus.ui.JMessage;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import prompt.engine.Engine;
import tschmidt.utils.ValidLocation;

public class TestBlastInstallation extends Engine {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * @param args
	 * @throws Exception 
	 */
	public static void main(String[] args) throws Exception {
		TestBlastInstallation self = new TestBlastInstallation();
		self.run();

	}

	boolean silent = false;
	
	Logger logger = Logger.getLogger(this.getClass().getName());
	
	void msgWarn(String msg) {
		logger.warn(msg);
		if (!silent) {
			 JMessage.showMessageDialog(null, msg, JOptionPane.WARNING_MESSAGE, JOptionPane.DEFAULT_OPTION);
		}
	}
	
	void msgWarn(String msg, Exception e) {
		logger.warn(msg, e);
		if (!silent) {
			 JMessage.showMessageDialog(null, msg, JOptionPane.WARNING_MESSAGE, JOptionPane.DEFAULT_OPTION,e);
		}
	}	
	
	boolean worked = false;
	
	@Override
	public void run() throws Exception {
		long start = System.currentTimeMillis();
		String qryFileName = this.getClass().getSimpleName() + ".qry.fasta";
		URL url = getClass().getResource(qryFileName);
		if (url == null) {
			msgWarn("<html>No test query sequence found:<br>"+qryFileName+"</hmtl>");
			return;
		}
        String qry = ValidLocation.readFromUrl(url);
        //logger.info("Query sequence\n"+qry);
        
        String dbFileName = this.getClass().getSimpleName() + ".db.fasta";
        URL url2 = getClass().getResource(dbFileName);
		if (url2 == null) {
			msgWarn("<html>No test database sequences found:<br>"+dbFileName+"</hmtl>");
			return;
		}
		String db = ValidLocation.readFromUrl(url2);
		//logger.info("DB sequences\n"+db);
		
		
		//Check formatdb /////////////////
		logger.info("Create BLAST database file");
		File seqDB = null;
		try {
		  seqDB = File.createTempFile("blastDB", ".fasta");
		  FileWriter fw = new FileWriter(seqDB);
		  fw.write(db);
		  fw.close();
		} catch (Exception e) {
			msgWarn("Could not create temporary database file",e);
			return;
		}
		
		String cmd1 = "formatdb -p T -t MapBlast -o T -i" + seqDB.getPath();
		logger.info(cmd1);
		Process p = null;
		try {
			p = Runtime.getRuntime().exec(cmd1);
		} catch (IOException e) {
			msgWarn("<html>Could not execute the BLAST tool <i>formatdb</i>.<br>Check that formatdb can be found in your path</html>",e);
			return;
		}
		
		BufferedReader bf = new BufferedReader(new InputStreamReader(p.getInputStream()));
		BufferedReader bf2 = new BufferedReader(new InputStreamReader(p.getErrorStream()));
		
		String outLine = bf.readLine();
		while ((outLine!=null)) {
			logger.info("stdout: "+outLine);
			outLine = bf.readLine();
		}
		
		StringBuffer stderr = new StringBuffer();
		outLine = bf2.readLine();	
		while ((outLine!=null)) {
			logger.info("stderr: "+outLine);
			stderr.append(outLine+"<br>");
			outLine = bf2.readLine();
		}
		
		int formatdbRetCode = p.waitFor();
		bf.close();
		bf2.close();
		// = p.exitValue();
		logger.info("Process exitValue="+formatdbRetCode);
		if ( (formatdbRetCode!=0) || (stderr.length()>0) ) {
			msgWarn("<html><b><i>formatdb</i> returned the error code "+formatdbRetCode+".</b><br>Check your installation and the BLAST documentation about this error code" +
					"<br><br>Error msg returned:<br>"+stderr.toString()+"</html>");
			return;
		}
		
		
		logger.info("Create QUERY file");
		File seqQry = null;
		try {
		  seqQry = File.createTempFile("blastQry", ".fasta");
		  FileWriter fw = new FileWriter(seqQry);
		  fw.write(qry);
		  fw.close();
		} catch (Exception e) {
			msgWarn("Could not create temporary query file",e);
			return;
		}
		
		
		
		//RUN BLAST ////////////////////////////
		String cmd2 = "blastall -p blastp -d " + seqDB.getPath() + " -i " + seqQry.getPath()+" -b 1 -m 8 ";
		logger.info(cmd2);
		
		Process p2 = null;
		try {
			p2 = Runtime.getRuntime().exec(cmd2);
		} catch (IOException e) {
			msgWarn("<html>Could not execute the BLAST tool <i>blastall</i>.<br>Check that blastall can be found in your path</html>",e);
			return;
		}
		
		bf = new BufferedReader(new InputStreamReader(p2.getInputStream()));
		bf2 = new BufferedReader(new InputStreamReader(p2.getErrorStream()));
		
		StringBuffer stdout = new StringBuffer();
		outLine = bf.readLine();
		while ((outLine!=null)) {
			logger.info("stdout: "+outLine);
			stdout.append(outLine);
			outLine = bf.readLine();
		}
		bf.close();
		stderr = new StringBuffer();
		outLine = bf2.readLine();	
		while ((outLine!=null)) {
			logger.info("stderr: "+outLine);
			stderr.append(outLine+"<br>");
			outLine = bf2.readLine();
		}
		bf2.close();		
		
		int blastRetCode = p.exitValue();
		logger.info("Process exitValue="+formatdbRetCode);
		if ( (blastRetCode!=0) || (stderr.length()>0) ) {
			String part2 = "<br><ul><li>Tip1: if the BLOSUM Matrix is not found. check that you have created the ini file as described in the BLAST docs.</li>" +
			"<li>Tip2: in the ini-file the path must not have quote signs</li></ul>" +
			"<br>Error text returned:<br>"+stderr.toString()+"</html>";
			
			if (blastRetCode!=0)
			msgWarn("<html><b><i>blastall</i> returned the error code "+blastRetCode+".</b><br>Check your installation and the BLAST documentation about this error code" +part2);
			
			else 
				msgWarn("<html><b><i>blastall</i> returned error messages.</b><br>Check your installation and the BLAST documentation" +part2);		
			return;
		}
		
		//CHECK IF HIT FOUND
		if (! stdout.toString().startsWith("gi_2367363")) {
			msgWarn("The hit that we expected was not found, is the test data set correct?");		
		}
	
		//del temp.
		try {
			logger.debug("delete temporary database sequences '"+seqDB.getAbsolutePath()+"'");
			seqDB.delete();
		} catch (Exception e) {
			msgWarn("Could not delete temporary database file",e);
			return;
		}
		try {
			logger.debug("delete temporary query sequence '"+seqQry.getAbsolutePath()+"'");
			seqQry.delete();
		} catch (Exception e) {
			msgWarn("Could not delete temporary query file",e);
			return;
		}
		long end = System.currentTimeMillis();
		
		String msg = "<html>Congratulations: the test BLAST run finished successfully!<br>" +
				"Milliseconds needed: "+(end-start)+"</html>";
        logger.info(msg);
        if (!supressSuccess) {
			 JMessage.showMessageDialog(null, msg, JOptionPane.INFORMATION_MESSAGE, JOptionPane.DEFAULT_OPTION);
		}
        worked=true;
	}

	boolean supressSuccess = false;

	public boolean isSilent() {
		return silent;
	}


	/**
	 * Fehler nur ins log, oder auch zusaetzliche Msg Dlgs
	 * @param silent
	 */
	public void setSilent(boolean silent) {
		this.silent = silent;
	}

	/**
	 * @return true, if the last run() finished successfully
	 */
	public boolean hasWorked() {
		return worked;
	}

	@Override
	public String toString() {
		return null;
	}

	/**If you set this to true, then the success dlg will never appear
	 * @param supressSuccess
	 */
	public void setSupressSuccess(boolean supressSuccess) {
		this.supressSuccess = supressSuccess;
	}

}
