/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: TestFastaInstallation.java,v 1.1 2006/04/13 10:50:52 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.seqproperties.mapper;

import gr.zeus.ui.JMessage;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import prompt.engine.Engine2;
import prompt.input.IGenericXMLFile;
import prompt.input.IInput;
import prompt.input.ISeqProperties;
import prompt.input.classes.FlatFile;
import tschmidt.iter.TupelIter;
import tschmidt.mapper.MapFasta;

/**
 * FASTA seq against another
 * 
 * @author hager
 * 
 */
public class TestFastaInstallation extends Engine2 {

	private static final long serialVersionUID = 8078791463371642170L;

	private ISeqProperties seqA;

	private ISeqProperties seqB;

	String resultStr;

	// String blastParam = null;

	int sequenceType = -1;

	public static final int UNKNOWN = -1;

	public static final int PROTEIN = 1;

	public static final int NUCLEOTID = 2;

	public TestFastaInstallation() {
		FlatFile fileA = new FlatFile();
		try {
			fileA.init("TestFastaInstallation.qry.fasta");

			FlatFile fileB = new FlatFile();
			fileB.init("TestFastaInstallation.db.fasta");
			init(fileA, fileB);
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	@Override
	public void init(IInput setA, IInput setB) throws Exception {
		super.init(setA, setB);
		try {
			this.seqA = ((ISeqProperties) setA);
		} catch (ClassCastException e) {
			String errmsg = "The input '" + setA.getDescription()
					+ " must be able to provide sequences";
			throw new ClassCastException(errmsg);
		}
		try {
			this.seqB = ((ISeqProperties) setB);
		} catch (ClassCastException e) {
			String errmsg = "The input '" + setB.getDescription()
					+ " must be able to provide sequence information";
			throw new ClassCastException(errmsg);
		}
	}

	@Override
	public String toString() {
		return resultStr;
	}

	@Override
	public void run() throws Exception {
		Logger logger = Logger.getLogger(TestFastaInstallation.class);

		// TestBlastInstallation testBlast = new TestBlastInstallation();
		// testBlast.setSupressSuccess(true);
		// testBlast.setSilent(false);
		// testBlast.run();
		// if (!testBlast.hasWorked()) {
		// String errmsg = "The Blast Installation Test returned errors. Please
		// correct this and try again.";
		// logger.fatal(errmsg);
		// JMessage.showMessageDialog(null, errmsg,
		// JOptionPane.INFORMATION_MESSAGE, JOptionPane.DEFAULT_OPTION);
		// return;
		// }

		// hier noch auswahl ob aminosr oder nucleotid sqs

		MapFasta mapper = null;
		boolean type = true;

		List listA = this.seqA.getSequences();
		if ((listA == null) || (listA.size() == 0)) {
			String errmsg = "<b>The input '" + seqA.getDescription()
					+ "' does not contain any sequences</b>";
			if (IGenericXMLFile.class.isAssignableFrom(seqA.getClass())) {
				errmsg = errmsg
						+ "<br>The Generic XML file needs a <b>setdef</b> property that defines "
						+ "the proteins within the set,<br> and must contain sequences for this analysis";
			}
			throw new ClassCastException(errmsg);
		}

		List listB = this.seqB.getSequences();
		if ((listB == null) || (listB.size() == 0)) {
			String errmsg = "<b>The input '" + seqB.getDescription()
					+ "' does not contain any sequences</b>";
			if (IGenericXMLFile.class.isAssignableFrom(seqA.getClass())) {
				errmsg = errmsg
						+ "<br>The Generic XML file needs a <b>setdef</b> property that defines "
						+ "the proteins within the set,<br> and must contain sequences for this analysis";
			}
			throw new ClassCastException(errmsg);
		}

		String params = "";
		if (sequenceType == UNKNOWN) {
			// ask the user
			BlastDialogReturn ret = BlastDialog.showDialog(seqA
					.getDescription(), seqB.getDescription());
			if (ret.canceled) {
				String errmsg = "Canceled by the user";
				logger.fatal(errmsg);
				JMessage.showMessageDialog(null, errmsg,
						JOptionPane.INFORMATION_MESSAGE,
						JOptionPane.DEFAULT_OPTION);
				return;
			}
			if (ret.sequenceType == BlastDialogReturn.NUCLEOTID)
				type = false;
			if (ret.choosenQuerySet == 1) { // vertausche a und b
				ISeqProperties tmp = seqA;
				seqA = seqB;
				seqB = tmp;

			}

			if (ret.paramStr != null)
				params = ret.paramStr;

		}
		logger.debug("Choosen parameters : '" + params + "'");
		mapper = new MapFasta(type, params);

		// FIXME: check if sequences are available!!
		desc1 = seqA.getDescription();
		desc2 = seqB.getDescription();

		logger.debug("start data retrieval");
		TupelIter inputA = new TupelIter(seqA.getSequencesNames(), listA);
		mapper.setSequenceIteratorQuery(inputA);

		TupelIter inputB = new TupelIter(seqB.getSequencesNames(), listB);
		try {
			mapper.setSequenceIteratorDatabase(inputB);
		} catch (IOException e) {
			if (e.getMessage().contains("formatdb")) {
				String errmsg = "The program <b>formatdb</b> could not executed<br>Check that it is in your path and has permissions to run";
				logger.fatal(errmsg);
				JMessage.showMessageDialog(null, errmsg,
						JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION,
						e);
				return;
			} else
				throw e;
		}

		logger.debug("start mapping");

		StringBuffer sb = new StringBuffer();
		sb.append("QUERY_ID\tSUBJECT_ID\tALIGNMENT_SIZE\tSCORE\tE_VALUE\t");
		sb.append("PERCENTAGE_IDENTITY\tQ_START\tQ_END\tS_START\tS_END");
		sb.append("\n");
		Iterator resIter = mapper.getResultIterator();
		int i = 0;
		long checktime = System.currentTimeMillis();
		while (resIter.hasNext()) {
			i++;
			try {
				ArrayList res = (ArrayList) resIter.next();
				if (res == null) {
					logger.warn("result gleich null, input error");
					sb.append(i + "#null\n");
				} else {
					sb.append("" + res.get(0));
					if (res.size() > 1) {
						for (int j = 1; j < res.size(); j++) {
							String field = (String) res.get(j);
							System.out.println("field: " + field);
							sb.append("\t" + field);
						}
					}
					sb.append("\n");
				}
				long dur = System.currentTimeMillis() - checktime;
				if (dur > 60000) {
					logger.info("FASTA finished until query protein " + i
							+ " of " + seqA.getDescription() + " vs. "
							+ seqB.getDescription());
					checktime = System.currentTimeMillis();
				}
			} catch (Exception e) {
				logger.error("Error at query number " + i, e);

			}
			// logger.debug(res.get(0)+"\t"+res.get(1));
		}
		resultStr = sb.toString();
		logger.debug("finished mapping");
	}

	public int getSequenceType() {
		return sequenceType;
	}

	/**
	 * If the sequence type is not set, the a dialog will pop up in the
	 * run-method
	 * 
	 * @param sequenceType
	 */
	public void setSequenceType(int sequenceType) {
		this.sequenceType = sequenceType;
	}
	
	public static void main(String[] args) {
		new TestFastaInstallation();
	}

}
