/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: PropertyComparison.java,v 1.6 2006/03/19 11:58:11 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.swissProt;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Formatter;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.TreeMap;

import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.WindowConstants;
import javax.swing.plaf.basic.BasicComboBoxRenderer;

import org.apache.log4j.Logger;
import org.jfree.ui.RefineryUtilities;

import prompt.engine.Engine2;
import prompt.engine.IEngineResults;
import prompt.input.IInput;
import prompt.input.classes.swissProt.internals.SortHashtable;
import prompt.input.swissProt.ISwissProt;
import prompt.util.graph.Graph;
import prompt.util.graph.Graph2;


public class PropertyComparison extends Engine2 implements Serializable{

    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private boolean printToConsole = true;
    private String de, os, oc, og, medline, pubmed, kw, ft, dom;
    private Hashtable deHash, osHash, ocHash, ogHash, medlineHash, pubmedHash, kwHash, ftHash, domHash;
    private Hashtable deFilteredHash, osFilteredHash, ocFilteredHash, ogFilteredHash, medlineFilteredHash, pubmedFilteredHash, kwFilteredHash, ftFilteredHash, domFilteredHash;
    private ISwissProt setA;
    private ISwissProt setB;
    private int sizeSetA;
    private int sizeSetB;
    private String descA;
    private String descB;
    private float ratioThreshold = 0.5f;
    private float abundanceThreshold = 0.01f;
    private float infinityThreshold = 0.01f;
    private int kBest = Integer.MAX_VALUE;
    private int keyColumnLength = 95;
    private int compareColumn = 6;
   //private String tmpPath;
  
    public void init(IInput setA, IInput setB) {
        
        try {
            this.setA = ((ISwissProt) setA);
        } catch (ClassCastException e) {
           String errmsg = "<b>The input '"+setA.getDescription()+" must of the type SWISS-PROT or UNIPROT</b>";
           throw new ClassCastException(errmsg);
        }        
        try {
            this.setB = ((ISwissProt) setB);
        } catch (ClassCastException e) {
           String errmsg = "<b>The input '"+setB.getDescription()+" must be type SWISS-PROT or UNIPROT</b>";
           throw new ClassCastException(errmsg);
        }
         
        this.setA = (ISwissProt) setA;
        this.setB = (ISwissProt) setB;
        this.sizeSetA = this.setA.getSequencesNames().size();
        this.sizeSetB = this.setB.getSequencesNames().size();
        
      
        if (sizeSetA==0) {
            String errmsg="<b>The input '"+setA.getDescription()+"' does not contain any proteins</b>";
            throw new ClassCastException(errmsg);
        }
        
        
        if (sizeSetB==0) {
            String errmsg="<b>The input '"+setB.getDescription()+"' does not contain any proteins</b>";
            throw new ClassCastException(errmsg);
        }
        
        
        
        
        
        this.descA=setA.getDescription();
        this.descB=setB.getDescription();
       // this.tmpPath = this.setA.getTmpPath();
        initHashtable();
        ratioThreshold = 0.5f;
        abundanceThreshold = 0.01f;
        infinityThreshold = 0.01f;
        kBest = Integer.MAX_VALUE;
    }

    private void initHashtable() {
   
        deHash = joinTables((Hashtable)setA.getDE().getDESummary().clone(),(Hashtable)setB.getDE().getDESummary().clone());
        osHash = joinTables((Hashtable)setA.getOS().getOSSummary().clone(), (Hashtable)setB.getOS().getOSSummary().clone());
        ocHash = joinTables((Hashtable)setA.getOC().getOCSummary().clone(), (Hashtable)setB.getOC().getOCSummary().clone());
        ogHash = joinTables((Hashtable)setA.getOG().getOGSummary().clone(), (Hashtable)setB.getOG().getOGSummary().clone());
        medlineHash = joinTables((Hashtable)setA.getRX().getMedlineSummary().clone(),(Hashtable) setB.getRX().getMedlineSummary().clone());
        pubmedHash = joinTables((Hashtable)setA.getRX().getPubMedSummary().clone(), (Hashtable)setB.getRX().getPubMedSummary().clone());
        kwHash = joinTables((Hashtable)setA.getKW().getKWSummary().clone(), (Hashtable)setB.getKW().getKWSummary().clone());
        ftHash = joinTables((Hashtable)setA.getFT().getFTSummary().clone(), (Hashtable)setB.getFT().getFTSummary().clone());
        domHash = joinTables((Hashtable)setA.getDOM().getDOMSummary().clone(), (Hashtable)setB.getDOM().getDOMSummary().clone());
    }
    
    public static void main(String[] args){
    	
    	PropertyComparison tet = new PropertyComparison();
    	tet.run();
    }
    
    public void run() {
    	
//    	 deFilteredHash = (Hashtable) deHash.clone();
//         de = differenceAnalysis(deFilteredHash, "EC (Enzyme Commission) numbers", ratioThreshold, abundanceThreshold, infinityThreshold, kBest, compareColumn);
//         
//         osFilteredHash = (Hashtable) osHash.clone();
//         os = differenceAnalysis(osFilteredHash, "OS (Organism Species)", ratioThreshold, abundanceThreshold, infinityThreshold, kBest, compareColumn);
//         
//         ocFilteredHash = (Hashtable) ocHash.clone();
//         oc = differenceAnalysis(ocFilteredHash, "OC (Organism Classification)", ratioThreshold, abundanceThreshold, infinityThreshold, kBest, compareColumn);
//         
//         ogFilteredHash = (Hashtable) ogHash.clone();
//         og = differenceAnalysis(ogFilteredHash, "Organelle", ratioThreshold, abundanceThreshold, infinityThreshold, kBest, compareColumn);
//         
//         medlineFilteredHash = (Hashtable) medlineHash.clone();
//         medline = differenceAnalysis(medlineFilteredHash, "MEDLINE IDs", ratioThreshold, abundanceThreshold, infinityThreshold, kBest, compareColumn);
//         
//         pubmedFilteredHash = (Hashtable) pubmedHash.clone();
//         pubmed = differenceAnalysis(pubmedFilteredHash, "PubMed IDs", ratioThreshold, abundanceThreshold, infinityThreshold, kBest, compareColumn);
//         
//         kwFilteredHash = (Hashtable) kwHash.clone();
//         kw = differenceAnalysis(kwFilteredHash, "KW (Keywords)", ratioThreshold, abundanceThreshold, infinityThreshold, kBest, compareColumn);
//         
//         ftFilteredHash = (Hashtable) ftHash.clone();
//         ft = differenceAnalysis(ftFilteredHash, "FT (Feature Table) key names", ratioThreshold, abundanceThreshold, infinityThreshold, kBest, compareColumn);
//         
        /* Uncomment the lower part to activate the ThresholdSetupForm, and also comment out the upper part!
         *   */ 
         
        // ThresholdSetupForm test =   
        	 new ThresholdSetupForm();
       
//        while(!test.getStatus()){
//            
//        }
        
    }
    
 
    public IEngineResults[] getResultsAll() throws Exception {  	
    	ArrayList descriptions = new ArrayList();
    	descriptions.add(setA.getDescription());
    	descriptions.add(setB.getDescription());
    	SwissProtEngineResult[] output = new SwissProtEngineResult[] {
    			new SwissProtEngineResult("EC (Enzyme Commission) numbers", de, deHash, descriptions,this.getClass().getName()+":ec"),
    			new SwissProtEngineResult("OS (Organism Species)", os, osFilteredHash,  descriptions,this.getClass().getName()+":os"),
    			new SwissProtEngineResult("OC (Organism Classification)", oc, ocHash,  descriptions,this.getClass().getName()+":oc"),
    			new SwissProtEngineResult("OG (Organelle)", og, ogFilteredHash,  descriptions,this.getClass().getName()+":og"),
    			new SwissProtEngineResult("MEDLINE IDs", medline, medlineFilteredHash, descriptions,this.getClass().getName()+":medline"),
    			new SwissProtEngineResult("PubMed IDs", pubmed, pubmedFilteredHash,  descriptions,this.getClass().getName()+":pubmed"),
    			new SwissProtEngineResult("KW (Keywords)", kw, kwFilteredHash,  descriptions,this.getClass().getName()+":kw"),
    			new SwissProtEngineResult("FT (Feature Table) key names", ft, ftFilteredHash, descriptions, this.getClass().getName()+":ft"),
    			new SwissProtEngineResult("DOM (Domains)", dom, domFilteredHash,  descriptions, this.getClass().getName()+":dom")
    			};
    	
    	return output;
    }

    

    
    public int hashCode() {
        return 0;
    }

    public Graph2 joinTrees(Graph tree1, Graph tree2) {

        Graph2 joined = new Graph2();
        Hashtable nodesTree1 = tree1.getNodes();

        return null;
    }

    protected Hashtable joinTables(Hashtable group1, Hashtable group2) {

        float absA;
        float absB;
        float relA;
        float relB;
        float absRatio;
        float relRatio;
        float logRatio;

        String curKey;
        Hashtable joined = new Hashtable();
        Iterator keys = group1.keySet().iterator();

        // iterate over all hashtable keys of protein set A
        while (keys.hasNext()) {
            curKey = (String) keys.next();
            if (group2.containsKey(curKey)) {
                // current key of protein set A also appears in hashtable key
                // set of protein set B
                absA = ((Integer) group1.get(curKey)).floatValue();
                absB = ((Integer) group2.get(curKey)).floatValue();
                // calculate relative frequencies
                relA = absA / sizeSetA;
                relB = absB / sizeSetB;
                // calculate absolute ratios between protein set A and B
                // minor value as numerator, major value as denominator
                if (absA < absB) {
                    absRatio = absA / absB;
                } else {
                    absRatio = absB / absA;
                }
                // calculate relative ratios between protein set A and B
                // minor value as numerator, major value as denominator
                if (relA < relB) {
                    relRatio = relA / relB;
                } else {
                    relRatio = relB / relA;
                }
                // calculate logarithmized relative ratios between protein set A and B
                logRatio = (float) (Math.log((double) relA / (double) relB) / Math.log(2.0));
                // insert statistics for current key in new hashtable
                joined.put(curKey, new float[] { absA, absB, relA, relB, absRatio, relRatio, logRatio });
                // remove current key from hashtable key set of protein set B (key already processed)
                group2.remove(curKey);
            } else {
                // current key of protein set A does not appear in hashtable key set of protein set B
                absA = ((Integer) group1.get(curKey)).floatValue();
                relA = absA / sizeSetA;
                // insert statistics for current key in new hashtable
                // absolute and relative ratios set to negative infinity
                joined.put(curKey, new float[] { absA, 0, relA, 0, Float.NEGATIVE_INFINITY, Float.NEGATIVE_INFINITY, Float.NEGATIVE_INFINITY });
            }
        }
        keys = group2.keySet().iterator();
        // iterate over remaining hashtable keys of protein set B
        while (keys.hasNext()) {
            // current key of protein set B does not appear in hashtable key set of protein set A
            curKey = (String) keys.next();
            absB = ((Integer) group2.get(curKey)).floatValue();
            relB = absB / sizeSetB;
            // insert statistics for current key in new hashtable
            // absolute and relative ratios set to negative infinity
            joined.put(curKey, new float[] { 0, absB, 0, relB, Float.NEGATIVE_INFINITY, Float.NEGATIVE_INFINITY, Float.NEGATIVE_INFINITY });
        }
        return joined;
    }

    /**
     * 
     * @param joined
     *            joinTables(group1, group2) result hashtable            count++;

     * @param ratioThreshold
     *            threshold (between 0 and 1), relative ratio has to be satisfied
     * @param abundanceThreshold
     *            minimum fraction (between 0 an 1) of combined absoluted frequencies with regard to size of both protein sets has to be satisfied
     * @param infinityThreshold
     *            minimum fraction (between 0 and 1) of absolute frequency with regard to size of protein set has to be satisfied
     * @param kBest
     *            display only k best keys  
     * @param compareColumn
     *            which column in array shall be considered to sort hashtable
     */
    protected String differenceAnalysis(Hashtable joined, String header, float ratioThreshold, float abundanceThreshold, float infinityThreshold, int kBest, int compareColumn) {
    	String key;
        float[] calc;
        
        SortHashtable toSort = new SortHashtable(joined, compareColumn);
    	TreeMap map = toSort.sort();

        Formatter formatter = new Formatter();
        if (printToConsole) {
            formatter.format("%1$-"+keyColumnLength+"."+keyColumnLength+"s     %2$4s     %3$4s     %4$5s     %5$5s     %6$9s     %7$9s     %8$9s %n %n", new String[] { header, "absA", "absB", "relA", "relB", "absRatio", "relRatio", "logRatio" });
        }
        // header line of result stringt
        String result = header + "\tabs "+descA+"\tabs "+descB+"\trel " +descA+"\trel "+descB+" \tabsRatio\trelRatio\tlogRatio\n";
    	int count = 0;
        Iterator iterator = map.keySet().iterator();
        while (iterator.hasNext() && count < kBest) {
        	key = (String) iterator.next();
            calc = (float[]) joined.get(key);
            if (calc[5] <= ratioThreshold) {
                // relative frequency ratio threshold satisfied
                if (calc[5] == Float.NEGATIVE_INFINITY) {
                    // only one protein set contains current key
                    if (calc[0] == 0) {
                        // absolute frequency of protein set A equals zero
                        if (calc[1] / sizeSetB >= infinityThreshold) {
                            // absolute frequency of protein set B satisfies infinity threshold
                            if (printToConsole) {
                                formatter.format("%1$-"+keyColumnLength+"."+keyColumnLength+"s     %2$4.0f     %3$4.0f     %4$5.3f     %5$5.3f     %6$9.3f     %7$9.3f     %8$9.3f %n", new Object[] {key, new Float((float) calc[0]), new Float((float) calc[1]), new Float((float) calc[2]), new Float((float) calc[3]), new Float((float) calc[4]), new Float((float) calc[5]), new Float((float) calc[6]) });
                            }
                            // enlarge result string
                            result += key + "\t" + calc[0] + "\t" + calc[1] + "\t" + calc[2] + "\t" + calc[3] + "\t" + calc[4] + "\t" + calc[5] + "\t" + calc[6] + "\n";
                            count++;
                        } else {
                        	// remove hashtable entries that do not satisfy given infinity threshold
                        	joined.remove(key);
                        }
                    } else {
                        // absolute frequency of protein set B equals zero
                        if (calc[0] / sizeSetA >= infinityThreshold) {
                            // absolute frequency of protein set A satisfies infinity threshold
                            if (printToConsole) {
                                formatter.format("%1$-"+keyColumnLength+"."+keyColumnLength+"s     %2$4.0f     %3$4.0f     %4$5.3f     %5$5.3f     %6$9.3f     %7$9.3f     %8$9.3f %n", new Object[] {key, new Float((float) calc[0]), new Float((float) calc[1]), new Float((float) calc[2]), new Float((float) calc[3]), new Float((float) calc[4]), new Float((float) calc[5]), new Float((float) calc[6]) });
                            }
                            // enlarge result string
                            result += key + "\t" + calc[0] + "\t" + calc[1] + "\t" + calc[2] + "\t" + calc[3] + "\t" + calc[4] + "\t" + calc[5] + "\t" + calc[6] + "\n";
                            count++;
                        } else {
                        	// remove hashtable entries that do not satisfy given infinity threshold
                        	joined.remove(key);
                        }
                    }
                } else {
                    // both protein sets contain current key and satisfy given relative frequency ratio threshold
                    if ((calc[0] + calc[1]) / (sizeSetA + sizeSetB) >= abundanceThreshold) {
                        // absolute frequency of protein set A and B satisfy abundance threshold
                        if (printToConsole) {
                            formatter.format("%1$-"+keyColumnLength+"."+keyColumnLength+"s     %2$4.0f     %3$4.0f     %4$5.3f     %5$5.3f     %6$9.3f     %7$9.3f     %8$9.3f %n", new Object[] { key, new Float((float) calc[0]), new Float((float) calc[1]), new Float((float) calc[2]), new Float((float) calc[3]), new Float((float) calc[4]), new Float((float) calc[5]), new Float((float) calc[6]) });
                        }
                        // enlarge result string
                        result += key + "\t" + calc[0] + "\t" + calc[1] + "\t" + calc[2] + "\t" + calc[3] + "\t" + calc[4] + "\t" + calc[5] + "\t" + calc[6] + "\n";
                        count++;
                    } else {
                    	// remove hashtable entries that do not satisfy given abundance threshold
                    	joined.remove(key);
                    }
                }
            } else {
            	// remove hashtable entries that do not satisfy given ratio threshold
            	joined.remove(key);
            }
        }
        while (iterator.hasNext()){
        	// if k best keys have been taken, left keys in hashtable must be removed
        	joined.remove(iterator.next());
        }
        if (printToConsole) {
            formatter.format("%n %n", new Object[] {});
            Logger logger = Logger.getLogger(this.getClass());
            logger.debug(formatter);
        }
        // remove last newline to avoid empty line in DataTable
        result = result.replaceAll("\n$", "");
        return result;
    }

    public float getAbundanceThreshold() {
        return abundanceThreshold;
    }

    public void setAbundanceThreshold(float abundanceThreshold) {
        this.abundanceThreshold = abundanceThreshold;
    }

    public float getInfinityThreshold() {
        return infinityThreshold;
    }

    public void setInfinityThreshold(float infinityThreshold) {
        this.infinityThreshold = infinityThreshold;
    }

    public float getRatioThreshold() {
        return ratioThreshold;
    }

    public void setRatioThreshold(float ratioThreshold) {
        this.ratioThreshold = ratioThreshold;
    }

    
    public class ThresholdSetupForm extends javax.swing.JFrame {
        


//        private float ratioThreshold = 0.5f;
//        private float abundanceThreshold = 0.01f;
//        private float infinityThreshold = 0.01f;
        
        /**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		private int kBest = Integer.MAX_VALUE;
        private boolean ready =false;
        /** Creates new form NewJFrame */
  public boolean getStatus(){
      
      return ready;
  }
        public ThresholdSetupForm() {
            initComponents();
            
        }
        
        /** This method is called from within the constructor to
         * initialize the form.
         * WARNING: Do NOT modify this code. The content of this method is
         * always regenerated by the Form Editor.
         */
        // <editor-fold defaultstate="collapsed" desc=" Generated Code ">
    
        private void initComponents() {
            
        	dialog = new JDialog(this,true);
            //jPanel1 = new javax.swing.JPanel();
        	dialog.setModal(true);
            kBestTextField = new javax.swing.JTextField();
            ratioTextField = new javax.swing.JTextField();
            abundanceTextField = new javax.swing.JTextField();
            infinityTextField = new javax.swing.JTextField();
            kBestCheckBox = new javax.swing.JCheckBox();
            okButton = new javax.swing.JButton();
            cancelButton = new javax.swing.JButton();
            sortComboBox = new javax.swing.JComboBox();
            jTextPane1 = new JTextPane();
            kBestDescription = new JEditorPane();
            kBestDescription.setContentType("text/html");
            //kBestDescription.setSize(205,55);
            kBestDescription.setBounds(10, 120, 250, 60);
            kBestDescription.setText("<html>Specify kBest to display only the k best values.</html>");
            kBestDescription.setEditable(false);
            dialog.add(kBestDescription);
            abundanceDescription = new JEditorPane();
            abundanceDescription.setContentType("text/html");
            //abundanceDescription.setSize(205,60);
            abundanceDescription.setBounds(10, 185, 250, 60);
            abundanceDescription.setText("<html>Specify the minimum relative frequency for the combined SwissProt objects.</html>");
            abundanceDescription.setEditable(false);

            dialog.add(abundanceDescription);
            ratioDescription = new JEditorPane();
            ratioDescription.setContentType("text/html");
            //ratioDescription.setSize(205,55);
            ratioDescription.setBounds(10, 250, 250, 60);
            ratioDescription.setText("<html>Specify the maximum relative ratio.</html>");
            ratioDescription.setEditable(false);
            dialog.add(ratioDescription); 
            minRelDescription = new JEditorPane();
            minRelDescription.setContentType("text/html");
            //minRelDescription.setSize(205,55);
            minRelDescription.setBounds(10, 315, 250, 60);
            minRelDescription.setText("<html>Specify the minimum relative frequency if an property entry only appears in one SwissProt object.</html>");
            minRelDescription.setEditable(false);
            dialog.add(minRelDescription);
            consoleScrollPane = new JScrollPane();
            consoleScrollPane.setBounds(10,385,355,175);
            
            console = new JEditorPane();
            console.setContentType("text/html");
            console.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.RAISED), "Information", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 1, 12)));
            console.setEditable(false);           
            
            consoleScrollPane.setViewportView(console);
            dialog.getContentPane().add(consoleScrollPane);
          
            setTitle("Setup");
            setState(1);
            dialog.setLayout(null);

            kBestTextField.setEditable(false);
           
            kBestTextField.setBorder(new javax.swing.border.TitledBorder("kBest"));
            kBestTextField.setName("kBestTextField");
            dialog.add(kBestTextField);
            kBestTextField.setBounds(290, 120,75, 60);
            kBestTextField.setFont(new Font("Serif", Font.PLAIN, 16));
            kBestTextField.getAccessibleContext().setAccessibleDescription("kBest");
            kBestTextField.addMouseMotionListener(new MouseMotionListener(){
            	public void mouseMoved(MouseEvent evt){console.setText("<html><font size=5 ; color=red><u><b>kBest Threshold</b></u></font><br>" +
            			"Specify the maximum number of results that shall be displayed (disable to show all results). The k best entries will be chosen depending on the sorting type selected below.</html>");}
            	public void mouseDragged(MouseEvent evt){}
            });
           
            ratioTextField.setBorder(new javax.swing.border.TitledBorder("maxRelRatio"));
            ratioTextField.setName("ratioTextField");
            dialog.add(ratioTextField);
            ratioTextField.setBounds(270, 250, 95, 60);
            ratioTextField.setText(ratioThreshold+"");
            ratioTextField.setFont(new Font("Serif", Font.PLAIN, 16));
            ratioTextField.addMouseMotionListener(new MouseMotionListener(){
            	public void mouseMoved(MouseEvent evt){console.setText("<html><font size=5 ; color=red><u><b>maxRelRatio Threshold</b></u></font><br>" +
            			"The maximum relative frequency ratio (between 0 and 1) has to be satisfied.<br><br>" +
            			"In detail, the formula<br>" +
            			"<p><img src=\"file:../studioBase/src/bea/engine/swissProt/maxRelRatio.gif\", alt=\"maxRelRatio\"></p><br>" +
//            			"<b>&nbsp;&nbsp;(freqRelative(setA) / freqRelative(setB)) &lt maxRelRatio</b><br>" +
            			"has to be true.</font></html>");}
            	public void mouseDragged(MouseEvent evt){}
            });
            abundanceTextField.setBorder(new javax.swing.border.TitledBorder("abundance"));
            dialog.add(abundanceTextField);
            abundanceTextField.setBounds(270, 185, 95, 60);
            abundanceTextField.setText(abundanceThreshold+"");
            abundanceTextField.setFont(new Font("Serif", Font.PLAIN, 16));
            abundanceTextField.addMouseMotionListener(new MouseMotionListener(){
            	public void mouseMoved(MouseEvent evt){console.setText("<html><font size=5 ; color=red><u><b>abundance Threshold</b></u></font><br>" +
            			"The minimum relative frequency (between 0 an 1) that has to be satisfied of the accumulated absolute frequencies of an property entry in both SwissProt objects with respect to the number of property entries in both SwissProt objects.<br><br>" +
            			"In detail, the formula<br>" +
            			"<p><img src=\"file:../studioBase/src/bea/engine/swissProt/abundance.gif\", alt=\"abundance\"></p><br>" +
//            			"<b>&nbsp;&nbsp;(freqAbsolute(setA) + freqAbsolute(setB)) / (size(setA) + size(setB)) >= abundance</b><br>" +
            			"has to be true.</html>");}
            	public void mouseDragged(MouseEvent evt){}
            });
            infinityTextField.setBorder(new javax.swing.border.TitledBorder("minRelRatio"));
            dialog.add(infinityTextField);
            infinityTextField.setBounds(270, 315, 95, 60);
            infinityTextField.setText(infinityThreshold+"");
            infinityTextField.setFont(new Font("Serif", Font.PLAIN, 16));
            infinityTextField.addMouseMotionListener(new MouseMotionListener(){
            	public void mouseMoved(MouseEvent evt){console.setText("<html><font size=5 ; color=red><u><b>minRelRatio Threshold</b></u></font><br>" +
            			"The minimum relative frequency (between 0 an 1) that has to be satisfied of an property entry in one SwissProt object if the relative frequency of the corresponding property entry in the other SwissProt object equals 0 (and therefore no meaningful ratios can be calculated).<br><br>" +
            			"In detail, the formula<br>" +
            			"<p><img src=\"file:../studioBase/src/bea/engine/swissProt/minRelRatio.gif\", alt=\"minRelRatio\"></p><br>" +            			
//            			"<b>&nbsp;&nbsp;(freqAbsolute(setA) / size(setA)) >= minRelRatio</b> or <b><br>&nbsp;&nbsp;freqRelative(setA) >= minRelRatio</b><br>" +
            			"has to be true.</html>");}
            	public void mouseDragged(MouseEvent evt){}
            });
            kBestCheckBox.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    kBestCheckBoxActionPerformed(evt);
                }
            });
            kBestCheckBox.setBounds(270, 140, 20, 20);
            dialog.add(kBestCheckBox);
            okButton.setBounds(200, 585, 50, 30);

            okButton.setFont(new java.awt.Font("SansSerif", 1, 10));
            okButton.setText("Ok");
            okButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    okButtonActionPerformed(evt);
                }
            });

            
            dialog.add(okButton);
     

            cancelButton.setFont(new java.awt.Font("SansSerif", 1, 10));
            cancelButton.setText("Cancel");
            cancelButton.setPreferredSize(new java.awt.Dimension(46, 2));
            cancelButton.addActionListener(new java.awt.event.ActionListener(){
            	public void actionPerformed(java.awt.event.ActionEvent evt){
            		cancelButtonActionPerformed(evt);
            	}
            });
            dialog.add(cancelButton);
            cancelButton.setBounds(270, 585, 70, 30);
            
            sortComboBox.setBorder(new javax.swing.border.TitledBorder("sort by"));
            dialog.add(sortComboBox);  
            sortComboBox.setBounds(20, 570, 150, 50);
            sortComboBox.addItem(new String("absolute frequency (" + setA.getDescription() + ")"));
            sortComboBox.addItem(new String("absolute frequency (" + setB.getDescription() + ")"));
            sortComboBox.addItem(new String("relative frequency (" + setA.getDescription() + ")"));
            sortComboBox.addItem(new String("relative frequency (" + setB.getDescription() + ")"));
            sortComboBox.addItem(new String("absolute ratio"));
            sortComboBox.addItem(new String("relative ratio"));
            sortComboBox.addItem(new String("log ratio"));
            
            sortComboBox.setSelectedIndex(5);
            sortComboBox.setToolTipText("<html>Select the value which shall be used to sort the results.</html>");
            sortComboBox.setRenderer(new BasicComboBoxRenderer() {
                public Component getListCellRendererComponent( JList list, 
                        Object value, int index, boolean isSelected, boolean cellHasFocus) {
                   if (isSelected) {
                     setBackground(list.getSelectionBackground());
                     setForeground(list.getSelectionForeground());      
                     if (-1 < index) {
                       list.setToolTipText(sortComboBox.getItemAt(index).toString());
                     }
                   } else {
                     setBackground(list.getBackground());
                     setForeground(list.getForeground());
                   }	
                   setFont(list.getFont());
                   setText((value == null) ? "" : value.toString());     
                   return this;
                 }  
            });
//			sortComboBox.setToolTipText(e.getItem().toString());
                  
        
			jTextPane1.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.RAISED), "Description", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 1, 12)));
            jTextPane1.setText("This engine compares two sets of SwissProt files. To manipulate and configure the result of the comparison, you can configure the parameters below.The values set at the start are the default values. Draw mouse over each parameter to see extra information, wich will be displayed in the information area.");
            jTextPane1.setEditable(false);
            JScrollPane jScrollTextPanel1 = new JScrollPane();
            jScrollTextPanel1.setBounds(10, 10, 355, 100);
            jScrollTextPanel1.setViewportView(jTextPane1);
            dialog.add(jScrollTextPanel1);
            dialog.setTitle("PropertyComparison Setup");
            setPreferredSize(new Dimension(220,300));
            RefineryUtilities.centerFrameOnScreen(this);
            setExtendedState(Frame.MAXIMIZED_BOTH);
            this.setAlwaysOnTop(true);
            this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
            
            
            
            dialog.setMinimumSize(new Dimension(220,300));
           
            dialog.setSize(385,660);
            RefineryUtilities.centerFrameOnScreen(dialog);
            dialog.setVisible(true);
          
           // setVisible(true);
        }
      
        private boolean getFieldInput(){
            boolean setupOk = true;
            
            if(kBestTextField.isEditable()){
                
                String kBestString = kBestTextField.getText();
                if(kBestString.length()==0){
                    
                    setupOk =false;
                }else{
                    this.kBest=new Integer(kBestString).intValue();
                    if(kBest<1){setupOk=false;}
                }
                
                    
             }
             
             String infinityString = infinityTextField.getText();
             String abundanceString = abundanceTextField.getText();
             String ratioString = ratioTextField.getText();
             if(infinityString.length()==0 || ratioString.length()==0 || abundanceString.length()==0){
                 setupOk =false;
             }else{
            	
                 
                 infinityThreshold = new Float(infinityString).floatValue();
                 ratioThreshold = new Float(ratioString).floatValue();
                 abundanceThreshold = new Float(abundanceString).floatValue();
                 compareColumn= sortComboBox.getSelectedIndex();
                 if(0>infinityThreshold || infinityThreshold>1){setupOk=false;}
                 if(0>ratioThreshold || ratioThreshold>1){setupOk=false;}
                 if(0>abundanceThreshold || abundanceThreshold>1){setupOk=false;}
                 Logger logger = Logger.getLogger(this.getClass());
                 logger.debug(compareColumn);
             }
            
            return setupOk;
        }
        private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt){
        	this.dispose();
        }
        private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {                                         
         
          if(getFieldInput()){
              
           /*   logger.debug("compareColumn "+compareColumn);
              logger.debug("ratioThreshold "+ratioThreshold);
              logger.debug("infinityThreshold "+infinityThreshold);
              System.exit(kBest);*/
              
              deFilteredHash = (Hashtable) deHash.clone();
              de = differenceAnalysis(deFilteredHash, "EC (Enzyme Commission) numbers", ratioThreshold, abundanceThreshold, infinityThreshold, kBest, compareColumn);
              
              osFilteredHash = (Hashtable) osHash.clone();
              os = differenceAnalysis(osFilteredHash, "OS (Organism Species)", ratioThreshold, abundanceThreshold, infinityThreshold, kBest, compareColumn);
              
              ocFilteredHash = (Hashtable) ocHash.clone();
              oc = differenceAnalysis(ocFilteredHash, "OC (Organism Classification)", ratioThreshold, abundanceThreshold, infinityThreshold, kBest, compareColumn);
              
              ogFilteredHash = (Hashtable) ogHash.clone();
              og = differenceAnalysis(ogFilteredHash, "Organelle", ratioThreshold, abundanceThreshold, infinityThreshold, kBest, compareColumn);
              
              medlineFilteredHash = (Hashtable) medlineHash.clone();
              medline = differenceAnalysis(medlineFilteredHash, "MEDLINE IDs", ratioThreshold, abundanceThreshold, infinityThreshold, kBest, compareColumn);
              
              pubmedFilteredHash = (Hashtable) pubmedHash.clone();
              pubmed = differenceAnalysis(pubmedFilteredHash, "PubMed IDs", ratioThreshold, abundanceThreshold, infinityThreshold, kBest, compareColumn);
              
              kwFilteredHash = (Hashtable) kwHash.clone();
              kw = differenceAnalysis(kwFilteredHash, "KW (Keywords)", ratioThreshold, abundanceThreshold, infinityThreshold, kBest, compareColumn);
              
              ftFilteredHash = (Hashtable) ftHash.clone();
              ft = differenceAnalysis(ftFilteredHash, "FT (Feature Table) key names", ratioThreshold, abundanceThreshold, infinityThreshold, kBest, compareColumn);
              
              domFilteredHash = (Hashtable) domHash.clone();
              dom = differenceAnalysis(domFilteredHash, "DOM (Domains)", ratioThreshold, abundanceThreshold, infinityThreshold, kBest, compareColumn);
              
              ready=true;
           //   this.setVisible(false);
              this.dispose();
          }else{
              JOptionPane.showMessageDialog(null, "At least one threshold field is empty or entered values are not in allowed range!","Warning!",JOptionPane.OK_OPTION);
              }
              
            
            
         
        }                                        
        
        private void kBestCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {                                              
            
            
            
            if(this.kBestCheckBox.isSelected()){
               
                this.kBestTextField.setEditable(true);
            }
            else{
                
            	this.kBestTextField.setText("");
                this.kBestTextField.setEditable(false);
                kBest = Integer.MAX_VALUE;
            }
        }                                             
        
        /**
         * @param args the command line arguments
         */
//        public static void main(String args[]) {
//            java.awt.EventQueue.invokeLater(new Runnable() {
//                public void run() {
//                    new ThresholdSetupForm().setVisible(true);
//                }
//            });
//      ;
//        }
        
        // Variables declaration - do not modify
        private javax.swing.JTextField abundanceTextField;
        private javax.swing.JButton cancelButton;
        private javax.swing.JTextField infinityTextField;
        private javax.swing.JComboBox sortComboBox;
        private javax.swing.JDialog dialog;       
        private javax.swing.JCheckBox kBestCheckBox;
        private javax.swing.JTextField kBestTextField;
        private javax.swing.JButton okButton;
        private javax.swing.JTextField ratioTextField;
        private javax.swing.JTextPane jTextPane1;
        private JScrollPane  consoleScrollPane;
        
        private JEditorPane console;
        private JEditorPane abundanceDescription;
        private JEditorPane minRelDescription;
        private JEditorPane ratioDescription;
        private JEditorPane kBestDescription;
        // End of variables declaration
        
    }
}
