/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: PropertyComparisonN.java,v 1.4 2006/03/02 21:13:23 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.engine.swissProt;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.TreeMap;

import prompt.engine.IEngineN;
import prompt.engine.IEngineResults;
import prompt.input.IInput;
import prompt.input.classes.swissProt.internals.SortHashtable;
import prompt.input.filter.IFilterValue;
import prompt.input.swissProt.ISwissProt;


public class PropertyComparisonN implements IEngineN {

    /*
     * (non-Javadoc)
     * 
     * @see bea.engine.IEngineN#init(prompt.input.IInput[])
     */
    private final String[] ALPHABET = new String[] { "A", "B", "C", "D", "E",
            "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R",
            "S", "T", "U", "V", "W", "X", "Y", "Z", };

    private String de, os, oc, og, medline, pubmed, kw, ft;

    private Hashtable deHash, osHash, ocHash, ogHash, medlineHash, pubmedHash,
            kwHash, ftHash;

    private Hashtable deFilteredHash, osFilteredHash, ocFilteredHash,
            ogFilteredHash, medlineFilteredHash, pubmedFilteredHash,
            kwFilteredHash, ftFilteredHash;

    private ISwissProt[] SPsets;

    private int[] SPsetsSize;

    private int nrSets;

    private int wholeSize = 0;

    // private float ratioThreshold = 0.5f;
    private float abundanceThreshold = 0.01f;

    // private float infinityThreshold = 0.01f;
    //private String tmpPath;

    private ArrayList descriptions;

    public void init(IInput[] inputArray) throws Exception {
        // TODO Auto-generated method stub
        this.descriptions = new ArrayList();
        this.SPsets = (ISwissProt[]) inputArray;
        this.nrSets = SPsets.length;
        SPsetsSize = new int[nrSets];
        for (int i = 0; i < nrSets; i++) {

            SPsetsSize[i] = SPsets[i].getSequencesNames().size();
            descriptions.add(SPsets[i].getDescription());
        }

        //tmpPath = SPsets[0].getTmpPath();

        for (int i = 0; i < SPsetsSize.length; i++) {

            this.wholeSize = +SPsetsSize[i];
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see bea.engine.IEngine#writeResultsToTmpFile()
     */
    private void initHashTables() {

        Hashtable[] propertyResults = new Hashtable[nrSets];
        for (int i = 0; i < nrSets; i++) {

            propertyResults[i] = this.SPsets[i].getDE().getDESummary();
        }
        this.deHash = joinTables(propertyResults);

        for (int i = 0; i < nrSets; i++) {

            propertyResults[i] = this.SPsets[i].getOG().getOGSummary();
        }
        this.ogHash = joinTables(propertyResults);

        for (int i = 0; i < nrSets; i++) {

            propertyResults[i] = this.SPsets[i].getOC().getOCSummary();
        }
        this.ocHash = joinTables(propertyResults);

        for (int i = 0; i < nrSets; i++) {

            propertyResults[i] = this.SPsets[i].getKW().getKWSummary();
        }
        this.kwHash = joinTables(propertyResults);

        for (int i = 0; i < nrSets; i++) {

            propertyResults[i] = this.SPsets[i].getFT().getFTSummary();
        }
        this.ftHash = joinTables(propertyResults);

        for (int i = 0; i < nrSets; i++) {

            propertyResults[i] = this.SPsets[i].getOS().getOSSummary();
        }
        this.osHash = joinTables(propertyResults);

        for (int i = 0; i < nrSets; i++) {

            propertyResults[i] = this.SPsets[i].getRX().getMedlineSummary();
        }
        this.medlineHash = joinTables(propertyResults);

        for (int i = 0; i < nrSets; i++) {

            propertyResults[i] = this.SPsets[i].getRX().getPubMedSummary();
        }
        this.pubmedHash = joinTables(propertyResults);

    }

    private Hashtable joinTables(Hashtable[] resultTables) {

        float absA;
        float absB;
        float relA;
        float relB;
        String curKey;
        Hashtable output = new Hashtable();

        // Hashtable temp = this.SPsets[0].getDE().getDESummary();
        float[] valueArray;
        Iterator it = resultTables[0].keySet().iterator();

        while (it.hasNext()) {

            valueArray = new float[2 * nrSets];
            curKey = (String) it.next();
            absA = ((Integer) resultTables[0].get(curKey)).floatValue();
            relA = absA / this.SPsetsSize[0];
            valueArray[0] = absA;
            valueArray[nrSets] = relA;
            output.put(curKey, valueArray);
        }

        for (int i = 1; i < nrSets; i++) {

            // temp =this.SPsets[i].getDE().getDESummary();
            it = resultTables[i].keySet().iterator();
            while (it.hasNext()) {

                curKey = (String) it.next();
                absA = ((Integer) resultTables[i].get(curKey)).floatValue();
                relA = absA / this.SPsetsSize[i];
                if (output.contains(curKey)) {

                    valueArray = (float[]) output.get(curKey);
                    valueArray[i] = absA;
                    valueArray[nrSets + i] = relA;
                    output.put(curKey, valueArray);
                } else {
                    valueArray = new float[2 * nrSets];
                    valueArray[i] = absA;
                    valueArray[nrSets + i] = relA;
                    output.put(curKey, valueArray);

                }

            }
        }
        return output;
    }

    public String writeResultsToTmpFile() throws Exception {
        // TODO Auto-generated method stub
        return null;
    }

    protected String DifferenceAnalysis(Hashtable joined, String header,
            float abundanceThreshold) {

        String result = header;
        String key;
        float[] calc;

        for (int i = 0; i < this.nrSets; i++) {

            result += "\tabs" + this.ALPHABET[i];
        }
        for (int i = 0; i < nrSets; i++) {

            result += "\rel" + this.ALPHABET[i];
        }
        result += "\n";
        SortHashtable toSort = new SortHashtable(joined, 0);
        TreeMap map = toSort.sort();
        float tempAbsoluteSum;
        Iterator it = map.keySet().iterator();

        while (it.hasNext()) {

            tempAbsoluteSum = 0.0f;
            key = (String) it.next();
            calc = (float[]) map.get(key);

            for (int i = 0; i < calc.length / 2; i++) {
                tempAbsoluteSum = +calc[i];
            }

            if (tempAbsoluteSum / this.wholeSize >= abundanceThreshold) {
                // absolute frequency of protein sets satisfy abundance
                // threshold

                // enlarge result string
                result = key;
                for (int i = 0; i < nrSets; i++) {
                    result += "\t" + calc[i];
                }
                result += key + "\t" + calc[0] + "\t" + calc[1] + "\t"
                        + calc[2] + "\t" + calc[3] + "\t" + calc[4] + "\t"
                        + calc[5] + "\t" + calc[6] + "\n";

            } else {
                // remove hashtable entries that do not satisfy given abundance
                // threshold
                joined.remove(key);
            }
        }

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see bea.engine.IEngine#setRownames()
     */
    public void setRownames() {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     * 
     * @see bea.engine.IEngine#getResults()
     */
    public IEngineResults getResults() throws Exception {
        // TODO Auto-generated method stub
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see bea.engine.IEngine#getResultsAll()
     */
    public IEngineResults[] getResultsAll() throws Exception {

        SwissProtEngineResult[] output = new SwissProtEngineResult[] {
                new SwissProtEngineResult("Description", de, deHash,
                         descriptions),
                new SwissProtEngineResult("OrganismSpecies", os,
                        osFilteredHash, descriptions),
                new SwissProtEngineResult("OrganismClassification", oc, ocHash,
                        descriptions),
                new SwissProtEngineResult("medline", medline,
                        medlineFilteredHash,  descriptions),
                new SwissProtEngineResult("pubmed", pubmed, pubmedFilteredHash,
                         descriptions),
                new SwissProtEngineResult("Keyword", kw, kwFilteredHash,
                         descriptions),
                new SwissProtEngineResult("FeatureTable", ft, ftFilteredHash,
                         descriptions) };
        // DataTable deTable = new DataTable(new
        // SwissProtEngineResult("Description", de, deHash, this.tmpPath), "EC",
        // setA.getDescription(), setB.getDescription());
        // deTable.showTable();
        // DataTable kwTable = new DataTable(new
        // SwissProtEngineResult("Keyword", kw, kwFilteredHash, this.tmpPath),
        // "KW", setA.getDescription(), setB.getDescription());
        // kwTable.showTable();
        return output;
    }

    /*
     * (non-Javadoc)
     * 
     * @see prompt.input.IJobTemplate#run()
     */
    public void run() throws Exception {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     * 
     * @see prompt.input.filter.IFilterValueAble#getFilterValue()
     */
    public IFilterValue getFilterValue() {
        // TODO Auto-generated method stub
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see prompt.input.filter.IFilterValueAble#setFilterValue(prompt.input.filter.IFilterValue)
     */
    public void setFilterValue(IFilterValue filter) {
        // TODO Auto-generated method stub

    }

	public IInput[] getInputsAll() throws Exception {
		// TODO Auto-generated method stub
		return null;
	}

}
