/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: IAnnotationData.java,v 1.2 2006/01/05 02:19:13 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input;

import java.util.HashMap;

/**
 * @author tschmidt
 * Format TAB-Files: first column= protein code<br>
 * Format XML (see source!):<br>
 * 
 * <protein code="xxx">
 *   <anno>Some value here</anno>
 *   <anno>123</anno>
 * </protein>
 * <protein>
 * ...
 * </protein>
 */
public interface IAnnotationData extends IDataFile {

    /**
     * Reads input file so that it the values
     * can be accessed via the method
     * getAnnotations<br><!-- html doc -->
     * 
     * &nbsp;&lt;annotations&gt;<br>
     &lt;<b>protein</b> code=&quot;xxx&quot;&gt;<br>
     &nbsp;&nbsp;&nbsp;&nbsp; &lt;anno&gt;Some value here&lt;/anno&gt;<br>
     &nbsp;&nbsp;&nbsp;&nbsp; &lt;anno&gt;123&lt;/anno&gt;<br>
     &nbsp; &lt;<b>/protein</b>&gt;<br>
     &nbsp; &lt;protein&gt;<br>
     &nbsp;&nbsp; ...<br>
     &nbsp; &lt;/protein&gt;<br>
     &lt;/annotations&gt;
     
     * @param filename path and filename 
     * @param fileType 0=flatfile, 1=xml
     */
    public void init(String filename, int fileType);

    /**
     * returns the annotations. Before calling this 
     * method the init method must have been called
     * @return HashMap
     * key = protein code (as String)<br>
     * value = String-Array (all values annotated with this
     * protein)
     */
    public HashMap getAnnotations();

}
