/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: IDesignability.java,v 1.2 2006/01/05 02:19:12 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input;

import java.util.List;

/**
 * @author Schmidt Thorsten <a href="mailto:t.schmidt@wzw.tum.de>t.schmidt@wzw.tum.de</a>
 *
 */
public interface IDesignability {

  /**
   * Protein designability was measured by counting the number of families in each fold contained in a
   * given protein and taking the minimum. For example, if protein A contains three domains with
   * folds F1, F2 and F3 and these folds in turn contain 8,3 and 7 families, respectively, proteins 
   * A's minimum family count would be 3. By recording the minimum family count of the folds ins proteins,
   * we assessed their designability by computing the designability of their least designable fold.
   * @param maxEvalue, recommended 1E-6, all SCOP fold assignments equal or better than
   * this threshold are incorporated
   * @return LinkedList of Integers
   */
  public List getDesignabilityMinFamilies(String maxEvalue); 

  /**
  * Similar to getDesignabilityMinFamilies,
  * but returns the average number of families instead the minimum number.<br>
  * For example, if protein A contains three domains with
   * folds F1, F2 and F3 and these folds in turn contain 8,3 and 7 families, respectively, proteins 
   * A's minimum family count would be (8+3+7)/3.
  * @param maxEvalue, recommended 1E-6, all SCOP fold assignments equal or better than
  * this threshold are incorporated
  * @return LinkedList of Doubles
  * @see #getDesignabilityMinFamilies(String)
  */
 public List getDesignabilityAvgFamilies(String maxEvalue);   
  
}
