/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: IDisorder.java,v 1.2 2006/01/05 02:19:11 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input;

/**
 * @author Schmidt Thorsten <a href="mailto:t.schmidt@wzw.tum.de>t.schmidt@wzw.tum.de</a>
 *
 */
public interface IDisorder {

  
  /**
   * Returns the disordered regions of each protein
   * @return String[][] array with the following columns:<br>
   * + 0: unique-prot_data_id,<br>
   * + and 1: begin-, 2: end-, 3:length of disordered region<br>
   * <b>Note:</b>Each protein can have multiple regions of disordered streches i.e. multiple array rows.<br>
   * In case of PEDANT, the unique-prot_data_id is the prot_data_id
   */
  public String[][] getDisorderDetails();
  
  
  
  /**
   * Returns the percentage of disordered residues and the average length of the disordered streches of each protein. <br>
   * The first array column is the unique protein identifier, the second and third the actual values
   * @return String[][] array with the following columns:<br>
   * + 0: unique-prot_data_id, <br>
   * + 1: percentage of disordered residues of each protein, <br>
   * + 2: average length (in aa) of each disordered stretch<br>
   * + 3: number of disordered streches<br>
   * + 4: minimum stretch length<br>
   * + 5: maximum stretch length<br>
   * In case of PEDANT, the unique-prot_data_id is the prot_data_id
   */
  public String[][] getDisordePercentageOfResidues();
  
  
  /**
   * average percentage of disordered residues of all proteins
   * @return double
   */
  public double getDisorderPercentageAll();
  
}
