/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: IEcNum.java,v 1.2 2006/01/05 02:19:11 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input;

import java.util.HashMap;

/**
 * @author Schmidt Thorsten <a href="mailto:t.schmidt@wzw.tum.de>t.schmidt@wzw.tum.de</a>
 *
 */
public interface IEcNum {

	/**
	 * Im Fall von PEDANT2: ist die Tabelle ecnum ueberhaupt da 
	 * und ist was drin, dann true
	 * @return boolean
	 */
	public boolean isEcNumAvailable();
	
	
	 /**
	 * Returns for each protein all Enzyme Category (EC) <b>top Levels</b> that are better
	 * than the choosen threshold<br>
	 * <b>Example:</b><br>
	 * prot_data.code -> 2;3;1;....<br>
	 * @param eValThreshold
	 * @return HashMap key=prot_data.code, value=Strichpunkt getrennte Liste von
	 * EC Top Levels
	 */
	public HashMap getEcLevel1Ids(String eValThreshold);
	

	/**
	 * Returns for each protein all Enzyme Category (EC) numbers of the <b>first two levels</b> that are better
	 * than the choosen threshold:<br>
	 * <b>Example:</b><br>
	 * prot_data.code -> 2.1;3.2;....<br>
	 * @param eValThreshold
	 * @return HashMap key=prot_data.code, value=Strichpunkt getrennte Liste von
	 * EC nummern (nur die ersten beiden Ebenen)
	 */
	public HashMap getEcLevel2Ids(String eValThreshold);
	


	
	
}
