/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: IFlatFile.java,v 1.2 2006/01/05 02:19:12 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input;

import java.util.List;

/**
 * @author Schmidt Thorsten <a href="mailto:t.schmidt@wzw.tum.de>t.schmidt@wzw.tum.de</a>
 *
 */
public interface IFlatFile extends ISeqProperties,IDataFile,IInput {
	
  final long serialVersionUID = -6849794470754667700L;
	
    /**
     * @param filename
     *   full path and filename of the Protein-Fasta file to read
     */
    public void init(String filename) throws Exception;
    
    
    /**
     * @param filename
     *                  full path and filename of the file to read
     * @param format
     *                  fasta, genbank,embl or swissprot
     * @param alphabetName
     *                  dna,rna or protein
     * @throws Exception
     */
    public void init(String filename, String format, String alphabetName) throws Exception;
    
	
	public List getAnnotation();
	
	
	/**
	 * @param format: the desired format
	 * @return true if this format is supported
	 */
	public boolean setFormat(String format);
	
	/**
	 * @param alphabet
	 * @return true if this alphabet is supported
	 */
	public boolean setAlphabet(String alphabet);
}
