/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: IFuncat.java,v 1.4 2006/04/10 13:24:43 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input;

import java.util.HashMap;

/**
 * FunCat annotations
 * 
 * @author Schmidt Thorsten <a href="mailto:t.schmidt@wzw.tum.de>t.schmidt@wzw.tum.de</a>
 */
public interface IFuncat {
  
  
	/**
	 * Im Fall von PEDANT2: ist die Tabelle funcat ueberhaupt da 
	 * und ist was drin, dann true
	 * @return boolean
	 */
	public boolean isFunCatAvailable();
	

  
  /**
	 * Returns for each protein all FunCat TOP Levels that are better
	 * than the choosen threshold<br>
	 * <b>Example:</b><br>
	 * prot_data.code -> 1;99;4;....<br>
	 * @param eValThreshold
	 * @return HashMap key=prot_data.code, value=Strichpunkt getrennte Liste von
	 * Funcat Top Level Numbers
	 */
	public HashMap getFuncatLevel1Ids(String eValThreshold);
	
	/**
	 * Returns plain human-readable descriptions of the Funcat TOP Levels that are within the pedant dataset<br>
	 * <b>Example:</b><br>
	 * classnr -> plain description<br>
	 * 01 -> METABOLISM<br>
	 * 99 -> UNASSIGNED<br>
	 * @param eValThreshold
	 * @return HashMap
	 */
	public HashMap getFuncatLevel1Desc();
	
	
	/**
	 * Returns for each protein all Funcat numbers of the first two levels that are better
	 * than the choosen threshold:<br>
	 * <b>Example:</b><br>
	 * prot_data.code -> 01.01;04.05;....<br>
	 * @param eValThreshold
	 * @return HashMap key=prot_data.code, value=Strichpunkt getrennte Liste von
	 * Funcat nummern (nur die ersten beiden Ebenen)
	 */
	public HashMap getFuncatLevel2Ids(String eValThreshold);
	
	/**
	 * Returns plain human-readable descriptions of Funcat Numbers that are within the pedant dataset:<br>
	 * <b>Example:</b><br>
	 * level1.level2 -> plain description<br>
	 * 01.01 -> amino acid metabolism<br>
	 * 04.01 -> rRNA transcription<br>
	 * @param eValThreshold
	 * @return HashMap
	 */
	public HashMap getFuncatLevel2Desc();
  
	/**
	 * @param eValThreshold
	 * @return HashMap
	 * @see #getFuncatLevel1Ids(String)
	 */
	public HashMap getFuncatLevel3Ids(String eValThreshold);
	
	/**
	 * @return HashMap
	 * @see #getFuncatLevel1Desc()
	 */
	public HashMap getFuncatLevel3Desc();
	
	/**
	 * @param eValThreshold
	 * @return HashMap
	 * @see #getFuncatLevel1Ids(String)
	 */
	public HashMap getFuncatLevel4Ids(String eValThreshold);
	
	/**
	 * @return HashMap
	 * @see #getFuncatLevel1Desc()
	 */
	public HashMap getFuncatLevel4Desc();

}
