package prompt.input;

public interface IFuncatExt {

	  /**
	   * liefert ein array von prot_data_ids von allen Proteinen die in der Tabelle funcat in der spalte fun_name EXAKT den suchstr haben, unter gruppen
	   * werden dabei nicht beruecksichtigt
	   * 
	   * @param str
	   * @return int array of prot_data_ids
	   */
	  public int[] getProteinsWithFunNum(String str);

	  /**
	   * liefert DISTINCT prot_data_id FROM funcat WHERE fun_num LIKE 'query%'
	   * 
	   * @param str
	   *          e.g. 04
	   * @return int array
	   */
	  public int[] getProteinsWithFunNumAtTheStart(String str);

	  /**
	   * Metabolism <br>
	   * Works only with Funcat schema 1.3!, <br>
	   * Returns all prot_data_id from the table funcat which have such a functional annotation: <br>
	   * 01.* METABOLISM
	   * 
	   * @return int array of prot_data_ids
	   */
	  public int[] getFuncat13_01xMetabolism();

	  /**
	   * Energy <br>
	   * Works only with Funcat schema 1.3!, <br>
	   * Returns all prot_data_id from the table funcat which have such a functional annotation: <br>
	   * 02.* ENERGY
	   * 
	   * @return int array of prot_data_ids
	   */
	  public int[] getFuncat13_02xEnergy();

	  /**
	   * Transcription <br>
	   * Works only with Funcat schema 1.3!, <br>
	   * Returns all prot_data_id from the table funcat which have such a functional annotation: <br>
	   * 04.* TRANSCRIPTION
	   * 
	   * @return int array of prot_data_ids
	   */
	  public int[] getFuncat13_04xTranscription();

	  /**
	   * Protein Synthesis <br>
	   * Works only with Funcat schema 1.3!, <br>
	   * Returns all prot_data_id from the table funcat which have such a functional annotation: <br>
	   * 05.* PROTEIN SYNTHESIS
	   * 
	   * @return int array of prot_data_ids
	   */
	  public int[] getFuncat13_05xProteinSynthesis();

	  
	  /**
	   * Protein Fate (folding, modification, destination)<br>
	   * Works only with Funcat schema 1.3!, <br>
	   * Returns all prot_data_id from the table funcat which have such a functional annotation: <br>
	   * 06.* PROTEIN FATE (folding, modification, destination) 
	   * 
	   * @return int array of prot_data_ids
	   */  
	  public int[] getFuncat13_06xProteinFate();

	  
	  /**
	   * Cellular transport and transport mechanisms <br>
	   * Works only with Funcat schema 1.3!, <br>
	   * Returns all prot_data_id from the table funcat which have such a functional annotation: <br>
	   * 08.* CELLULAR TRANSPORT AND TRANSPORT MECHANISMS
	   * 
	   * @return int array of prot_data_ids
	   */  
	  public int[] getFuncat13_08xCellularTransport();

	  
	  /**
	   * Cell rescue, defense and virulence<br>
	   * Works only with Funcat schema 1.3!, <br>
	   * Returns all prot_data_id from the table funcat which have such a functional annotation: <br>
	   * 11.* CELL RESCUE, DEFENSE AND VIRULENCE  
	   * 
	   * @return int array of prot_data_ids
	   */    
	  public int[] getFuncat13_11xCellResuce();

	  
	  /**
	   * Control of cellular organization<br>
	   * Works only with Funcat schema 1.3!, <br>
	   * Returns all prot_data_id from the table funcat which have such a functional annotation: <br>
	   * 30.* CONTROL OF CELLULAR ORGANIZATION   
	   * 
	   * @return int array of prot_data_ids
	   */    
	  public int[] getFuncat13_30xCellularOrganization();

	  
	  /**
	   * Subcellular localisation<br>
	   * Works only with Funcat schema 1.3!, <br>
	   * Returns all prot_data_id from the table funcat which have such a functional annotation: <br>
	   * 40.* SUBCELLULAR LOCALISATION    
	   * 
	   * @return int array of prot_data_ids
	   */   
	  public int[] getFuncat13_40xSubcellularLocalisation();

	  
	  /**
	   * Proteins with binding function or cofactor requirement (structural or catalytic)<br>
	   * Works only with Funcat schema 1.3!, <br>
	   * Returns all prot_data_id from the table funcat which have such a functional annotation: <br>
	   * 63.* PROTEIN WITH BINDING FUNCTION OR COFACTOR REQUIREMENT (structural or catalytic)
	   * 
	   * @return int array of prot_data_ids
	   */     
	  public int[] getFuncat13_63xProteinsWithBindingFunction();

	  
	  /**
	   * Transport facilitation<br>
	   * Works only with Funcat schema 1.3!, <br>
	   * Returns all prot_data_id from the table funcat which have such a functional annotation: <br>
	   * 67.* TRANSPORT FACILITATION 
	   * 
	   * @return int array of prot_data_ids
	   */       
	  public int[] getFuncat13_67xTransportFacilitation();

	  


	 

	  /**
	   * Metabolism and related <br>
	   * Works only with Funcat schema 2.0!, <br>
	   * Returns all prot_data_id from the table funcat which have such a functional annotation: <br>
	   * 01.* METABOLISM <br>
	   * 02.* ENERGY <br>
	   * 04.* STORAGE PROTEIN <br>
	   * 
	   * @return int array of prot_data_ids
	   */
	  public int[] getFuncat0xMetabolism();

	  /**
	   * Information pathways <br>
	   * Works only with Funcat schema 2.0!, <br>
	   * Returns all prot_data_id from the table funcat which have such a functional annotation: <br>
	   * 10.*, CELL CYCLE AND DNA PROCESSING <br>
	   * 11.*, TRANSCRIPTION <br>
	   * 12.* PROTEIN SYNTHESIS <br>
	   * 14.*, PROTEIN FATE (folding, modification, destination) <br>
	   * 16.*, PROTEIN WITH BINDING FUNCTION OR COFACTOR REQUIREMENT (structural or catalytic) <br>
	   * 18.* PROTEIN ACTIVITY REGULATION <br>
	   * 
	   * @return int array of prot_data_ids
	   */
	  public int[] getFuncat1xInformation();

	  /**
	   * Transport facilitation <br>
	   * Works only with Funcat schema 2.0!, <br>
	   * Returns all prot_data_id from the table funcat which have such a functional annotation: <br>
	   * 20.*, CELLULAR TRANSPORT, TRANSPORT FACILITATION AND TRANSPORT ROUTES <br>
	   * 
	   * @return int array of prot_data_ids
	   */
	  public int[] getFuncat2xTransport();

	  /**
	   * Perception and response to stimuli <br>
	   * Works only with Funcat schema 2.0!, <br>
	   * Returns all prot_data_id from the table funcat which have such a functional annotation: <br>
	   * 30.*, CELLULAR COMMUNICATION/SIGNAL TRANSDUCTION MECHANISM <br>
	   * 32.*, CELL RESCUE, DEFENSE AND VIRULENCE 34.*, INTERACTION WITH THE CELLULAR ENVIRONMENT <br>
	   * 36.* INTERACTION WITH THE ENVIRONMENT (Systemic) <br>
	   * 38.*, TRANSPOSABLE ELEMENTS, VIRAL AND PLASMID PROTEINS <br>
	   * 
	   * @return int array of prot_data_ids
	   */
	  public int[] getFuncat3xPerception();

	  /**
	   * Developmental processes <br>
	   * Works only with Funcat schema 2.0!, <br>
	   * Returns all prot_data_id from the table funcat which have such a functional annotation: <br>
	   * 40.*, CELL FATE <br>
	   * 41.*, DEVELOPMENT (Systemic) <br>
	   * 42.* BIOGENESIS OF CELLULAR COMPONENTS <br>
	   * 43.*, CELL TYPE DIFFERENTIATION <br>
	   * 45.* TISSUE DIFFERENTIATION <br>
	   * 47.*, ORGAN DIFFERENTIATION <br>
	   * 
	   * @return int array of prot_data_ids
	   */
	  public int[] getFuncat4xDevelopmental();

	  /**
	   * Localization <br>
	   * Works only with Funcat schema 2.0!, <br>
	   * Returns all prot_data_id from the table funcat which have such a functional annotation: <br>
	   * 70.*, SUBCELLULAR LOCALIZATION <br>
	   * 73.*, CELL TYPE LOCALIZATION <br>
	   * 75.* TISSUE LOCALIZATION <br>
	   * 77.*, ORGAN LOCALIZATION <br>
	   * 78.* ubiquitous expression <br>
	   * 
	   * @return int array of prot_data_ids
	   */
	  public int[] getFuncat7xLocalization();

	  /**
	   * Experimentally uncharacterized proteins <br>
	   * Works with Funcat schema 1.3 AND 2.0!, <br>
	   * Returns all prot_data_id from the table funcat which have such a functional annotation: <br>
	   * 98.*, CLASSIFICATION NOT YET CLEAR-CUT <br>
	   * 99.*, UNCLASSIFIED PROTEINS <br>
	   * 
	   * @return int array of prot_data_ids
	   */
	  public int[] getFuncat9xUnknown();
	  
	  
	
}
