/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: IGenericXML.java,v 1.4 2006/02/14 17:16:56 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input;


/**
 * Basically a Generic XML&nbsp; input files look like:</p>
<font SIZE="2" face="Arial">
<p>&lt;<b>dataset</b> label=&quot;<b>Escherichia_coli_k12</b>&quot;&gt;<br>
&nbsp;&nbsp;&nbsp;&nbsp; &lt;<b>property</b> type=&quot;<b><i>numeric</i></b>&quot; id=&quot;<b>orf.length</b>&quot;&gt;<br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; &lt;input 
prot_id=&quot;0&quot; value=&quot;66&quot; /&gt;<br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; &lt;input 
prot_id=&quot;1&quot; value=&quot;2463&quot; /&gt;<br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; ...<br>
&nbsp;&nbsp;&nbsp; &lt;/property&gt;<br>
&nbsp;&nbsp;&nbsp; &lt;<b>property</b> type=&quot;<b><i>symbolic</i></b>&quot; id=&quot;<b>funcat.fun_num</b>&quot;&gt;<br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; &lt;input 
prot_id=&quot;0&quot; value=&quot;01.01.01&quot; /&gt;<br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; &lt;input 
prot_id=&quot;1&quot; value=&quot;01.01.04&quot; /&gt;<br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; ...<br>
&lt;/dataset&gt;</p>
</font>
 * 
 * 
 * @author Schmidt Thorsten <a href="mailto:t.schmidt@wzw.tum.de>t.schmidt@wzw.tum.de</a>
 *
 */
public interface IGenericXML extends ISeqProperties,  IInput{
  
  
	/**
	 * Returns all Properties of this GenericXML file
	 * @return array of IGenericProperty
	 */
	public abstract IGenericProperty[] getProperties();
	
	/**
	 * Returns the property that has the same ID (value of the getId() function)
	 * that the supplied parameter string. If not found or if no properties are
	 * available null is returned.
	 * @param propertyId
	 * @return IGenericProperty
	 */
	public abstract IGenericProperty getProperty(String propertyId);
	
	
	/**
	 * Returns the value of the attribute label of an Generic XML file
	 * @return String
	 */
	public abstract String getLabel();
	

}
