/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: IInterpro.java,v 1.3 2006/01/05 20:42:46 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input;

import java.util.HashMap;

public interface IInterpro {

	/**
	 * Im Fall von PEDANT2: ist die Tabelle <b>interpro</b> ueberhaupt da 
	 * und ist was drin, dann true
	 * @return boolean
	 */
	public boolean isInterproAvailable();
	
	/**
	 * Returns for each protein all <b>INTERPRO-Ids</b>
	 * <b>Example:</b><br>
	 * prot_data.code -> IPR001048;IPR001342; IPR001064..<br>
	 * @return HashMap key=prot_data.code, value=Strichpunkt getrennte Liste von
	 * Ids
	 */
	public HashMap getInterproIds();
	
	
	/**
	 * Returns plain human-readable descriptions of the <b>INTERPRO</b>
	 * identifiers that are within the pedant dataset:<br>
	 * <b>Example:</b><br>
	 * id -> plain description<br>
	 * IPR001048 -> Aspartate/glutamate/uridylate kinase<br>
	 * IPR001342 -> Aspartate kinase<br>
	 * @param eValThreshold
	 * @return HashMap
	 */
	public HashMap getInterproDesc();
}
