/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: IPedant.java,v 1.3 2006/04/12 12:33:16 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input;

import tschmidt.dbDialog.DBProperties;




/**
 * @author Schmidt Thorsten <a href="mailto:t.schmidt@wzw.tum.de>t.schmidt@wzw.tum.de</a>
 *
 */
public interface IPedant extends 
ISeqProperties,IInput,
IFuncat, IFuncatExt, ICath, IScop, IEcNum, 
IBlimps, IInterpro, IPfam, IProsite,
IDisorder, IDesignability, ITango {

  final long serialVersionUID = -6849794470754667700L;

	/**
	 * Initializes a PEDANT input object with the specified database name
	 * @param dbname String
	 */
	public void init(String dbname);
	
	
	/**
	 * Returns the current used PEDANT database name
	 * @uml.property name="databaseName"
	 */
	public abstract String getDatabaseName();

	/**
	 * Sets the PEDANT database name
	 * @uml.property name="databaseName"
	 */
	public void setDatabaseName(String str);

	
	
	 /**
	 * 
	 * Sets the prot_data table name.<br>
	 * Usually this method does not need to be called by a common user of this class, 
	 * however it allows to set up highly flexible views by using special prot_data subsets
	 * @param str String
	 */
	public void setDataTableName(String str);
	
	
	/**
	 * Gets the current used prot_data table name. Usually this is 'prot-data', if no Pedant filter is used or if no user views are set up
	 * @return String
	 */
	public String getDataTableName();


	public String getScopTable();
    
	public DBProperties getDbProperties();

	public void setDbProperties(DBProperties dbProperties);
	
    
/*	public abstract List getScop();
	public abstract List getFunCat();
	public abstract List getBlast();
	public abstract List getCog();
	public abstract List getPfam();
	public abstract List getEcnum();	
	public abstract List getKnown3d();	
	public abstract List getPirkw();
	public abstract List getTaxon();	
	public abstract List getBlimps();
	public abstract List getPros();
	public abstract List getRep();
	*/

}
