/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: IPfam.java,v 1.2 2006/01/05 02:19:13 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input;

import java.util.HashMap;

/**
 * @author Schmidt Thorsten <a href="mailto:t.schmidt@wzw.tum.de>t.schmidt@wzw.tum.de</a>
 *
 */
public interface IPfam {

	/**
	 * Im Fall von PEDANT2: ist die Tabelle <b>pfam</b> ueberhaupt da 
	 * und ist was drin, dann true
	 * @return boolean
	 */
	public boolean isPfamAvailable();
	
	/**
	 * Returns for each protein all <b>PFAM-Ids</b>
	 * that are better than the choosen threshold<br>
	 * <b>Example:</b><br>
	 * prot_data.code -> PF00696;PF01842; PF01842..<br>
	 * @param eValThreshold
	 * @return HashMap key=prot_data.code, value=Strichpunkt getrennte Liste von
	 * Ids
	 */
	public HashMap getPfamIds(String eValThreshold);
	
	
	/**
	 * Returns plain human-readable descriptions of the <b>PFAM</b>
	 * identifiers that are within the pedant dataset:<br>
	 * <b>Example:</b><br>
	 * id -> plain description<br>
	 * PF00696 -> Amino acid kinase family<br>
	 * PF01842 -> ACT domain<br>
	 * @param eValThreshold
	 * @return HashMap
	 */
	public HashMap getPfamDesc();
}
