/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: IProsite.java,v 1.2 2006/01/05 02:19:12 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input;

import java.util.HashMap;

/**
 * @author Schmidt Thorsten <a href="mailto:t.schmidt@wzw.tum.de>t.schmidt@wzw.tum.de</a>
 *
 */
public interface IProsite {

	/**
	 * Im Fall von PEDANT2: ist die Tabelle <b>pros</b> ueberhaupt da 
	 * und ist was drin, dann true
	 * @return boolean
	 */
	public boolean isPrositeAvailable();
	
	/**
	 * Returns for each protein all <b>Prosite-Ids (PDID)</b><br>
	 * Since a Prosite Pattern either matches or not <b>no threshold</b> is necessary here<br>
	 * <b>Example:</b><br>
	 * prot_data.code -> PDOC00800;PDOC00289;PDOC00545..<br>
	 * @param eValThreshold
	 * @return HashMap key=prot_data.code, value=Strichpunkt getrennte Liste von
	 * Ids
	 */
	public HashMap getPrositeIds();
	
	
	/**
	 * Returns plain human-readable descriptions of the <b>Prosite (PDID)</b>
	 * identifiers that are within the pedant dataset:<br>
	 * <b>Example:</b><br>
	 * id -> plain description<br>
	 * PDOC00800 -> HOMOSER_DHGENASE<br>
	 * PDOC00289 -> ASPARTOKINASE<br>
	 * @param eValThreshold
	 * @return HashMap
	 */
	public HashMap getPrositeDesc();
}
