/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: ISeqProperties.java,v 1.8 2006/04/06 11:56:09 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input;

import java.util.List;

import org.biojava.bio.BioException;
import org.biojava.bio.symbol.IllegalAlphabetException;
import org.biojava.bio.symbol.IllegalSymbolException;

import tschmidt.scales.Scale;

/**
 * @author Schmidt Thorsten <a
 *         href="mailto:t.schmidt@wzw.tum.de>t.schmidt@wzw.tum.de</a>
 * 
 */
public interface ISeqProperties extends IInput, Iterable {
    final long serialVersionUID = -6849794470754667700L;

    public abstract List getSequences();

    public abstract List getSequencesNames();

    public abstract List getAnnotation();

    /**
     * @return: Number of sequences hold by this Input Object.<br>
     */
    public abstract int getSize();

    /**
     * @return List of Integer()
     */
    public abstract List getLength();

    /**
     * @return List of Double()
     * @throws IllegalSymbolException
     */
    public abstract List getMass() throws IllegalSymbolException;

    /**
     * @return List of Double()
     * @throws IllegalAlphabetException
     * @throws BioException
     */
    public abstract List getPI() throws IllegalAlphabetException, BioException;

    /**
     * @return List of Double()
     * @throws IllegalAlphabetException
     * @throws BioException
     */
    public List getPIawayFromNeutral() throws IllegalAlphabetException, BioException;

    /**
     * @param scale
     * @return List of Float()
     */
    public abstract List getHydrophobicityAvg(Scale scale);

    /**
     * @param scale
     * @return List of Float()
     */
    public abstract List getHydrophobicitySum(Scale scale);

    /**
     * @return
     */
    public abstract List getAlternatingHPhobHPhil();

//    public Iterator getIterator();
}