/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: Annotation.java,v 1.4 2006/01/06 23:52:37 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input.classes;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.log4j.Logger;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import prompt.input.IAnnotation;


/**
 * @author tschmidt
 *
 */
public class Annotation implements IAnnotation {

	String filename;

	int type;

	public static final int TAB_DELIMITED = 0;

	public static final int XML_FILE = 1;

	/* (non-Javadoc)
	 * @see prompt.input.IAnnotationData#init(java.lang.String, int)
	 */
	public void init(String filename, int fileType) {
		this.filename = filename;
		this.type = fileType;

	}

	/**
	 * parst ein tab-delimited text file
	 * @return HashMap
	 * key = first column
	 * values = Object[] (of Strings) der anderen spalten
	 */
	private HashMap parseTab() {
		HashMap hash = new HashMap();
		try {
			BufferedReader in = new BufferedReader(new FileReader(filename));
			String line;
			while ((line = in.readLine()) != null) {
				StringTokenizer tokenizer = new StringTokenizer(line, "\t");
				ArrayList al = new ArrayList();
				boolean first = true;
				String code = null;
				while (tokenizer.hasMoreTokens()) {
					if (first) {
						code = tokenizer.nextToken();
						first = false;
					} else
						al.add(tokenizer.nextToken());
				}
				if (code != null)
					hash.put(code, al.toArray());
			}
			in.close();
		} catch (Exception ex) {
            Logger logger = Logger.getLogger(this.getClass());
            logger.error(ex);
		}
		return hash;
	}

	/**
	 * parst ein XML file. Der Aufbau der XML Datei ist in
	 * im Interface IAnnotationData festgelegt
	 * @see prompt.input.IAnnotation 
	 * @return HashMap
	 * 
	 */
	private HashMap parseXML() {

        Logger logger = Logger.getLogger(this.getClass());
		HashMap hash = new HashMap();
		File f = new File(filename);
		SAXBuilder builder = new SAXBuilder();
		Document doc = null;
		Element xmlRoot = null;
		try {
			doc = builder.build(f);
			xmlRoot = doc.getRootElement();
			if (!xmlRoot.getName().equals("annotations")) {
				logger.error("File '" + filename + "' is not a valid XML data file");
				return null;
			}
		} catch (JDOMException e) {
            logger.error(e);
			return null;
		} catch (IOException e) {
            logger.error(e);
			return null;
		}
		xmlRoot = doc.getRootElement();
		Object[] tmp = xmlRoot.getChildren("protein").toArray(); //read all protein nodes
		for (int i = 0; i < tmp.length; i++) {
			Element elProtein = (Element) tmp[i];
			String code = elProtein.getAttributeValue("code");

			ArrayList al = new ArrayList();
			List list = elProtein.getChildren("anno");
			Iterator iter = list.iterator();
			while (iter.hasNext()) {
				Element elAnno = (Element) iter.next();
				al.add(elAnno.getValue());
			}
			if (code != null)
				hash.put(code, al.toArray());
		}

		return hash;
	}

	/* (non-Javadoc)
	 * @see prompt.input.IAnnotationData#getAnnotations()
	 */
	public HashMap getAnnotations() {
		switch (type) {
		case TAB_DELIMITED:
			return parseTab();
		case XML_FILE:
			return parseXML();
		default:
			break;
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see prompt.input.IDataFile#setFilename(java.lang.String)
	 */
	public void setFilename(String filename) {
		this.filename = filename;
	}

	/* (non-Javadoc)
	 * @see prompt.input.IDataFile#getFilename()
	 */
	public String getFilename() {
		return filename;
	}

}
