/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: Filter.java,v 1.3 2006/04/10 13:24:17 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input.classes;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.TreeMap;
import java.util.regex.Pattern;

import prompt.input.IInput;
import prompt.input.filter.IFilterId;



/**
 * @author schmidt
 * Klassen die das Interface <i>IFilter</i> implementieren koennen eine Instanz von dieser Klasse erzeugen und dann 
 * auf diese Weise diese Methoden wiederverwenden<br>
 */
public class Filter implements IFilterId {
  private final static long serialVersionUID = -6849794470754667700L;
	protected TreeMap hash;
	protected boolean type;
	protected IInput input;
	
	public Filter() {
		hash = new TreeMap();
		type = true;
	}

	
	
	/* (non-Javadoc)
	 * @see de.tum.binfo.proj.input.IFilter#filterAddSingleId(java.lang.String)
	 */
	public void addSingleId(String id) {
		if (id!=null) {
			String tmp = id.trim().toUpperCase();
			hash.put(tmp, new Integer(1));
		}
	}

	/* (non-Javadoc)
	 * @see de.tum.binfo.proj.input.IFilter#filterAddArray(java.lang.String[])
	 */
	public void addArray(String[] ids) {
		for (int i = 0; i < ids.length; i++) {
			String string = ids[i];
			addSingleId(string);
		}

	}

	/* (non-Javadoc)
	 * @see de.tum.binfo.proj.input.IFilter#filterAddList(java.util.List)
	 */
	public void addList(List list) {
		for (Iterator iter = list.iterator(); iter.hasNext();) {
			String element = (String) iter.next();
			addSingleId(element);
		}

	}

	/* (non-Javadoc)
	 * @see de.tum.binfo.proj.input.IFilter#filterAddStringList(java.lang.String)
	 */
	public void addStringList(String str) {
		//Parse Lines
		String[] lines = Pattern.compile("\\\n").split(str);
		if ((lines != null) && (lines.length > 0)) {
			for (int i = 0; i < lines.length; i++) {
				addSingleId(lines[i]);
			}
		}
	}

	/* (non-Javadoc)
	 * @see de.tum.binfo.proj.input.IFilter#filterIsInSet(java.lang.String)
	 */
	public boolean contains(String id) {
		if (id==null) return false;
		return hash.containsKey(id.trim().toUpperCase());
	}

	public void saveToFile(String filename) throws IOException {
		//TODO: erste Zeile den Typ noch rein!
		// output : Unicode to Cp850 (MS-DOS Latin-1)
		FileOutputStream fos = new FileOutputStream(filename + ".filter");
		Writer w = new BufferedWriter(new OutputStreamWriter(fos, "Cp850"));
		Set ids = hash.keySet();
		for (Iterator iter = ids.iterator(); iter.hasNext();) {
			String element = (String) iter.next();
			w.write(element + "\n");
		}
		w.flush();
		w.close();
	}

	public void loadFromFile(String filename) throws IOException {
//		TODO: erste Zeile den Typ noch rein!
		hash = null;
		hash = new TreeMap();
		FileInputStream fin = new FileInputStream(filename+".filter");
		BufferedReader myInput = new BufferedReader(new InputStreamReader(fin));
		String thisLine;
		while ((thisLine = myInput.readLine()) != null) {
			addSingleId(thisLine);
		}
		myInput.close();
		fin.close();
	}

	/**
	 * @param id
	 * @return true if the id passes the filter test, this means that the object's getter methods should return the entry with this id
	 */
	public boolean pass(String id)
	{
		if (type){
			return contains(id);
		}
		else {
			return (! contains(id)) ;
		}
	}
	
	/**
	 * Sets which entries should be pass the method filterPass.
	 * @param b:<br>
	 * <b>true</b>: (=default) only entries whose id is in the filter set will be returned by the getter-methods (filterPass=true)<br>
	 * <b>false</b>: all entries will be returned <b>except</b> those whose id is in the filter set 
	 */
	public void setPass(boolean b)
	{
	  type=b;	
	}

    /* (non-Javadoc)
     * @see prompt.input.filter.IFilterId#setDestination(de.tum.binfo.proj.input.IInput)
     */
    public void setDestination(IInput dest) {
      input=dest;
        
    }

	
	

}
