/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: GenBankRemote.java,v 1.8 2006/04/06 11:56:08 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input.classes;

import gr.zeus.ui.JMessage;

import java.awt.Component;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JOptionPane;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.blueprint.seqhound.Seqhound;
import org.blueprint.seqhound.SeqhoundException;
import org.blueprint.seqhound.SeqhoundFactory;
import org.blueprint.seqhound.SeqhoundLogicException;
import org.blueprint.seqhound.SeqhoundProperties;

import prompt.ui.DialogFactory;
import prompt.ui.SaveUI;
import tschmidt.RemoteQueries.FastaSeq;
import tschmidt.RemoteQueries.Exceptions.ParserError;
import tschmidt.utils.ValidLocation;

/**
 * Haelt Ids im Speicher, die Sequencen werden auf der Platte gecacht
 * 
 * @author tschmidt
 * 
 */
public class GenBankRemote extends SeqProperties {

    private static final long serialVersionUID = -2931264739657740842L;

    String filename;

    public void init(String filename) {
        this.filename = filename;
    }

    // private static DiskHashtable seqHash = new DiskHashtable(new
    // File(System.getProperty("java.io.tmpdir")));

    private static Hashtable seqHash = new Hashtable();

    private List ids = null;

    @Override
    public void run() throws Exception {
        Logger logger = Logger.getLogger(GenBankRemote.class);

        Logger logger2 = Logger.getLogger("org.blueprint.seqhound");
        logger2.setLevel((Level) Level.INFO);
        Logger logger3 = Logger.getLogger("org.apache");
        logger3.setLevel((Level) Level.INFO);

        Logger logger4 = Logger.getLogger("httpclient");
        logger4.setLevel((Level) Level.INFO);

        if (filename == null) {
            // msg to user
            JMessage.showMessageDialog(null, "<html>Choose a text file with GenBank Ids (like gi_12345)<br>"
                    + "Each row must contain one identifer.</html>", JOptionPane.INFORMATION_MESSAGE,
                    JOptionPane.DEFAULT_OPTION);
            // choose file with genebank-id, one id at each row
            filename = SaveUI.load("Choose File with GenBank Ids", ".txt");
        }
        if (filename == null) {
            logger.info("canceled");
            return;
        }

        String[] idsTmp = ValidLocation.readFileLines(filename);
        ids = new LinkedList();
        for (String string : idsTmp) {
            string = string.trim().toUpperCase();
            ids.add(string);
        }
      
        setDescription(DialogFactory.showInputDescriptionDialogFile(filename));
    }

    public List getSequences() {
        // all inlcusive um serialisierbarkeit zu gewaehren
        SeqhoundProperties shp = null;
        SeqhoundFactory shf = null;
        Seqhound sh = null;
        if (shp == null) {
            try {
                shp = new SeqhoundProperties();
            } catch (SeqhoundException e) {
                Logger logger = Logger.getLogger(GenBankRemote.class);
                logger.error("initWebservice1 ", e);
                return null;
            }
        }
        if (shf == null) {
            shf = new SeqhoundFactory();
            try {
                sh = shf.createSeqhound(shp);
            } catch (SeqhoundLogicException e1) {
                Logger logger = Logger.getLogger(GenBankRemote.class);
                logger.error("initWebservice2 ", e1);
                return null;
            } catch (SeqhoundException e1) {
                Logger logger = Logger.getLogger(GenBankRemote.class);
                logger.error("initWebservice3 ", e1);
                return null;
            }
        }

        LinkedList list = new LinkedList();
        Logger logger = Logger.getLogger(GenBankRemote.class);
        for (Object id : ids) {
            Object seq = null;
            try {
                seq = seqHash.get(id);
            } catch (Exception e) {
                logger.error("cache error loading (id='" + id + "')", e);
            }
            if (seq == null) {
                logger.info("download sequence of id: '" + id + "'");
                // seq = get(id);
                if (id == null) {
                    return null;
                }
                int gid = -1;
                if (id instanceof String) {
                    String tmp = (String) id;
                    tmp = tmp.replaceAll("\\D", "");
                    try {
                        gid = Integer.parseInt(tmp);
                    } catch (Exception e) {
                        // Logger logger =
                        // Logger.getLogger(GenBankRemote.class);
                        logger.error("error parsing number in get '" + id + "'", e);
                        return null;
                    }
                }
                if (id instanceof Integer) {
                    gid = ((Integer) id).intValue();
                }
                // Logger logger = Logger.getLogger(GenBankRemote.class);
                seq = null;
                try {
                    logger.debug("get seq for '" + gid + "'");
                    seq = sh.SHoundGetFasta(gid);
                } catch (SeqhoundLogicException e) {
                    logger.error("error get '" + id + "'", e);
                } catch (SeqhoundException e) {
                    logger.error("error get '" + id + "'", e);
                }

                if (seq != null) {
                    // entferne erste zeile, da dort nochmal der fasta header
                    // ist
                    String seq2 = (String) seq;
                    FastaSeq fasta = new FastaSeq();
                    try {
                        fasta.init(seq2);
                    } catch (ParserError e) {
                        logger.debug("invalid fasta seq: '" + id + "'");
                    }
                    seq = fasta.getSequence();
                    logger.debug(seq);
                    seqHash.put(id, seq);
                }
            } else {
                logger.info("took from cache seq. of id: '" + id + "'");
            }
            list.add(seq);
        }
        return list;
    }

    public List getSequencesNames() {
        return ids;
    }

    public List getAnnotation() {
        return null;
    }

    public String getMetaData() {
        StringBuffer sb = new StringBuffer();
        sb.append("<html><table>");

        sb.append("<tr><td> load from file </td>");
        sb.append("<td>" + filename + "</td></tr>");

        sb.append("</html></table>");
        JMessage.showMessageDialog(null, sb.toString(), JOptionPane.INFORMATION_MESSAGE, JOptionPane.DEFAULT_OPTION);

        return sb.toString();
    }

    public Component parentWin = null;

    public void setParentWindow(Component win) {
        this.parentWin = win;
    }
    
    public Iterator iterator() {
        return new SeqIterator();
    }

    public class SeqIterator implements Iterator {
        SeqhoundProperties shp = null;
        SeqhoundFactory shf = null;
        Seqhound sh = null;
        Logger logger = Logger.getLogger(GenBankRemote.class);
        Iterator idIter;

        public SeqIterator() {
            if (shp == null) {
                try {
                    shp = new SeqhoundProperties();
                } catch (SeqhoundException e) {
                    logger.error("initWebservice1 ", e);
                }
            }
            if (shf == null) {
                shf = new SeqhoundFactory();
                try {
                    sh = shf.createSeqhound(shp);
                } catch (SeqhoundLogicException e1) {
                    logger.error("initWebservice2 ", e1);
                } catch (SeqhoundException e1) {
                    logger.error("initWebservice3 ", e1);
                }
            }
            idIter = ids.iterator();

        }

        public boolean hasNext() {
            return idIter.hasNext();
        }

        public Object next() {
            Object id = idIter.next();
            // for (Object id : ids) {
            Object seq = null;
            try {
                seq = seqHash.get(id);
            } catch (Exception e) {
                logger.error("cache error loading (id='" + id + "')", e);
            }
            if (seq == null) {
                logger.info("download sequence of id: '" + id + "'");
                // seq = get(id);
                if (id == null) {
                    return null;
                }
                int gid = -1;
                if (id instanceof String) {
                    String tmp = (String) id;
                    tmp = tmp.replaceAll("\\D", "");
                    try {
                        gid = Integer.parseInt(tmp);
                    } catch (Exception e) {
                        logger.error("error parsing number in get '" + id + "'", e);
                        return null;
                    }
                }
                if (id instanceof Integer) {
                    gid = ((Integer) id).intValue();
                }
                seq = null;
                try {
                    logger.debug("get seq for '" + gid + "'");
                    seq = sh.SHoundGetFasta(gid);
                } catch (SeqhoundLogicException e) {
                    logger.error("error get '" + id + "'", e);
                } catch (SeqhoundException e) {
                    logger.error("error get '" + id + "'", e);
                }

                if (seq != null) {
                    // entferne erste zeile, da dort nochmal der fasta
                    // header ist
                    String seq2 = (String) seq;
                    FastaSeq fasta = new FastaSeq();
                    try {
                        fasta.init(seq2);
                    } catch (ParserError e) {
                        logger.debug("invalid fasta seq: '" + id + "'");
                    }
                    seq = fasta.getSequence();
                    logger.debug(seq);
                    seqHash.put(id, seq);
                }
            } else {
                logger.info("took from cache seq. of id: '" + id + "'");
            }
            // }

            return new Object[] { id, seq };
        }

        public void remove() {
            throw new UnsupportedOperationException();
        }

    }

}
