/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: GenericProperty.java,v 1.10 2006/04/05 11:07:45 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input.classes;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

import org.apache.log4j.Logger;
import org.jdom.Element;

import prompt.input.IGenericProperty;




/**
 * @author tschmidt
 * 
 */
public class GenericProperty implements IGenericProperty,Serializable {


	private static final long serialVersionUID = 1L;
	
	String id = null;
	int type = YETUNKOWN;
	HashMap hash;
	
	/**
	 * Creates a GenericProperty from a JDOM-Tree property-XML Node
	 * 
	 * @param elProperty
	 *            Element of JDOM
	 * @deprecated
	 */
	public GenericProperty(Element elProperty) {
        Logger logger = Logger.getLogger(GenericProperty.class.getName());
        hash = new HashMap();
		String tmp = elProperty.getAttributeValue("type");
		tmp=tmp.trim().toLowerCase();
        
		if (tmp.equals("numeric")) type=NUMERIC;
		if (tmp.equals("symbolic")) type=SYMBOLIC;
        if (tmp.equals("setdef")) type=SETDEF;
        
		if (type==YETUNKOWN) {
            logger.warn("type could not identified");
        }
		
		id = elProperty.getAttributeValue("id");
	
		List list = elProperty.getChildren("input"); // read all inputs
		if (list!=null) {
        logger.debug("found "+list.size()+" input items in the property '"+id+"' of type '"+tmp+"'");
        for (Iterator iter = list.iterator(); iter.hasNext();) {
			Element input = (Element) iter.next();
			String protId = input.getAttributeValue("prot_id");
            if (protId==null) protId=input.getAttributeValue("id");;
			if (protId==null) {
			    logger.warn("no id attribute found within one input item of the property '"+id+"' of type '"+tmp+"'");
			    continue;
            }
            String value = input.getAttributeValue("value");
			// here evtl. warning falls symb or numeric und value ist leer/non
			// existent
            protId = protId.trim().toUpperCase();
            hash.put(protId,value);
		}
        } else {
            logger.warn("NO input items found in the property '"+id+"' of type '"+tmp+"'");
        }
	}


	private String getLocalName(XMLStreamReader parser) {
		String qname = parser.getLocalName();
		if ((qname==null) || (qname.trim().length()==0)) {
			return null;
		}
		qname = qname.toLowerCase().trim();
		return qname;
	}
	
	/**
	 * @param parser
	 * @return null if no attributes were found
	 */
	private HashMap<String,String> getAttributes(XMLStreamReader parser) {
		int anzAttr = parser.getAttributeCount();
		if (anzAttr==0) return null;
		HashMap<String,String> aHash = new HashMap<String,String>();
		for (int j = 0; j < anzAttr; j++) {
			String aName = parser.getAttributeLocalName(j);
			String aValue = parser.getAttributeValue(j);
			if ((aName==null) || (aName.length()==0)) continue;
			if ((aValue==null) || (aValue.length()==0)) continue;
			aName=aName.trim().toLowerCase();
		    aHash.put(aName,aValue);
		}
		return aHash;
	}
	
	/**
	 * prueft ob die attribute in aNames im Hash sind
	 * 
	 * @param hash
	 * @param aNames
	 * @return
	 */
	private boolean attributsOk(HashMap<String,String> hash,String[] aNames) {
		if (aNames==null) return true;
		if (aNames.length==0) return true;
		if (hash==null) return false;
		if (hash.size()==0) return false;
		if (hash.size()!=aNames.length) return false;
		for (int i = 0; i < aNames.length; i++) {
			String string = aNames[i];
			String val = hash.get(string);
			if (val==null) return false;
		}
		return true;
	}
	

	
	/**
	 * Creates a GenericProperty getting a XMLStreamReader pointing at a
	 * property node in an XML Stream.
	 * 
	 * @param parser
	 *            XMLStreamReader (open StAX parser, with the curser at the
	 *            property element)<br>
	 *            <b>Note: The state of the parser will be changed as the parser
	 *            moves forward!</b><br>
	 *            See also JSR-173 (Interfaces) and stax-1.2.0.jar (bea
	 *            implementierung)
	 */
	public GenericProperty(XMLStreamReader parser) {
		Logger logger = Logger.getLogger(this.getClass());
		hash = new HashMap();
		String nodeName = getLocalName(parser);
		logger.debug("nodeName='"+nodeName+"'");
		if (!nodeName.equals("property")) {
		   logger.fatal("Element name must be property");
		   return;	
		}
		HashMap<String,String> attr = getAttributes(parser);
		if (!attributsOk(attr,new String[]{"id","type"})) {
			logger.fatal("insufficient attributes in property tag");
			return;
		}
	    String typeValue = attr.get("type");
		if (typeValue.equals("numeric")) type=NUMERIC;
		if (typeValue.equals("symbolic")) type=SYMBOLIC;
	    if (typeValue.equals("setdef")) type=SETDEF;
		if (type==YETUNKOWN) {
	          logger.warn("property type could not identified");
	        }
		long anzInputs = 0;
		id = attr.get("id");
		attr.remove("id");
		attr.remove("type");
		attr.clear();
		attr=null;
		logger.info("property id='"+id+"' type="+type);
		boolean propOpen=true;
		boolean inpOpen=false;
		
		long lastPoint = System.currentTimeMillis();
		long eventToPoint = 0;
		long eventNr = 0;
		try {
			while (true) {
				if(!parser.hasNext()) {
					break;
				}
				int event = parser.next();
				eventToPoint++;
				eventNr++;
				 if (eventToPoint>5000) { //alle xxx Zeilen schauen wir ob es Zeit ist fuer eine Info msg
					 	//vermeidet das wir immer die Zeitabfragen mussen
	                	if ((System.currentTimeMillis()-lastPoint)>3000)  {
	                		logger.info("read until event number "+eventNr); 
	                		lastPoint = System.currentTimeMillis();
	                	}
	                	eventToPoint=0;
	             }
				 
				//logger.debug(parser.getLocalName());
				if (event==XMLStreamConstants.START_ELEMENT) {
					if (inpOpen) {
						logger.fatal("No nested tags are allowed");
						return;
					}
					String name = getLocalName(parser);
					if (!name.equals("input")) {
						logger.fatal("only input nodes are allowed here");
						return;
					}
					inpOpen=true;
					
					HashMap<String,String> attrInput = getAttributes(parser);
					if (!attributsOk(attrInput,new String[]{"id","value"})) {
						logger.fatal("insufficient attributes in input tag");
						continue;
					}
					String input_id = new String(attrInput.get("id"));
					String input_value = new String(attrInput.get("value"));
					attrInput.remove("id");
					attrInput.remove("value");
					attrInput.clear();
					attrInput= null;
					
					if (type==NUMERIC) {
						Object neu = input_value;
						try {
							neu = Double.valueOf(input_value);
						} catch (Exception e) {
							neu= input_value;
						}
						
						Object prev = hash.put(input_id,neu);
						if (prev!=null) {
							String old = prev+";";
							hash.put(input_id,old+input_value);
						}
						
					} else {
						Object prev = hash.put(input_id,input_value);
						if (prev!=null) {
							hash.put(input_id,prev+";"+input_value);
						}
					}
					anzInputs++;
				}
				if (event==XMLStreamConstants.END_ELEMENT) {
					if (inpOpen) { 
						inpOpen=false; 
					//	logger.debug("close input");
						continue;
					}
					if (propOpen) break; //es duerfen ausser input close tags nichts anderes kommen
				}
				
				
			}
		} catch (XMLStreamException e) {
			String errmsg = "XML Parsing error";
			logger.fatal(errmsg,e);
		}
		logger.info(" "+anzInputs+" input elements found in the property '"+id+"'");
		logger.info(hash.size()+" unique identifiers of the input elements");
		//laufe solange bis ein property end element kommt
		
	/*
	 * if (qname.equals("input")) { int anzAttr = parser.getAttributeCount();
	 * for (int j = 0; j < anzAttr; j++) { String aName =
	 * parser.getAttributeLocalName(j); String aValue =
	 * parser.getAttributeValue(j); logger.debug("aName='"+aName+"' ->
	 * '"+aValue+"'"); } }
	 */
	}
	
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see prompt.input.IGenericProperty#getType()
	 */
	public int getType() {
		return type;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see prompt.input.IGenericProperty#getId()
	 */
	public String getId() {
		return id;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see prompt.input.IGenericProperty#getValues()
	 */
	public HashMap getValues() {
		return hash;
	}

}
