/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: GenericXMLFile.java,v 1.21 2006/04/06 13:09:51 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input.classes;

import gr.zeus.ui.JMessage;

import java.awt.Component;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.NoSuchElementException;

import javax.swing.JOptionPane;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

import org.apache.log4j.Logger;

import prompt.input.IGenericProperty;
import prompt.input.IGenericXMLFile;
import prompt.input.ISeqProperties;
import prompt.ui.DialogFactory;
import prompt.ui.SaveUI;

/**
 * @author tschmidt
 * 
 */
public class GenericXMLFile extends SeqProperties implements ISeqProperties,
		IGenericXMLFile, Serializable {
	private final static long serialVersionUID = -6849794470754667700L;

	private String filename = null;

	private String label = null;

	private IGenericProperty[] properties = null;

	/*
	 * (non-Javadoc)
	 * 
	 * @see prompt.input.IGenericXMLFile#init(java.lang.String)
	 */
	public void init(String filename) throws Exception {
		this.filename = filename;
	}

	public String proposedDescription = null;

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.tum.binfo.jdrm.JobTemplate#run()
	 */
	public void run() throws Exception {
		Logger logger = Logger.getLogger(this.getClass());
		if (filename == null) {
			filename = SaveUI.load("Load GenericXML file", ".xml");
		}
		if (filename == null) {
			logger.error("canceled choosing a file");
			return;
		}
		if (!isFileOk()) {
			logger.error("File '" + filename
					+ "' is not ok, checked with the DTD/Schema definition");
			return;
		}

		// System.gc();
		// logger.debug("used=
		// "+(Runtime.getRuntime().totalMemory()-Runtime.getRuntime().freeMemory()));
		// parse now
		URL u = (new File(filename)).toURL();
		InputStream in = u.openStream();
		XMLInputFactory factory = XMLInputFactory.newInstance();
		XMLStreamReader parser = factory.createXMLStreamReader(in);
		ArrayList<IGenericProperty> all = new ArrayList<IGenericProperty>();

		int i = 0;
		while (parser.hasNext()) {
			i++;
			int event;
			try {
				event = parser.next();
			} catch (javax.xml.stream.XMLStreamException e) {
				logger.debug("break stream processing event=" + i);
				break;
			}

			if (event == XMLStreamConstants.START_DOCUMENT) {
				logger.debug("start element");
				logger.debug("xml version: " + parser.getVersion());
			}

			/*
			 * if (event== XMLStreamConstants.END_ELEMENT) { logger.debug("end
			 * element"); }
			 */
			if (event == XMLStreamConstants.START_ELEMENT) {
				String qname = parser.getLocalName();

				if ((qname == null) || (qname.trim().length() == 0)) {
					logger.warn("Empty tag this should never happens");
					continue;
				}
				qname = qname.toLowerCase().trim();
				// logger.debug(qname);

				if (qname.equals("property")) {
					IGenericProperty current = new GenericProperty(parser);
					all.add(current);
				}
			} // start element
			// if (i>10) System.exit(1);
		} // while hasNext
		parser.close();
		parser = null;
		factory = null;
		u = null;

		properties = new IGenericProperty[all.size()];

		for (int j = 0; j < all.size(); j++) {
			IGenericProperty currentProp = all.get(j);
			properties[j] = currentProp;
		}

		// logger.debug("used=
		// "+(Runtime.getRuntime().totalMemory()-Runtime.getRuntime().freeMemory()));
		//	
		// System.gc();
		// logger.debug("used=
		// "+(Runtime.getRuntime().totalMemory()-Runtime.getRuntime().freeMemory()));
		logger.debug("Finished parsing of GenericXML");
		if (proposedDescription == null)
			proposedDescription = filename;
		if (this.getDescription() == null)
			setDescription(DialogFactory
					.showInputDescriptionDialogFile(proposedDescription));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see prompt.input.IGenericXMLFile#getProperties()
	 */
	public IGenericProperty[] getProperties() {
		return properties;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see prompt.input.IGenericXMLFile#getLabel()
	 */
	public String getLabel() {
		return label;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see prompt.input.IGenericXMLFile#isFileOk()
	 */
	public boolean isFileOk() {
		// TODO: validate via DTD
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see prompt.input.IInput#getMetaData()
	 */
	public String getMetaData() {
		// evtl hier mehr infos falls schon geparst
		StringBuffer sb = new StringBuffer();
		sb.append("<html><table>");

		sb.append("<tr><td> load from file </td>");
		sb.append("<td>" + filename + "</td></tr>");

		sb.append("<tr><td> label </td>");
		sb.append("<td>" + label + "</td></tr>");

		if (properties != null) {
			sb.append("<tr><td> number of properties </td>");
			sb.append("<td>" + properties.length + "</td></tr>");
			for (int i = 0; i < properties.length; i++) {
				sb.append("<tr><td> " + properties[i].getId() + " ( "
						+ properties[i].getValues().size() + " ) </td>");
				String typeStr = "unknown type";
				switch (properties[i].getType()) {
				case IGenericProperty.NUMERIC:
					typeStr = "numeric";
					break;
				case IGenericProperty.SYMBOLIC:
					typeStr = "symbolic";
					break;
				case IGenericProperty.SETDEF:
					typeStr = "protein set definition";
					break;
				default:
					break;
				}

				sb.append("<td>" + typeStr + "</td></tr>");
			}
		}

		sb.append("</html></table>");

		JMessage.showMessageDialog(null, sb.toString(),
				JOptionPane.INFORMATION_MESSAGE, JOptionPane.DEFAULT_OPTION);

		return sb.toString();
	}

	// ONLY if a setdef input item is available, sonst geben diese Methode null
	// zurueck

	public List getSequences() {
		IGenericProperty[] properties = this.getProperties();
		if (properties == null)
			return null;
		LinkedList ret = new LinkedList();
		for (IGenericProperty property : properties) {
			if (property.getType() == IGenericProperty.SETDEF) {
				HashMap hash = property.getValues();
				for (Object key : hash.keySet()) {
					Object value = hash.get(key);
					if (value != null) {
						if (value instanceof String) {
							String value2 = (String) value;
							value2 = value2.replaceAll("\\\n", "");
							// logger.debug("replaced");
							ret.add(value2);
						} else {
							ret.add(value);
						}
					}
				}

				return ret;
			} // IS SETDEF
		}
		return null;
	}

	public List getSequencesNames() {
		IGenericProperty[] properties = this.getProperties();
		if (properties == null)
			return null;
		LinkedList ret = new LinkedList();
		for (IGenericProperty property : properties) {
			if (property.getType() == IGenericProperty.SETDEF) {
				HashMap hash = property.getValues();
				for (Object key : hash.keySet()) {
					ret.add(key);
				}

				return ret;
			} // IS SETDEF
		}
		return null;
	}

	public List getAnnotation() {
		IGenericProperty[] properties = this.getProperties();
		if (properties == null)
			return null;

		return null;
	}

	public void setFilename(String filename) {
		this.filename = filename;
	}

	public String getFilename() {

		return filename;
	}

	public Component parentWin = null;

	public void setParentWindow(Component win) {
		this.parentWin = win;
	}

	public Iterator iterator() {
		return new SeqIterator();
	}

	public class SeqIterator implements Iterator {
		Logger logger = Logger.getLogger(this.getClass());

		XMLStreamReader parser;

		boolean cached;

		public SeqIterator() {
			URL u;
			try {
				u = (new File(filename)).toURL();
				InputStream in = u.openStream();
				XMLInputFactory factory = XMLInputFactory.newInstance();
				parser = factory.createXMLStreamReader(in);
				String actProp = "";
				int event = 0;
				while (!actProp.equalsIgnoreCase("setdef") && parser.hasNext()) {
					event = parser.next();
					if (event == XMLStreamConstants.START_ELEMENT) {
						if (parser.getLocalName().equalsIgnoreCase("property")) {
							int c = parser.getAttributeCount();
							for (int i = 0; i < c; i++) {
								if (parser.getAttributeName(i).toString()
										.equalsIgnoreCase("type")
										&& parser.getAttributeValue(i)
												.equalsIgnoreCase("setdef")) {
									actProp = parser.getAttributeValue(i);

								}
							}
						}
					}
				}

			} catch (MalformedURLException e) {
				logger.error("Cannot parse path (" + filename + ")", e);
			} catch (IOException e) {
				logger.error("Cannot open file " + filename, e);
			} catch (XMLStreamException e) {
				logger.error("Error processing XML", e);
			}
		}

		public boolean hasNext() {
			int state = 0;
			if (cached) {
				return true;
			}
			try {
				if (!parser.hasNext()) {
					return false;
				}
				state = parser.next();
				while (state != XMLStreamConstants.START_ELEMENT) {
					if (!parser.hasNext()) {
						return false;
					}
					state = parser.next();
				}
				if (parser.getLocalName().equalsIgnoreCase("input")) {
					cached = true;
					return true;
				}
			} catch (XMLStreamException e) {
				logger.error("Error processing XML", e);
			}
			return false;
		}

		public Object next() {
			String[] ret = new String[2];
			if (!cached) {
				if (!hasNext()) {
					throw new NoSuchElementException();
				}
			}
			for (int i = 0; i < parser.getAttributeCount(); i++) {
				if (parser.getAttributeName(i).toString()
						.equalsIgnoreCase("id")) {
					ret[0] = parser.getAttributeValue(i);
				}
				if (parser.getAttributeName(i).toString().equalsIgnoreCase(
						"value")) {
					ret[1] = parser.getAttributeValue(i);
				}
			}
			cached = false;
			return ret;
		}

		public void remove() {
			throw new UnsupportedOperationException();
		}

	}
}
