/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: GenericXMLFile_JDOM.java,v 1.3 2006/04/06 11:56:09 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input.classes;

import gr.zeus.ui.JMessage;

import java.awt.Component;
import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import prompt.input.IGenericProperty;
import prompt.input.IGenericXMLFile;
import prompt.input.ISeqProperties;
import prompt.ui.DialogFactory;
import prompt.ui.SaveUI;

/**
 * @author tschmidt
 * 
 */
public class GenericXMLFile_JDOM extends SeqProperties implements ISeqProperties, IGenericXMLFile, Serializable {
    private final static long serialVersionUID = -6849794470754667700L;

    private String filename = null;

    private String label = null;

    private IGenericProperty[] properties = null;

    /*
     * (non-Javadoc)
     * 
     * @see prompt.input.IGenericXMLFile#init(java.lang.String)
     */
    public void init(String filename) throws Exception {
        this.filename = filename;
    }

    public String proposedDescription = null;

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.jdrm.JobTemplate#run()
     */
    public void run() throws Exception {
        Logger logger = Logger.getLogger(this.getClass());
        if (filename == null) {
            filename = SaveUI.load("Load GenericXML file", ".xml");
        }
        if (filename == null) {
            logger.error("canceled choosing a file");
            return;
        }
        if (!isFileOk()) {
            logger.error("File '" + filename + "' is not ok, checked with the DTD/Schema definition");
            return;
        }

        // parse now
        File f = new File(filename);
        SAXBuilder builder = new SAXBuilder();
        Document doc = null;
        Element xmlRoot = null;
        try {
            doc = builder.build(f);
            xmlRoot = doc.getRootElement();
            if (!xmlRoot.getName().equals("dataset")) {
                logger.error("File '" + filename + "' is not a valid Generic XML data file");
                return;
            }
        } catch (JDOMException e) {
            logger.error(e);
            return;
        } catch (IOException e) {
            logger.error(e);
            return;
        }

        xmlRoot = doc.getRootElement();
        this.label = xmlRoot.getAttributeValue("label");

        Object[] tmp = xmlRoot.getChildren("property").toArray(); // read all
                                                                    // properties
        properties = new IGenericProperty[tmp.length];

        for (int i = 0; i < tmp.length; i++) {
            Element elProperty = (Element) tmp[i];
            IGenericProperty currentProp = new GenericProperty(elProperty);
            properties[i] = currentProp;
        }

        logger.debug("Finished parsing of GenericXML");
        if (proposedDescription == null)
            proposedDescription = filename;
        if (this.getDescription() == null)
            setDescription(DialogFactory.showInputDescriptionDialogFile(proposedDescription));

    }

    /*
     * (non-Javadoc)
     * 
     * @see prompt.input.IGenericXMLFile#getProperties()
     */
    public IGenericProperty[] getProperties() {
        return properties;
    }

    /*
     * (non-Javadoc)
     * 
     * @see prompt.input.IGenericXMLFile#getLabel()
     */
    public String getLabel() {
        return label;
    }

    /*
     * (non-Javadoc)
     * 
     * @see prompt.input.IGenericXMLFile#isFileOk()
     */
    public boolean isFileOk() {
        // TODO: validate via DTD
        return true;
    }

    /*
     * (non-Javadoc)
     * 
     * @see prompt.input.IInput#getMetaData()
     */
    public String getMetaData() {
        // evtl hier mehr infos falls schon geparst
        StringBuffer sb = new StringBuffer();
        sb.append("<html><table>");

        sb.append("<tr><td> load from file </td>");
        sb.append("<td>" + filename + "</td></tr>");

        sb.append("<tr><td> label </td>");
        sb.append("<td>" + label + "</td></tr>");

        if (properties != null) {
            sb.append("<tr><td> number of properties </td>");
            sb.append("<td>" + properties.length + "</td></tr>");
            for (int i = 0; i < properties.length; i++) {
                sb.append("<tr><td> " + properties[i].getId() + "</td>");
                String typeStr = "unknown type";
                switch (properties[i].getType()) {
                case IGenericProperty.NUMERIC:
                    typeStr = "numeric";
                    break;
                case IGenericProperty.SYMBOLIC:
                    typeStr = "symbolic";
                    break;
                case IGenericProperty.SETDEF:
                    typeStr = "protein set definition";
                    break;
                default:
                    break;
                }

                sb.append("<td>" + typeStr + "</td></tr>");
            }
        }

        sb.append("</html></table>");

        JMessage.showMessageDialog(null, sb.toString(), JOptionPane.INFORMATION_MESSAGE, JOptionPane.DEFAULT_OPTION);

        return sb.toString();
    }

    // ONLY if a setdef input item is available, sonst geben diese Methode null
    // zurueck

    public List getSequences() {
        IGenericProperty[] properties = this.getProperties();
        if (properties == null)
            return null;
        LinkedList ret = new LinkedList();
        for (IGenericProperty property : properties) {
            if (property.getType() == IGenericProperty.SETDEF) {
                HashMap hash = property.getValues();
                for (Object key : hash.keySet()) {
                    Object value = hash.get(key);
                    if (value != null) {
                        if (value instanceof String) {
                            String value2 = (String) value;
                            value2 = value2.replaceAll("\\\n", "");
                            // logger.debug("replaced");
                            ret.add(value2);
                        } else {
                            ret.add(value);
                        }
                    }
                }

                return ret;
            } // IS SETDEF
        }
        return null;
    }

    public List getSequencesNames() {
        IGenericProperty[] properties = this.getProperties();
        if (properties == null)
            return null;
        LinkedList ret = new LinkedList();
        for (IGenericProperty property : properties) {
            if (property.getType() == IGenericProperty.SETDEF) {
                HashMap hash = property.getValues();
                for (Object key : hash.keySet()) {
                    ret.add(key);
                }

                return ret;
            } // IS SETDEF
        }
        return null;
    }

    public List getAnnotation() {
        IGenericProperty[] properties = this.getProperties();
        if (properties == null)
            return null;

        return null;
    }

    public void setFilename(String filename) {
        this.filename = filename;
    }

    public String getFilename() {

        return filename;
    }

    public Component parentWin = null;

    public void setParentWindow(Component win) {
        this.parentWin = win;
    }

    public Iterator iterator() {
        throw new UnsupportedOperationException();
    }
}
