/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: Input.java,v 1.3 2006/03/19 11:57:53 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input.classes;

import java.net.URL;

import prompt.IDescription;
import prompt.input.IInput;
import tschmidt.utils.ValidLocation;
import de.tum.binfo.jdrm.JobTemplate;

/*
 * PLEASE NOTE: 
 * 1) All further input classes must be derived from this input-class to
 *    allow the use of java reflections and the grid-engine support
 * 2) The design and the use(e.g. in the engine constructors)
 * 	  of any input-code SHOULD BE SOLEY DONE 
 *    BY INTERFACE DEFINITIONS
 * 3) use interface heritage to define your own input interface!
 * 4) This class must not be touched 
 */

public abstract class Input extends JobTemplate implements IInput, IDescription {
	private final static long serialVersionUID = -6849794470754667700L;

	public Input() {
	}

	public String inputDescription;

	/**
	 * 
	 * @uml.property name="inputDescription"
	 */
	/* (non-Javadoc)
	 * @see de.tum.binfo.proj.input.IInput#getInputDescription()
	 */
	public String getDescription() {
		return inputDescription;
	}

	/**
	 * 
	 * @uml.property name="inputDescription"
	 */
	/* (non-Javadoc)
	 * @see de.tum.binfo.proj.input.IInput#setInputDescription(java.lang.String)
	 */
	public void setDescription(String string) {
		inputDescription = string;
	}

	public int hashCode() {
		//Job specific stuff is only needed if gathering the input is a long running job
		return 0;
	}

	public String getMetaData;

	public String getClassDescription(Object sender) {
		// TODO: oberklassen durchsuchen, falls aktuelle klasse keine beschreibung hat
		URL url = getClass().getResource(this.getClass().getSimpleName() + ".html");
		if (url == null) {
			return "No description found :-(";
		}
        return ValidLocation.readFromUrl(url);
        
	}

}
