/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: Pedant.java,v 1.18 2006/04/06 11:56:09 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input.classes;

import gr.zeus.ui.JMessage;

import java.awt.Component;
import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import prompt.engine.genericxml.GenericHelper;
import prompt.input.IGenericProperty;
import prompt.input.IGenericXML;
import prompt.input.IPedant;
import prompt.ui.DialogFactory;
import tschmidt.dbDialog.DBDialog;
import tschmidt.dbDialog.DBProperties;
import tschmidt.utils.Db;

/**
 * @author Schmidt Thorsten <a
 *         href="mailto:t.schmidt@wzw.tum.de>t.schmidt@wzw.tum.de</a>
 * 
 */
public class Pedant extends SeqProperties implements IPedant, IGenericXML {

    private final static long serialVersionUID = -6849794470754667700L;

    DBProperties dbProperties = null;

    public Pedant() {
    }

    /**
     * @param dbname
     *            name of the pedant genome
     */
    public Pedant(String dbname) throws Exception {
        setDatabaseName(dbname);
        setDescription(dbname);
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IPedant#init(java.lang.String)
     */
    public void init(String dbname) {
        setDatabaseName(dbname);
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.jdrm.JobTemplate#run()
     */
    public void run() {

        if (dbProperties == null) {
            dbProperties = DBDialog.showDBDialog(null);

        }
        if (dbProperties == null) {
            Logger logger = Logger.getLogger(this.getClass().getName());
            logger.debug("canceled");
            return;
        }

        String newDesc = DialogFactory.showInputDescriptionDialog(dbProperties.getDatabaseName());
        setDescription(newDesc);

        super.setDescription(this.inputDescription);
        super.filename = filename;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IInput#getInputDescription()
     */
    public String getDescription() {
        return inputDescription;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IInput#getMetaData()
     */
    public String getMetaData() {
        StringBuffer sb = new StringBuffer();
        sb.append("<html><table>");

        sb.append("<tr><td> Database name </td>");
        sb.append("<td>" + getDatabaseName() + "</td></tr>");

        sb.append("<tr><td>host </td>");
        sb.append("<td>" + dbProperties.getHost() + "</td></tr>");

        sb.append("<tr><td>user </td>");
        sb.append("<td>" + dbProperties.getUsername() + "</td></tr>");

        sb.append("<tr><td>driver </td>");
        sb.append("<td>" + dbProperties.getDriver() + "</td></tr>");

        sb.append("</html></table>");
        JMessage.showMessageDialog(null, sb.toString(), JOptionPane.INFORMATION_MESSAGE, JOptionPane.DEFAULT_OPTION);

        return sb.toString();
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IInput#setInputDescription(java.lang.String)
     */
    public void setDescription(String string) {
        inputDescription = string;

    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IPedant#getDatabaseName()
     */
    public String getDatabaseName() {
        if (dbProperties != null)
            return dbProperties.getDatabaseName();
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IPedant#setDatabaseName(java.lang.String)
     */
    public void setDatabaseName(String str) {
        dbProperties = new DBProperties();
        dbProperties.setToSystemDefaults();
        dbProperties.setDatabaseName(str);
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.ISeqProperties#getSequences()
     */
    public List getSequences() {

        List seq = new LinkedList();
        String[] ret = null;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectStringArray(con, "SELECT dat FROM prot_data");
            for (int i = 0; i < ret.length; i++) {
                String current = ret[i];
                current = current.replaceAll("\\\n", "");
                seq.add(current);
            }

        } catch (SQLException e) {
            Logger logger = Logger.getLogger(this.getClass());
            logger.error(e);
        }

        return seq;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.ISeqProperties#getSequencesNames()
     */
    public List getSequencesNames() {
        List seq = new LinkedList();
        String[] ret = null;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectStringArray(con, "SELECT code FROM prot_data");
            for (int i = 0; i < ret.length; i++) {
                String current = ret[i];
                seq.add(current);
            }

        } catch (SQLException e) {
            Logger logger = Logger.getLogger(this.getClass());
            logger.error(e);
        }
        return seq;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IInput#getAnnotation()
     */
    public List getAnnotation() {
        List seq = new LinkedList();
        String[] ret = null;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectStringArray(con, "SELECT descr FROM prot_data");
            for (int i = 0; i < ret.length; i++) {
                String current = ret[i];
                seq.add(current);
            }

        } catch (SQLException e) {
            Logger logger = Logger.getLogger(this.getClass());
            logger.error(e);
        }
        return seq;
    }

    protected String dataTableName = "prot_data";

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IPedant#setDataTableName(java.lang.String)
     */
    public void setDataTableName(String str) {
        dataTableName = str;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IPedant#getDataTableName()
     */
    public String getDataTableName() {
        return dataTableName;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IPedant#getProteinsWithFunNum(java.lang.String)
     */
    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IPedant#getProteinsWithFunNum(java.lang.String)
     */
    public int[] getProteinsWithFunNum(String str) {
        String sql = "SELECT DISTINCT funcat.prot_data_id " + " FROM funcat," + getDataTableName()
                + " WHERE funcat.fun_num='" + str + "' AND " + " funcat.prot_data_id=" + getDataTableName() + ".id";
        // logger.debug(sql);
        int[] ret = null;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectIntArray(con, sql);
        } catch (SQLException e) {
            Logger logger = Logger.getLogger(this.getClass().getName());
            logger.debug("error query database", e);

            return null;
        }
        return ret;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IPedant#getProteinsWithFunNumAtTheStart(java.lang.String)
     */
    public int[] getProteinsWithFunNumAtTheStart(String str) {
        String sql = "SELECT DISTINCT funcat.prot_data_id " + " FROM funcat," + getDataTableName()
                + " WHERE funcat.fun_num LIKE '" + str + "%' AND " + " funcat.prot_data_id=" + getDataTableName()
                + ".id";
        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        int[] ret = null;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectIntArray(con, sql);
        } catch (SQLException e) {
            logger.debug("error query database", e);

            return null;
        }

        return ret;
    }

    /*
     * (non-Javadoc)
     * 
     * @see prompt.input.IDisorder#getDisorderDetails()
     */
    public String[][] getDisorderDetails() {
        String sql = "SELECT disorder.prot_data_id,disorder.begin,disorder.end,disorder.length " + " FROM disorder,"
                + getDataTableName() + " WHERE " + " disorder.prot_data_id=" + getDataTableName() + ".id";
        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        String[][] ret = null;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectStringArray2D(con, sql);
        } catch (SQLException e) {
            logger.debug("error query database", e);

            return null;
        }

        return ret;
    }

    /*
     * (non-Javadoc)
     * 
     * @see prompt.input.IDisorder#getDisordePercentageOfResidues()
     */
    public String[][] getDisordePercentageOfResidues() {
        String sql = "SELECT disorder.prot_data_id, sum(disorder.length)/rep.length*100 AS percentage_residues, "
                + "sum(disorder.length)/count(disorder.id) AS avg_strech_len, "
                + "count(disorder.id), max(disorder.length), min(disorder.length) " + " FROM disorder,rep,"
                + getDataTableName() + " WHERE " + getDataTableName() + ".id=disorder.prot_data_id "
                + " AND disorder.prot_data_id=rep.prot_data_id " + " GROUP BY disorder.prot_data_id";
        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        String[][] ret = null;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectStringArray2D(con, sql);
        } catch (SQLException e) {
            logger.debug("error query database", e);

            return null;
        }
        return ret;
    }

    /*
     * (non-Javadoc)
     * 
     * @see prompt.input.IDisorder#getDisorderPercentageAll()
     */
    public double getDisorderPercentageAll() {
        String sql = "SELECT avg(percentage_residues), avg( strech_len) FROM ( "
                + " SELECT (sum(disorder.length)/rep.length*100) AS percentage_residues, sum(disorder.length)/count(disorder.id) AS strech_len"
                + " FROM disorder,rep," + getDataTableName() + " WHERE " + getDataTableName()
                + ".id=disorder.prot_data_id " + " AND disorder.prot_data_id=rep.prot_data_id "
                + " GROUP BY disorder.prot_data_id) AS AA ";
        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        double ret = Double.NaN;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectDouble(con, sql);
        } catch (SQLException e) {
            logger.debug("error query database", e);
        }
        return ret;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IPedant#getFuncat0xMetabolism()
     */
    public int[] getFuncat0xMetabolism() {
        // getProteinsWithFunNumAtTheStart(String)
        String sql = "SELECT DISTINCT funcat.prot_data_id " + " FROM funcat," + getDataTableName()
                + " WHERE (funcat.fun_num LIKE '01%' OR funcat.fun_num LIKE '02%' OR funcat.fun_num LIKE '04%') AND "
                + " funcat.prot_data_id=" + getDataTableName() + ".id";
        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        int[] ret = null;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectIntArray(con, sql);
        } catch (SQLException e) {
            logger.debug("error query database", e);

            return null;
        }

        return ret;
    }

    /*
     * 
     * alter table funcat_data rename to funcat_data13
     * 
     * SELECT fun_num,fun_name FROM funcat where fun_num='04'
     * 
     * Funcat 2.0: 04= STORAGE Funcat 1.3: 04= TRANSCRIPTION
     * 
     */
    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IPedant#getFuncat1xInformation()
     */
    public int[] getFuncat1xInformation() {
        String sql = "SELECT DISTINCT funcat.prot_data_id " + " FROM funcat," + getDataTableName()
                + " WHERE (funcat.fun_num LIKE '10%' OR funcat.fun_num LIKE '11%' OR funcat.fun_num LIKE '12%'"
                + " OR funcat.fun_num LIKE '14%' OR funcat.fun_num LIKE '16%' OR funcat.fun_num LIKE '18%')"
                + " AND funcat.prot_data_id=" + getDataTableName() + ".id";
        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        int[] ret = null;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectIntArray(con, sql);
        } catch (SQLException e) {
            logger.debug("error query database", e);

            return null;
        }
        return ret;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IPedant#getFuncat2xTransport()
     */
    public int[] getFuncat2xTransport() {
        String sql = "SELECT DISTINCT funcat.prot_data_id " + " FROM funcat," + getDataTableName()
                + " WHERE funcat.fun_num LIKE '20%' AND " + " funcat.prot_data_id=" + getDataTableName() + ".id";
        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        int[] ret = null;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectIntArray(con, sql);
        } catch (SQLException e) {
            logger.debug("error query database", e);

            return null;
        }

        return ret;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IPedant#getFuncat3xPerception()
     */
    public int[] getFuncat3xPerception() {
        String sql = "SELECT DISTINCT funcat.prot_data_id "
                + " FROM funcat,"
                + getDataTableName()
                + " WHERE (funcat.fun_num LIKE '30%' OR funcat.fun_num LIKE '32%' OR funcat.fun_num LIKE '34%' OR funcat.fun_num LIKE '36%'"
                + " OR funcat.fun_num LIKE '38%')" + " AND funcat.prot_data_id=" + getDataTableName() + ".id";
        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        int[] ret = null;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectIntArray(con, sql);
        } catch (SQLException e) {
            logger.debug("error query database", e);

            return null;
        }
        return ret;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IPedant#getFuncat4xDevelopmental()
     */
    public int[] getFuncat4xDevelopmental() {
        String sql = "SELECT DISTINCT funcat.prot_data_id " + " FROM funcat," + getDataTableName()
                + " WHERE (funcat.fun_num LIKE '40%' OR funcat.fun_num LIKE '41%' OR funcat.fun_num LIKE '42%'"
                + " OR funcat.fun_num LIKE '43%' OR funcat.fun_num LIKE '45%' OR funcat.fun_num LIKE '47%')"
                + " AND funcat.prot_data_id=" + getDataTableName() + ".id";
        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        int[] ret = null;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectIntArray(con, sql);
        } catch (SQLException e) {
            logger.debug("error query database", e);
            return null;
        }
        return ret;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IPedant#getFuncat7xLocalization()
     */
    public int[] getFuncat7xLocalization() {
        String sql = "SELECT DISTINCT funcat.prot_data_id " + " FROM funcat," + getDataTableName()
                + " WHERE (funcat.fun_num LIKE '70%' OR funcat.fun_num LIKE '73%' OR funcat.fun_num LIKE '75%'"
                + " OR funcat.fun_num LIKE '77%' OR funcat.fun_num LIKE '78%')" + " AND funcat.prot_data_id="
                + getDataTableName() + ".id";
        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        int[] ret = null;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectIntArray(con, sql);
        } catch (SQLException e) {
            logger.debug("error query database", e);

            return null;
        }
        return ret;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IFuncat#getFuncat13_01xMetabolism()
     */
    public int[] getFuncat13_01xMetabolism() {
        return getProteinsWithFunNumAtTheStart("01");
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IFuncat#getFuncat13_02xEnergy()
     */
    public int[] getFuncat13_02xEnergy() {
        return getProteinsWithFunNumAtTheStart("02");
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IFuncat#getFuncat13_04xTranscription()
     */
    public int[] getFuncat13_04xTranscription() {
        return getProteinsWithFunNumAtTheStart("04");
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IFuncat#getFuncat13_05xProteinSynthesis()
     */
    public int[] getFuncat13_05xProteinSynthesis() {
        return getProteinsWithFunNumAtTheStart("05");
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IFuncat#getFuncat13_06xProteinFate()
     */
    public int[] getFuncat13_06xProteinFate() {
        return getProteinsWithFunNumAtTheStart("06");
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IFuncat#getFuncat13_08xCellularTransport()
     */
    public int[] getFuncat13_08xCellularTransport() {
        return getProteinsWithFunNumAtTheStart("08");
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IFuncat#getFuncat13_11xCellResuce()
     */
    public int[] getFuncat13_11xCellResuce() {
        return getProteinsWithFunNumAtTheStart("11");
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IFuncat#getFuncat13_30xCellularOrganization()
     */
    public int[] getFuncat13_30xCellularOrganization() {
        return getProteinsWithFunNumAtTheStart("30");
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IFuncat#getFuncat13_40xSubcellularLocalisation()
     */
    public int[] getFuncat13_40xSubcellularLocalisation() {
        return getProteinsWithFunNumAtTheStart("40");
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IFuncat#getFuncat13_63xProteinsWithBindingFunction()
     */
    public int[] getFuncat13_63xProteinsWithBindingFunction() {
        return getProteinsWithFunNumAtTheStart("63");
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IFuncat#getFuncat13_67xTransportFacilitation()
     */
    public int[] getFuncat13_67xTransportFacilitation() {
        return getProteinsWithFunNumAtTheStart("67");
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IFuncat#getFuncat9xUnknown()
     */
    public int[] getFuncat9xUnknown() {
        String sql = "SELECT DISTINCT funcat.prot_data_id " + " FROM funcat," + getDataTableName()
                + " WHERE (funcat.fun_num LIKE '98%' OR funcat.fun_num LIKE '99%') AND " + " funcat.prot_data_id="
                + getDataTableName() + ".id";
        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        int[] ret = null;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectIntArray(con, sql);
        } catch (SQLException e) {
            logger.debug("error query database", e);

            return null;
        }

        return ret;
    }

    public List getBetaAggregationStretches() throws NumberFormatException, IOException {
        String sql = "SELECT tango_beta_stretches " + "FROM `rep`," + getDataTableName() + " WHERE "
                + getDataTableName() + ".id=rep.prot_data_id " + "AND tango_beta_stretches>=0";
        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        List seq = new LinkedList();
        int[] ret = null;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectIntArray(con, sql);
            for (int i = 0; i < ret.length; i++) {
                int current = ret[i];
                seq.add(new Integer(current));
            }

        } catch (SQLException e) {

            logger.error(e);
        }

        return seq;

    }

    private boolean existTable(String dbTable) {
        try {
            Connection con = Db.connectDb(dbProperties);
            Db.selectString(con, "SELECT * FROM " + dbTable + " LIMIT 1");
        } catch (SQLException e) {
            return false;
        }
        return true;
    }

    public String getScopTable() {
        // exist table scop or scop1
        String tableA = "scop";
        if (!existTable(tableA)) {
            tableA = "scop1";
            if (!existTable(tableA)) {
                Logger logger = Logger.getLogger(this.getClass().getName());
                logger.warn("Table scop or scop1 do NOT exist or can not accessed in " + this.getDatabaseName());
                return null;
            }
        }
        return tableA;
    }

    /*
     * (non-Javadoc)
     * 
     * @see prompt.input.IDesignability#getDesignabilityMinFamilies(java.lang.String)
     */
    public List getDesignabilityMinFamilies(String maxEvalue) {
        // TODO: noch nonsense, denn die anz der families unabhaenig von
        // Datenbank
        String scop = getScopTable();
        String sql = "SELECT min(VAL) FROM ( " + " SELECT prot_data_id, fold, count(distinct family) AS VAL "
                + " FROM " + scop + "," + getDataTableName() + " WHERE " + getDataTableName() + ".id=prot_data_id "
                + " AND pval<=" + maxEvalue + " group by prot_data_id, fold " + ") AS Q1 GROUP BY prot_data_id ";
        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        List seq = new LinkedList();
        int[] ret = null;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectIntArray(con, sql);
            for (int i = 0; i < ret.length; i++) {
                int current = ret[i];
                seq.add(new Integer(current));
            }

        } catch (SQLException e) {

            logger.error(e);
        }

        return seq;
    }

    /*
     * (non-Javadoc)
     * 
     * @see prompt.input.IDesignability#getDesignabilityAvgFamilies(java.lang.String)
     */
    public List getDesignabilityAvgFamilies(String maxEvalue) {
        // TODO: noch nonsense
        String scop = getScopTable();
        String sql = "SELECT  avg(VAL) FROM ( " + " SELECT prot_data_id, fold, count(distinct family) AS VAL "
                + " FROM " + scop + "," + getDataTableName() + " WHERE " + getDataTableName() + ".id=prot_data_id "
                + " AND pval<=" + maxEvalue + " group by prot_data_id, fold " + ") AS Q1 GROUP BY prot_data_id ";
        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        List seq = new LinkedList();
        double[] ret = null;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectDoubleArray(con, sql);
            for (int i = 0; i < ret.length; i++) {
                double current = ret[i];
                seq.add(new Double(current));
            }
        } catch (SQLException e) {

            logger.error(e);
        }

        return seq;
    }

    public boolean isFunCatAvailable() {
        return existTable("funcat");
    }

    public DBProperties getDbProperties() {
        return dbProperties;
    }

    public void setDbProperties(DBProperties dbProperties) {
        this.dbProperties = dbProperties;
    }

    public boolean isCathAvailable() {
        return existTable("cath");
    }

    protected HashMap getSplittedIds(String table, String codeField, String filter, String pvalThreshold, int levels) {
        // Ask if we allow double annotations e.g. the same annoTerm more than
        // one time for the same protein

        String sql = "SELECT " + table + ".code, " + codeField + " FROM " + table + "," + getDataTableName()
                + " WHERE " + filter + "<" + pvalThreshold + " AND " + table + ".prot_data_id=" + getDataTableName()
                + ".id";
        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        HashMap hash = new HashMap();
        String[][] ret;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectStringArray2D(con, sql);
        } catch (SQLException e) {
            logger.debug("error query database", e);

            return null;
        }
        for (int i = 0; i < ret.length; i++) {
            String[] row = ret[i];
            if ((row != null) && (row.length == 2) && (row[0] != null) && (row[1] != null)) {
                String code = row[0];
                String[] cathLevels = row[1].split("\\.");
                if ((cathLevels != null) && (cathLevels.length >= levels)) {
                    String value = (String) hash.get(code);

                    String currentValue = cathLevels[0];
                    for (int j = 1; j < levels; j++) {
                        currentValue = currentValue + "." + cathLevels[j];
                    }
                    if (value == null)
                        value = currentValue;
                    else {
                        // TODO: Here check if value already included,
                        // if yes, then do not add again. That's all

                        boolean contain = false;

                        if (value.contains(";" + currentValue + ";")) {
                            contain = true; // in der Mitte
                        }
                        if (value.endsWith(";" + currentValue)) {
                            contain = true; // letztes element
                        }
                        if (value.contains(currentValue + ";")) { // contain=true
                            // value='99;10'
                            // currentValue='99'
                            contain = true; // genau das bereits als ersten in
                            // der liste von mehreren
                        }
                        if (value.equals(currentValue)) {
                            contain = true; // genau das bereits als ersten in
                            // der liste (nur ein eintrag)
                        }

                        if (contain == false) {
                            // logger.debug("contain=false value='"+value+"'
                            // currentValue='"+currentValue+"'");
                            value = value + ";" + currentValue;
                        } else {
                            // logger.debug("contain=true value='"+value+"'
                            // currentValue='"+currentValue+"'");
                        }

                    }
                    hash.put(code, value);
                } // else logger.debug("error splitting number in result data
                // at line "+i);
            } else
                logger.debug("error in result data at line " + i);
        }
        return hash;
    }

    protected HashMap getSplittedDesc(String table, String codeField, String descField, int levels) {
        String sql = "SELECT " + codeField + "," + descField + " FROM " + table + "," + getDataTableName() + " WHERE "
                + table + ".prot_data_id=" + getDataTableName() + ".id";

        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        HashMap hash = new HashMap();
        String[][] ret;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectStringArray2D(con, sql);
        } catch (SQLException e) {
            logger.debug("error query database", e);

            return null;
        }
        for (int i = 0; i < ret.length; i++) {
            String[] row = ret[i];
            if ((row != null) && (row.length == 2) && (row[0] != null) && (row[1] != null)) {
                String desc = row[1];
                String[] cathLevels = row[0].split("\\.");
                if ((cathLevels != null) && (cathLevels.length >= levels)) {
                    String currentValue = cathLevels[0];
                    for (int j = 1; j < levels; j++) {
                        currentValue = currentValue + "." + cathLevels[j];
                    }
                    hash.put(currentValue, desc);
                } // else logger.debug("error splitting cathnumber in result
                // data at line "+i);
            } else
                logger.debug("error in result data at line " + i);
        }
        return hash;
    }

    public HashMap getCathClassIds(String eValThreshold) {
        return this.getSplittedIds("cath", "cath_number", "pval", eValThreshold, 1);
    }

    public HashMap getCathClassDesc() {
        return this.getSplittedDesc("cath", "cath_number", "class", 1);
    }

    public HashMap getCathArcIds(String eValThreshold) {
        return this.getSplittedIds("cath", "cath_number", "pval", eValThreshold, 2);
    }

    public HashMap getCathArcDesc() {
        return this.getSplittedDesc("cath", "cath_number", "arc", 2);
    }

    public HashMap getCathTopolIds(String eValThreshold) {
        return this.getSplittedIds("cath", "cath_number", "pval", eValThreshold, 3);
    }

    public HashMap getCathTopolDesc() {
        return this.getSplittedDesc("cath", "cath_number", "topol", 3);
    }

    public HashMap getCathHomolIds(String eValThreshold) {
        return this.getSplittedIds("cath", "cath_number", "pval", eValThreshold, 4);
    }

    public HashMap getCathHomolDesc() {
        return this.getSplittedDesc("cath", "cath_number", "homol", 4);
    }

    public HashMap getFuncatLevel1Ids(String eValThreshold) {
        return this.getSplittedIds("funcat", "fun_num", "pval", eValThreshold, 1);
    }

    @SuppressWarnings("unchecked")
    protected HashMap getFuncatDescriptions(int levels) {
        String sql = "SELECT fun_num,fun_name FROM funcat," + getDataTableName() + " WHERE funcat.prot_data_id="
                + getDataTableName() + ".id";

        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        HashMap hash = new HashMap();
        String[][] ret;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectStringArray2D(con, sql);
        } catch (SQLException e) {
            logger.debug("error query database", e);

            return null;
        }
        for (int i = 0; i < ret.length; i++) {
            String[] row = ret[i];
            if ((row != null) && (row.length == 2) && (row[0] != null) && (row[1] != null)) {
                String desc = row[1];
                String[] cathLevels = row[0].split("\\.");
                if ((cathLevels != null) && (cathLevels.length == levels)) {
                    String currentValue = cathLevels[0];
                    for (int j = 1; j < levels; j++) {
                        currentValue = currentValue + "." + cathLevels[j];
                    }
                    hash.put(currentValue, desc);
                } // else logger.debug("error splitting cathnumber in result
                // data at line "+i);
            } else
                logger.debug("error in result data at line " + i);
        }
        return hash;
    }

    public HashMap getFuncatLevel1Desc() {
        return getFuncatDescriptions(1);
    }

    public HashMap getFuncatLevel2Ids(String eValThreshold) {
        return this.getSplittedIds("funcat", "fun_num", "pval", eValThreshold, 2);
    }

    public HashMap getFuncatLevel2Desc() {
        return getFuncatDescriptions(2);
    }

    public HashMap getFuncatLevel3Ids(String eValThreshold) {
        return this.getSplittedIds("funcat", "fun_num", "pval", eValThreshold, 3);
    }

    public HashMap getFuncatLevel3Desc() {
        return getFuncatDescriptions(3);
    }

    public HashMap getFuncatLevel4Ids(String eValThreshold) {
        return this.getSplittedIds("funcat", "fun_num", "pval", eValThreshold, 4);
    }

    public HashMap getFuncatLevel4Desc() {
        return getFuncatDescriptions(4);
    }

    public boolean isScopAvailable() {
        return existTable(getScopTable());
    }

    public HashMap getScopClassIds(String eValThreshold) {
        return this.getSplittedIds(getScopTable(), "scop_number", "pval", eValThreshold, 1);
    }

    public HashMap getScopClassDesc() {
        return this.getSplittedDesc("scop", "scop_number", "class", 1);
    }

    public HashMap getScopFoldIds(String eValThreshold) {
        return this.getSplittedIds(getScopTable(), "scop_number", "pval", eValThreshold, 2);
    }

    public HashMap getScopFoldDesc() {
        return this.getSplittedDesc("scop", "scop_number", "fold", 2);
    }

    public HashMap getScopSuperfamilyIds(String eValThreshold) {
        return this.getSplittedIds(getScopTable(), "scop_number", "pval", eValThreshold, 3);
    }

    public HashMap getScopSuperfamilyDesc() {
        return this.getSplittedDesc("scop", "scop_number", "superfamily", 3);
    }

    public HashMap getScopFamilyIds(String eValThreshold) {
        return this.getSplittedIds(getScopTable(), "scop_number", "pval", eValThreshold, 4);
    }

    public HashMap getScopFamilyDesc() {
        return this.getSplittedDesc("scop", "scop_number", "family", 4);
    }

    public boolean isEcNumAvailable() {
        return existTable("ecnum");
    }

    public HashMap getEcLevel1Ids(String eValThreshold) {
        return this.getSplittedIds("ecnum", "ecnum", "pval", eValThreshold, 1);
    }

    public HashMap getEcLevel2Ids(String eValThreshold) {
        return this.getSplittedIds("ecnum", "ecnum", "pval", eValThreshold, 2);
    }

    public boolean isBlimpsAvailable() {
        return existTable("blimps");
    }

    @SuppressWarnings("unchecked")
    public HashMap getBlimpsIds(String eValThreshold) {
        String sql = "SELECT blimps.code, blimpsid FROM blimps," + getDataTableName() + " WHERE pval<" + eValThreshold
                + " AND blimps.prot_data_id=" + getDataTableName() + ".id";
        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        HashMap hash = new HashMap();
        String[][] ret;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectStringArray2D(con, sql);
        } catch (SQLException e) {
            logger.debug("error query database", e);

            return null;
        }
        for (int i = 0; i < ret.length; i++) {
            String[] row = ret[i];
            if ((row != null) && (row.length == 2) && (row[0] != null) && (row[1] != null)) {
                String code = row[0];
                String currentValue = row[1].trim();

                // falls letztes Zeichen alpabetischer untertyp, dann
                // abschneiden
                String last = currentValue.substring(currentValue.length() - 1);
                try {
                    Integer.parseInt(last);
                } catch (NumberFormatException e) {
                    currentValue = currentValue.substring(0, currentValue.length() - 1);
                }
                String value = (String) hash.get(code);
                if (value == null)
                    value = currentValue;
                else
                    value = value + ";" + currentValue;
                hash.put(code, value);

            } else
                logger.debug("error in result data at line " + i);
        }
        return hash;
    }

    @SuppressWarnings("unchecked")
    public HashMap getBlimpsDesc() {
        String sql = "SELECT  blimpsid,descr FROM blimps";
        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        HashMap hash = new HashMap();
        String[][] ret;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectStringArray2D(con, sql);
        } catch (SQLException e) {
            logger.debug("error query database", e);

            return null;
        }
        for (int i = 0; i < ret.length; i++) {
            String[] row = ret[i];
            if ((row != null) && (row.length == 2) && (row[0] != null) && (row[1] != null)) {
                String blimpsid = row[0];
                String descr = row[1].trim();

                // falls letztes Zeichen alpabetischer untertyp, dann
                // abschneiden
                String last = blimpsid.substring(blimpsid.length() - 1);
                try {
                    Integer.parseInt(last);
                } catch (NumberFormatException e) {
                    blimpsid = blimpsid.substring(0, blimpsid.length() - 1);
                }
                hash.put(blimpsid, descr);

            } else
                logger.debug("error in result data at line " + i);
        }
        return hash;
    }

    protected HashMap getIds(String table, String codeField, String filter, String pvalThreshold) {
        String sql = "SELECT " + table + ".code, " + codeField + " FROM " + table + "," + getDataTableName()
                + " WHERE " + filter + "<" + pvalThreshold + " AND " + table + ".prot_data_id=" + getDataTableName()
                + ".id";
        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        HashMap hash = new HashMap();
        String[][] ret;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectStringArray2D(con, sql);
        } catch (SQLException e) {
            logger.debug("error query database", e);

            return null;
        }
        for (int i = 0; i < ret.length; i++) {
            String[] row = ret[i];
            if ((row != null) && (row.length == 2) && (row[0] != null) && (row[1] != null)) {
                String code = row[0];
                String currentValue = row[1].trim();

                String value = (String) hash.get(code);
                if (value == null)
                    value = currentValue;
                else
                    value = value + ";" + currentValue;
                hash.put(code, value);

            } else
                logger.debug("error in result data at line " + i);
        }
        return hash;
    }

    protected HashMap getIds(String table, String codeField) {
        String sql = "SELECT " + table + ".code, " + codeField + " FROM " + table + "," + getDataTableName()
                + " WHERE " + table + ".prot_data_id=" + getDataTableName() + ".id";
        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        HashMap hash = new HashMap();
        String[][] ret;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectStringArray2D(con, sql);
        } catch (SQLException e) {
            logger.debug("error query database", e);

            return null;
        }
        for (int i = 0; i < ret.length; i++) {
            String[] row = ret[i];
            if ((row != null) && (row.length == 2) && (row[0] != null) && (row[1] != null)) {
                String code = row[0];
                String currentValue = row[1].trim();

                String value = (String) hash.get(code);
                if (value == null)
                    value = currentValue;
                else
                    value = value + ";" + currentValue;
                hash.put(code, value);

            } else
                logger.debug("error in result data at line " + i);
        }
        return hash;
    }

    protected HashMap getDesc(String table, String codeField, String descField) {
        String sql = "SELECT " + codeField + "," + table + "." + descField + " FROM " + table + ","
                + getDataTableName() + " WHERE " + table + ".prot_data_id=" + getDataTableName() + ".id";

        Logger logger = Logger.getLogger(this.getClass().getName());
        logger.debug(sql);
        HashMap hash = new HashMap();
        String[][] ret;
        try {
            Connection con = Db.connectDb(dbProperties);
            ret = Db.selectStringArray2D(con, sql);
        } catch (SQLException e) {
            logger.debug("error query database", e);

            return null;
        }
        for (int i = 0; i < ret.length; i++) {
            String[] row = ret[i];
            if ((row != null) && (row.length == 2) && (row[0] != null) && (row[1] != null)) {
                String desc = row[1];
                String code = row[0];
                hash.put(code, desc);
            } else
                logger.debug("error in result data at line " + i);
        }
        return hash;
    }

    public boolean isInterproAvailable() {
        return existTable("interpro");
    }

    public HashMap getInterproIds() {
        return getIds("interpro", "ipid");
        // return getIds("interpro", "ipid", "pval", eValThreshold);
    }

    public HashMap getInterproDesc() {
        return getDesc("interpro", "ipid", "descr");
    }

    public boolean isPfamAvailable() {
        return existTable("pfam");
    }

    public HashMap getPfamIds(String eValThreshold) {
        return getIds("pfam", "hmmid", "pval", eValThreshold);
    }

    public HashMap getPfamDesc() {
        return getDesc("pfam", "hmmid", "descr");
    }

    public boolean isPrositeAvailable() {
        return existTable("pros");
    }

    public HashMap getPrositeIds() {
        return getIds("pros", "pdid");
    }

    public HashMap getPrositeDesc() {
        return getDesc("pros", "pdid", "descr");
    }

    public IGenericProperty[] getProperties() {
        ArrayList al = new ArrayList();
        if (this.isBlimpsAvailable()) {
            al.add(new PedantProxy(this, "Blimps", IGenericProperty.SYMBOLIC));
        }
        if (this.isCathAvailable()) {
            al.add(new PedantProxy(this, "CathArc", IGenericProperty.SYMBOLIC));
            al.add(new PedantProxy(this, "CathClass", IGenericProperty.SYMBOLIC));
            al.add(new PedantProxy(this, "CathHomol", IGenericProperty.SYMBOLIC));
            al.add(new PedantProxy(this, "CathTopol", IGenericProperty.SYMBOLIC));
        }
        if (this.isScopAvailable()) {
            al.add(new PedantProxy(this, "ScopClass", IGenericProperty.SYMBOLIC));
            al.add(new PedantProxy(this, "ScopFamiliy", IGenericProperty.SYMBOLIC));
            al.add(new PedantProxy(this, "ScopFold", IGenericProperty.SYMBOLIC));
            al.add(new PedantProxy(this, "ScopSuperFamily", IGenericProperty.SYMBOLIC));
        }
        if (this.isEcNumAvailable()) {
            al.add(new PedantProxy(this, "EcLevel1", IGenericProperty.SYMBOLIC));
            al.add(new PedantProxy(this, "EcLevel2", IGenericProperty.SYMBOLIC));
        }
        if (this.isFunCatAvailable()) {
            al.add(new PedantProxy(this, "FuncatLevel1", IGenericProperty.SYMBOLIC));
            al.add(new PedantProxy(this, "FuncatLevel2", IGenericProperty.SYMBOLIC));
            al.add(new PedantProxy(this, "FuncatLevel3", IGenericProperty.SYMBOLIC));
            al.add(new PedantProxy(this, "FuncatLevel4", IGenericProperty.SYMBOLIC));
        }
        if (this.isInterproAvailable()) {
            al.add(new PedantProxy(this, "Interpro", IGenericProperty.SYMBOLIC));
        }
        if (this.isPfamAvailable()) {
            al.add(new PedantProxy(this, "Pfam", IGenericProperty.SYMBOLIC));
        }
        if (this.isPrositeAvailable()) {
            al.add(new PedantProxy(this, "Prosite", IGenericProperty.SYMBOLIC));
        }
        al.add(new PedantProxy(this, "SeqDescriptions", IGenericProperty.SYMBOLIC));

        IGenericProperty[] ret = new IGenericProperty[al.size()];
        for (int i = 0; i < ret.length; i++) {
            ret[i] = (IGenericProperty) al.get(i);
        }

        Class superClazz = super.getClass();
        if (IGenericXML.class.isAssignableFrom(superClazz)) {
            ret = GenericHelper.mergePropertyArrays(super.getProperties(), ret);
        }
        return ret;
    }

    public String getLabel() {
        return getDatabaseName();
    }

    class PedantProxy implements IGenericProperty {
        IPedant backref;

        int type = IGenericProperty.YETUNKOWN;

        String method = "unknown";

        public PedantProxy(IPedant backref, String method, int type) {
            this.backref = backref;
            this.method = method;
            this.type = type;
        }

        public int getType() {
            return type;
        }

        public String getId() {
            return method;
        }

        public HashMap getValues() {
            /*
             * je nachdem was fuer eine methode rufe eine andere Methode des
             * SeqProperties Objeckts auf das als backref bekannt ist.
             */
            try {

                if (method.equals("ScopClass")) {
                    String eValue = (String) JOptionPane.showInputDialog(null, "Enter e-value threshold: "
                            + method.toUpperCase(), "1e-4");
                    if (eValue == null)
                        return null;
                    return backref.getScopClassIds(eValue);
                }
                if (method.equals("ScopFamiliy")) {
                    String eValue = (String) JOptionPane.showInputDialog(null, "Enter e-value threshold: "
                            + method.toUpperCase(), "1e-4");
                    if (eValue == null)
                        return null;
                    return backref.getScopFamilyIds(eValue);
                }
                if (method.equals("ScopFold")) {
                    String eValue = (String) JOptionPane.showInputDialog(null, "Enter e-value threshold: "
                            + method.toUpperCase(), "1e-4");
                    if (eValue == null)
                        return null;
                    return backref.getScopFoldIds(eValue);
                }
                if (method.equals("ScopSuperFamily")) {
                    String eValue = (String) JOptionPane.showInputDialog(null, "Enter e-value threshold: "
                            + method.toUpperCase(), "1e-4");
                    if (eValue == null)
                        return null;
                    return backref.getScopSuperfamilyIds(eValue);
                }

                if (method.equals("CathClass")) {
                    String eValue = (String) JOptionPane.showInputDialog(null, "Enter e-value threshold: "
                            + method.toUpperCase(), "1e-4");
                    if (eValue == null)
                        return null;
                    return backref.getCathClassIds(eValue);
                }
                if (method.equals("CathArc")) {
                    String eValue = (String) JOptionPane.showInputDialog(null, "Enter e-value threshold: "
                            + method.toUpperCase(), "1e-4");
                    if (eValue == null)
                        return null;
                    return backref.getCathArcIds(eValue);
                }
                if (method.equals("CathTopol")) {
                    String eValue = (String) JOptionPane.showInputDialog(null, "Enter e-value threshold: "
                            + method.toUpperCase(), "1e-4");
                    if (eValue == null)
                        return null;
                    return backref.getCathTopolIds(eValue);
                }
                if (method.equals("CathHomol")) {
                    String eValue = (String) JOptionPane.showInputDialog(null, "Enter e-value threshold: "
                            + method.toUpperCase(), "1e-4");
                    if (eValue == null)
                        return null;
                    return backref.getCathHomolIds(eValue);
                }
                if (method.equals("EcLevel1")) {
                    String eValue = (String) JOptionPane.showInputDialog(null, "Enter e-value threshold: "
                            + method.toUpperCase(), "1e-4");
                    if (eValue == null)
                        return null;
                    return backref.getEcLevel1Ids(eValue);
                }
                if (method.equals("EcLevel2")) {
                    String eValue = (String) JOptionPane.showInputDialog(null, "Enter e-value threshold: "
                            + method.toUpperCase(), "1e-4");
                    if (eValue == null)
                        return null;
                    return backref.getEcLevel2Ids(eValue);
                }

                if (method.equals("FuncatLevel1")) {
                    String eValue = (String) JOptionPane.showInputDialog(null, "Enter e-value threshold: "
                            + method.toUpperCase(), "1e-4");
                    if (eValue == null)
                        return null;
                    return backref.getFuncatLevel1Ids(eValue);
                }
                if (method.equals("FuncatLevel2")) {
                    String eValue = (String) JOptionPane.showInputDialog(null, "Enter e-value threshold: "
                            + method.toUpperCase(), "1e-4");
                    if (eValue == null)
                        return null;
                    return backref.getFuncatLevel2Ids(eValue);
                }
                if (method.equals("FuncatLevel3")) {
                    String eValue = (String) JOptionPane.showInputDialog(null, "Enter e-value threshold: "
                            + method.toUpperCase(), "1e-4");
                    if (eValue == null)
                        return null;
                    return backref.getFuncatLevel3Ids(eValue);
                }
                if (method.equals("FuncatLevel4")) {
                    String eValue = (String) JOptionPane.showInputDialog(null, "Enter e-value threshold: "
                            + method.toUpperCase(), "1e-4");
                    if (eValue == null)
                        return null;
                    return backref.getFuncatLevel4Ids(eValue);
                }

                if (method.equals("Blimps")) {
                    String eValue = (String) JOptionPane.showInputDialog(null, "Enter e-value threshold: "
                            + method.toUpperCase(), "1e-4");
                    if (eValue == null)
                        return null;
                    return backref.getBlimpsIds(eValue);
                }
                if (method.equals("Interpro")) {
                    return backref.getInterproIds();
                }

                if (method.equals("Pfam")) {
                    String eValue = (String) JOptionPane.showInputDialog(null, "Enter e-value threshold: "
                            + method.toUpperCase(), "1e-4");
                    if (eValue == null)
                        return null;
                    return backref.getPfamIds(eValue);
                }
                if (method.equals("Prosite")) {
                    return backref.getPrositeIds();
                }
                if (method.equals("SeqDescriptions")) {
                    return GenericHelper.makeHash(backref, backref.getAnnotation());
                }

                /*
                 * Blimps Interpro Pfam Prosite CathArc CathClass CathHomol
                 * CathTopol EcLevel1 EcLevel2 FuncatLevel1 FuncatLevel2
                 * 
                 * ScopClass ScopFamiliy ScopFold ScopSuperFamily
                 */

            } catch (Exception e) {
                Logger logger = Logger.getLogger(SeqPropertiesProxy.class);
                logger.fatal("error reading property: ", e);
            }

            return null;
        }

    }

    public Component parentWin = null;

    public void setParentWindow(Component win) {
        this.parentWin = win;
    }

    public Iterator iterator() {
        return new SeqIterator();
    }

    public class SeqIterator implements Iterator {

        ResultSet rs;
        boolean cached = false;

        PreparedStatement preStmt;
        Connection con;
        boolean hasNextElement = false;

        public SeqIterator() {
            try {
                con = Db.connectDb(dbProperties);
                Statement stmt = con.createStatement();
                preStmt = con.prepareStatement("SELECT dat FROM prot_data WHERE code = ?");
                rs = stmt.executeQuery("SELECT code FROM prot_data");
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }

        public boolean hasNext() {
           
            if (cached) {
                return true;
            }
            try {
                hasNextElement = rs.next();
                cached = true;
            } catch (SQLException e) {
                hasNextElement = false;
                return false;
            }
            if (!hasNextElement) {
                try {
                    rs.close();
                    preStmt.close();
                    con.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            }
            return hasNextElement;
        }

        public Object next() {
            String nextSequence = null, nextName = null;
            try {
                if (!cached) {
                   hasNext();
                }
                if(!hasNextElement) {
                    rs.close();
                    preStmt.close();
                    con.close();
                    return null;
                }
                nextName = rs.getString(1);
                preStmt.setString(1, nextName);
                ResultSet nextElementRS = preStmt.executeQuery();
                nextElementRS.next();
                nextSequence = nextElementRS.getString(1);
            } catch (SQLException e) {
                e.printStackTrace();
            }
            cached = false;
            return new Object[] { nextName, nextSequence};
        }

        public void remove() {
            throw new UnsupportedOperationException();
        }
    }
}
