/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: PedantFilter.java,v 1.4 2006/03/19 11:57:54 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input.classes;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.TreeMap;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;

import prompt.input.IInput;
import prompt.input.IPedant;
import prompt.input.filter.IPedantFilter;
import tschmidt.MetaDb.DbTable;
import tschmidt.MetaDb.TableReader;
import tschmidt.utils.Db;


/**
 * in den auto-generated methods muss jetzt jeweils eine Methode aufgerufen
 * werden die a) einen neuen View-Table erzeugt b) in der caller-classe den TN=
 * Table Name fuer prot_data neu setzt
 * 
 * @author schmidt
 *  
 */
public class PedantFilter extends Filter implements IPedantFilter {
    private Logger logger = Logger.getLogger(this.getClass().getName());

    private final static long serialVersionUID = -6849794470754667700L;

    protected String viewTableName = null;

    //FIXME: id feld noch indizieren mit KEY
    protected final String DEFAULT_PROT_DATA_TABLE_SQL = "(`id` int(11) NOT NULL default '0')";

    /*
     * protected final String DEFAULT_PROT_DATA_TABLE_SQL = "(" + "`id` int(11)
     * NOT NULL default '0'," + "`contig_data_id` int(11) NOT NULL default '0'," +
     * "`contig_data_code` varchar(100) NOT NULL default ''," + "`code`
     * varchar(100) NOT NULL default ''," + "`descr` varchar(255) default NULL," +
     * "`dat` longtext," + "`created` varchar(100) NOT NULL default ''," +
     * "PRIMARY KEY (`id`)," + "KEY `contig_data_id` (`contig_data_id`)," + "KEY
     * `contig_data_code` (`contig_data_code`)," + "KEY `code` (`code`)," + "KEY
     * `descr` (`descr`)" + ")";
     */

    protected Random rnd = new Random();

    public PedantFilter() {
        super();

    }

    /*
     * (non-Javadoc)
     * 
     * @see prompt.input.filter.IFilterId#setDestination(de.tum.binfo.proj.input.IInput)
     */
    public void setDestination(IInput dest) {
        super.setDestination(dest);
        updateViewTable();
    }

    public void delViewTable() {
        //del viewTable
        if ((input != null) && (viewTableName != null)) {
            String dbname = ((IPedant) input).getDatabaseName();
            try {
                logger.debug(dbname+": drop view table '" + viewTableName + "' for '" + input.getDescription() + "'");
                Connection connection = Db.connect(Db.getCurrentDefaultUrl() + ((IPedant) input).getDatabaseName());
                Statement stmt = connection.createStatement();
                String sql = "DROP TABLE " + viewTableName;
                stmt.executeUpdate(sql);
                viewTableName = null;
                ((IPedant) input).setDataTableName("prot_data");
            } catch (Exception e) {
                logger.debug(dbname+":error dropping view table '" + viewTableName + "'");
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#finalize()
     */
    protected void finalize() throws Throwable {
        delViewTable();
        super.finalize();
    }

    /**
     * Unterprogramm von updateViewTable, no err-checking
     * 
     * @param tableName
     * @return
     */
    private boolean existTable(String tableName) {
        TableReader tr = new TableReader();
        tr.getTables();
        Enumeration tablesEnum = tr.getTables().elements();
        while (tablesEnum.hasMoreElements()) {
            DbTable nextTable = (DbTable) tablesEnum.nextElement();
            if (nextTable.getTableName().equals(tableName)) {
                return true;
            }
        }
        return false;
    }

    private void setViewTableName(String str) {
        viewTableName = str;
        ((IPedant) input).setDataTableName(viewTableName);
    }

    /**
     * wird immer aufgerufen von setDestination, oder wenn der Filter veraendert
     * wird zum Beispiel durch add*
     */
    protected void updateViewTable() {

        // allgemeine checks
        if (input == null) {
            logger.debug("Not connected to a Pedant database yet");
            return;
        }
        if (!(input instanceof IPedant)) {
            logger.fatal("This filter works only with PEDANT databases");
            return;
        }
        String dbname = ((IPedant) input).getDatabaseName();
        if (dbname == null) {
            logger.fatal("Pedant input object's database name can not be resolved");
            return;
        }
        Connection connection = null;
        try {
            connection = Db.connect(Db.getCurrentDefaultUrl() + dbname);
        } catch (SQLException e1) {
            logger.fatal("Can not connect to Pedant database '" + dbname + "'", e1);
            return;
        }

        // no kein view Name, dann generiere einen
        if (viewTableName == null) {
            logger.debug(dbname+": set up viewDataTableName");
            int tries = 10;
            for (int i = 0; i < tries; i++) {
                double random = rnd.nextDouble() * rnd.nextDouble();
                random = random * 10000000;
                int random_int = (int) Math.floor(random);
                String tmpName = "view_" + random_int;
                logger.debug(dbname+": try table name '" + tmpName + "'");
                // check if this table name is possible, i.e. keine Kollision
                if (!existTable(tmpName)) {
                    setViewTableName(tmpName);
                    break;
                }
            }
            if (viewTableName == null) {
                logger.fatal(dbname+": we could not create a unique table name");
                return;
            }
        }

        //        logger.info("updateViewTable '" + viewTableName + "' input.getdesc='"
        // + input.getDescription() + "' db='" + dbname + "'");

        //alten table weg
        if (existTable(viewTableName)) {
            //dropTable
            String sql = "DROP TABLE `" + viewTableName + "`";
            try {
                Statement stmt = connection.createStatement();
                stmt.executeUpdate(sql);
                stmt.close();
            } catch (SQLException e) {
                logger.fatal(dbname+": error dropping table '" + viewTableName + "' ", e);
            }
        }

        //try to create table, and finish if successed
        try {
            Statement stmt = connection.createStatement();
            String sql = "CREATE TABLE IF NOT EXISTS `" + viewTableName + "` " + DEFAULT_PROT_DATA_TABLE_SQL;
            //logger.debug(sql);
            stmt.executeUpdate(sql);
            stmt.close();

        } catch (SQLException e) {
            logger.fatal(dbname+": error creating view table '" + viewTableName + "' ", e);
            viewTableName = null;
            return;
        }

        // gehe ueber alle prot_data id, if pass(id) diese zum neuen ViewTable
        // hinzufuegen
        logger.debug(dbname+": START fill view table '"+viewTableName+"'");
        try {
            Statement stmt = connection.createStatement();
            stmt.executeQuery("SELECT id FROM prot_data");
            ResultSet rs = stmt.getResultSet();
            while (rs.next()) {
                String value = rs.getString(1);
                if (pass(value)) {
                    Statement update = connection.createStatement();
                    String sql = "INSERT INTO " + viewTableName + " SET id=" + value;
                    update.executeUpdate(sql);
                    update.close();
                }
            }
            stmt.close();
        } catch (SQLException e) {
            logger.fatal(dbname+": error copying data to view table '" + viewTableName + "' ", e);
        }
        logger.debug(dbname+": FINISHED fill view table '"+viewTableName+"'");
    }

    /*
     * (non-Javadoc)
     * 
     * @see prompt.input.filter.IFilterId#addArray(java.lang.String[])
     */
    public void addArray(String[] ids) {
        //hier kein aufruf von addSingleId da sonst jedesmal ein
        // updateViewTabel aufgerufen wird
        for (int i = 0; i < ids.length; i++) {
            String string = ids[i];
            hash.put(string, new Integer(1));
        }
        updateViewTable();
    }

    /*
     * (non-Javadoc)
     * 
     * @see prompt.input.filter.IFilterId#addList(java.util.List)
     */
    public void addList(List list) {
        for (Iterator iter = list.iterator(); iter.hasNext();) {
            String element = (String) iter.next();
            hash.put(element, new Integer(1));
        }
        updateViewTable();
    }

    /*
     * (non-Javadoc)
     * 
     * @see prompt.input.filter.IFilterId#addSingleId(java.lang.String)
     */
    public void addSingleId(String id) {
        super.addSingleId(id);
        updateViewTable();
    }

    /*
     * (non-Javadoc)
     * 
     * @see prompt.input.filter.IFilterId#addStringList(java.lang.String)
     */
    public void addStringList(String str) {
        String[] lines = Pattern.compile("\\\n").split(str);
        if ((lines != null) && (lines.length > 0)) {
            for (int i = 0; i < lines.length; i++) {
                hash.put(lines[i], new Integer(1));
            }
        }
        updateViewTable();
    }

    /*
     * (non-Javadoc)
     * 
     * @see prompt.input.filter.IFilterId#loadFromFile(java.lang.String)
     */
    public void loadFromFile(String filename) throws IOException {
        //TODO: erste Zeile den Typ noch rein!
        hash = null;
        hash = new TreeMap();
        FileInputStream fin = new FileInputStream(filename + ".filter");
        BufferedReader myInput = new BufferedReader(new InputStreamReader(fin));
        String thisLine;
        while ((thisLine = myInput.readLine()) != null) {
            hash.put(thisLine, new Integer(1));
        }
        myInput.close();
        fin.close();

        updateViewTable();
    }

    /*
     * (non-Javadoc)
     * 
     * @see prompt.input.filter.IFilterId#setPass(boolean)
     */
    public void setPass(boolean b) {
        super.setPass(b);
        updateViewTable();
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IPedantFilter#filterAddFromSqlQuery(java.lang.String)
     */
    public void filterAddFromSqlQuery(String dbname, String sql) {
        try {
            String[] ret = Db.selectStringArray(dbname, sql);
            addArray(ret);
        } catch (SQLException e) {
            Logger logger = Logger.getLogger(this.getClass());
            logger.error(e);
        }
    }

}
