/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: PedantWithFilter.java,v 1.4 2006/03/19 11:57:54 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input.classes;


import gr.zeus.ui.JMessage;

import java.io.File;
import java.net.MalformedURLException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import prompt.input.IPedantWithFilter;
import prompt.input.filter.IFilterId;
import prompt.input.filter.IPedantFilter;
import prompt.ui.idDialog.IDDialog;
import prompt.ui.idDialog.IdDialogProperties;
import tschmidt.utils.Db;
import tschmidt.utils.ValidLocation;



/**
 * @author schmidt
 */
public class PedantWithFilter extends Pedant implements IPedantWithFilter {
  private final static long serialVersionUID = -6849794470754667700L;
	/**
	 * 
	 * @uml.property name="filter"
	 * @uml.associationEnd multiplicity="(0 1)"
	 */
	protected IFilterId filter; // = new PedantFilter();


	public PedantWithFilter() {
		super();
	}

	
	
    @Override
	public void run() {
		super.run();
		if (this.getDatabaseName()==null) return; //user canceled database dialog
		
		if (filter==null) {
			IdDialogProperties prop = IDDialog.showIDDialog(null);
			if (prop==null) return; //user canceled
			
			IPedantFilter myFilter = new PedantFilter();
			
	
			if(prop.getIdFile()!=null){
				//load from file
				File f = prop.getIdFile();
				String[] ids;
				try {
					ids = ValidLocation.readLinesFromUrl(f.toURL());
				} catch (MalformedURLException e) {
					Logger logger = Logger.getLogger(this.getClass());
					String errmsg = "Error loading ids from file ";
					logger.error(errmsg,e);
					JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION, e);

					return;
				}
				if (ids==null) {
					Logger logger = Logger.getLogger(this.getClass());
					String errmsg = "Error loading ids from file ";
					logger.error(errmsg);
					JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
					return;
				}
				myFilter.addArray(ids);
				
			}else{
				myFilter.addList(prop.getIds());
			}
			myFilter.setPass(!prop.getInverseSelection());
			
			this.setFilterId(myFilter);
			
			ExitHandlerPFilter eh = new ExitHandlerPFilter();
			eh.filter= myFilter;
			Runtime.getRuntime().addShutdownHook(eh);
			
		}
		

	     
	}



	/* (non-Javadoc)
     * @see java.lang.Object#finalize()
     */
    protected void finalize() throws Throwable {
        if ((filter!=null) && (filter instanceof IPedantFilter)) {
            ((IPedantFilter) filter).delViewTable();
        }
        super.finalize();
    }
	/**
	 * @param dbname
	 * @throws Exception
	 */
	public PedantWithFilter(String dbname) throws Exception {
		super(dbname);
	}

	public List getSequences() {
		if (filter == null)
			return super.getSequences();

		List seq = new LinkedList();
		String[][] ret = null;
		try {
            Connection con = Db.connectDb(dbProperties);
			ret =
				Db.selectStringArray2D(con, "SELECT id,dat FROM prot_data");
			for (int i = 0; i < ret.length; i++) {
				String[] line = ret[i];
				if (line[0] != null) {
					if (filter.pass(line[0])) {
						line[1] = line[1].replaceAll("\\\n", "");
						seq.add(line[1]);
					}
				}

			}

		} catch (SQLException e) {
            Logger logger = Logger.getLogger(this.getClass());
            logger.error(e);
		}

		return seq;
	}

	public List getSequencesNames() {
		if (filter == null)
			return super.getSequencesNames();

		List seq = new LinkedList();
		String[][] ret = null;
		try {
            Connection con = Db.connectDb(dbProperties);
			ret =
							Db.selectStringArray2D(con, "SELECT id,code FROM prot_data");
						for (int i = 0; i < ret.length; i++) {
							String[] line = ret[i];
							if (line[0] != null) {
								if (filter.pass(line[0])) {
									seq.add(line[1]);
								}
							}

						}

		} catch (SQLException e) {
            Logger logger = Logger.getLogger(this.getClass());
            logger.error(e);
		}
		return seq;
	}

	/* (non-Javadoc)
	 * @see de.tum.binfo.proj.input.IInput#getAnnotation()
	 */
	public List getAnnotation() {
		List seq = new LinkedList();
		String[][] ret = null;
		try {
            Connection con = Db.connectDb(dbProperties);
			ret =
							Db.selectStringArray2D(con, "SELECT id,descr FROM prot_data");
						for (int i = 0; i < ret.length; i++) {
							String[] line = ret[i];
							if (line[0] != null) {
								if (filter.pass(line[0])) {
									seq.add(line[1]);
								}
							}

						}

		} catch (SQLException e) {
            Logger logger = Logger.getLogger(this.getClass());
            logger.error(e);
		}
		return seq;
	}

	/**
	 * 
	 * @uml.property name="filter"
	 */
	/* (non-Javadoc)
	 * @see de.tum.binfo.proj.input.IFilterAble#getFilter()
	 */
	public IFilterId getFilterId() {
		return filter;
	}

	/**
	 * 
	 * @uml.property name="filter"
	 */
	/* (non-Javadoc)
	 * @see de.tum.binfo.proj.input.IFilterAble#setFilter(de.tum.binfo.proj.input.IFilter)
	 */
	public void setFilterId(IFilterId filter) {
		this.filter = filter;
		//so kennt der Filter die Pedant DB
		this.filter.setDestination(this);
	}

}

class ExitHandlerPFilter extends Thread {

  	public IPedantFilter filter;
	  
	  public ExitHandlerPFilter()
	  {
	    super();
	  }
	  
	  public void run() {
		   Logger logger = Logger.getLogger(this.getClass());
		   logger.debug("started to delete temporary filter ");
		   filter.delViewTable();
		   logger.debug("finished deleting temporary filter");
	  }
}
	    
