/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: SequenceSet.java,v 1.7 2006/04/06 11:56:08 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input.classes;

import java.awt.Component;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;

import prompt.engine.IEngineResults;
import prompt.input.ISequenceSet;



/**
 * @author schmidt
 * 
 */
public class SequenceSet extends SeqProperties implements ISequenceSet {

    private final static long serialVersionUID = -6849794470754667700L;
    Logger logger = Logger.getLogger(SequenceSet.class);

    protected List listSequences;
    protected List listNames;
    protected List listAnnotations;

    public SequenceSet() {
        listSequences = new LinkedList();
        listNames = new LinkedList();
        listAnnotations = new LinkedList();
   
    }

    protected void parseStr(String str) {
        listSequences = new LinkedList();
        // Parse Sequences
        String[] lines = Pattern.compile("\\\n").split(str);
        logger.debug(lines.length + " Sequence lines found");
        if ((lines != null) && (lines.length > 0)) {
            for (int i = 0; i < lines.length; i++) {
                listSequences.add(lines[i]);
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.ISequenceSet#init(java.io.File)
     */
    public void init(File file) throws IOException {
        FileInputStream fin = new FileInputStream(file);
        listSequences = new LinkedList();
        BufferedReader myInput = new BufferedReader(new InputStreamReader(fin));
        String thisLine;
        while ((thisLine = myInput.readLine()) != null) {
            listSequences.add(thisLine);
        }
        myInput.close();
        fin.close();
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.ISequenceSet#init(java.lang.String)
     */
    public void init(String str) {
        parseStr(str);
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.ISequenceSet#init(de.tum.binfo.proj.engine.EngineResults)
     */
    public void init(IEngineResults result) {
        parseStr(result.getData());

    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.jdrm.JobTemplate#run()
     */
    public void run() throws Exception {
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IInput#getSequences()
     */
    public List getSequences() {
        return listSequences;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IInput#getSequencesNames()
     */
    public List getSequencesNames() {
        return listNames;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IInput#getAnnotation()
     */
    public List getAnnotation() {
        return listAnnotations;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.IInput#getMetaData()
     */
    public String getMetaData() {
        // TODO Auto-generated method stub
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.proj.input.ISequenceSet#addSequence(java.lang.String,
     *      java.lang.String, java.lang.String)
     */
    public boolean addSequence(String seq, String name, String annotation) {
        boolean b1 = listSequences.add(seq);
        boolean b2 = listNames.add(name);
        boolean b3 = listAnnotations.add(annotation);
        return (b1 && b2 && b3);
    }

    public Component parentWin = null;

    public void setParentWindow(Component win) {
        this.parentWin = win;
    }

    public Iterator iterator() {
        // TODO Auto-generated method stub
        return null;
    }

    public class SeqIterator implements Iterator {
        File file;
        BufferedReader myInput;
        public SeqIterator(File file) {
            FileInputStream fin=null;
            try {
                fin = new FileInputStream(file);
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            }
            myInput = new BufferedReader(new InputStreamReader(fin));
        }

        public boolean hasNext() {
            return false;
        }

        public Object next() {
      
            return null;
        }

        public void remove() {

        }

    }
}
