/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: SwissProt.java,v 1.8 2006/04/06 11:56:09 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input.classes.swissProt;

import gr.zeus.ui.JMessage;

import java.awt.Component;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileFilter;
import java.io.FileReader;
import java.io.Serializable;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import prompt.IDescription;
import prompt.input.classes.SeqProperties;
import prompt.input.classes.swissProt.internals.Description;
import prompt.input.classes.swissProt.internals.Domain;
import prompt.input.classes.swissProt.internals.Downloader;
import prompt.input.classes.swissProt.internals.FeatureTable;
import prompt.input.classes.swissProt.internals.Keywords;
import prompt.input.classes.swissProt.internals.Organelle;
import prompt.input.classes.swissProt.internals.OrganismClassification;
import prompt.input.classes.swissProt.internals.OrganismSpecies;
import prompt.input.classes.swissProt.internals.Reference;
import prompt.input.classes.swissProt.internals.SwissProtFileReader;
import prompt.input.swissProt.ISwissProt;
import prompt.input.swissProt.ISwissProtFileReader;
import prompt.ui.DialogFactory;
import prompt.ui.SaveUI;
import tschmidt.utils.ValidLocation;

public class SwissProt extends SeqProperties implements ISwissProt,
		Serializable, IDescription {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private String filePath;

	private int format = -1;

	public static final int FORMAT_FLAT = 0;

	public static final int FORMAT_XML = 1;

	public static String formatToString(int fmt) {
		switch (fmt) {
		case FORMAT_FLAT:
			return "txt";
		case FORMAT_XML:
			return "xml";
		default:
			break;
		}
		return "unknown";
	}

	private int inputType = -1; // 0=AC; 1=folder; 2=single/multiple file

	public static final int INPUT_TYPE_ACLIST = 0;

	public static final int INPUT_TYPE_FOLDER_OF_FILES = 1;

	public static final int INPUT_TYPE_FILE = 2;

	private String description;

	/**
	 * List of Accession Codes
	 */
	private List<String> seqIds;

	private List<String> seqs;

	private List<String> seqDes;

	/**
	 * run method canceled
	 */
	private boolean canceled = false;

	// private static String lastDir;

	// Daten: werden vom SwissProtFileReader gesetzt
	private Description de;

	private Organelle og;

	private OrganismClassification oc;

	private Keywords kw;

	private FeatureTable ft;

	private OrganismSpecies os;

	private Reference rx;

	private Domain dom;

	private ISwissProtFileReader readingResult;

	public void setInputType(int type) {
		if (type < 3 && type >= 0) {
			this.inputType = type;
		} else {
			Logger logger = Logger.getLogger(this.getClass().getName());
			logger.debug("The value for the input type must be 0,1 or 2 !!!");
		}
	}

	public int getInputType() {
		return this.inputType;
	}

	public SwissProt() {
	}

	public String getDescription() {
		return description;
	}

	ArrayList<File> filesToParse = null;

	/*
	 * INIT METHODS TEMPORARILY DISABLED
	 * 
	 * public void init(ArrayList set, boolean isXML) { reader = new
	 * SwissProtFileReader(isXML,false); reader.setInput(set);
	 * //reader.setIsMultipleFile(false);
	 * if(isXML){this.setFormat("xml");}else{this.setFormat("txt");} initCalled =
	 * true; }
	 * 
	 * public void init(File multipleFile, boolean isXML) { reader = new
	 * SwissProtFileReader(isXML,false); ArrayList set = new ArrayList();
	 * set.add(multipleFile); // checken ob die �bergabe passt bei der sp�teren
	 * readFile verarbeitung reader.setInput(set);
	 * //reader.setIsMultipleFile(true); initCalled = true; }
	 * 
	 * public void init(String path) { ArrayList ACs1 = new ArrayList(); try {
	 * BufferedReader in = new BufferedReader(new FileReader(new File(path)));
	 * while (in.ready()) { ACs1.add(in.readLine()); } } catch
	 * (FileNotFoundException e) {e.printStackTrace();} catch (IOException e1)
	 * {e1.printStackTrace();} this.acNr=ACs1; this.netLoad=true; reader = new
	 * SwissProtFileReader(isXML,false); //reader.setInput(ACs1);
	 * reader.setIsMultipleFile(false); initCalled = true; }
	 * 
	 * public void init(ArrayList set) { try { String a = (String) set.get(0);
	 * netLoad = true; this.acNr = set; } catch (ClassCastException e) {
	 * 
	 * this.acNr = parseFileNames(set); File a = (File) set.get(0); String
	 * filePath = a.getAbsolutePath(); String ending =
	 * filePath.substring(filePath.length() - 3, filePath.length()); if
	 * (!(ending.equalsIgnoreCase("xml"))) { isXML = false; } } reader = new
	 * SwissProtFileReader(isXML,false); reader.setInput(set);
	 * //reader.setIsMultipleFile(false); initCalled = true; }
	 */

	public void setDescription(String string) {
		this.description = string;
	}

	public String getMetaData() {
		StringBuffer sb = new StringBuffer();
		sb.append("<html><table>");

		// Input type
		sb.append("<tr><td>input type</td><td>");
		switch (inputType) {
		case INPUT_TYPE_ACLIST:
			sb.append("list of identifiers");
			break;
		case INPUT_TYPE_FILE:
			sb.append("file (with or many entries)");
			break;
		case INPUT_TYPE_FOLDER_OF_FILES:
			sb.append("folder with multiple data files");
			break;
		default:
			sb.append("unknown");
			break;
		}
		sb.append("</td></tr>");

		// Format
		sb.append("<tr><td>format</td><td>");
		switch (format) {
		case FORMAT_FLAT:
			sb.append("flat");
			break;
		case FORMAT_XML:
			sb.append("xml");
			break;
		default:
			sb.append("unknown");
			break;
		}
		sb.append("</td></tr>");

		sb.append("<tr><td>Path</td><td>" + this.filePath + "</td></tr>");
		sb.append("</table></html>");
		JMessage.showMessageDialog(null, sb.toString(),
				JOptionPane.INFORMATION_MESSAGE, JOptionPane.DEFAULT_OPTION);
		return sb.toString();
	}

	public void run() throws Exception {
		Logger logger = Logger.getLogger(this.getClass().getName());

		// Check if format and input type is set
		if (format < 0) {
			throw new Exception("Format not set");
		}
		if (inputType < 0) {
			throw new Exception("Input not set");
		}

		// FILES OR FOLDER bekanntgeben, dann user fragen
		if (filePath == null) {
			if (inputType == INPUT_TYPE_FOLDER_OF_FILES) {
				String title = "containing ." + formatToString(format)
						+ " files";
				filePath = SaveUI.loadDir(title);
			}
			if (inputType == INPUT_TYPE_ACLIST) {
				filePath = SaveUI.load("Load text file with accessions", "txt");
			}
			if (inputType == INPUT_TYPE_FILE) {
				String suffix = "*";
				if (format == FORMAT_FLAT)
					suffix = "txt";
				if (format == FORMAT_XML)
					suffix = "xml";
				filePath = SaveUI.load("Open file", suffix);
			}
		}
		if (filePath == null) {
			String errmsg = "Canceled by user";
			logger.fatal(errmsg);
			JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE,
					JOptionPane.DEFAULT_OPTION);
			return;
		}

		// BEGINN DER VERARBEITUNG,
		SwissProtFileReader reader = null;

		if (inputType == INPUT_TYPE_ACLIST) {
			ArrayList<String> acNr = new ArrayList();
			BufferedReader in = new BufferedReader(new FileReader(new File(
					filePath)));
			while (in.ready()) {
				acNr.add(in.readLine().trim().toUpperCase());
			}

			Downloader dl = new Downloader();
			dl.parentWin = this.parentWin;
			logger.debug("Parent win not null");
			filesToParse = dl.get(acNr, true);
			if (filesToParse == null) {
				String errmsg = "no files to parse (src: download)";
				logger.fatal(errmsg);
				JMessage.showMessageDialog(null, errmsg,
						JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
				return;
			}
			reader = new SwissProtFileReader(FORMAT_XML);
			reader.setInput(filesToParse);
		}

		if (inputType == INPUT_TYPE_FOLDER_OF_FILES) {

			File directory = new File(filePath);
			File[] files = directory.listFiles(new FileFilter() {
				public boolean accept(File f) {
					if (f.getName().endsWith("." + formatToString(format))) {
						return true;
					} else {
						return false;
					}
				}
			});
			logger.debug(files.length + " files with the suffix '"
					+ formatToString(format) + "' found");
			if (files.length == 0) {
				String errmsg = "no files to parse found";
				logger.fatal(errmsg);
				JMessage.showMessageDialog(null, errmsg,
						JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
				return;
			}
			filesToParse = new ArrayList<File>();
			for (File file : files) {
				filesToParse.add(file);
			}
			if (filesToParse == null) {
				String errmsg = "no files to parse";
				logger.fatal(errmsg);
				JMessage.showMessageDialog(null, errmsg,
						JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
				return;
			}
			reader = new SwissProtFileReader(format);
			reader.setInput(filesToParse);
		}

		if (inputType == INPUT_TYPE_FILE) {
			filesToParse = new ArrayList<File>();
			File temp = new File(this.filePath);
			filesToParse.add(temp);
			reader = new SwissProtFileReader(format);
			reader.setInput(filesToParse);
		}

		// ACTUAL READING
		try {
			readingResult = reader.readFile();
			de = readingResult.getDE();
			og = readingResult.getOG();
			oc = readingResult.getOC();
			os = readingResult.getOS();
			kw = readingResult.getKW();
			rx = readingResult.getRX();
			ft = readingResult.getFT();
			dom = readingResult.getDOM();

			seqs = readingResult.getSequences();
			seqIds = readingResult.getSequencesNames();
			seqDes = readingResult.getAnnotation();

		} catch (Exception e) {

			String errmsg = "<html>File not valid/not found<br>"
					+ e.getLocalizedMessage() + "</html>";
			logger.info(errmsg, e);
			JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE,
					JOptionPane.DEFAULT_OPTION, e);

			return;
		}

		setDescription(DialogFactory.showInputDescriptionDialogFile(filePath));
	}

	public boolean setFormat(int format) {
		this.format = format;
		return true;
	}

	public boolean setFilename(String filename) {
		this.filePath = filename;
		return true;
	}

	public Description getDE() {
		return de;
	}

	public void setDE(Description de) {
		this.de = de;
	}

	public FeatureTable getFT() {
		return ft;
	}

	public void setFT(FeatureTable ft) {
		this.ft = ft;
	}

	public Keywords getKW() {
		return kw;
	}

	public Domain getDOM() {
		return dom;
	}

	public void setKW(Keywords kw) {
		this.kw = kw;
	}

	public OrganismClassification getOC() {
		return oc;
	}

	public void setOC(OrganismClassification oc) {
		this.oc = oc;
	}

	public Organelle getOG() {
		return og;
	}

	public void setOG(Organelle og) {
		this.og = og;
	}

	public OrganismSpecies getOS() {
		return os;
	}

	public void setOS(OrganismSpecies os) {
		this.os = os;
	}

	public void setDOM(Domain dom) {
		this.dom = dom;
	}

	public Reference getRX() {
		return rx;
	}

	public void setRX(Reference rx) {
		this.rx = rx;
	}

	public List getSequences() {
		/*
		 * if (sequences == null) { this.sequences =
		 * SwissProtSequenceParser.parseSequence(this.filePath); return
		 * this.sequences[0]; } else return this.sequences[0];
		 */
		return seqs;
	}

	public List getSequencesNames() {
		/*
		 * if (sequences == null) { this.sequences =
		 * SwissProtSequenceParser.parseSequence(this.filePath); return
		 * this.sequences[1]; } else return this.sequences[1];
		 */
		return seqIds;
	}

	public List getAnnotation() {
		/*
		 * if (sequences == null) { this.sequences =
		 * SwissProtSequenceParser.parseSequence(this.filePath); return
		 * this.sequences[2]; } else return this.sequences[2];
		 */
		return seqDes;
	}

	public String getClassDescription(Object sender) {
		// TODO: oberklassen durchsuchen, falls aktuelle klasse keine
		// beschreibung hat
		URL url = getClass().getResource(
				this.getClass().getSimpleName() + ".html");
		if (url == null) {
			return "No description found :-(";
		}
		return ValidLocation.readFromUrl(url);

	}

	public Component parentWin = null;

	public void setParentWindow(Component win) {
		this.parentWin = win;
	}

	public Iterator iterator() {
		return readingResult.getIterator();
	}

	/*
	 * public ArrayList getAcNr() { // TODO Auto-generated method stub return
	 * (ArrayList) getSequencesNames(); }
	 */
}
