package prompt.input.classes.swissProt.internals;

import java.awt.Component;
import java.io.File;
import java.io.FileWriter;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

public class Downloader {

	public Component parentWin = null;

	public void setParentWindow(Component win) {
		this.parentWin = win;
	}

	public  int askUser(String msg, String title) {
		Logger logger = Logger.getLogger(this.getClass().getName());
		if (parentWin != null)
			logger.debug("Parent win not null");
		else
			logger.debug("Parent win IS NULL");

		return JOptionPane.showConfirmDialog(parentWin, msg, title, JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);

	}

	public ArrayList<File> get(ArrayList<String> ids, boolean asXml) {
		Logger logger = Logger.getLogger(this.getClass().getName());

		String urlStart, urlEnd = null;
		String tmp = File.listRoots()[0] + "tmp" + File.separator + "Swissprot" + File.separator;

		File localTmp = new File(System.getProperty("java.io.tmpdir"));
		logger.debug("localTmp='" + localTmp.getAbsolutePath() + "'");
		if (!localTmp.exists()) {
			localTmp = new File(tmp);
			localTmp.mkdirs();
			tmp = localTmp.getAbsolutePath() + File.separator;
		} else {
			tmp = localTmp.getAbsolutePath() + File.separator;
		}
		logger.debug("tmp-caching dir ='" + tmp + "'");

		String ending; // filetype (xml or flat)
		if (!asXml) {
			urlStart = "http://www.expasy.org/uniprot/";
			urlEnd = ".txt";
			ending = ".txt";
			/*
			 * if(!isMultipleFile){ this.parsed = new SwissProtFlatFileReader();
			 * }else{this.parsed = new SwissProtOneFlatFileReader();}
			 */
		} else {
			urlStart = "http://www.ebi.uniprot.org/entry/";
			urlEnd = "?format=xml&ascii";
			ending = ".xml";
			/*
			 * if(!isMultipleFile){ this.parsed = new SwissProtXMLFileReader();
			 * }else{this.parsed = new SwissProtOneXmlFileReader();}
			 */
		}

		logger.info("Downloading SwissProt " + ending + " files (this could last several minutes)");
		ArrayList<File> files = new ArrayList<File>();
		logger.info("number of files: "+ids.size());
		for (int j = 0; j < ids.size(); j++) {
			String currentAc = ids.get(j);
			if (currentAc == null) {
				logger.warn("The " + j + " accession is NULL");
				continue;
			}
			currentAc = currentAc.trim();
			if (currentAc.length() == 0) {
				logger.warn("The " + j + " accession is empty");
				continue;
			}

			StringBuffer fileBuf = new StringBuffer("");
			File curFile = new File(tmp + currentAc + ending);
			if (!curFile.exists()) { // try to download
				try {

					URL sp = new URL(urlStart + currentAc + urlEnd);
					InputStream in = sp.openStream();
					byte[] cache = new byte[32768];
					int read = 0;
					read = in.read(cache);
					while (read > 0) {
						fileBuf.append(new String(cache, 0, read));
						read = in.read(cache);
					}
				} catch (Exception e) {
					String msg = "<html>An downloading error occured at Accession '" + currentAc + "'.<br>"
							+ e.getLocalizedMessage() + "<br>" + "Do you want to continue?</hmtl>";
					logger.warn(msg);
					if (askUser(msg, "Continue?") == JOptionPane.NO_OPTION)
						return null;
				}

				String buf = fileBuf.toString().toUpperCase();
				if (buf.contains("<HTML>")) {
					// HTML Error page
					String msg = "<html>It seems that accession '" + currentAc + "' was not found.<br>"
							+ "Do you want to continue?</hmtl>";
					logger.warn(msg);
					if (askUser(msg, "Continue?") == JOptionPane.NO_OPTION)
						return null;

				} else {
					try {
						FileWriter out = new FileWriter(curFile);
						out.write(fileBuf.toString());
						out.close();
					} catch (Exception e) {
						String msg = "<html>Error saving download of accession '" + currentAc + "'.<br>"
								+ e.getLocalizedMessage() + "<br>" + "Do you want to continue?</hmtl>";
						logger.warn(msg);
						if (askUser(msg, "Continue?") == JOptionPane.NO_OPTION)
							return null;
					}
				}
			} // end download block

			// existiert es jetzt nach dem versuch es runterzuladen
			if (curFile.exists())
				files.add(curFile);

		} // forall ids
		return files;
	}

}
