/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: Literature.java,v 1.2 2006/01/05 02:19:08 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input.classes.swissProt.internals;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Literature {

	public static String getArticleTitle(String id) throws IOException {
		
		// parse XML view ("<" displayed as "&lt;", ">" displayed as "&gt;") 
		String urlStart = "http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?cmd=Text&db=PubMed&uid=";
		String urlEnd = "&dopt=XML";

		// build url for specific id
		URL url = new URL(urlStart + id + urlEnd);
		// read in XML file for specific id
		InputStream in = url.openStream();
		byte[] cache = new byte[32768];
		String articleAsXML = "";
		int read = 0;
		read = in.read(cache);
		while (read > 0) {
			articleAsXML += new String(cache, 0, read);
			read = in.read(cache);
		}

		// build pattern to extract specific article title
		String match = "";
		Pattern p = Pattern.compile("^.*&lt;ArticleTitle&gt;(.*)&lt;/ArticleTitle&gt;.*$", Pattern.DOTALL);
		Matcher m = p.matcher(articleAsXML);
		if (m.matches()){
			match = m.group(1).trim();
		}		
		return match;
	}
}
