/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: SortHashtable.java,v 1.3 2006/03/02 13:23:59 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input.classes.swissProt.internals;

import java.util.Comparator;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.TreeMap;

import org.apache.log4j.Logger;


public class SortHashtable implements Comparator{
	Logger logger = Logger.getLogger(this.getClass().getName());
	
	private Hashtable toSort;
	private int compareColumn;
	private TreeMap map;
	
	/**
	 * 
	 * @param toSort
	 * @param column
	 * @param descending
	 */
	public SortHashtable(Hashtable toSort, int compareColumn){
		this.toSort = toSort;
		this.compareColumn = compareColumn;
		this.map = new TreeMap(this);
	}
		
	public TreeMap sort() {
		map.putAll(toSort);
		return map;
	}
	
	/**
	 * 
	 * @param o1
	 * @param o2
	 * @return
	 */
	public int compare(Object o1, Object o2){
		float[] array1 = (float[])(toSort.get(o1));
		float[] array2 = (float[])(toSort.get(o2));
		try {
			if (compareColumn == 0){
				// sort concerning absolute frequencies in protein set A
				return (array1[compareColumn] < array2[compareColumn]) ? 1 : (array1[compareColumn] > array2[compareColumn]) ? -1 : (array1[1] > array2[1]) ? 1 : (array1[1] < array2[1]) ? -1 : ((String) o1).compareTo((String) o2);
			}
			if (compareColumn == 1){
				// sort concerning absolute frequencies in protein set B
				return (array2[compareColumn] < array1[compareColumn]) ? -1 : (array2[compareColumn] > array1[compareColumn]) ? 1 : (array2[0] > array1[0]) ? -1 : (array2[0] < array1[0]) ? 1 : -((String) o2).compareTo((String) o1);
			}
			if (compareColumn == 2){
				// sort concerning relative frequencies in protein set A
				return (array1[compareColumn] < array2[compareColumn]) ? 1 : (array1[compareColumn] > array2[compareColumn]) ? -1 : (array1[3] > array2[3]) ? 1 : (array1[3] < array2[3]) ? -1 : ((String) o1).compareTo((String) o2);
			}
			if (compareColumn == 3){
				// sort concerning relative frequencies in protein set B
				return (array2[compareColumn] < array1[compareColumn]) ? -1 : (array2[compareColumn] > array1[compareColumn]) ? 1 : (array2[2] > array1[2]) ? -1 : (array2[2] < array1[2]) ? 1 : -((String) o2).compareTo((String) o1);
			}
			if (compareColumn == 4){
				// sort concerning absolute frequency ratios
				if (array1[compareColumn] != Float.NEGATIVE_INFINITY){
					if (array2[compareColumn] != Float.NEGATIVE_INFINITY){
						return (array1[compareColumn] > array2[compareColumn]) ? 1 : (array1[compareColumn] < array2[compareColumn]) ? -1 : (array1[0] < array2[0]) ? 1 : (array1[0] < array2[0]) ? -1 : ((String) o1).compareTo((String) o2);
					}
					else{
						return -1;
					}
				}
				else{
					if (array2[compareColumn] != Float.NEGATIVE_INFINITY){
						return 1;
					}
					else{
						return (array1[2] < array2[2]) ? 1 : (array1[2] > array2[2]) ? -1 : (array1[3] < array2[3]) ? 1 : (array1[3] > array2[3]) ? -1 : ((String) o1).compareTo((String) o2);
					}
				}			
			}
			if (compareColumn == 5){
				// sort concerning relative frequency ratios
				if (array1[compareColumn] != Float.NEGATIVE_INFINITY){
					if (array2[compareColumn] != Float.NEGATIVE_INFINITY){
						return (array1[compareColumn] > array2[compareColumn]) ? 1 : (array1[compareColumn] < array2[compareColumn]) ? -1 : (array1[0] < array2[0]) ? 1 : (array1[0] < array2[0]) ? -1 : ((String) o1).compareTo((String) o2);
					}
					else{
						return -1;
					}
				}
				else{
					if (array2[compareColumn] != Float.NEGATIVE_INFINITY){
						return 1;
					}
					else{
						return (array1[2] < array2[2]) ? 1 : (array1[2] > array2[2]) ? -1 : (array1[3] < array2[3]) ? 1 : (array1[3] > array2[3]) ? -1 : ((String) o1).compareTo((String) o2);
					}
				}			
			}
			if (compareColumn == 6){
				// sort concerning logarithmized relative frequency ratios
				if (array1[compareColumn] != Float.NEGATIVE_INFINITY){
					if (array2[compareColumn] != Float.NEGATIVE_INFINITY){
						if (array1[compareColumn] > 0){
							if (array2[compareColumn] > 0){
								return (array1[compareColumn] < array2[compareColumn]) ? 1 : (array1[compareColumn] > array2[compareColumn]) ? -1 : ((String) o1).compareTo((String) o2);
							}
							else{
								return -1;
							}
						}
						else{
							if (array2[compareColumn] > 0){
								return 1;
							}
							else{
								return (array1[compareColumn] > array2[compareColumn]) ? 1 : (array1[compareColumn] < array2[compareColumn]) ? -1 : ((String) o1).compareTo((String) o2);
							}
						}
					}
					else{
						return -1;
					}
				}
				else{
					if (array2[compareColumn] != Float.NEGATIVE_INFINITY){
						return 1;
					}
					else{
						return (array1[2] < array2[2]) ? 1 : (array1[2] > array2[2]) ? -1 : (array1[3] < array2[3]) ? 1 : (array1[3] > array2[3]) ? -1 : ((String) o1).compareTo((String) o2);						
					}
				}
			}
			else {
				// sort keys lexicographically
				return ((String) o1).compareTo((String) o2);
			}			
		}
		catch(Exception e){
			logger.debug(e);
			return 0;
		}
	}
	
	public void toString(TreeMap map){
		String key;
		float[] values;
		String data;
		
		Iterator keys = map.keySet().iterator();
		while(keys.hasNext()){
			key = keys.next().toString();
			values = (float[]) map.get(key);
			data = "";
			data += key + "\t";
			for (int i=0; i<values.length; i++){
				data += values[i] + "\t";
			}
			data += "\n";
			System.out.print(data);
		}
	}
	
	public static void main(String[] args) {
		Hashtable hash = new Hashtable();
		hash.put("eins", new float[]{0.0f, 2.0f, 0.0f, 0.0051020407f, Float.NEGATIVE_INFINITY, Float.NEGATIVE_INFINITY, Float.NEGATIVE_INFINITY});
		hash.put("zwei", new float[]{8.0f, 39.0f, 0.040816326f, 0.039f, 0.20512821f, 0.95550007f, 0.06567216f});
		hash.put("drei", new float[]{0.0f, 1.0f, 0.0f, 0.0010f, Float.NEGATIVE_INFINITY, Float.NEGATIVE_INFINITY, Float.NEGATIVE_INFINITY});
		hash.put("vier", new float[]{1.0f, 3.0f, 0.0051020407f, 0.0030f, 0.33333334f, 0.588f, 0.7661119f});
		hash.put("fünf", new float[]{4.0f, 10.0f, 0.020408163f, 0.01f, 0.4f, 0.49f, 1.0291463f});
		hash.put("sechs", new float[]{1.0f, 0.0f, 0.0051020407f, 0.0f, Float.NEGATIVE_INFINITY, Float.NEGATIVE_INFINITY, Float.NEGATIVE_INFINITY});
		hash.put("sieben", new float[]{1.0f, 7.0f, 0.0051020407f, 0.0070f, 0.14285715f, 0.72886294f, -0.45628056f});
		hash.put("acht", new float[]{1.0f, 0.0f, 0.0051020407f, 0.0f, Float.NEGATIVE_INFINITY, Float.NEGATIVE_INFINITY, Float.NEGATIVE_INFINITY});
		hash.put("neun", new float[]{0.0f, 2.0f, 0.0f, 0.0051020407f, Float.NEGATIVE_INFINITY, Float.NEGATIVE_INFINITY, Float.NEGATIVE_INFINITY});
		hash.put("zehn", new float[]{1.0f, 2.0f, 0.0051020407f, 0.0020f, 0.5f, 0.39200002f, 1.3510743f});
		hash.put("elf", new float[]{1.0f, 2.0f, 0.0051020407f, 0.0020f, 0.5f, 0.39200002f, 1.3510743f});
				
		SortHashtable x = new SortHashtable(hash, 5);
		TreeMap map = x.sort();
		x.toString(map);
	}
	
}
