/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: SwissProtXMLFileReader.java,v 1.7 2006/04/06 11:56:09 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input.classes.swissProt.internals;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.help.UnsupportedOperationException;

import org.apache.log4j.Logger;
import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.input.SAXBuilder;
import org.jdom.xpath.XPath;

import prompt.input.swissProt.ISwissProtFileReader;

public class SwissProtXMLFileReader implements ISwissProtFileReader,
		Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	Logger logger = Logger.getLogger(this.getClass().getName());

	// true, if results shall be printed to console - false, if not
	private boolean printToConsole = false;

	private Description de;

	private OrganismSpecies os;

	private Organelle og;

	private OrganismClassification oc;

	private Reference rx;

	private Keywords kw;

	private FeatureTable ft;

	private Domain dom;

	private String error = "";

	private List<String> seqIds;

	private List<String> seqs;

	private List<String> seqDes;

	public SwissProtXMLFileReader() {
		de = new Description();
		os = new OrganismSpecies();
		og = new Organelle();
		oc = new OrganismClassification();
		rx = new Reference();
		kw = new Keywords();
		ft = new FeatureTable();
		dom = new Domain();
	}

	public void parseSwissProtFileArray(ArrayList<File> proteinGroupArray)
			throws FileNotFoundException, JDOMException {
		int countWarnings = 0;
		seqIds = new LinkedList();
		seqs = new LinkedList();
		seqDes = new LinkedList();

		// parse every swissprot file contained in proteinGroupArray
		for (int i = 0; i < proteinGroupArray.size(); i++) {
			SAXBuilder builder = new SAXBuilder();
			try {
				error = (proteinGroupArray.get(i).getName());
				Document doc = builder.build(proteinGroupArray.get(i));
				Element root = doc.getRootElement();

				/*
				 * test if file contains more than one entry XPath entry =
				 * XPath.newInstance("//ns:entry"); entry.addNamespace("ns",
				 * "http://uniprot.org/uniprot"); int numberOfEntries =
				 * entry.selectNodes(root).size();
				 */

				// Namespace ns = new Namespace("ns",
				// "http://uniprot.org/uniprot");
				// logger.debug(root);
				Namespace ns = root.getNamespace();
				// logger.debug(ns);

				List entryNodes = root.getChildren("entry", ns);
				int numberOfEntries = entryNodes.size();
				int buf = 0;
				for (Object object : entryNodes) {
					buf++;

					logger.debug(buf + " " + object);
					// logger.debug(((Element) object).getNamespacePrefix());
					// logger.debug(((Element) object).getNamespaceURI());
				}

				if (numberOfEntries > 1) {
					Logger logger = Logger.getLogger(this.getClass().getName());
					logger
							.info("WARNING: multiple entries in "
									+ proteinGroupArray.get(i).getName().split(
											"\\.")[0] + " ");

					countWarnings++;
				}

				Hashtable ecHash = new Hashtable();
				Hashtable osHash = new Hashtable();
				Hashtable ogHash = new Hashtable();
				Hashtable ocHash = new Hashtable();
				Hashtable medlineHash = new Hashtable();
				Hashtable pubMedHash = new Hashtable();
				Hashtable kwHash = new Hashtable();
				Hashtable ftHash = new Hashtable();
				Hashtable domHash = new Hashtable();

				// iterate over all entries in file

				for (int j = 0; j < numberOfEntries; j++) {

					if (printToConsole) {
						logger.debug("entry " + j);
					}

					// AC
					// -----------------------------------------------------------------------------

					XPath accession = XPath.newInstance("//ns:entry[" + (j + 1)
							+ "]/ns:accession[1]");
					accession.addNamespace("ns", "http://uniprot.org/uniprot");
					Iterator acList = accession.selectNodes(root).iterator();
					int za1 = 0;
					while (acList.hasNext()) {
						String ac = ((Element) acList.next()).getValue();
						logger.debug("AC\t" + ac);
						if (za1 == 0)
							seqIds.add(ac);
						za1++;
					}

					// SEQ
					XPath seq = XPath.newInstance("//ns:entry[" + (j + 1)
							+ "]/ns:sequence[1]");
					seq.addNamespace("ns", "http://uniprot.org/uniprot");
					Iterator seqList = seq.selectNodes(root).iterator();
					int za2 = 0;
					while (seqList.hasNext()) {
						String curSeq = ((Element) seqList.next()).getValue();
						curSeq.trim().toUpperCase();
						curSeq = curSeq.replaceAll("\\W", "");
						if (printToConsole) {
							logger.debug("sequence found");
						}
						// logger.debug("SEQ\t" + curSeq);
						if (za2 == 0)
							seqs.add(curSeq);
						za2++;
					}

					// ID------------------------------------------------------------------------------

					XPath name = XPath.newInstance("//ns:entry[" + (j + 1)
							+ "]/ns:name");
					name.addNamespace("ns", "http://uniprot.org/uniprot");
					Iterator idList = name.selectNodes(root).iterator();
					StringBuffer nameBuf = new StringBuffer("");
					while (idList.hasNext()) {
						String curName = ((Element) idList.next()).getValue();
						if (printToConsole) {
							logger.debug("ID\t" + curName);
						}
						nameBuf.append(curName + " ");
					}
					seqDes.add(nameBuf.toString());

					// EC
					// -----------------------------------------------------------------------------
					XPath dbReference = XPath.newInstance("//ns:entry["
							+ (j + 1) + "]/ns:dbReference[@type=\"EC\"]/@id");
					dbReference
							.addNamespace("ns", "http://uniprot.org/uniprot");
					Iterator ecList = dbReference.selectNodes(root).iterator();
					while (ecList.hasNext()) {
						String tmpDE = ((Attribute) ecList.next()).getValue();
						// to generate unique ec numbers
						ecHash.put(tmpDE, new Integer(0));
						if (printToConsole) {
							logger.debug("EC\t" + tmpDE);
						}
					}

					// OS
					// -----------------------------------------------------------------------------
					XPath scientificOrganism = XPath.newInstance("//ns:entry["
							+ (j + 1)
							+ "]/ns:organism/ns:name[@type=\"scientific\"]");
					scientificOrganism.addNamespace("ns",
							"http://uniprot.org/uniprot");
					Iterator osList = scientificOrganism.selectNodes(root)
							.iterator();
					while (osList.hasNext()) {
						String tmpOS = ((Element) osList.next()).getValue();
						// to generate unique organism species
						osHash.put(tmpOS, new Integer(0));
						if (printToConsole) {
							logger.debug("OS\t" + tmpOS);
						}
					}

					// OG
					// -----------------------------------------------------------------------------
					XPath organelle = XPath.newInstance("//ns:entry[" + (j + 1)
							+ "]/ns:geneLocation/@type");
					organelle.addNamespace("ns", "http://uniprot.org/uniprot");
					List ogList = organelle.selectNodes(root);
					if (!(ogList.isEmpty())) {
						String tmpOG = ((Attribute) ogList.get(0)).getValue();
						// to generate unique organelles
						ogHash.put(tmpOG, new Integer(0));
						if (printToConsole) {
							logger.debug("OG\t" + tmpOG);
						}
					}

					// OC
					// -----------------------------------------------------------------------------
					XPath organismClassification = XPath
							.newInstance("//ns:entry[" + (j + 1)
									+ "]/ns:organism/ns:lineage/ns:taxon");
					organismClassification.addNamespace("ns",
							"http://uniprot.org/uniprot");
					Iterator ocList = organismClassification.selectNodes(root)
							.iterator();
					String tmpOC = "";
					while (ocList.hasNext()) {
						tmpOC = tmpOC + "."
								+ ((Element) ocList.next()).getValue();
					}
					// remove leading "."
					tmpOC = tmpOC.substring(1);
					// to generate unique organism classifications
					ocHash.put(tmpOC, new Integer(0));
					if (printToConsole) {
						logger.debug("OC\t" + tmpOC);
					}

					// RX
					// -----------------------------------------------------------------------------
					XPath medline = XPath
							.newInstance("//ns:entry["
									+ (j + 1)
									+ "]/ns:reference/descendant::ns:dbReference[@type=\"MEDLINE\"]/@id");
					medline.addNamespace("ns", "http://uniprot.org/uniprot");
					Iterator mList = medline.selectNodes(root).iterator();
					while (mList.hasNext()) {
						String tmpMedline = ((Attribute) mList.next())
								.getValue();
						// to generate unique medline identifiers
						medlineHash.put(tmpMedline, new Integer(0));
						if (printToConsole) {
							logger.debug("MEDLINE\t" + tmpMedline);
						}
					}
					XPath pubMed = XPath
							.newInstance("//ns:entry["
									+ (j + 1)
									+ "]/ns:reference/descendant::ns:dbReference[@type=\"PubMed\"]/@id");
					pubMed.addNamespace("ns", "http://uniprot.org/uniprot");
					Iterator pList = pubMed.selectNodes(root).iterator();
					while (pList.hasNext()) {
						String tmpPubMed = ((Attribute) pList.next())
								.getValue();
						// to generate unique pubmed identifiers
						pubMedHash.put(tmpPubMed, new Integer(0));
						if (printToConsole) {
							logger.debug("PubMed\t" + tmpPubMed);
						}
					}

					// KW
					// -----------------------------------------------------------------------------
					XPath keyword = XPath.newInstance("//ns:entry[" + (j + 1)
							+ "]/ns:keyword");
					keyword.addNamespace("ns", "http://uniprot.org/uniprot");
					Iterator kwList = keyword.selectNodes(root).iterator();
					while (kwList.hasNext()) {
						String tmpKW = ((Element) kwList.next()).getValue();
						// to generate unique keywords
						kwHash.put(tmpKW, new Integer(0));
						if (printToConsole) {
							logger.debug("KW\t" + tmpKW);
						}
					}

					// FT
					// -----------------------------------------------------------------------------
					XPath feature = XPath.newInstance("//ns:entry[" + (j + 1)
							+ "]/ns:feature/@type");
					feature.addNamespace("ns", "http://uniprot.org/uniprot");
					Iterator ftList = feature.selectNodes(root).iterator();
					Hashtable ftWithoutDuplicates = new Hashtable();
					while (ftList.hasNext()) {
						String tmpFT = ((Attribute) ftList.next()).getValue();
						// to generate unique feature table key names for one
						// entry
						ftWithoutDuplicates.put(tmpFT, new Integer(0));
						// to generate unique feature table key names for all
						// entries in a file
						ftHash.put(tmpFT, new Integer(0));
					}
					String[] ftArrayWithoutDuplicates = new String[ftWithoutDuplicates
							.keySet().size()];
					ftWithoutDuplicates.keySet().toArray(
							ftArrayWithoutDuplicates);
					for (int k = 0; k < ftArrayWithoutDuplicates.length; k++) {
						if (printToConsole) {
							logger.debug("FT\t" + ftArrayWithoutDuplicates[k]);
						}
					}
					// Dom------------------------------------------------------------------------------
					XPath domain = XPath
							.newInstance("//ns:entry["
									+ (j + 1)
									+ "]/ns:dbReference[@type=\"Pfam\" or @type=\"InterPro\" or @type=\"PROSITE\" or @type=\"SMART\" or @type=\"PRINTS\"]/@id");
					domain.addNamespace("ns", "http://uniprot.org/uniprot");
					Iterator domList = domain.selectNodes(root).iterator();
					Hashtable domWithoutDuplicates = new Hashtable();
					while (domList.hasNext()) {
						String tmpDOM = ((Attribute) domList.next()).getValue();
						// to generate unique feature table key names for one
						// entry
						domWithoutDuplicates.put(tmpDOM, new Integer(0)); // to
						// generate
						// unique
						// feature
						// table
						// key
						// names
						// for
						// all
						// entries in a file
						domHash.put(tmpDOM, new Integer(0));
					}
					String[] domArrayWithoutDuplicates = new String[domWithoutDuplicates
							.keySet().size()];
					domWithoutDuplicates.keySet().toArray(
							domArrayWithoutDuplicates);
					for (int k = 0; k < domArrayWithoutDuplicates.length; k++) {
						if (printToConsole) {
							logger
									.debug("DOM\t"
											+ domArrayWithoutDuplicates[k]);
						}
					}
				}
				// add information to corresponding property objects
				if (printToConsole) {
					logger
							.debug("\nFollowing information will be stored in property objects:\n");
				}
				String tmp;
				Iterator t;

				// EC
				// -----------------------------------------------------------------------------
				t = ecHash.keySet().iterator();
				while (t.hasNext()) {
					tmp = (String) t.next();
					// add ec number to Description object
					de.newDEEntry(tmp);
					if (printToConsole) {
						logger.debug("EC\t" + tmp);
					}
				}

				// OS
				// -----------------------------------------------------------------------------
				t = osHash.keySet().iterator();
				while (t.hasNext()) {
					tmp = (String) t.next();
					// add organism species to OrganismSpecies object
					os.newOSEntry(tmp);
					if (printToConsole) {
						logger.debug("OS\t" + tmp);
					}
				}

				// OG
				// -----------------------------------------------------------------------------
				t = ogHash.keySet().iterator();
				while (t.hasNext()) {
					tmp = (String) t.next();
					// add organelle to Organelle object
					og.newOGEntry(tmp);
					if (printToConsole) {
						logger.debug("OG\t" + tmp);
					}
				}

				// OC
				// -----------------------------------------------------------------------------
				t = ocHash.keySet().iterator();
				while (t.hasNext()) {
					tmp = (String) t.next();
					// add organism classification to OrganismClassification
					// object
					oc.newOCEntry(tmp);
					if (printToConsole) {
						logger.debug("OC\t" + tmp);
					}
				}

				// RX
				// -----------------------------------------------------------------------------
				t = medlineHash.keySet().iterator();
				while (t.hasNext()) {
					tmp = (String) t.next();
					// add MEDLINE id to Reference object
					rx.newMedlineEntry(tmp);
					if (printToConsole) {
						logger.debug("MEDLINE\t" + tmp);
					}
				}
				t = pubMedHash.keySet().iterator();
				while (t.hasNext()) {
					tmp = (String) t.next();
					// add PubMed id to Reference object
					rx.newPubMedEntry(tmp);
					if (printToConsole) {
						logger.debug("PubMed\t" + tmp);
					}
				}

				// KW
				// -----------------------------------------------------------------------------
				String[] kwArray = new String[kwHash.size()];
				kwHash.keySet().toArray(kwArray);
				// add keywords to Keywords object
				kw.newKWEntry(kwArray);
				if (printToConsole) {
					for (int j = 0; j < kwArray.length; j++) {
						logger.debug("KW\t" + kwArray[j]);
					}
				}

				// FT
				// -----------------------------------------------------------------------------
				String[] ftArray = new String[ftHash.size()];
				ftHash.keySet().toArray(ftArray);
				// add feature table key words to FeatureTable object
				ft.newFTEntry(ftArray);
				if (printToConsole) {
					for (int j = 0; j < ftArray.length; j++) {
						logger.debug("FT\t" + ftArray[j]);
					}
				}
				// Dom
				// -----------------------------------------------------------------------------
				String[] domArray = new String[domHash.size()];
				domHash.keySet().toArray(domArray);
				// add domain id to DomainTable object
				dom.newDomainEntry(domArray);
				if (printToConsole) {
					for (int j = 0; j < domArray.length; j++) {
						logger.debug("DOM\t" + domArray[j]);
					}
				}
				if (printToConsole) {
					logger.debug("argl");
				}
			} catch (JDOMException e) {
				// JOptionPane.showMessageDialog(null,"File format not valid, or
				// file not found! Please check your input
				// files!","Warning!",JOptionPane.OK_OPTION);
				Logger logger = Logger.getLogger(this.getClass().getName());
				logger.info("File-format WARNING:" + this.error
						+ " is not a valid XML file!!!!!!");

				logger.debug(error);
				throw (new JDOMException());

			} catch (IOException e) {
				// JOptionPane.showMessageDialog(null,"File format not valid, or
				// file not found! Please check your input
				// files!","Warning!",JOptionPane.OK_OPTION);
				throw (new FileNotFoundException());
				// logger.debug(e);
			}
		}
		Logger logger = Logger.getLogger(this.getClass().getName());

		logger.info("SUMMARY: " + countWarnings + " warning(s) reading "
				+ proteinGroupArray.size() + " file(s)\n");
	}

	private void listChildren(Element current, int depth) {
		printSpaces(depth);
		logger.debug(current.getName());
		List children = current.getChildren();
		Iterator iterator = children.iterator();
		while (iterator.hasNext()) {
			Element child = (Element) iterator.next();
			listChildren(child, depth + 1);
		}
	}

	private void printSpaces(int n) {
		for (int i = 0; i < n; i++) {
			System.out.print(" ");
		}
	}

	public Description getDE() {
		return de;
	}

	public FeatureTable getFT() {
		return ft;
	}

	public Keywords getKW() {
		return kw;
	}

	public OrganismClassification getOC() {
		return oc;
	}

	public Organelle getOG() {
		return og;
	}

	public OrganismSpecies getOS() {
		return os;
	}

	public Reference getRX() {
		return rx;
	}

	public Domain getDOM() {
		return dom;
	}

	/*
	 * public static void main(String[] args) throws Exception{ File testfile =
	 * new File("C:\\Dokumente und Einstellungen\\IDIOT\\Desktop\\P0A4D0.xml");
	 * File[] testfileArray = new File[]{testfile}; SwissProtXMLFileReader
	 * readin = new SwissProtXMLFileReader();
	 * readin.parseSwissProtFileArray(testfileArray); }
	 */

	/*
	 * public ArrayList getAC() { logger.debug("TO BE DONW ARGL ARGL FUCKING
	 * STUDENTS"); // TODO Auto-generated method stub return null; }
	 */

	public List getSequences() {
		return seqs;
	}

	public List getSequencesNames() {
		return seqIds;
	}

	public List getAnnotation() {
		return seqDes;
	}

	public ArrayList getAC() {
		Logger logger = Logger.getLogger(this.getClass().getName());
		logger.debug("damm student, is this necessary at all?");
		return null;
	}

	public Iterator getIterator() {
		throw new UnsupportedOperationException();
	}

}
