/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: IFilterId.java,v 1.3 2006/04/10 13:24:16 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.input.filter;

import java.io.IOException;
import java.util.List;

import prompt.input.IInput;



/**
 * 
 * Basis interface of all filters. 
 * 
 * @author schmidt
 */
public interface IFilterId {
    
    /**
     * Mittels dieser Methode kann dem Filter 
     * mitgeteilt werden welches Objekt er filtert
     * @param dest IInput
     */
    public void setDestination(IInput dest);
	
	public void addSingleId(String id);
	
	public void addArray(String[] ids);
	
	public void addList(List list);
	
	public void addStringList(String str);
	
	/**
	 * Looks if an identifier is in our filter set.<br>
	 * Ignores case and trims tailing blanks
	 * @param id
	 * @return true or false
	 */
	public boolean contains(String id);
	
	
	/**
	 * @param id
	 * @return true if the id passes the filter test, this means that the object's getter methods should return the entry with this id
	 * Note: should use the contains method, e.g. ignore case and
	 * tailing blanks of the identifiers
	 */
	public boolean pass(String id);
	
	/**
	 * Sets which entries should be pass the method filterPass.
	 * @param b:<br>
	 * <b>true</b>: (=default) only entries whose id is in the filter set will be returned by the getter-methods (filterPass=true)<br>
	 * <b>false</b>: all entries will be returned <b>except</b> those whose id is in the filter set 
	 */
	public void setPass(boolean b);
	
	
	public void saveToFile(String filename) throws IOException;
	
	public void loadFromFile(String filename) throws IOException;
}
