/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: Ansari.java,v 1.4 2006/03/20 00:59:16 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.statistic;

import prompt.engine.IEngineResults;
import prompt.statistic.pdef.Checkbox;
import prompt.statistic.pdef.Radiobutton;
import prompt.statistic.pdef.Spinner;
import prompt.statistic.pdef.Textarea;
import prompt.statistic.pdef.Textfield;
import tschmidt.rinterface.RException;
import tschmidt.rinterface.RParser;


public class Ansari extends Test {
  private final static long serialVersionUID = -6849794470754667700L;
  public final String testName = "ansari";

	/**
	 *
	 */
	public Ansari() {
		super();
		super.testName = testName;
	}

	/**
	 * @param data
	 * @throws Exception
	 */
	public void init(IEngineResults data) throws Exception {
		super.init(data);
		super.testName = testName;
	}

	/**
	 * @param data
	 * @param testParams
	 * @param prefix
	 * @throws Exception
	 */
	public  void init(IEngineResults data, String testParams, String prefix) throws Exception {
		super.init(data, testParams, prefix);
		super.testName = testName;
	}

	/**
	 * @param data
	 * @param string
	 * @throws Exception
	 */
	public  void init(IEngineResults data, String string) throws Exception {
		super.init(data, string);
		super.testName = testName;
	}

	protected void setParameter() {
		paramDescription =
			new Object[] {
				"Ansari",
				"Performs the Ansari-Bradley two-sample test for\n"
					+ "a difference in scale parameters",
				"Suppose that 'x' and 'y' are independent samples from distributions with\n"
					+ "densities f((t-m)/s)/s and f(t-m), respectively, where m is an unknown\n"
					+ "nuisance parameter and s, the ratio of scales, is the parameter of interest.\n"
					+ "The Ansari-Bradley test is used for testing the null that s equals 1, the\n"
					+ "two-sided alternative being that s != 1 (the distributions differ only in\n"
					+ "variance), and the one-sided alternatives being s > 1 (the distribution\n"
					+ "underlying 'x' has a larger variance, \"greater\") or s < 1 (\"less\").\n\n"
					+ "By default (if 'exact' is not specified), an exact p-value is\n"
					+ "computed if both samples contain less than 50 finite values and\n"
					+ "there are no ties.  Otherwise, a normal approximation is used.\n\n"
					+ "Optionally, a nonparametric confidence interval and an estimator\n"
					+ "for s are computed.  If exact p-values are available, an exact\n"
					+ "confidence interval is obtained by the algorithm described in\n"
					+ " Bauer (1972), and the Hodges-Lehmann estimator is employed.\n"
					+ "Otherwise, the returned confidence interval and point estimate are\n"
					+ "based on normal approximations.\n",
				new Textarea(
					"Dataselect",
					"dataselect",
					DATASELECTHELP,
					""),
				new Radiobutton(
					"Alternative",
					"alternative",
					"character string specifying the alternative hypothesis,\n"
						+ "must be one of \"two.sided\" (default), \"greater\" or \"less\".\n"
						+ "You can specify just the initial letter",
					new String[] { "Two sided", "Greater", "Less" },
					null),
				new Checkbox(
					"Exact",
					"exact",
					"a logical indicating whether an exact p-value should be computed",
					false),
				new Checkbox(
					"Confidence interval",
					"conf.int",
					" a logical,indicating whether a confidence interval should be computed",
					false),
				new Spinner(
					"Confidence level",
					"conf.level",
					"confidence level of the interval",
					new double[] { 0.95, 0.01, 1.00, 0.01 }),
				new Textfield(
					"Formula",
					"formula",
					"a formula of the form 'lhs ~ rhs' where 'lhs' is a numeric variable\n"
						+ "giving the data values and 'rhs' a factor with two levels giving the\n"
						+ "corresponding groups",
					""),
				new Textfield(
					"Data",
					"data",
					"an optional data frame containing the variables in the model formula",
					""),
				new Textfield(
					"Subset",
					"subset",
					"an optional vector specifying a subset of observations to be used",
					""),
				new Textfield(
					"NA Action",
					"na.action",
					"a function which indicates what should happen when the data contain 'NA's.\n"
						+ "Defaults to 'getOption(\"na.action\")'",
					"")
		};

	}

	public String toString() {
		return null;
	}

	public int hashCode() {
		return 0;
	}

	public void run() throws RException {
		RParser p = new RParser();
		String statement =
			"testm = as.matrix(data)\n"
				+ "ret = ansari.test("
				+ testParams
				+ ")\n";
		performTest(statement);
		output = p.parseROutput(getProcessingMessages());
	}
}
