/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: Bartlett.java,v 1.4 2006/03/20 00:59:13 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.statistic;

import prompt.engine.IEngineResults;
import prompt.statistic.pdef.Textarea;
import prompt.statistic.pdef.Textfield;
import tschmidt.rinterface.RException;
import tschmidt.rinterface.RParser;


public class Bartlett extends Test {
  private final static long serialVersionUID = -6849794470754667700L;
	public final String testName = "bartlett";

	public Bartlett() {
		super();
		super.testName = testName;
	}

	/**
	 * @param data
	 * @throws Exception
	 */
	public void init(IEngineResults data) throws Exception {
		super.init(data);
		super.testName = testName;
	}

	/**
	 * @param data
	 * @param testParams
	 * @param prefix
	 * @throws Exception
	 */
	public void init(IEngineResults data, String testParams, String prefix) throws Exception {
		super.init(data, testParams, prefix);
		super.testName = testName;
	}

	/**
	 * @param data
	 * @param string
	 * @throws Exception
	 */
	public Bartlett(IEngineResults data, String string) throws Exception {
		super.init(data, string);
		super.testName = testName;
	}

	protected void setParameter() {
		paramDescription =
			new Object[] {
				"Bartlett Test",
				" Performs Bartlett's test of the null that the\n"
					+ "variances in each of the groups (samples)\n"
					+ "are the same.",
				"If 'x' is a list, its elements are taken as the samples or fitted"
					+ "linear models to be compared for homogeneity of variances.  In"
					+ "this case, the elements must either all be numeric data vectors or"
					+ "fitted linear model objects, 'g' is ignored, and one can simply"
					+ "use 'bartlett.test(x)' to perform the test.  If the samples are"
					+ "not yet contained in a list, use 'bartlett.test(list(x, ...))'.\n"
					+ "Otherwise, 'x' must be a numeric data vector, and 'g' must be a "
					+ "vector or factor object of the same length as 'x' giving the group"
					+ "for the corresponding elements of 'x'.",
				new Textarea(
					"Dataselect",
					"dataselect",
					DATASELECTHELP,
					""),
				new Textfield(
					"Formula",
					"formula",
					"a formula of the form 'lhs ~ rhs' where 'lhs' is a numeric variable\n"
						+ "giving the data values and 'rhs' a factor with two levels giving the\n"
						+ "corresponding groups",
					""),
				new Textfield(
					"Data",
					"data",
					"an optional data frame containing the variables in the model formula",
					""),
				new Textfield(
					"Subset",
					"subset",
					"an optional vector specifying a subset of observations to be used",
					""),
				new Textfield(
					"NA Action",
					"na.action",
					"a function which indicates what should happen when the data contain 'NA's.\n"
						+ "Defaults to 'getOption(\"na.action\")'",
					"")};
	}

	public String toString() {
		return null;
	}

	public int hashCode() {
		return 0;
	}

	public void run() throws RException {
		RParser p = new RParser();
		String statement =
			"testm = as.matrix(data)\n"
				+ "ret = bartlett.test("
				+ testParams
				+ ")\n";
		performTest(statement);
		output = p.parseROutput(getProcessingMessages());
	}
}
