/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: ChiSq.java,v 1.4 2006/03/20 00:59:14 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.statistic;

import prompt.engine.IEngineResults;
import prompt.statistic.pdef.Checkbox;
import prompt.statistic.pdef.Textarea;
import prompt.statistic.pdef.Textfield;
import tschmidt.rinterface.RParser;

public class ChiSq extends Test {
	public final String testName = "chisq";

	 private final static long serialVersionUID = -6849794470754667700L;
	/**
		 *
		 */
	public ChiSq() {
		super();
		super.testName = testName;

	}

	/**
	 * @param data
	 * @throws Exception
	 */
	public void init(IEngineResults data) throws Exception {
		super.init(data);
		super.testName = testName;

	}

	/**
	 * @param data
	 * @param testParams
	 * @param prefix
	 * @throws Exception
	 */
	public void init(IEngineResults data, String testParams, String prefix) throws Exception {
		super.init(data, testParams, prefix);
		super.testName = testName;

	}

	/**
	 * @param data
	 * @param string
	 * @throws Exception
	 */
	public void init(IEngineResults data, String string) throws Exception {
		super.init(data, string);
		super.testName = testName;
			}
	/* (non-Javadoc)
	 * @see prompt.statistic.Test#setParameter()
	 */
	protected void setParameter() {
		paramDescription =
			new Object[] {
				"Pearson's Chi-squared Test",
				"'chisq.test' performs chi-squared tests\n"
					+ "on contingency tables.",
				"If 'x' is a matrix with one row or column, or if 'x' is a vector"
					+ "and 'y' is not given, 'x' is treated as a one-dimensional"
					+ "contingency table.  In this case, the hypothesis tested is whether"
					+ "the population probabilities equal those in 'p', or are all equal"
					+ "if 'p' is not given.\n"
					+ "If 'x' is a matrix with at least two rows and columns, it is taken"
					+ "as a two-dimensional contingency table, and hence its entries"
					+ "should be nonnegative integers.  Otherwise, 'x' and 'y' must be"
					+ "vectors or factors of the same length; incomplete cases are"
					+ "removed, the objects are coerced into factor objects, and the"
					+ "contingency table is computed from these.  Then, Pearson's"
					+ "chi-squared test of the null that the joint distribution of the"
					+ "cell counts in a 2-dimensional contingency table is the product of"
					+ "the row and column marginals is performed.  If 'simulate.p.value'"
					+ "is 'FALSE', the p-value is computed from the asymptotic"
					+ "chi-squared distribution of the test statistic; continuity"
					+ "correction is only used in the 2-by-2 case if 'correct' is 'TRUE'."
					+ "Otherwise, if 'simulate.p.value' is 'TRUE', the p-value is"
					+ "computed by Monte Carlo simulation with 'B' replicates.  This"
					+ "isdone by random sampling from the set of all contingency tables"
					+ "with given marginals, and works only if the marginals are"
					+ "positive.  (A C translation of the algorithm of Patefield (1981)"
					+ "is used.",
				new Textarea(
					"Dataselect",
					"dataselect",
					DATASELECTHELP,
					""),
				new Checkbox(
					"Correct",
					"correct",
					"a logical indicating whether to apply continuity correction when computing the test statistic.",
					true),
				new Textfield(
					"p",
					"p",
					"a vector of probabilities of the same length of 'x'.",
					""),
				new Checkbox(
					"Simulate p value",
					"simulate.p.value",
					"a logical indicating whether to compute p-values by Monte Carlo simulation.",
					false),
				new Textfield(
					"B",
					"B",
					"an integer specifying the number of replicates used in the Monte Carlo simulation.",
					"")};
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see de.tum.binfo.jdrm.JobTemplate#run()
	 */
	public void run() throws Exception {
		RParser p = new RParser();
		String statement =
			"testm = as.matrix(data)\n"
				+ "ret = chisq.test("
				+ testParams
				+ ")\n";
		performTest(statement);
		output = p.parseROutput(getProcessingMessages());
	}
}
