/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: Fisher.java,v 1.4 2006/03/20 00:59:14 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.statistic;

import prompt.engine.IEngineResults;
import prompt.statistic.pdef.Checkbox;
import prompt.statistic.pdef.Radiobutton;
import prompt.statistic.pdef.Spinner;
import prompt.statistic.pdef.Textarea;
import prompt.statistic.pdef.Textfield;
import tschmidt.rinterface.RParser;


public class Fisher extends Test {
	public final String testName = "fisher";
	 private final static long serialVersionUID = -6849794470754667700L;
	/**
		 *
		 */
	public Fisher() {
		super();
		super.testName = testName;
		// TODO Auto-generated constructor stub
	}

	/**
	 * @param data
	 * @throws Exception
	 */
	public void init(IEngineResults data) throws Exception {
		super.init(data);
		super.testName = testName;
			// TODO Auto-generated constructor stub
	}

	/**
	 * @param data
	 * @param testParams
	 * @param prefix
	 * @throws Exception
	 */
	public void init(IEngineResults data, String testParams, String prefix) throws Exception {
		super.init(data, testParams, prefix);
		super.testName = testName;
			// TODO Auto-generated constructor stub
	}

	/**
	 * @param data
	 * @param string
	 * @throws Exception
	 */
	public void init(IEngineResults data, String string) throws Exception {
		super.init(data, string);
		super.testName = testName;
			}
	/* (non-Javadoc)
	 * @see prompt.statistic.Test#setParameter()
	 */
	protected void setParameter() {
		paramDescription =
			new Object[] {
				"Fisher's Test",
				"Performs Fisher's exact test for testing\n"
					+ "the null of independence of rows and columns\n"
					+ "in a contingency table with fixed marginals.",
				"If 'x' is a matrix, it is taken as a two-dimensional contingency"
					+ "table, and hence its entries should be nonnegative integers."
					+ "Otherwise, both 'x' and 'y' must be vectors of the same length."
					+ "Incomplete cases are removed, the vectors are coerced into factor"
					+ "objects, and the contingency table is computed from these."
					+ "In the one-sided 2 by 2 cases, p-values are obtained directly"
					+ "using the hypergeometric distribution.  Otherwise, computations"
					+ "are based on a C version of the FORTRAN subroutine FEXACT which"
					+ "implements the network developed by Mehta and Patel (1986) and"
					+ "improved by Clarkson, Fan & Joe (1993).  The FORTRAN code can be"
					+ "obtained from <URL: http://www.netlib.org/toms/643>.  Note this"
					+ "fails (with an error message) when the entries of the table are"
					+ "too large.\n"
					+ "In the 2 by 2 case, the null of conditional independence is"
					+ "equivalent to the hypothesis that the odds ratio equals one."
					+ "Exact inference can be based on observing that in general, given"
					+ "all marginal totals fixed, the first element of the contingency"
					+ "table has a non-central hypergeometric distribution with"
					+ "non-centrality parameter given by the odds ratio (Fisher, 1935).",
				new Textarea(
					"Dataselect",
					"dataselect",
					DATASELECTHELP,
					""),
				new Textfield(
					"Workspace",
					"workspace",
					"an integer specifying the size of the workspace used in the network algorithm.",
					""),
				new Checkbox(
					"Hybrid",
					"hybrid",
					"a logical indicating whether the exact probabilities"
						+ "(default) or a hybrid approximation thereof should be"
						+ " computed. In the hybrid case, asymptotic chi-squared"
						+ "probabilities are only used provided that the \"Cochran\""
						+ "conditions are satisfied.",
					false),
				new Textfield(
					"Control",
					"control",
					"a list with named components for low level algorithm control.",
					""),
				new Textfield(
					"Or",
					"or",
					"the hypothesized odds ratio.  Only used in the 2 by 2 case.",
					""),
				new Radiobutton(
					"Alternative",
					"alternative",
					"character string specifying the alternative hypothesis,\n"
						+ "must be one of \"two.sided\" (default), \"greater\" or \"less\".\n"
						+ "You can specify just the initial letter",
					new String[] { "Two sided", "Greater", "Less" },
					null),
				new Spinner(
					"Confidence level",
					"conf.level",
					"confidence level of the interval",
					new double[] { 0.95, 0.01, 1.00, 0.01 })
		};

	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see de.tum.binfo.jdrm.JobTemplate#run()
	 */
	public void run() throws Exception {
		RParser p = new RParser();
		String statement =
			"testm = as.matrix(data)\n"
				+ "ret = fisher.test("
				+ testParams
				+ ")\n";
		performTest(statement);
		output = p.parseROutput(getProcessingMessages());

	}

}
