/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: Fligner.java,v 1.4 2006/03/20 00:59:13 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.statistic;

import prompt.engine.IEngineResults;
import prompt.statistic.pdef.Textarea;
import prompt.statistic.pdef.Textfield;
import tschmidt.rinterface.RParser;

public class Fligner extends Test {
	public final String testName = "fligner";
	 private final static long serialVersionUID = -6849794470754667700L;
	/**
		 *
		 */
	public Fligner() {
		super();
		super.testName = testName;
			// TODO Auto-generated constructor stub
	}

	/**
	 * @param data
	 * @throws Exception
	 */
	public void init(IEngineResults data) throws Exception {
		super.init(data);
		super.testName = testName;
				// TODO Auto-generated constructor stub
	}

	/**
	 * @param data
	 * @param testParams
	 * @param prefix
	 * @throws Exception
	 */
	public void init(IEngineResults data, String testParams, String prefix) throws Exception {
		super.init(data, testParams, prefix);
		super.testName = testName;
			// TODO Auto-generated constructor stub
	}

	/**
	 * @param data
	 * @param string
	 * @throws Exception
	 */
	public void init(IEngineResults data, String string) throws Exception {
		super.init(data, string);
		super.testName = testName;
			}
	/* (non-Javadoc)
	 * @see prompt.statistic.Test#setParameter()
	 */
	protected void setParameter() {
		paramDescription =
			new Object[] {
				"Fligner Test",
				"Performs a Fligner-Killeen (median) test\n"
					+ "of the null that the variances in each of the\n"
					+ "groups (samples) are the same.",
				" If 'x' is a list, its elements are taken as the samples to be"
					+ "compared for homogeneity of variances, and hence have to be"
					+ "numeric data vectors.  In this case, 'g' is ignored, and one can"
					+ "simply use 'fligner.test(x)' to perform the test.  If the samples"
					+ "are not yet contained in a list, use 'fligner.test(list(x, ...))'.\n"
					+ "Otherwise, 'x' must be a numeric data vector, and 'g' must be a"
					+ "vector or factor object of the same length as 'x' giving the group"
					+ "for the corresponding elements of 'x'.\n"
					+ "The Fligner-Killeen (median) test has been determined in a"
					+ "simulation study as one of the many tests for homogeneity of"
					+ "variances which is most robust against departures from normality,"
					+ "see Conover, Johnson & Johnson (1981).  It is a k-sample simple"
					+ "linear rank which uses the ranks of the absolute values of the"
					+ "centered samples and weights a(i) = qnorm((1 + i/(n+1))/2).  The"
					+ "version implemented here uses median centering in each of the"
					+ "samples (F-K:med X^2 in the reference).",
				new Textarea(
					"Dataselect",
					"dataselect",
					DATASELECTHELP,
					""),
				new Textfield(
					"Formula",
					"formula",
					"a formula of the form 'lhs ~ rhs' where 'lhs' is a numeric variable\n"
						+ "giving the data values and 'rhs' a factor with two levels giving the\n"
						+ "corresponding groups",
					""),
				new Textfield(
					"Data",
					"data",
					"an optional data frame containing the variables in the model formula",
					""),
				new Textfield(
					"Subset",
					"subset",
					"an optional vector specifying a subset of observations to be used",
					""),
				new Textfield(
					"NA Action",
					"na.action",
					"a function which indicates what should happen when the data contain 'NA's.\n"
						+ "Defaults to 'getOption(\"na.action\")'",
					"")};

	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see de.tum.binfo.jdrm.JobTemplate#run()
	 */
	public void run() throws Exception {
		RParser p = new RParser();
		String statement =
			"testm = as.matrix(data)\n"
				+ "ret = fligner.test("
				+ testParams
				+ ")\n";
		performTest(statement);
		output = p.parseROutput(getProcessingMessages());

	}

}
