/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: ITest.java,v 1.2 2006/01/05 02:19:15 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.statistic;

import java.util.Hashtable;

import prompt.engine.IEngineResults;
import prompt.input.IJobTemplate;



/**
 * @author Schmidt
 *
 */
public interface ITest extends IJobTemplate {
    final long serialVersionUID = -6849794470754667700L;
  
    public abstract void init(IEngineResults res) throws Exception;

    public abstract void init(IEngineResults data, String testParams)
            throws Exception;

    public abstract void init(IEngineResults data, String testParams,
            String prefix) throws Exception;

    /**
     * Returns all processing and debug messages which have been created during the last 
     * run method
     * @return String
     */
    public abstract String getProcessingMessages();

    public abstract Hashtable getTestResults();

    public abstract IEngineResults getResults();

    public abstract String toString();

    /**
     * @return an array with all the data needed to create
     * 			<code>TestCustomizer</code> and 
     * 			<code>TestDetailDisplay</code>
     */
    public abstract Object[] getParamDescription();

    /**
     * @return	the resultdata itself.
     */
    public abstract IEngineResults getData();
    
    public String getTestName();

    /**
     * @param results
     */
    public abstract void setData(IEngineResults results);
    
    /**
     * Shall we delete temporary files?
     * @param del , default = true;
     */
    public void cleanUpTmpFiles(boolean del);
    
    public abstract double getPValue();
}
