/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: KS.java,v 1.6 2006/03/20 00:59:15 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.statistic;

import java.util.Iterator;
import java.util.List;

import prompt.engine.EngineResults;
import prompt.engine.IEngineResults;
import prompt.statistic.pdef.Checkbox;
import prompt.statistic.pdef.Radiobutton;
import prompt.statistic.pdef.Textarea;
import tschmidt.rinterface.RExceptionNotEnoughData;
import tschmidt.rinterface.RParser;



public class KS extends Test {
	public final String testName = "ks";
	 private final static long serialVersionUID = -6849794470754667700L;
	/**
		 *
		 */
	public KS() {
		super();
		super.testName = testName;
			// TODO Auto-generated constructor stub
	}

	/**
	 * @param data
	 * @throws Exception
	 */
	public void init(IEngineResults data) throws Exception {
		super.init(data);
		super.testName = testName;
		// TODO Auto-generated constructor stub
	}

	/**
	 * @param data
	 * @param testParams
	 * @param prefix
	 * @throws Exception
	 */
	public void init(IEngineResults data, String testParams, String prefix) throws Exception {
		super.init(data, testParams, prefix);
		super.testName = testName;
		// TODO Auto-generated constructor stub
	}

	/**
	 * @param data
	 * @param string
	 * @throws Exception
	 */
	public void init(IEngineResults data, String string) throws Exception {
		super.init(data, string);
		super.testName = testName;
		}
	/* (non-Javadoc)
	 * @see prompt.statistic.Test#setParameter()
	 */
	protected void setParameter() {
		paramDescription =
			new Object[] {
				"Kolmogorov-Smirnov Test",
				"Performs one or two sample Kolmogorov-Smirnov tests.",
				"If 'y' is numeric, a two-sample test of the null hypothesis that"
					+ "'x' and 'y' were drawn from the same _continuous_ distribution is"
					+ "performed.\n"
					+ "Alternatively, 'y' can be a character string naming a continuous"
					+ "distribution function.  In this case, a one-sample test is carried"
					+ "out of the null that the distribution function which generated 'x'"
					+ "is distribution 'y' with parameters specified by '...'."
					+ "The presence of ties generates a warning, since continuous"
					+ "distributions do not generate them.\n"
					+ "The possible values '\"two.sided\"', '\"less\"' and '\"greater\"' of"
					+ " 'alternative' specify the null hypothesis that the true"
					+ "distribution function of 'x' is equal to, not less than or not"
					+ "greater than the hypothesized distribution function (one-sample"
					+ "case) or the distribution function of 'y' (two-sample case),"
					+ "respectively.\n"
					+ "Exact p-values are only available for the two-sided two-sample"
					+ "test with no ties.  In that case, if 'exact = NULL' (the default)"
					+ "an exact p-value is computed if the product of the sample sizes is"
					+ "less than 10000.  Otherwise, asymptotic distributions are used"
					+ "whose approximations may be inaccurate in small samples.\n"
					+ "Ifa single-sample test is used, the parameters specified in '...'"
					+ "must be pre-specified and not estimated from the data. There is"
					+ "some more refined distribution theory for the KS test with"
					+ "estimated parameters (see Durbin, 1973), but that is not"
					+ "implemented in 'ks.test'.",
				new Textarea(
					"Dataselect",
					"dataselect",
					DATASELECTHELP,
					""),
			new Radiobutton(
					"Alternative",
					"alternative",
					"character string specifying the alternative hypothesis,\n"
						+ "must be one of \"two.sided\" (default), \"greater\" or \"less\".\n"
						+ "You can specify just the initial letter",
					new String[] { "Two sided", "Greater", "Less" },
					null),
				new Checkbox(
					"Exact",
					"exact",
					"a logical indicating whether an exact p-value should be computed",
					false),
				};
	}


	/**
	 * Compares two list of Numbers
	 * @param listA: List of Number
	 * @param listB: List of Number
	 * @param mu
	 * @throws Exception
	 */
	public void init(List listA, List listB) throws Exception
	{
	  if (listA==null) {
	    throw new RExceptionNotEnoughData("listA is null");
	  }
	  if (listB==null) {
	    throw new RExceptionNotEnoughData("listB is null");
	  }
	  if (listA.size()<10) {
	    throw new RExceptionNotEnoughData("Less than 10 data values from setA");
	  }
	  if (listB.size()<10) {
	    throw new RExceptionNotEnoughData("Less than 10 data values from setB");
	  }

	  StringBuffer str = new StringBuffer();
      str.append("DATASET\tVALUE\n");
      for (Iterator iter1 = listA.iterator(); iter1.hasNext();) {
          Number element = (Number) iter1.next();
          str.append("setA\t" + element.toString() + "\n");
      }
      for (Iterator iter2 = listB.iterator(); iter2.hasNext();) {
          Number element = (Number) iter2.next();
          str.append("setB\t" + element.toString() + "\n");
      }
      IEngineResults testData = new EngineResults();

      testData.setResult(str.toString(), "setA", "setB", false, this.getClass().getName());

      //logger.debug(testData);
      init(testData,"data[data$DATASET==\"setA\",2] ,  data[data$DATASET==\"setB\",2]");
	}



	  public void init(Number[] listA, Number[] listB) throws Exception {
	        if (listA == null) {
	            throw new RExceptionNotEnoughData("listA is null");
	        }
	        if (listB == null) {
	            throw new RExceptionNotEnoughData("listB is null");
	        }
	        if (listA.length < 10) {
	            throw new RExceptionNotEnoughData("Less than 10 data values from setA");
	        }
	        if (listB.length < 10) {
	            throw new RExceptionNotEnoughData("Less than 10 data values from setB");
	        }

	        StringBuffer str = new StringBuffer();
	        str.append("DATASET\tVALUE\n");
	        for (int i = 0; i < listA.length; i++) {
	            Number element = listA[i];
	            str.append("setA\t" + element.toString() + "\n");
	        }
	        for (int i = 0; i < listB.length; i++) {
	            Number element = listB[i];
	            str.append("setB\t" + element.toString() + "\n");
	        }
	        IEngineResults testData = new EngineResults();
	        testData.setResult(str.toString(), "setA", "setB", false, this.getClass().getName());
	        init(testData,"data[data$DATASET==\"setA\",2] ,  data[data$DATASET==\"setB\",2]");


	    }


	  public void init(double[] listA, double[] listB) throws Exception {
	        if (listA == null) {
	            throw new RExceptionNotEnoughData("listA is null");
	        }
	        if (listB == null) {
	            throw new RExceptionNotEnoughData("listB is null");
	        }
	        if (listA.length < 10) {
	            throw new RExceptionNotEnoughData("Less than 10 data values from setA");
	        }
	        if (listB.length < 10) {
	            throw new RExceptionNotEnoughData("Less than 10 data values from setB");
	        }

	        StringBuffer str = new StringBuffer();
	        str.append("DATASET\tVALUE\n");
	        for (int i = 0; i < listA.length; i++) {
	            double element = listA[i];
	            str.append("setA\t" + element + "\n");
	        }
	        for (int i = 0; i < listB.length; i++) {
	            double element = listB[i];
	            str.append("setB\t" + element + "\n");
	        }
	        IEngineResults testData = new EngineResults();
	        testData.setResult(str.toString(), "setA", "setB", false, this.getClass().getName());
	        init(testData,"data[data$DATASET==\"setA\",2] ,  data[data$DATASET==\"setB\",2]");


	    }



	  public void init(String[] listA, String[] listB) throws Exception {
	        if (listA == null) {
	            throw new RExceptionNotEnoughData("listA is null");
	        }
	        if (listB == null) {
	            throw new RExceptionNotEnoughData("listB is null");
	        }
	        if (listA.length < 10) {
	            throw new RExceptionNotEnoughData("Less than 10 data values from setA");
	        }
	        if (listB.length < 10) {
	            throw new RExceptionNotEnoughData("Less than 10 data values from setB");
	        }

	        StringBuffer str = new StringBuffer();
	        str.append("DATASET\tVALUE\n");
	        for (int i = 0; i < listA.length; i++) {
	            String element = listA[i];
	            str.append("setA\t" + element + "\n");
	        }
	        for (int i = 0; i < listB.length; i++) {
	            String element = listB[i];
	            str.append("setB\t" + element + "\n");
	        }
	        IEngineResults testData = new EngineResults();
	        testData.setResult(str.toString(), "setA", "setB", false, this.getClass().getName());
	        init(testData,"data[data$DATASET==\"setA\",2] ,  data[data$DATASET==\"setB\",2]");
	    }

	  public void init(Object[] listA, Object[] listB) throws Exception {
        if (listA == null) {
            throw new RExceptionNotEnoughData("listA is null");
        }
        if (listB == null) {
            throw new RExceptionNotEnoughData("listB is null");
        }
        if (listA.length < 10) {
            throw new RExceptionNotEnoughData("Less than 10 data values from setA");
        }
        if (listB.length < 10) {
            throw new RExceptionNotEnoughData("Less than 10 data values from setB");
        }

        StringBuffer str = new StringBuffer();
        str.append("DATASET\tVALUE\n");
        for (int i = 0; i < listA.length; i++) {
            Object element = listA[i];
            str.append("setA\t" + element.toString() + "\n");
        }
        for (int i = 0; i < listB.length; i++) {
            Object element = listB[i];
            str.append("setB\t" + element.toString() + "\n");
        }
        IEngineResults testData = new EngineResults();
        testData.setResult(str.toString(), "setA", "setB", false, this.getClass().getName());
        init(testData,"data[data$DATASET==\"setA\",2] ,  data[data$DATASET==\"setB\",2]");
    }

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see de.tum.binfo.jdrm.JobTemplate#run()
	 */
	public void run() throws Exception {
		RParser p = new RParser();
		String statement =
			"testm = as.matrix(data)\n" + "ret = ks.test(" + testParams + ")\n";
		performTest(statement);
		output = p.parseROutput(getProcessingMessages());
	}
}
