/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: Kruskal.java,v 1.4 2006/03/20 00:59:32 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.statistic;

import prompt.engine.IEngineResults;
import prompt.statistic.pdef.Textarea;
import prompt.statistic.pdef.Textfield;
import tschmidt.rinterface.RParser;

public class Kruskal extends Test {
	public final String testName = "kruskal";
	 private final static long serialVersionUID = -6849794470754667700L;
	/**
		 *
		 */
	public Kruskal() {
		super();
		super.testName = testName;
			// TODO Auto-generated constructor stub
	}

	/**
	 * @param data
	 * @throws Exception
	 */
	public void init(IEngineResults data) throws Exception {
		super.init(data);
		super.testName = testName;
		// TODO Auto-generated constructor stub
	}

	/**
	 * @param data
	 * @param testParams
	 * @param prefix
	 * @throws Exception
	 */
	public void init(IEngineResults data, String testParams, String prefix) throws Exception {
		super.init(data, testParams, prefix);
		super.testName = testName;
			// TODO Auto-generated constructor stub
	}

	/**
	 * @param data
	 * @param string
	 * @throws Exception
	 */
	public void init(IEngineResults data, String string) throws Exception {
		super.init(data, string);
		super.testName = testName;
		}
	/* (non-Javadoc)
	 * @see prompt.statistic.Test#setParameter()
	 */
	protected void setParameter() {
		paramDescription =
			new Object[] {
				"Kruskal-Wallis Test",
				"Performs a Kruskal-Wallis rank sum test.",
				"'kruskal.test' performs a Kruskal-Wallis rank sum test of the null"
					+ "that the location parameters of the distribution of 'x' are the"
					+ "same in each group (sample).  The alternative is that they differ"
					+ "in at least one.\n"
					+ "If 'x' is a list, its elements are taken as the samples to be"
					+ "compared, and hence have to be numeric data vectors.  In this"
					+ "case, 'g' is ignored, and one can simply use 'kruskal.test(x)' to"
					+ "perform the test.  If the samples are not yet contained in a list,"
					+ "use 'kruskal.test(list(x, ...))'.\n"
					+ "Otherwise, 'x' must be a numeric data vector, and 'g' must be a"
					+ "vector or factor object of the same length as 'x' giving the group"
					+ "for the corresponding elements of 'x'.",
				new Textarea(
					"Dataselect",
					"dataselect",
					DATASELECTHELP,
					""),
			new Textfield(
					"g",
					"g",
					"a vector or factor object giving the group for the\n"
						+ "corresponding elements of 'x'.  Ignored if 'x' is a list.",
					""),
				new Textfield(
					"Formula",
					"formula",
					"a formula of the form 'lhs ~ rhs' where 'lhs' is a numeric variable\n"
						+ "giving the data values and 'rhs' a factor with two levels giving the\n"
						+ "corresponding groups",
					""),
				new Textfield(
					"Data",
					"data",
					"an optional data frame containing the variables in the model formula",
					""),
				new Textfield(
					"Subset",
					"subset",
					"an optional vector specifying a subset of observations to be used",
					""),
				new Textfield(
					"NA Action",
					"na.action",
					"a function which indicates what should happen when the data contain 'NA's.\n"
						+ "Defaults to 'getOption(\"na.action\")'",
					"")};
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see de.tum.binfo.jdrm.JobTemplate#run()
	 */
	public void run() throws Exception {
		RParser p = new RParser();
		String statement =
			"testm = as.matrix(data)\n"
				+ "ret = kruskal.test("
				+ testParams
				+ ")\n";
		performTest(statement);
		output = p.parseROutput(getProcessingMessages());
	}
}


