/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: MannWhitney.java,v 1.6 2006/03/20 00:59:13 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.statistic;



import java.util.Iterator;
import java.util.List;

import prompt.engine.EngineResults;
import prompt.engine.IEngineResults;
import prompt.statistic.pdef.Checkbox;
import prompt.statistic.pdef.Radiobutton;
import prompt.statistic.pdef.Spinner;
import prompt.statistic.pdef.Textarea;
import prompt.statistic.pdef.Textfield;
import tschmidt.rinterface.RException;
import tschmidt.rinterface.RExceptionNotEnoughData;
import tschmidt.rinterface.RParser;



/**
 * Mann-Whitney rank test for Non-Gaussian distributions
 * Compares two unpaired groups
 * Null Hypothesis: x and y distributions do not differ (by parameter mu [default=0])
 **/
public class MannWhitney extends Test {
	public final String testName = "wilcox";
	 private final static long serialVersionUID = -6849794470754667700L;
	/**
	 *
	 */
	public MannWhitney() {
		super();
		super.testName = testName;
		// TODO Auto-generated constructor stub
	}

	/**
	 * @param result
	 * @throws Exception
	 */
	public void init(IEngineResults result) throws Exception {
		super.init(result);
		super.testName = testName;
		// TODO Auto-generated constructor stub
	}


	/**
	 * @param data
	 * @param testParams
	 * @param prefix
	 * @throws Exception
	 */
	public void init(IEngineResults data, String testParams, String prefix) throws Exception {
		super.init(data, testParams, prefix);
		super.testName = testName;
	}


	/**
	 * Compares two list of Numbers with the MannWhitney test
	 * @param listA: List of Number
	 * @param listB: List of Number
	 * @param mu
	 * @throws Exception
	 */
	public void init(List listA, List listB, float mu) throws Exception
	{
	  if (listA==null) {
	    throw new RExceptionNotEnoughData("listA is null");
	  }
	  if (listB==null) {
	    throw new RExceptionNotEnoughData("listB is null");
	  }
	  if (listA.size()<10) {
	    throw new RExceptionNotEnoughData("Less than 10 data values from setA");
	  }
	  if (listB.size()<10) {
	    throw new RExceptionNotEnoughData("Less than 10 data values from setB");
	  }

	  StringBuffer str = new StringBuffer();
      str.append("DATASET\tVALUE\n");
      for (Iterator iter1 = listA.iterator(); iter1.hasNext();) {
          Number element = (Number) iter1.next();
          str.append("setA\t" + element.toString() + "\n");
      }
      for (Iterator iter2 = listB.iterator(); iter2.hasNext();) {
          Number element = (Number) iter2.next();
          str.append("setB\t" + element.toString() + "\n");
      }
      IEngineResults testData = new EngineResults();

      testData.setResult(str.toString(), "setA", "setB", false, this.getClass().getName());

      //logger.debug(testData);
      init(testData,"data[data$DATASET==\"setA\",2] ,  data[data$DATASET==\"setB\",2]");
	}
	  public void init(Number[] listA, Number[] listB, float mu) throws Exception {
	        if (listA == null) {
	            throw new RExceptionNotEnoughData("listA is null");
	        }
	        if (listB == null) {
	            throw new RExceptionNotEnoughData("listB is null");
	        }
	        if (listA.length < 10) {
	            throw new RExceptionNotEnoughData("Less than 10 data values from setA");
	        }
	        if (listB.length < 10) {
	            throw new RExceptionNotEnoughData("Less than 10 data values from setB");
	        }

	        StringBuffer str = new StringBuffer();
	        str.append("DATASET\tVALUE\n");
	        for (int i = 0; i < listA.length; i++) {
	            Number element = listA[i];
	            str.append("setA\t" + element.toString() + "\n");
	        }
	        for (int i = 0; i < listB.length; i++) {
	            Number element = listB[i];
	            str.append("setB\t" + element.toString() + "\n");
	        }
	        IEngineResults testData = new EngineResults();
	        testData.setResult(str.toString(), "setA", "setB", false, this.getClass().getName());
	        init(testData,"data[data$DATASET==\"setA\",2] ,  data[data$DATASET==\"setB\",2]");


	    }


	  public void init(double[] listA, double[] listB, float mu) throws Exception {
	        if (listA == null) {
	            throw new RExceptionNotEnoughData("listA is null");
	        }
	        if (listB == null) {
	            throw new RExceptionNotEnoughData("listB is null");
	        }
	        if (listA.length < 10) {
	            throw new RExceptionNotEnoughData("Less than 10 data values from setA");
	        }
	        if (listB.length < 10) {
	            throw new RExceptionNotEnoughData("Less than 10 data values from setB");
	        }

	        StringBuffer str = new StringBuffer();
	        str.append("DATASET\tVALUE\n");
	        for (int i = 0; i < listA.length; i++) {
	            double element = listA[i];
	            str.append("setA\t" + element + "\n");
	        }
	        for (int i = 0; i < listB.length; i++) {
	            double element = listB[i];
	            str.append("setB\t" + element + "\n");
	        }
	        IEngineResults testData = new EngineResults();
	        testData.setResult(str.toString(), "setA", "setB", false, this.getClass().getName());
	        init(testData,"data[data$DATASET==\"setA\",2] ,  data[data$DATASET==\"setB\",2]");


	    }



	  public void init(String[] listA, String[] listB,  float mu) throws Exception {
	        if (listA == null) {
	            throw new RExceptionNotEnoughData("listA is null");
	        }
	        if (listB == null) {
	            throw new RExceptionNotEnoughData("listB is null");
	        }
	        if (listA.length < 10) {
	            throw new RExceptionNotEnoughData("Less than 10 data values from setA");
	        }
	        if (listB.length < 10) {
	            throw new RExceptionNotEnoughData("Less than 10 data values from setB");
	        }

	        StringBuffer str = new StringBuffer();
	        str.append("DATASET\tVALUE\n");
	        for (int i = 0; i < listA.length; i++) {
	            String element = listA[i];
	            str.append("setA\t" + element + "\n");
	        }
	        for (int i = 0; i < listB.length; i++) {
	            String element = listB[i];
	            str.append("setB\t" + element + "\n");
	        }
	        IEngineResults testData = new EngineResults();
	        testData.setResult(str.toString(), "setA", "setB", false, this.getClass().getName());
	        init(testData,"data[data$DATASET==\"setA\",2] ,  data[data$DATASET==\"setB\",2]");


	    }

	  public void init(Object[] listA, Object[] listB,  float mu) throws Exception {
        if (listA == null) {
            throw new RExceptionNotEnoughData("listA is null");
        }
        if (listB == null) {
            throw new RExceptionNotEnoughData("listB is null");
        }
        if (listA.length < 10) {
            throw new RExceptionNotEnoughData("Less than 10 data values from setA");
        }
        if (listB.length < 10) {
            throw new RExceptionNotEnoughData("Less than 10 data values from setB");
        }

        StringBuffer str = new StringBuffer();
        str.append("DATASET\tVALUE\n");
        for (int i = 0; i < listA.length; i++) {
            Object element = listA[i];
            str.append("setA\t" + element.toString() + "\n");
        }
        for (int i = 0; i < listB.length; i++) {
            Object element = listB[i];
            str.append("setB\t" + element.toString() + "\n");
        }
        IEngineResults testData = new EngineResults();
        testData.setResult(str.toString(), "setA", "setB", false, this.getClass().getName());
        init(testData,"data[data$DATASET==\"setA\",2] ,  data[data$DATASET==\"setB\",2]");


    }

	/**
	 * @param data
	 * @param string
	 * @throws Exception
	 */
	public void init(IEngineResults data, String string) throws Exception {
		super.init(data, string);
		super.testName = testName;
		}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return 0;
	}

	/* (non-Javadoc)
	 * @see de.tum.binfo.jdrm.JobTemplate#run()
	 */
	public void run() throws RException {
		RParser p = new RParser();
		if (testParams==null) {
		  throw new RException("No test parameters given");
		}
		String statement =
			"testm = as.matrix(data)\n"
				+ "ret = wilcox.test("
				+ testParams
				+ ",paired=FALSE)\n";
		//logger.debug(statement);
		performTest(statement);
		output = p.parseROutput(getProcessingMessages());
	}



	/* (non-Javadoc)
	 * @see prompt.statistic.Test#setParameter()
	 */
	protected void setParameter() {
		paramDescription =
			new Object[] {
				"Mann Whitney",
				"Performs one and two sample Wilcoxon\n"
					+ "tests on vectors of data.",
				"The formula interface is only applicable for the 2-sample tests.\n"
					+ "If only 'x' is given, or if both 'x' and 'y' are given and 'paired' is 'TRUE',\n"
					+ "a Wilcoxon signed rank test of the null that the distribution of 'x' \n"
					+ "(in the one sample case) or of 'x-y' (in the paired two sample case)\n"
					+ "is symmetric about 'mu' is performed.\n\n"
					+ "Otherwise, if both 'x' and 'y' are given and 'paired' is 'FALSE', a \n"
					+ "Wilcoxon rank sum test (equivalent to the Mann-Whitney test) is carried\n"
					+ "out.  In this case, the null hypothesis is that the location of the distributions\n"
					+ "of 'x' and 'y' differ by 'mu'.\n\n"
					+ "By default (if 'exact' is not specified), an exact p-value is computed\n"
					+ "if the samples contain less than 50 finite values and there are no ties.\n"
					+ "Otherwise, a normal approximation is used.\n\n"
					+ "Optionally (if argument 'conf.int' is true), a nonparametric confidence\n"
					+ "interval and an estimator for the pseudomedian (one-sample case) or\n"
					+ "for the difference of the location parameters 'x-y' is computed. \n"
					+ "(The pseudomedian of a distribution F is the median of the distribution\n"
					+ "of (u+v)/2, where u and v are independent, each with distribution F.\n"
					+ "If F is symmetric, then the pseudomedian and median coincide.\n"
					+ "See Hollander & Wolfe (1973), page 34. If exact p-values are available,\n"
					+ "an exact confidence interval is obtained by the algorithm described in\n"
					+ " Bauer (1972), and the Hodges-Lehmann estimator is employed.\n"
					+ "Otherwise, the returned confidence interval and point estimate are based\n"
					+ "on normal approximations.",
				new Textarea(
					"Dataselect",
					"dataselect",
					DATASELECTHELP,
					""),
				new Radiobutton(
					"Alternative",
					"alternative",
					"character string specifying the alternative hypothesis,\n"
						+ "must be one of \"two.sided\" (default), \"greater\" or \"less\".\n"
						+ "You can specify just the initial letter",
					new String[] { "Two sided", "Greater", "Less" },
					null),
				new Textfield(
					"Mu",
					"mu",
					"a number specifying an optional location parameter",
					""),
				new Checkbox(
					"Paired",
					"paired",
					"a logical indicating whether you want a paired test",
					false),
				new Checkbox(
					"Exact",
					"exact",
					"a logical indicating whether an exact p-value should be computed",
					false),
				new Checkbox(
					"Correct",
					"correct",
					"a logical indicating whether to apply continuity correction in the\n"
						+ "normal approximation for the p-value",
					false),
				new Checkbox(
					"Confidence interval",
					"conf.int",
					"a logical indicating whether a confidence interval should becomputed",
					false),
				new Spinner(
					"Confidence level",
					"conf.level",
					"confidence level of the interval",
					new double[] { 0.95, 0.01, 1.00, 0.01 }),
				new Textfield(
					"Formula",
					"formula",
					"a formula of the form 'lhs ~ rhs' where 'lhs' is a numeric variable\n"
						+ "giving the data values and 'rhs' a factor with two levels giving the\n"
						+ "corresponding groups",
					""),
				new Textfield(
					"Data",
					"data",
					"an optional data frame containing the variables in the model formula",
					""),
				new Textfield(
					"Subset",
					"subset",
					"an optional vector specifying a subset of observations to be used",
					""),
				new Textfield(
					"NA Action",
					"na.action",
					"a function which indicates what should happen when the data contain 'NA's.\n"
						+ "Defaults to 'getOption(\"na.action\")'",
					"")
		};

	}

}
