/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: MantelHaen.java,v 1.4 2006/03/20 00:59:15 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.statistic;

import prompt.engine.IEngineResults;
import prompt.statistic.pdef.Checkbox;
import prompt.statistic.pdef.Radiobutton;
import prompt.statistic.pdef.Spinner;
import prompt.statistic.pdef.Textarea;
import tschmidt.rinterface.RParser;

public class MantelHaen extends Test {
	public final String testName = "mantelhaen";
	 private final static long serialVersionUID = -6849794470754667700L;
	/**
		 *
		 */
	public MantelHaen() {
		super();
		super.testName = testName;
		// TODO Auto-generated constructor stub
	}

	/**
	 * @param data
	 * @throws Exception
	 */
	public void init(IEngineResults data) throws Exception {
		super.init(data);
		super.testName = testName;
			// TODO Auto-generated constructor stub
	}

	/**
	 * @param data
	 * @param testParams
	 * @param prefix
	 * @throws Exception
	 */
	public void init(IEngineResults data, String testParams, String prefix) throws Exception {
		super.init(data, testParams, prefix);
		super.testName = testName;
			// TODO Auto-generated constructor stub
	}

	/**
	 * @param data
	 * @param string
	 * @throws Exception
	 */
	public MantelHaen(IEngineResults data, String string) throws Exception {
		super.init(data, string);
		super.testName = testName;
		}
	/* (non-Javadoc)
	 * @see prompt.statistic.Test#setParameter()
	 */
	protected void setParameter() {
		paramDescription =
			new Object[] {
				"Cochran-Mantel-Haenszel Test",
				"Performs a Cochran-Mantel-Haenszel\n"
					+ "chi-squared test of the null that two nominal\n"
					+ "variables are conditionally independent in each\n"
					+ "stratum, assuming that there is no three-way\n"
					+ "interaction.",
				"",
				new Textarea(
					"Dataselect",
					"dataselect",
					DATASELECTHELP,
					""),
				new Radiobutton(
					"Alternative",
					"alternative",
					"character string specifying the alternative hypothesis,\n"
						+ "must be one of \"two.sided\" (default), \"greater\" or \"less\".\n"
						+ "You can specify just the initial letter",
					new String[] { "Two sided", "Greater", "Less" },
					null),
				new Checkbox(
					"Correct",
					"correct",
					"a logical indicating whether to apply continuity correction\n"
						+ "when computing the test statistic. Only used in the 2 by 2 by\n"
						+ "K case.",
					false),
				new Checkbox(
					"Exact",
					"exact",
					"a logical indicating whether the Mantel-Haenszel test or the\n"
						+ "exact conditional test (given the strata margins) should be\n"
						+ "computed. Only used in the 2 by 2 by K case.",
					false),
				new Spinner(
					"Confidence level",
					"conf.level",
					"confidence level of the interval",
					new double[] { 0.95, 0.01, 1.00, 0.01 })
		};
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see de.tum.binfo.jdrm.JobTemplate#run()
	 */
	public void run() throws Exception {
		RParser p = new RParser();
		String statement =
			"testm = as.matrix(data)\n"
				+ "ret = mantelhaen.test("
				+ testParams
				+ ")\n";
		performTest(statement);
		output = p.parseROutput(getProcessingMessages());

	}

}
