/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: Mood.java,v 1.4 2006/03/20 00:59:14 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.statistic;

import prompt.engine.IEngineResults;
import prompt.statistic.pdef.Radiobutton;
import prompt.statistic.pdef.Textarea;
import prompt.statistic.pdef.Textfield;
import tschmidt.rinterface.RParser;

public class Mood extends Test {
	public final String testName = "mood";
	 private final static long serialVersionUID = -6849794470754667700L;
	/**
		 *
		 */
	public Mood() {
		super();
		super.testName = testName;
		// TODO Auto-generated constructor stub
	}

	/**
	 * @param data
	 * @throws Exception
	 */
	public void init(IEngineResults data) throws Exception {
		super.init(data);
		super.testName = testName;
		// TODO Auto-generated constructor stub
	}

	/**
	 * @param data
	 * @param testParams
	 * @param prefix
	 * @throws Exception
	 */
	public void init(IEngineResults data, String testParams, String prefix) throws Exception {
		super.init(data, testParams, prefix);
		super.testName = testName;
		// TODO Auto-generated constructor stub
	}

	/**
	 * @param data
	 * @param string
	 * @throws Exception
	 */
	public void init(IEngineResults data, String string) throws Exception {
		super.init(data, string);
		super.testName = testName;
		}
	/* (non-Javadoc)
	 * @see prompt.statistic.Test#setParameter()
	 */
	protected void setParameter() {
		paramDescription =
			new Object[] {
				"Mood Test",
				"Performs Mood's two-sample test for a\n"
					+ "difference in scale parameters.",
				"The underlying model is that the two samples are drawn from f(x-l)"
					+ "and f((x-l)/s)/s, respectively, where l is a common location"
					+ "parameter and s is a scale parameter.\n"
					+ "The null hypothesis is s = 1.\n"
					+ "There are more useful tests for this problem.",
				new Textarea(
					"Dataselect",
					"dataselect",
					DATASELECTHELP,
					""),
				new Radiobutton(
					"Alternative",
					"alternative",
					"character string specifying the alternative hypothesis,\n"
						+ "must be one of \"two.sided\" (default), \"greater\" or \"less\".\n"
						+ "You can specify just the initial letter",
					new String[] { "Two sided", "Greater", "Less" },
					null),
				new Textfield(
					"Formula",
					"formula",
					"a formula of the form 'lhs ~ rhs' where 'lhs' is a numeric variable\n"
						+ "giving the data values and 'rhs' a factor with two levels giving the\n"
						+ "corresponding groups",
					""),
				new Textfield(
					"Data",
					"data",
					"an optional data frame containing the variables in the model formula",
					""),
				new Textfield(
					"Subset",
					"subset",
					"an optional vector specifying a subset of observations to be used",
					""),
				new Textfield(
					"NA Action",
					"na.action",
					"a function which indicates what should happen when the data contain 'NA's.\n"
						+ "Defaults to 'getOption(\"na.action\")'",
					"")
		};

	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see de.tum.binfo.jdrm.JobTemplate#run()
	 */
	public void run() throws Exception {
		RParser p = new RParser();
		String statement =
			"testm = as.matrix(data)\n"
				+ "ret = mood.test("
				+ testParams
				+ ")\n";
		performTest(statement);
		output = p.parseROutput(getProcessingMessages());

	}

}
