/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: PairwiseWilcox.java,v 1.4 2006/03/20 00:59:11 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.statistic;

import prompt.engine.IEngineResults;
import prompt.statistic.pdef.Textarea;
import prompt.statistic.pdef.Textfield;
import tschmidt.rinterface.RParser;

public class PairwiseWilcox extends Test {
	public final String testName = "pairwise.wilcox";
	 private final static long serialVersionUID = -6849794470754667700L;
	/**
		 *
		 */
	public PairwiseWilcox() {
		super();
		super.testName = testName;
	}

	/**
	 * @param data
	 * @throws Exception
	 */
	public void init(IEngineResults data) throws Exception {
		super.init(data);
		super.testName = testName;
	}

	/**
	 * @param data
	 * @param testParams
	 * @param prefix
	 * @throws Exception
	 */
	public void init(
		IEngineResults data,
		String testParams,
		String prefix) throws Exception {
		super.init(data, testParams, prefix);
		super.testName = testName;
	}

	/**
	 * @param data
	 * @param string
	 * @throws Exception
	 */
	public void init(IEngineResults data, String string) throws Exception {
		super.init(data, string);
		super.testName = testName;
		}

	protected void setParameter() {
		paramDescription =
			new Object[] {
				"Pairwise Wilcox Test",
				"Calculate pairwise comparisons between\n"
					+ "group levels with corrections for multiple testing.",
				"",
				new Textarea(
					"Dataselect",
					"dataselect",
					DATASELECTHELP,
					""),
				new Textfield(
					"p adjust method",
					"p.adjust.method",
					"Method for adjusting p values (see 'p.adjust')",
					"")};

	}

	public String toString() {
		return null;
	}

	public int hashCode() {
		return 0;
	}

	public void run() throws Exception {
		RParser p = new RParser();
		String statement =
			"testm = as.matrix(data)\n"
				+ "ret = pairwise.wilcox.test("
				+ testParams
				+ ")\n";
		performTest(statement);
		output = p.parseROutput(getProcessingMessages());
	}
}
