/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: PearsonCorTest.java,v 1.5 2006/03/20 00:59:11 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.statistic;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;

import prompt.engine.EngineResults;
import prompt.engine.IEngineResults;
import prompt.statistic.pdef.Checkbox;
import prompt.statistic.pdef.Radiobutton;
import prompt.statistic.pdef.Spinner;
import prompt.statistic.pdef.Textarea;
import prompt.statistic.pdef.Textfield;
import tschmidt.rinterface.RException;
import tschmidt.rinterface.RExceptionNotEnoughData;
import tschmidt.rinterface.RParser;


/**
 * .9 to 1 very high correlation<br>
 * .7 to .9 high correlation<br>
 * .5 to .7 moderate correlation<br>
 * .3 to .5 low correlation<br>
 * .0 to .3 little if any correlation
 *
 */
public class PearsonCorTest extends Test {
	public final String testName = "cor";
	 private final static long serialVersionUID = -6849794470754667700L;
	/**
	 * @param dataArr
	 * @param desc
	 * @param testParams
	 * @throws Exception
	 */
	public PearsonCorTest(double[] dataArr, String desc, String testParams)
		throws Exception {
		data = new EngineResults();
		StringBuffer strBuff = new StringBuffer();
		this.testParams = testParams;
		for (int i = 0; i < dataArr.length; i++) {
			strBuff.append(dataArr[i] + "\n");
		}
		data.setResult(strBuff.toString(), desc, null, false,this.getClass().getName());
	}

	/**
	 *
	 */
	public PearsonCorTest() {
		super();
		super.testName = testName;
	}

	/**
	 * @param data
	 * @throws Exception
	 */
	public void init(IEngineResults data) throws Exception {
		super.init(data);
		super.testName = testName;
	}
	public void init(IEngineResults data, String testParams)
		throws Exception {
		super.init(data, testParams);
		this.testParams = testParams;
		super.testName = testName;
		}

	public void init(IEngineResults data, String testParams, String prefix)
		throws Exception {
		super.init(data,testParams,prefix);
		super.testName = testName;
		}

	/**
	 * Compares two list of Numbers with the this test
	 * @param listA: List of Number
	 * @param listB: List of Number
	 * @throws Exception
	 */
	public void init(List listA, List listB) throws Exception
	{
	  if (listA==null) {
	    throw new RExceptionNotEnoughData("listA is null");
	  }
	  if (listB==null) {
	    throw new RExceptionNotEnoughData("listB is null");
	  }
	  if (listA.size()<5) {
	    throw new RExceptionNotEnoughData("Less than 5 data values from setA");
	  }
	  if (listB.size()<5) {
	    throw new RExceptionNotEnoughData("Less than 5 data values from setB");
	  }

	  StringBuffer str = new StringBuffer();
      str.append("DATASET\tVALUE\n");
      for (Iterator iter1 = listA.iterator(); iter1.hasNext();) {
          Number element = (Number) iter1.next();
          str.append("setA\t" + element.toString() + "\n");
      }
      for (Iterator iter2 = listB.iterator(); iter2.hasNext();) {
          Number element = (Number) iter2.next();
          str.append("setB\t" + element.toString() + "\n");
      }
      IEngineResults testData = new EngineResults();

      testData.setResult(str.toString(), "setA", "setB", false, this.getClass().getName());

      //logger.debug(testData);
      init(testData,"data[data$DATASET==\"setA\",2] ,  data[data$DATASET==\"setB\",2]");
	}


	  public void init(Number[] listA, Number[] listB) throws Exception {
	        if (listA == null) {
	            throw new RExceptionNotEnoughData("listA is null");
	        }
	        if (listB == null) {
	            throw new RExceptionNotEnoughData("listB is null");
	        }
	        if (listA.length < 10) {
	            throw new RExceptionNotEnoughData("Less than 10 data values from setA");
	        }
	        if (listB.length < 10) {
	            throw new RExceptionNotEnoughData("Less than 10 data values from setB");
	        }

	        StringBuffer str = new StringBuffer();
	        str.append("DATASET\tVALUE\n");
	        for (int i = 0; i < listA.length; i++) {
	            Number element = listA[i];
	            str.append("setA\t" + element.toString() + "\n");
	        }
	        for (int i = 0; i < listB.length; i++) {
	            Number element = listB[i];
	            str.append("setB\t" + element.toString() + "\n");
	        }
	        IEngineResults testData = new EngineResults();
	        testData.setResult(str.toString(), "setA", "setB", false, this.getClass().getName());
	        init(testData,"data[data$DATASET==\"setA\",2] ,  data[data$DATASET==\"setB\",2]");


	    }



	public int hashCode() {
		return 0;
	}

	public void run() throws RException {
		RParser p = new RParser();
		String statement =
			"testm = as.matrix(data)\n"
				+ "ret = cor.test("
				+ testParams
				+ ")\n";
		performTest(statement);
		output = p.parseROutput(getProcessingMessages());
	}

	public Hashtable getTestResults() {
		return output;
	}

	public String toString() {
		return null;
	}

	protected void setParameter() {
		paramDescription =
			new Object[] {
				"Pearson Correlation Test",
				"Test for association between paired samples,\n"
					+ "using one of Pearson's product moment correlation\n"
					+ "coefficient, Kendall's tau or Spearman's rho.",
				"The three methods each estimate the association between paired\n"
					+ "samples and compute a test of the value being zero.  They use\n"
					+ "different measures of association, all in the range [-1, 1] with 0\n"
					+ "indicating no association.  These are sometimes referred to as\n"
					+ "tests of no _correlation_, but that term is often confined to the\n"
					+ "default method.\n\n"
					+ "If 'method' is \" pearson \", the test statistic is based on Pearson's\n"
					+ "product moment correlation coefficient 'cor(x, y)' and follows a t\n"
					+ "distribution with 'length(x)-2' degrees of freedom if the samples follow\n"
					+ " independent normal distributions.  If there are at least 4 complete\n"
					+ "pairs of observation, an asymptotic confidence interval is given based\n"
					+ " on Fisher's Z transform.\n\n"
					+ "If 'method' is \" kendall \" or \" spearman \", Kendall's tau or\n"
					+ "Spearman's rho statistic is used to estimate a rank-based measure\n"
					+ "of association.  These tests may be used if the data do notnecessarily\n"
					+ "come from a bivariate normal distribution. For Kendall 'stest, by\n"
					+ "default (if 'exact'is NULL), an exact p - value is computed if there\n"
					+ " are less than 50 paired samples containing finite values and there\n"
					+ " are no ties. Otherwise, the test statistic is the estimate scaled to\n"
					+ "zero mean and unit variance, and is approximately normally distributed.\n"
					+ "For Spearman 'stest, p - values are computed using algorithm\n"
					+ "AS 89.",
				new Textarea(
					"Dataselect",
					"dataselect",
					DATASELECTHELP,
					""),
				new Radiobutton(
					"Alternative",
					"alternative",
					"character string specifying the alternative hypothesis,\n"
						+ "must be one of \"two.sided\" (default), \"greater\" or \"less\".\n"
						+ "You can specify just the initial letter",
					new String[] { "Two sided", "Greater", "Less" },
					null),
				new Textfield(
					"Method",
					"method",
					"a character string indicating which correlation coefficient\n"
						+ "is to be  used for the test.  One of '\"pearson\"', '\"kendall\"',\n"
						+ " or '\"spearman\"', can be abbreviated",
					""),
				new Checkbox(
					"Exact",
					"exact",
					"a logical indicating whether an exact p-value should be computed",
					false),
				new Spinner(
					"Confidence level",
					"conf.level",
					"confidence level of the interval",
					new double[] { 0.95, 0.01, 1.00, 0.01 }),
				new Textfield(
					"Formula",
					"formula",
					"a formula of the form 'lhs ~ rhs' where 'lhs' is a numeric variable\n"
						+ "giving the data values and 'rhs' a factor with two levels giving the\n"
						+ "corresponding groups",
					""),
				new Textfield(
					"Data",
					"data",
					"an optional data frame containing the variables in the model formula",
					""),
				new Textfield(
					"Subset",
					"subset",
					"an optional vector specifying a subset of observations to be used",
					""),
				new Textfield(
					"NA Action",
					"na.action",
					"a function which indicates what should happen when the data contain 'NA's.\n"
						+ "Defaults to 'getOption(\"na.action\")'",
					"")
		};
	}
}
