/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: PowerTTest.java,v 1.4 2006/03/20 00:59:31 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.statistic;

import prompt.engine.IEngineResults;
import prompt.statistic.pdef.Checkbox;
import prompt.statistic.pdef.Radiobutton;
import prompt.statistic.pdef.Spinner;
import prompt.statistic.pdef.Textarea;
import prompt.statistic.pdef.Textfield;
import tschmidt.rinterface.RParser;


public class PowerTTest extends Test {
	public final String testName = "power.t";
	 private final static long serialVersionUID = -6849794470754667700L;
	/**
		 *
		 */
	public PowerTTest() {
		super();
		super.testName = testName;
	}

	/**
	 * @param data
	 * @throws Exception
	 */
	public void init(IEngineResults data) throws Exception {
		super.init(data);
		super.testName = testName;
	}

	/**
	 * @param data
	 * @param testParams
	 * @param prefix
	 * @throws Exception
	 */
	public void init(IEngineResults data, String testParams, String prefix) throws Exception {
		super.init(data, testParams, prefix);
		super.testName = testName;
	}

	/**
	 * @param data
	 * @param string
	 * @throws Exception
	 */
	public void init(IEngineResults data, String string) throws Exception {
		super.init(data, string);
		super.testName = testName;
			}
	protected void setParameter() {
		paramDescription =
			new Object[] {
				"Power T Test",
				"Compute power of test, or determine parameters\n"
					+ "to obtain target power.",
				"Exactly one of the parameters 'n', 'delta', 'power', 'sd', and"
					+ "'sig.level' must be passed as NULL, and that parameter is"
					+ "determined from the others. Notice that the last two have non-NULL"
					+ "defaults so NULL must be explicitly passed if you want to compute"
					+ "them.\n"
					+ "If 'strict=TRUE' is used, the power will include the probability"
					+ "of rejection in the opposite direction of the true effect, in the"
					+ "two-sided  case. Without this the power will be half the"
					+ "significance level if the  true difference is zero.",
				new Textarea(
					"Dataselect",
					"dataselect",
					DATASELECTHELP,
					""),
				new Textfield(
					"n",
					"n",
					"Number of observations (per group)",
					""),
				new Textfield("Delta", "delta", "True difference in means", ""),
				new Textfield("Sd", "sd", "Standard deviation", ""),
				new Spinner(
					"Significance level",
					"sig.level",
					"Significance level (Type I error probability)",
					new double[] { 0.95, 0.01, 1.00, 0.01 }),
				new Textfield(
					"Power",
					"power",
					"Power of test (1 minus Type II error probability)",
					""),
				new Radiobutton(
					"Type",
					"type",
					"Type of t test",
					new String[] { "Two sample", "One sample", "Paired" },
					null),
				new Radiobutton(
					"Alternative",
					"alternative",
					"character string specifying the alternative hypothesis,\n"
						+ "must be one of \"two.sided\" (default), \"greater\" or \"less\".\n"
						+ "You can specify just the initial letter",
					new String[] { "Two sided", "One sided" },
					null),
				new Checkbox(
					"Strict",
					"strict",
					"Use strict interpretation in two-sided case",
					false)
		};
	}

	public String toString() {
		return null;
	}

	public int hashCode() {
		return 0;
	}

	public void run() throws Exception {
		RParser p = new RParser();
		String statement =
			"testm = as.matrix(data)\n"
				+ "ret = power.t.test("
				+ testParams
				+ ")\n";
		performTest(statement);
		output = p.parseROutput(getProcessingMessages());
	}
}
