/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: Prop.java,v 1.4 2006/03/20 00:59:14 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.statistic;

import prompt.engine.IEngineResults;
import prompt.statistic.pdef.Checkbox;
import prompt.statistic.pdef.Radiobutton;
import prompt.statistic.pdef.Spinner;
import prompt.statistic.pdef.Textarea;
import prompt.statistic.pdef.Textfield;
import tschmidt.rinterface.RParser;

public class Prop extends Test {
	public final String testName = "prop";
	 private final static long serialVersionUID = -6849794470754667700L;
	/**
	 *
	 */
	public Prop() {
		super();
		super.testName = testName;
	}

	/**
	 * @param data
	 * @throws Exception
	 */
	public void init(IEngineResults data) throws Exception {
		super.init(data);
		super.testName = testName;
	}

	/**
	 * @param data
	 * @param testParams
	 * @param prefix
	 * @throws Exception
	 */
	public void init(IEngineResults data, String testParams, String prefix) throws Exception {
		super.init(data, testParams, prefix);
		super.testName = testName;
	}

	/**
	 * @param data
	 * @param string
	 * @throws Exception
	 */
	public void init(IEngineResults data, String string) throws Exception {
		super.init(data, string);
		super.testName = testName;
	}

	protected void setParameter() {
		paramDescription =
			new Object[] {
				"Prop Test",
				"'prop.test' can be used for testing the null\n"
					+ "that the proportions(probabilities of success)\n"
					+ "in several groups are the same, or that they\n"
					+ "equal certain given values.",
				"Only groups with finite numbers of successes and failures are"
					+ "used. Counts of successes and failures must be nonnegative and"
					+ "hence not greater than the corresponding numbers of trials which"
					+ "must be positive.  All finite counts should be integers.\n"
					+ "If 'p' is 'NULL' and there is more than one group, the null tested"
					+ "is that the proportions in each group are the same.  If there are"
					+ "two groups, the alternatives are that the probability of success"
					+ "in the first group is less than, not equal to, or greater than the"
					+ "probability of success in the second group, as specified by"
					+ "'alternative'.  A confidence interval for the difference of"
					+ "proportions with confidence level as specified by 'conf.level' and"
					+ "clipped to [-1,1] is returned.  Continuity correction is used only"
					+ "if it does not exceed the difference of the sample proportions in"
					+ "absolute value.  Otherwise, if there are more than 2 groups, the"
					+ "alternative is always '\"two.sided\"', the returned confidence"
					+ "interval is 'NULL', and continuity correction is never used.\n"
					+ "If there is only one group, then the null tested is that the"
					+ "underlying probability of success is 'p', or .5 if 'p' is not"
					+ "given.  The alternative is that the probability of success if less"
					+ "than, not equal to, or greater than 'p' or 0.5, respectively, as"
					+ "specified by 'alternative'.  A confidence interval for the"
					+ "underlying proportion with confidence level as specified by"
					+ "conf.level' and clipped to [0,1] is returned.  Continuity"
					+ "correction is used only if it does not exceed the difference"
					+ "between sample and null proportions in absolute value. The"
					+ "confidence interval  is computed by inverting the score test.\n"
					+ "Finally, if 'p' is given and there are more than 2 groups, the"
					+ "null tested is that the underlying probabilities of success are"
					+ "those given by 'p'.  The alternative is always '\"two.sided\"', the"
					+ "returned confidence interval is 'NULL', and continuity correction"
					+ "is never used.",
				new Textarea(
					"Dataselect",
					"dataselect",
					DATASELECTHELP,
					""),
				new Textfield(
					"p",
					"p",
					"a vector of probabilities of success.  The length of 'p' must\n"
						+ "be the same as the number of groups specified by 'x', and its\n"
						+ "elements must be greater than 0 and less than 1.",
					""),
				new Radiobutton(
					"Alternative",
					"alternative",
					"character string specifying the alternative hypothesis,\n"
						+ "must be one of \"two.sided\" (default), \"greater\" or \"less\".\n"
						+ "You can specify just the initial letter",
					new String[] { "Two sided", "Greater", "Less" },
					null),
				new Spinner(
					"Confidence level",
					"conf.level",
					"confidence level of the returned confidence interval.  Must\n"
						+ "be a single number between 0 and 1.  Only used when testing\n"
						+ "the null that a single proportion equals a given value, or\n"
						+ "that two proportions are equal; ignored otherwise.",
					new double[] { 0.95, 0.01, 1.00, 0.01 }),
				new Checkbox(
					"Correct",
					"correct",
					" a logical indicating whether Yates' continuity correction"
						+ "should be applied.",
					false),
				};

	}

	public String toString() {
		return null;
	}

	public int hashCode() {
		return 0;
	}

	public void run() throws Exception {
		RParser p = new RParser();
		String statement =
			"testm = as.matrix(data)\n"
				+ "ret = prop.test("
				+ testParams
				+ ")\n";
		performTest(statement);
		output = p.parseROutput(getProcessingMessages());
	}
}
