/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: Quade.java,v 1.4 2006/03/20 00:59:13 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.statistic;

import prompt.engine.IEngineResults;
import prompt.statistic.pdef.Textarea;
import prompt.statistic.pdef.Textfield;
import tschmidt.rinterface.RParser;


public class Quade extends Test {
    public final String testName = "quade";
    private final static long serialVersionUID = -6849794470754667700L;
    public Quade() {
		super();
		super.testName = testName;
	}

	/**
	 * @param data
	 * @throws Exception
	 */
	public void init(IEngineResults data) throws Exception {
		super.init(data);
		super.testName = testName;
	}

	/**
	 * @param data
	 * @param testParams
	 * @param prefix
	 * @throws Exception
	 */
	public void init(IEngineResults data, String testParams, String prefix) throws Exception {
		super.init(data, testParams, prefix);
		super.testName = testName;
	}

	/**
	 * @param data
	 * @param string
	 * @throws Exception
	 */
	public void init(IEngineResults data, String string) throws Exception {
		super.init(data, string);
		super.testName = testName;
	}

	protected void setParameter() {
		paramDescription =
			new Object[] {
				"Quade Test",
				"Performs a Quade test with unreplicated\n"
					+ "blocked data.",
				"'quade.test' can be used for analyzing unreplicated complete block"
					+ "designs (i.e., there is exactly one observation in 'y' for each"
					+ "combination of levels of 'groups' and 'blocks') where the"
					+ "normality assumption may be violated.\n"
					+ "The null hypothesis is that apart from an effect of 'blocks', the"
					+ "location parameter of 'y' is the same in each of the 'groups'.\n"
					+ "If 'y' is a matrix, 'groups' and 'blocks' are obtained from the"
					+ "column and row indices, respectively.  'NA''s are not allowed in"
					+ "'groups' or 'blocks';  if 'y' contains 'NA''s, corresponding"
					+ "blocks are removed.",
				new Textarea(
					"Dataselect",
					"dataselect",
					DATASELECTHELP,
					""),
				new Textfield(
					"Groups",
					"groups",
					"a vector giving the group for the corresponding elements of\n"
						+ "'y' if this is a vector;  ignored if 'y' is a matrix. If not\n"
						+ "a factor object, it is coerced to one.",
					""),
				new Textfield(
					"Blocks",
					"blocks",
					"a vector giving the block for the corresponding elements of\n"
						+ "'y' if this is a vector;  ignored if 'y' is a matrix. If not\n"
						+ "a factor object, it is coerced to one.",
					"")};

	}

	public String toString() {
		return null;
	}

	public int hashCode() {
		return 0;
	}

	public void run() throws Exception {
		RParser p = new RParser();
		String statement =
			"testm = as.matrix(data)\n"
				+ "ret = quade.test("
				+ testParams
				+ ")\n";
		performTest(statement);
		output = p.parseROutput(getProcessingMessages());
	}
}
