/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: ShapiroTest.java,v 1.5 2006/03/20 00:59:12 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.statistic;

import java.util.Iterator;
import java.util.List;

import prompt.engine.EngineResults;
import prompt.engine.IEngineResults;
import prompt.statistic.pdef.Radiobutton;
import prompt.statistic.pdef.Textarea;
import tschmidt.rinterface.RException;
import tschmidt.rinterface.RExceptionNotEnoughData;
import tschmidt.rinterface.RParser;



/**
 *
 * null hyp.: values are gaussian distributed
 * p<alpha significantly deny null hypo. => not normal distributed
 *
 */
public class ShapiroTest extends Test {
  private final static long serialVersionUID = -6849794470754667700L;
	public void init(double[] dataArr, String desc, String testParams)
		throws Exception {
		data = new EngineResults();
		StringBuffer strBuff = new StringBuffer();
		this.testParams = testParams;
		for (int i = 0; i < dataArr.length; i++) {
			strBuff.append(dataArr[i] + "\n");
		}
		data.setResult(strBuff.toString(), desc, null, false,this.getClass().getName());
	}

	/**
	 *
	 */
	public ShapiroTest() {
		super();
	}

	/**
	 * @param data
	 * @throws Exception
	 */
	public void init(IEngineResults data) throws Exception {
		super.init(data);
		// TODO Auto-generated constructor stub
	}

	public void init(IEngineResults data, String testParams)
		throws Exception {
		super.init(data);
		this.testParams = testParams;
	}
	public void init(IEngineResults data, String testParams, String prefix) throws Exception {
		super.init(data);
		this.testParams = testParams;
		this.prefix = prefix;
	}

	public void init(List list) throws Exception
	{
	  if (list==null) {
	    throw new RExceptionNotEnoughData("listA is null");
	  }

	  if (list.size()<3) {
	    throw new RExceptionNotEnoughData("Less than 3 data values");
	  }
	  if (list.size()>=5000) {
	    throw new RExceptionNotEnoughData("Only up to 5000 values possible. found here: "+list.size());
	  }

	  StringBuffer str = new StringBuffer();
      str.append("VALUE\n");
      for (Iterator iter1 = list.iterator(); iter1.hasNext();) {
          Number element = (Number) iter1.next();
          str.append(element.toString() + "\n");
      }

      IEngineResults testData = new EngineResults();
      testData.setResult(str.toString(), "values", "", false, this.getClass().getName());

      //logger.debug(testData);
      init(testData,"data[,1]");
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return 0;
	}

	/* (non-Javadoc)
	 * @see de.tum.binfo.jdrm.JobTemplate#run()
	 */
	public void run() throws RException {
		RParser p = new RParser();
		String statement =
			"testm = as.matrix(data)\n"
				+ "ret = shapiro.test("
				+ testParams
				+ ")\n";
		performTest(statement);
		output = p.parseROutput(getProcessingMessages());
	}

	/* (non-Javadoc)
	 * @see prompt.statistic.Test#toString()
	 */
	public String toString() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see prompt.statistic.Test#setParameter()
	 */
	protected void setParameter() {
		paramDescription =
			new Object[] {
				"Shapiro Normality Test",
				" Performs the Shapiro-Wilk test for normality.",
				"",
				new Textarea("Data select", "NA", DATASELECTHELP, ""),
				new Radiobutton(
					"Alternative",
					"alternative",
					"character string specifying the alternative hypothesis,\n"
						+ "must be one of \"two.sided\" (default), \"greater\" or \"less\".\n"
						+ "You can specify just the initial letter",
					new String[] { "Two sided", "Greater", "Less" },
					null)
		};

	}

}
