/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: TTest.java,v 1.5 2006/03/20 00:59:26 hager Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.statistic;

import java.util.Hashtable;

import prompt.engine.IEngineResults;
import prompt.statistic.pdef.Checkbox;
import prompt.statistic.pdef.Radiobutton;
import prompt.statistic.pdef.Spinner;
import prompt.statistic.pdef.Textarea;
import prompt.statistic.pdef.Textfield;
import tschmidt.rinterface.RException;
import tschmidt.rinterface.RParser;




public class TTest extends Test {
	public final String testName = "t";
	 private final static long serialVersionUID = -6849794470754667700L;
	public TTest() {
		super();
		super.testName = testName;
	}

	/**
	 * @param data
	 * @param string
	 * @throws Exception
	 */
	public void init(IEngineResults data, String string) throws Exception {
		super.init(data, string);
		super.testName = testName;
	}

	public void init(IEngineResults data, String string, String prefix) throws Exception {
		super.init(data, string, prefix);
		super.testName = testName;
	}
	/*
		public TTest(double[] dataArr, String desc, String testParams)
			throws Exception {
			data = new IEngineResults();
			StringBuffer strBuff = new StringBuffer();
			this.testParams = testParams;
			for (int i = 0; i < dataArr.length; i++) {
				strBuff.append(dataArr[i] + "\n");
			}
			data.setResult(strBuff.toString(), desc, null, false);
		}
	*/
	public int hashCode() {
		return 0;
	}

	public void run() throws RException {
        System.out.println("TTEST RUN");
 		RParser p = new RParser();
		if (super.data != null) {
			String statement =
				"testm = as.matrix(data)\n"
					+ "ret = t.test("
					+ testParams
					+ ")\n";
            System.out.println("%%%%%%%%%%%%%%%%%%%%5");
            System.out.println(statement);
 			performTest(statement);
		} else
			performTest(null);
		output = p.parseROutput(getProcessingMessages());

	}

	public Hashtable getTestResults() {
		return output;
	}
	/* (non-Javadoc)
	 * @see prompt.statistic.Test#toString()
	 */
	public String toString() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see prompt.statistic.Test#setParameter()
	 */
	protected void setParameter() {
		paramDescription =
			new Object[] {
				"T Test",
				"Performs one and two sample t-tests\n" + "on vectors of data",
				"The formula interface is only applicable for the 2-sample tests."
					+ "If 'paired' is 'TRUE' then both 'x' and 'y' must be specified and"
					+ " they must be the same length.  Missing values are removed"
					+ " (in pairs if 'paired' is 'TRUE').  If 'var.equal' is 'TRUE' then the"
					+ " pooled estimate of the variance is used.  By default, if 'var.equal'"
					+ " is 'FALSE' then the variance is estimated separately for both"
					+ "groups and the Welch modification to the degrees of freedom is"
					+ " used.",
				new Textarea(
					"Dataselect",
					"dataselect",
					DATASELECTHELP,
					""),
				new Radiobutton(
					"Alternative",
					"alternative",
					"character string specifying the alternative hypothesis,\n"
						+ "must be one of \"two.sided\" (default), \"greater\" or \"less\".\n"
						+ "You can specify just the initial letter",
					new String[] { "Two sided", "Greater", "Less" },
					"Two sided"),
				new Textfield(
					"Mu",
					"mu",
					"a number indicating the true value of the mean (or difference\n"
						+ "in means if you are performing a two sample test)",
					"0"),
				new Checkbox(
					"Paired",
					"paired",
					"a logical indicating whether you want a paired t-test",
					false),
				new Checkbox(
					"Equal variances",
					"var.equal",
					"a logical variable indicating whether to treat the two\n"
						+ "variances as being equal. If 'TRUE' then the pooled variance\n"
						+ "is used to estimate the variance otherwise the Welch (or\n"
						+ "Satterthwaite) approximation to the degrees of freedom is\n"
						+ "used",
					false),
				new Spinner(
					"Conf Level",
					"conf.level",
					"confidence level of the interval",
					new double[] { 0.95, 0.01, 1.00, 0.01 }),
				new Textfield(
					"Formula",
					"formula",
					"a formula of the form 'lhs ~ rhs' where 'lhs' is a numeric\n"
						+ "variable giving the data values and 'rhs' a factor with two\n"
						+ "levels giving the corresponding groups",
					""),
				new Textfield(
					"Data",
					"data",
					"an optional data frame containing the variables in the model\n"
						+ "formula",
					""),
				new Textfield(
					"Subset",
					"subset",
					"an optional vector specifying a subset of observations to be\n"
						+ "used",
					""),
				new Textfield(
					"NA Action",
					"na.action",
					"a function which indicates what should happen when the data\n"
						+ "contain 'NA's",
					"")
		};

	}
}
