/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: Test.java,v 1.11 2006/03/20 19:42:19 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.statistic;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;

import org.apache.log4j.Logger;

import prompt.IDescription;
import prompt.engine.EngineResults;
import prompt.engine.IEngineResults;
import tschmidt.rinterface.RException;
import tschmidt.rinterface.RPiper;
import de.tum.binfo.jdrm.JobTemplate;


public abstract class Test
	extends JobTemplate
	implements java.io.Serializable, ITest, IDescription {
    private static Logger logger = Logger.getLogger(Test.class);
    private static final long serialVersionUID = -6849794470754667700L;
    protected boolean delTmpFiles =true;
	protected String processingMessages; // the stuff R returns running RPiper
	protected Object[] paramDescription;
	protected IEngineResults data;
	public String testParams;
	public String prefix;
	protected Hashtable output = new Hashtable();
	public String testName;

	protected abstract void setParameter();
	
	protected static final String DATASELECTHELP =
		"R syntax, the Engine result objects are accessed with result and their number for example result0$AA==... ";
				
	
	public String getClassDescription(Object sender) {
	  String res = "<font color='green'><b>"+paramDescription[0]+"</b></font><br><br>";
	  res = res + paramDescription[1];
	  return res;
	}

	public double getPValue() {
	    String pstr = (String) getTestResults().get("$p.value");
	    double d =  Double.parseDouble(pstr);
	    return d;
	}
	
	   protected boolean WINDOWS;
	public Test() {
		setParameter();
        String os = System.getProperty("os.name");
    	if (os.matches(".*?Windows.*")) {
    		WINDOWS = true;
    	} else {
    	    WINDOWS = false;
    	}
	}

	public void init(IEngineResults res) throws Exception {
	    this.data = res;
	    setParameter();
		
	}

	public void init(IEngineResults data, String testParams) throws Exception {
		setParameter();
		this.data = data;
		this.testParams = testParams;
	}

	public void init(IEngineResults data, String testParams, String prefix)throws Exception  {
		setParameter();
		this.data = data;
		this.testParams = testParams;
		this.prefix = prefix;
	}

	/**
	 * Returns all processing and debug messages which have been created during the last 
	 * run method
	 * @return String
	 */
	public String getProcessingMessages() {
		return processingMessages;
	}

	 protected void performTest(String rcmds) throws RException {
		File file = null;
		String fullDataFileName = "";
		String dataReadCmd = "";
		logger.debug("performTest started");
		
		if (data != null) {
			//logger.debug("write data start");
			try {
			    if (data.getData()==null) {
			      throw new RException("EngineResult method getData does not return any data");
			    }
				file = File.createTempFile("Rdat", null);
				BufferedWriter bw = new BufferedWriter(new FileWriter(file));
				bw.write(data.getData());
				bw.close();
				dataReadCmd = file.toString();
			} catch (IOException z) {
				logger.error(
					"Can not write to file '" + file.toString() + "' ",z);
               
			}
			fullDataFileName=file.getAbsolutePath();
		//	logger.debug("write data finished");
			//if windows einfach / durch doppelte ersetzen
			
			if (WINDOWS) {
				//logger.debug("working under windows");
				fullDataFileName = fullDataFileName.replaceAll("\\\\", "/");
			}
			
			

			if (data.hasRownames()) {
				dataReadCmd =
					"data = read.table(\""
						+ fullDataFileName
						+ "\", header=TRUE, check.names=F, row.names=1, sep='\t')\n";
			} else {
				dataReadCmd =
					"data = read.table(\""
						+ fullDataFileName
						+ "\", header = TRUE, check.names=F, sep='\t')\n";
			}
		}else {
		
		dataReadCmd = prefix;
		rcmds = testParams;
		}
		if (dataReadCmd==null) {
		    throw new RException("dataReadCmd is NULL\n"+"'"+rcmds+"'\n");
		}
		
		if (WINDOWS) {
			dataReadCmd = dataReadCmd.replaceAll("\\\\", "/");
			//FIXME: stoert sich mit obriger sep=\"\\t\" angabe, work-around wie jetzt,
			//ist aber sup optimal
		}
		
		
		//logger.debug("aaa:"+dataReadCmd);
		//logger.debug("create new RPiper");
		RPiper rp = new RPiper();
		rp.trySPlus=true;
		try {
			logger.debug("start submit R code");
			this.processingMessages =
			
				rp.submitRCode(
					dataReadCmd
						+ rcmds
						+ "for(i in attributes(ret)$names) {\n"
						+ "print(\"###\")\n"
						+ "print(ret[i])\n"
						+ "}\n"
						+ "print(\"ENDTEST\")\n");
			logger.debug("finished submitting R code");
		//	logger.debug("processingMessage='"+processingMessages+"'");
		} catch (IOException e1) {
		
            logger.error("error running R code:",e1);
		}

		
		if (data != null) {
			data = null;
			try {
				if (delTmpFiles) {
				File f = new File(fullDataFileName);
			//	logger.debug("del tmp file "+f.getAbsolutePath());
//				if (f.exists()) {
//					f.delete();
//				}
			    }
			} catch (Exception e) {
				logger.warn("error removing tmp file ",e);
			}
		}
		logger.debug("performTest finished");
	}

	public Hashtable getTestResults() {
		return output;
	}
	
	public IEngineResults getResults() {
		String testResult = "RESULT\tVALUE\n";
		Enumeration enu = output.keys();
		while(enu.hasMoreElements()) {
			Object actKey = enu.nextElement();
			testResult = testResult + actKey.toString() + "\t" + output.get(actKey) + "\n";
		}
		IEngineResults results = new EngineResults();
		try {
			results.setResult(testResult, null, null, false,this.getClass().getName());
		} catch (Exception e) {
            logger.error(e);
		}
     //   logger.debug("set result description");
        results.setResultDescription("<html><b>Test results:</b><br>" +
                "This result contains the applied test method, the used parameter and" +
                "the resulted test statistics and p-value.<br>" +
                "Information about the data that was tested could be found in the meta-" +
                "descriptions. To view this information in the GUI use the " +
                "right mouse click on the result entry and choose <b>Show meta information</b>." +
                "");
		return results;
	}
	
	public String toString(){
	  return getResults().toString();
	}

	/**
	 * @return an array with all the data needed to create
	 * 			<code>TestCustomizer</code> and 
	 * 			<code>TestDetailDisplay</code>
	 */
	public Object[] getParamDescription() {
		return paramDescription;
	}

	/**
	 * @return	the resultdata itself.
	 */
	public IEngineResults getData() {
		return data;
	}

	/**
	 * @param results
	 */
	public void setData(IEngineResults results) {
		data = results;
	}

	public void cleanUpTmpFiles(boolean del)
	{
	 this.delTmpFiles = del;   
	}
	
	public String getTestName()
	{
	  return testName;
	}
	
}
