/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: Compare2Distributions.java,v 1.5 2006/01/06 23:52:38 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.statistic.ext;

import java.util.Arrays;

import org.apache.log4j.Logger;

import prompt.engine.IEngineResults;
import prompt.statistic.ITest;
import prompt.statistic.Test;
import prompt.ui.AaDialog;
import prompt.util.DynamicLoading;



/**
 * Vergleicht alle Elemente einer Verteilung von 2 Mengen.
 * Es werden dabei nur die Elemente berücksichtigt die in beiden Mengen mit der
 * Häufigkeit x vorkommen.
 */
public class Compare2Distributions extends Test {
  
  private static final long serialVersionUID = -6849794470754667700L;
	static Logger logger = Logger.getLogger(Compare2Distributions.class);
	protected String[] aminoAlphabet;
	//protected Vector pValues = new Vector();
	protected int minimumOccurrences = 10;

	public void init(IEngineResults res) throws Exception {
		super.init(res);
	}

	public static void main(String[] args) {

	}

	protected void setParameter() {
	}

//	public String toString() {
//	   logger.debug("Here");
//		StringBuffer strBuff = new StringBuffer();
//		strBuff.append("AA\t P_VALUE\n");
//
//		for (int i = 0; i < pValues.size(); i++) {
//		
//			strBuff.append(aminoAlphabet[i] + "\t");
//			strBuff.append(pValues.get(i) + "\n");
//		}
//		return strBuff.toString();
//	}

	public int hashCode() {
		return 0;
	}

	public String testname;

	public void run() throws Exception {
		if (testname == null) {
			logger.error("Bitte gewünschten Test angeben");
			return;
		}

		aminoAlphabet =
			AaDialog.getElementsFromDistributionInBoth(this.data,minimumOccurrences);
			Arrays.sort(aminoAlphabet);
		//logger.info("Testing "+aminoAlphabet.length+" elements");
		for (int i = 0; i < aminoAlphabet.length; i++) {
			String cmd =
				"data[data$AA==\""
					+ aminoAlphabet[i]
					+ "\" & data$DATASET==\"setA\",3],data[data$AA==\""
					+ aminoAlphabet[i]
					+ "\" & data$DATASET==\"setB\",3]";
			//logger.debug(data);

			ITest test = DynamicLoading.createTestInstance(testname);
			test.init(data,cmd);
			
			
			//logger.debug(test.getData());
			//logger.debug("showed data inside of the new test object");

			try {
				test.run();
			} catch (Exception e) {
				
				logger.error(e);
				return;
			}
			float f =
				(new Float((String) test.getTestResults().get("$p.value")))
					.floatValue();
			output.put(aminoAlphabet[i],new Float(f));
			if (f <= 0.05) {
				logger.info(
					aminoAlphabet[i] + " significant\t p value: " + f);
			}
			else {
				logger.info(
								aminoAlphabet[i] + " NOT significant\t p value: " + f);
			}

		}
	
	}
//	/**
//	 * @return Vector of Float() 
//	 */
//	public Vector getPValues() {
//		return pValues;
//	}

	/**
	 * @param string
	 */
	public void setTestname(String string) {
		testname = string;
	}

	/**
	 * @return s <code>String</code> containing the amino alphabeth
	 */
	public String[] getAminoAlphabet() {
		return aminoAlphabet;
	}

	/**
	 * @param i
	 */
	public void setMinimumOccurrences(int i) {
		minimumOccurrences = i;
	}

}
