/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: AaDialog.java,v 1.4 2006/03/19 11:57:54 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.ui;

import java.util.Arrays;
import java.util.Enumeration;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import prompt.engine.IEngineResults;
import tschmidt.utils.SimpleHashInteger;
import tschmidt.utils.StringSequence;


/**
 * changed to data related selection,
 * 
 */
public class AaDialog extends JFrame {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	protected String filename;

	static Logger logger = Logger.getLogger(AaDialog.class);

	public static String[] getElementsFromDistribution(IEngineResults res) {

		StringSequence seq = new StringSequence();
		SimpleHashInteger hash = new SimpleHashInteger();

		String result = res.getData();
		String[] resultLines = result.split("\n", result.length());
		for (int i = 1; i < resultLines.length; i++) {
			String[] cols = resultLines[i].split("\t");
			//logger.debug(cols[1]);		
			/*StringTokenizer tokenizer = new StringTokenizer(resultLines[i]);
			while (tokenizer.hasMoreTokens()) {
			
				line.addElement(tokenizer.nextToken());
			}*/
			if ((cols != null) && (cols.length > 1)) {
				hash.inc(cols[1]);
			}
		}
		Enumeration elements = hash.keys();
		while (elements.hasMoreElements()) {
			String element = (String) elements.nextElement();
			seq.add(element);
			//logger.debug(element);
		}
		logger.info("getElementsFromDistribution - Number " + seq.size());
		return seq.toArray();
	}

	public static String[] getElementsFromDistributionInBoth(
		IEngineResults res,
		int minimumOccurrence) {

		StringSequence seq = new StringSequence();
		SimpleHashInteger hashA = new SimpleHashInteger();
		SimpleHashInteger hashB = new SimpleHashInteger();
		int total = 0;
		String result = res.getData();
		String[] resultLines = result.split("\n", result.length());
		for (int i = 1; i < resultLines.length; i++) {
			String[] cols = resultLines[i].split("\t");

			if ((cols != null) && (cols.length > 1)) {
				if (cols[0].equals("setA")) {
					hashA.inc(cols[1]);

				}
				if (cols[0].equals("setB")) {
					hashB.inc(cols[1]);

				}
			}
		}

		Enumeration elements = hashA.keys();
		while (elements.hasMoreElements()) {
			String element = (String) elements.nextElement();
			total++;
			if (hashA.get(element) >= minimumOccurrence) {
				if ((hashB.containsKey(element))
					&& (hashB.get(element) >= minimumOccurrence)) {
					seq.add(element);
				}
			}
			//logger.debug(element);
		}
		logger.info(
			"getElementsFromDistribution - #different elements in setA: "
				+ total);

		logger.info(
			"getElementsFromDistribution - Occurences in both more than "
				+ minimumOccurrence
				+ " times: "
				+ seq.size());
		return seq.toArray();
	}

	public String showAaDialog(Object[] possibilities) {
		Arrays.sort(possibilities);
		String s =
			(String) JOptionPane.showInputDialog(
				this,
				"Select an element to plot",
				"Elements",
				JOptionPane.QUESTION_MESSAGE,
				null,
				possibilities,
				"A");
			return s;
	}

	public int showSelectSetDialog(String str1, String str2) {
		int ret = 0;
		String s =
			(String) JOptionPane.showInputDialog(
				this,
				"Select the set which you want to be used",
				"Data descriptions",
				JOptionPane.QUESTION_MESSAGE,
				null,
				new String[] { str1, str2 },
				str1);
		if (s != null) {
			if (s.equals(str1)) {
				ret = 1;
			}
			if (s.equals(str2)) {
				ret = 2;
			}
			logger.debug(s + " " + ret);
		}
		return ret;
	}
}
