/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: CheckBoxDlg.java,v 1.7 2006/03/19 11:57:55 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.ui;

import java.awt.Component;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListCellRenderer;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

import org.apache.log4j.Logger;



public class CheckBoxDlg extends JDialog {
    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	Logger logger = Logger.getLogger(this.getClass());
    private JPanel jContentPane = null;
    JList jList = null;
    private JButton jButtonCancel = null;
    private JButton jButtonOk = null;
    private JLabel jLabel = null;
    
    boolean canceled = true;
    
    protected Data[] options;
    /**
     * This method initializes jList	
     * 	
     * @return javax.swing.JList	
     */
    private JList getJList() {
        if (jList == null) {
            jList = new JList(options);
            jList.setAutoscrolls(true);
            
            CheckListCellRenderer renderer = new CheckListCellRenderer();
            jList.setCellRenderer(renderer);
            jList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

            CheckListener lst = new CheckListener(this);
            jList.addMouseListener(lst);
            jList.addKeyListener(lst);
            
            
        }
        return jList;
    }

    /**
     * This method initializes jButton	
     * 	
     * @return javax.swing.JButton	
     */
    private JButton getJButton() {
        if (jButtonCancel == null) {
            jButtonCancel = new JButton();
            jButtonCancel.setBounds(new java.awt.Rectangle(239,191,78,25));
            jButtonCancel.setText("Cancel");
            jButtonCancel.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    //logger.debug("cancel()"); 
                    dispose();
                }
            });
        }
        return jButtonCancel;
    }

    
    
    
    /**
     * This method initializes jButton1	
     * 	
     * @return javax.swing.JButton	
     */
    private JButton getJButton1() {
        if (jButtonOk == null) {
            jButtonOk = new JButton();
            jButtonOk.setBounds(new java.awt.Rectangle(330,190,77,25));
            jButtonOk.setText("Ok");
            jButtonOk.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    //logger.debug("ok()"); 
                    canceled=false;
                    dispose();
                }
            });
        }
        return jButtonOk;
    }

    protected boolean[] selected;
    private JScrollPane jScrollPane = null;

    protected void recalcTotal()
    {
        ListModel model = jList.getModel();
        for (int k=0; k < model.getSize(); k++)
        {
            Data data = (Data)model.getElementAt(k);
            if (data==null) continue;
            if (data.isSelected()) selected[k]=true; else selected[k]=false;
               
        }
    }
    
   public static boolean[] showDlg(String[] list) {
       CheckBoxDlg self = new CheckBoxDlg(list,"Property Chooser","Choose the properties that should be used by your method");
    
       self.setModal(true);
       self.setVisible(true);
       if (self.canceled==true)  return null;

       return self.selected;
   }
    
   public static boolean[] showDlg(String[] list, String title, String header) {
       CheckBoxDlg self = new CheckBoxDlg(list,title,header);
    
       self.setModal(true);
       self.setVisible(true);
       if (self.canceled==true)  return null;

       return self.selected;
   }
   
    /**
 * This method initializes jScrollPane	
 * 	
 * @return javax.swing.JScrollPane	
 */
private JScrollPane getJScrollPane() {
    if (jScrollPane == null) {
        jScrollPane = new JScrollPane();
        jScrollPane.setBounds(new java.awt.Rectangle(15,36,389,145));
        jScrollPane.setViewportView(getJList());
    }
    return jScrollPane;
}

    /**
     * @param args
     */
    public static void main(String[] args) {
        Logger logger = Logger.getLogger(CheckBoxDlg.class);
        // TODO Auto-generated method stub
        boolean[] res = showDlg(new String[]{"aa","bb","cc","j","jj","1","3","3","5","88","hakdf","a","3d" } );
        if (res==null)  logger.debug("canceld"); else {
            for (int i = 0; i < res.length; i++) {
                logger.debug(res[i]);
            }
            
        }
    }

    
    String title;
    String header;
    
    /**
     * This is the default constructor
     */
    protected CheckBoxDlg(String[] data, String titlep, String headerp) {
        super();
        if (titlep==null) this.title=""; else title=titlep;
        if (headerp==null) this.header=""; else header=headerp;
        
        
        options = new Data[data.length];
        selected = new boolean[data.length];
        for (int i = 0; i < options.length; i++) {
           options[i] = new Data(data[i]);
        }
        initialize();
    }

    /**
     * This method initializes this
     * 
     * @return void
     */
    private void initialize() {
        this.setSize(426, 257);
        this.setTitle(title);
        this.setContentPane(getJContentPane());
    }

    /**
     * This method initializes jContentPane
     * 
     * @return javax.swing.JPanel
     */
    private JPanel getJContentPane() {
        if (jContentPane == null) {
            jLabel = new JLabel();
            jLabel.setBounds(new java.awt.Rectangle(15,6,389,18));
            jLabel.setText(header);
            jContentPane = new JPanel();
            jContentPane.setLayout(null);
            jContentPane.add(getJButton(), null);
            jContentPane.add(jLabel, null);
            jContentPane.add(getJButton1(), null);
            jContentPane.add(getJScrollPane(), null);
        }
        return jContentPane;
    }

    
   
    
    
}  //  @jve:decl-index=0:visual-constraint="10,10"


class CheckListCellRenderer extends JCheckBox implements ListCellRenderer
{
    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	protected static Border m_noFocusBorder = 
        new EmptyBorder(1, 1, 1, 1);

    public CheckListCellRenderer()
    {
        super();
        setOpaque(true);
        setBorder(m_noFocusBorder);
    }

    public Component getListCellRendererComponent(JList list,
        Object value, int index, boolean isSelected, boolean cellHasFocus)
    {
        setText(value.toString());

        setBackground(isSelected ? list.getSelectionBackground() : 
            list.getBackground());
        setForeground(isSelected ? list.getSelectionForeground() : 
            list.getForeground());

        Data data = (Data)value;
        setSelected(data.isSelected());

        setFont(list.getFont());
        setBorder((cellHasFocus) ? 
            UIManager.getBorder("List.focusCellHighlightBorder")
             : m_noFocusBorder);

        return this;
    }
}

class CheckListener implements MouseListener, KeyListener
{
    protected CheckBoxDlg m_parent;
    protected JList m_list;

    public CheckListener(CheckBoxDlg dlg)
    {
        m_parent = dlg;
        m_list = dlg.jList;
    }

    public void mouseClicked(MouseEvent e)
    {
        if (e.getX() < 20)
            doCheck();
    }

    public void mousePressed(MouseEvent e) {}

    public void mouseReleased(MouseEvent e) {}

    public void mouseEntered(MouseEvent e) {}

    public void mouseExited(MouseEvent e) {}

    public void keyPressed(KeyEvent e)
    {
        if (e.getKeyChar() == ' ')
            doCheck();
        if (e.getKeyCode()==(KeyEvent.VK_CONTROL)) {
        //	System.out.println("invert selection");
        	ListModel list = m_list.getModel();
        	for (int i = 0; i < list.getSize(); i++) {
        		Data data = (Data) list.getElementAt(i);
        		   data.invertSelected();
				
			}

         m_list.repaint();
         m_parent.recalcTotal();
        }
        
    }

    public void keyTyped(KeyEvent e) {}
    public void keyReleased(KeyEvent e) {}

    protected void doCheck()
    {
        int index = m_list.getSelectedIndex();
        if (index < 0)
            return;
        Data data = (Data)m_list.getModel().
            getElementAt(index);
        data.invertSelected();
        m_list.repaint();
        m_parent.recalcTotal();
    }
}

class Data
{
    protected String m_name;
    
    protected boolean m_selected;

    public Data(String name)
    {
    m_name = name;
   
    m_selected = false;
    }

    public String getName() { return m_name; }

    

    public void setSelected(boolean selected) { m_selected = selected;}

    public void invertSelected() { m_selected = !m_selected; }

    public boolean isSelected() { return m_selected; }

    public String toString() { return m_name; }
}
