package prompt.ui;

/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universit�t M�nchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: DataTable.java,v 1.11 2006/03/19 11:57:56 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.util.ArrayList;
import java.util.StringTokenizer;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;

import org.apache.log4j.Logger;

import prompt.engine.IEngineResults;
// import com.eteks.jeks.JeksTableModel;


/**
 * @author schmidt, fixed different col number bug
 * @author schrader
 * 
 */
public class DataTable extends JFrame implements ComponentListener {
	
	Logger logger = Logger.getLogger(this.getClass().getName());
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * 
	 */

	protected JScrollPane scrollPane;

	protected String title;

	protected String result;

	protected StringTokenizer tokenizer;

	protected DefaultTableModel tableModel;

	protected JLabel titleLabel;

	protected DataTableModelNew tablemodel;

	protected IEngineResults results;

	protected DataTableNew myTable;

	protected JTable tab;

	public DataTable(IEngineResults results, String title) {
		this.title = title;
		this.results = results;
		result = results.getData();
		this.addComponentListener(this);
		createTable();
	}
	
	/**
	 * Constructs the table model.
	 */
	protected void createTable() {
		String[] resultLines = result.split("\n", result.length());
		ArrayList all = new ArrayList();

		int maxCols = 1;
		for (int i = 0; i < resultLines.length; i++) {
			ArrayList line = new ArrayList();
			// logger.debug("CheckDataTableLines: "+resultLines[i]);
			
			tokenizer = new StringTokenizer(resultLines[i], "\t");

			int curCols = 0;
			while (tokenizer.hasMoreTokens()) {
				line.add(tokenizer.nextToken());
				curCols++;
			}
			if (curCols > maxCols)
				maxCols = curCols;
			
			all.add(line);
		}

		tablemodel = new DataTableModelNew(all.size(), maxCols);

		if (all.size() > 1) {
			for (int j = 1; j < all.size(); j++) {
				ArrayList currentRow = (ArrayList) all.get(j);
				for (int i = 0; i < currentRow.size(); i++) {
					tablemodel.setValueAt(currentRow.get(i), j - 1, i);
				}
			}
		} else {
			System.err.println("no data rows");
		}

		TableSorter sorter = new TableSorter(tablemodel);
		tab = new JTable(sorter);
		sorter.setTableHeader(tab.getTableHeader());

		if (all.size() > 0) {
			// column names
			ArrayList firstRow = (ArrayList) all.get(0);

			// Sets the Column Names
			for (int i = 0; i < firstRow.size(); i++) {
				if(firstRow.size()<((ArrayList) all.get(1)).size()){
					tab.getColumnModel().getColumn(i + 1).setHeaderValue(
							firstRow.get(i));
				}else{
					tab.getColumnModel().getColumn(i).setHeaderValue(
							firstRow.get(i));
				}
				/*if (firstRow.get(i).toString().matches("\".*\"")) {
					// Binning => Spalte um eins nach rechts verschieben
					tab.getColumnModel().getColumn(i + 1).setHeaderValue(
							firstRow.get(i));
				} else {
					tab.getColumnModel().getColumn(i).setHeaderValue(
							firstRow.get(i));
				}*/
			}

		} else {
			System.err.println("not even a frist row of header lines");
		}

		// Sets the perfect column width
		Font f = tab.getFont();
		FontMetrics fm = tab.getFontMetrics(f);
		for (int i = 0; i < tab.getColumnCount(); i++) {
			Object obj = tab.getValueAt(0, i);
			if (obj != null) {
				int width = Math.max(fm.stringWidth((String) tab
						.getColumnModel().getColumn(i).getHeaderValue()), fm
						.stringWidth((String) tab.getValueAt(0, i)));
				tab.getColumnModel().getColumn(i).setPreferredWidth(width + 10);
			} else
				logger.debug("no data in row 0, col " + i);
		}

		if (results.getResultDescription() != null) {
			myTable = new DataTableNew(results.getResultDescription(), tab);
		} else {
			myTable = new DataTableNew("No result description", tab);
		}

		// myTable.setTable(tab);
		if (title != null) {
			myTable.setTitle(this.title);
		}

	}

	/**
	 * Makes the layout and sets this (as <code>DataTable</code> inherits from
	 * <code>JFrame</code>, so is a <code>JFrame</code>) visible.
	 * 
	 * @throws Exception
	 */
	public void showTable() {
		myTable.setMetaData(results.getMetaData());
		// myTable.setResult(result);
		myTable.setVisible(true);
	}

	/**
	 * Returns the <code>DefaultTableModel</code> which is created by
	 * <code>createTable()</code>
	 * 
	 * @return a <code>DefaultTableModel</code> with the result data from the
	 *         <code>EngineResults</code>
	 */
	public DefaultTableModel getTableModel() {
		return tableModel;
	}

	/**
	 * Returns the result data from an <code>EngineResults</code>.
	 * <p>
	 * Equal to <code>EngineResults#getData()</code>
	 * 
	 * @return the result data in a tabbed and linebreaked format
	 */
	public String getRawData() {
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.ComponentListener#componentResized(java.awt.event.ComponentEvent)
	 */
	public void componentResized(ComponentEvent e) {
		// TODO Auto-generated method stub

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.ComponentListener#componentMoved(java.awt.event.ComponentEvent)
	 */
	public void componentMoved(ComponentEvent e) {
		// TODO Auto-generated method stub

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.ComponentListener#componentShown(java.awt.event.ComponentEvent)
	 */
	public void componentShown(ComponentEvent e) {
		// TODO Auto-generated method stub

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.ComponentListener#componentHidden(java.awt.event.ComponentEvent)
	 */
	public void componentHidden(ComponentEvent e) {
		// TODO Auto-generated method stub

	}

}
