package prompt.ui;


import java.io.IOException;
import java.util.Hashtable;

import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;

import org.apache.log4j.Logger;

/**
 * A table model storing its values in a hashtable with keys of <code>JeksCell</code> class.
 * This allows to have very big tables with a lot of <code>null</code> values.
 *
 * @version 1.0
 * @author  Emmanuel Puybaret
 * @since   Jeks 1.0
 */
public class DataTableModelNew extends AbstractTableModel
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	Logger logger = Logger.getLogger(this.getClass().getName());
	
  private int    rowCount;
  private int    columnCount;

  private Hashtable cellValues;

  /**
   * Creates a table model with <code>Short.MAX_VALUE</code> rows and columns.
   */
  public DataTableModelNew ()
  {
    this (Short.MAX_VALUE, Short.MAX_VALUE);
  }

  /**
   * Creates a table model with <code>rowCount</code> rows and
   * <code>columnCount</code> columns.
   */
  public DataTableModelNew (int rowCount, int columnCount)
  {
    this.rowCount     = rowCount;
    this.columnCount  = columnCount;

    cellValues = new Hashtable ();
  }

  public int getRowCount ()
  {
    return rowCount;
  }

  public int getColumnCount ()
  {
    return columnCount;
  }

  public Object getValueAt (int row, int column)
  {
    // row and column index are checked but storing in a Hashtable
    // won't cause real problems
    if (row >= getRowCount ()){
    	logger.debug("Check1Row: "+row +" > "+getRowCount());
    	//throw new ArrayIndexOutOfBoundsException (row);
    }
    //logger.debug("Check2Row: "+row +" > "+getRowCount());
      
    if (column > getColumnCount ()){
    	logger.debug("Check3Column: "+column +" > "+getColumnCount());
      //throw new ArrayIndexOutOfBoundsException (column);
    }
   // logger.debug("Check4Column: "+column +" > "+getColumnCount());
    return cellValues.get (new JeksCell (row, column));
  }

  public boolean isCellEditable (int row, int column)
  {
    return true;
  }

  public void setValueAt (Object value, int row, int column)
  {
    // row and column index are checked but storing in a Hashtable
    // won't cause real problems
    if (row >= getRowCount ())
      throw new ArrayIndexOutOfBoundsException (row);
    if (column >= getColumnCount ())
      throw new ArrayIndexOutOfBoundsException (column);
    JeksCell cell = new JeksCell (row, column);
    if (   value == null
        || "".equals (value))
      cellValues.remove (cell);
    else
      cellValues.put (cell, value);

    fireTableChanged (new TableModelEvent (this, row, row, column));
  }

  private void writeObject (java.io.ObjectOutputStream out) throws IOException
  {
    out.writeInt (rowCount);
    out.writeInt (columnCount);
    out.writeObject (cellValues);
    // Don't record listeners
  }

  private void readObject (java.io.ObjectInputStream in) throws IOException, ClassNotFoundException
  {
    rowCount    = in.readInt ();
    columnCount = in.readInt ();
    cellValues  = (Hashtable)in.readObject ();
  }
}

