/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: DataTableSimple.java,v 1.4 2006/03/19 11:57:55 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.ui;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.table.DefaultTableModel;

import prompt.engine.IEngineResults;



/**
 * Each <code>Engine</code> produces a result which is stored
 * in an <code>EngineResults</code> object. The result data itself
 * can be obtained by calling <code>getData()</code> in 
 * <code>EngineResults</code> which returns a tab and linebreak
 * formatted <code>String</code>.
 * <p>
 * <code>DataTable</code> creates a table and fills it with the
 * result data.  
 *  
 */
public class DataTableSimple extends JFrame implements ComponentListener{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	protected String result;
	protected JTable table;
	protected StringTokenizer tokenizer;
	protected Vector line = new Vector();
	protected Vector all = new Vector();
	protected JScrollPane scrollPane;
	protected String title;
	protected DefaultTableModel tableModel;
	protected JLabel label;
	protected JPanel panel;
	protected JButton button;
	protected JTextArea textArea;

	protected GridBagLayout layout;
	protected GridBagConstraints c;

	public DataTableSimple(IEngineResults results, String title) {
		this.title = title;
		result = results.getData();
		this.addComponentListener(this);
		createTable();
		//showTable();
	}
	
	/**
	 * Constructs the table model.
	 */
	protected void createTable() {
		String[] resultLines = result.split("\n", result.length());
		//logger.debug("+"+resultLines[0]+"+");
		for (int i = 0; i < resultLines.length; i++) {
			tokenizer = new StringTokenizer(resultLines[i], "\t");
			while (tokenizer.hasMoreTokens()) {
				line.addElement(tokenizer.nextToken());
			}
			all.addElement(line.clone());
			line.clear();
		}
		
		//column names
		line = (Vector) all.elementAt(0);
		//kill first line of table
		all.removeElementAt(0);
		table = new JTable(all, line);
		//table.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
		
		tableModel = new DefaultTableModel(all, line);
	}
	
	/**
	 * Makes the layout and sets this (as <code>DataTable</code> 
	 * inherits from <code>JFrame</code>, so is a
	 * <code>JFrame</code>) visible. 
	 */
	public void showTable() {
	    this.setSize(800,600);
	    JPanel rootPanel = new JPanel();
	    layout = new GridBagLayout();
		c = new GridBagConstraints();
		textArea = new JTextArea();
		textArea.setBackground(getContentPane().getBackground());
		textArea.setText(title);
		textArea.setFont(new Font("dialog.bold", Font.BOLD, 15));
		textArea.setLineWrap(true);
		textArea.setWrapStyleWord(true);
		textArea.setEditable(false);
		textArea.setRows(2);
		textArea.setColumns(30);
		//label = new JLabel(title);
		//label.setFont(new Font("Tahoma", Font.BOLD, 10));
		scrollPane = new JScrollPane(table);
		scrollPane.setMinimumSize(new Dimension(300,300));
		
//		table.doLayout();
		//table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		table.setDragEnabled(false);
//		getContentPane().setLayout(layout);
		rootPanel.setLayout(layout);

		c.insets = new Insets(20, 10, 20, 10);
		c.gridx = 0;
		c.gridy = 0;
		c.fill = GridBagConstraints.BOTH;
		layout.setConstraints(textArea, c);
//		getContentPane().add(textArea);
		rootPanel.add(textArea);

		c.insets = new Insets(0, 0, 0, 0);
		c.gridx = 0;
		c.gridy = 1;
		c.anchor = GridBagConstraints.NORTHWEST;
		c.fill = GridBagConstraints.BOTH;
		layout.setConstraints(scrollPane, c);
//		getContentPane().add(scrollPane);
		rootPanel.add(scrollPane);

		c.gridx = 0;
		c.gridy = 2;
		c.fill = GridBagConstraints.NONE;
		button = new JButton("Close");
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		});
		layout.setConstraints(button, c);
//		getContentPane().add(button);
		rootPanel.add(button);
		setTitle("DataTable - " + title);
		table.doLayout();
	
		this.setContentPane(rootPanel);
		
//		getContentPane().setLayout(new BorderLayout());
//		getContentPane().add(rootPanel, BorderLayout.CENTER);
//		pack();
		setVisible(true);
	}
	
	/**
	 * Returns the <code>DefaultTableModel</code> which
	 * is created by <code>createTable()</code>
	 * 
	 * @return	a <code>DefaultTableModel</code> with the result data
	 * 			from the <code>EngineResults</code>
	 */
	public DefaultTableModel getTableModel() {
		return tableModel;
	}
	
	/**
	 * Returns the result data from an <code>EngineResults</code>.
	 * <p>
	 * Equal to <code>EngineResults#getData()</code>
	 * 
	 * @return	the result data in a tabbed and linebreaked
	 * 			format
	 */
	public String getRawData() {
		return result;
	}

    /* (non-Javadoc)
     * @see java.awt.event.ComponentListener#componentResized(java.awt.event.ComponentEvent)
     */
    public void componentResized(ComponentEvent e) {
        table.doLayout();
    }

    /* (non-Javadoc)
     * @see java.awt.event.ComponentListener#componentMoved(java.awt.event.ComponentEvent)
     */
    public void componentMoved(ComponentEvent e) {
        // TODO Auto-generated method stub
        
    }

    /* (non-Javadoc)
     * @see java.awt.event.ComponentListener#componentShown(java.awt.event.ComponentEvent)
     */
    public void componentShown(ComponentEvent e) {
        // TODO Auto-generated method stub
        
    }

    /* (non-Javadoc)
     * @see java.awt.event.ComponentListener#componentHidden(java.awt.event.ComponentEvent)
     */
    public void componentHidden(ComponentEvent e) {
        // TODO Auto-generated method stub
        
    }
}
