/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id$
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.ui;

import java.io.File;

/**
 * An implementation of the abstract <code>FileFilter</code> 
 * class. A filter is assigned to a filechooser and decides which
 * file(types) have to be displyed.
 * 
 */
public class FileFilterAwt implements java.io.FilenameFilter  {

    protected String format;

    public FileFilterAwt() {
    }

    public FileFilterAwt(String format) {
        this.format = format;
    }

    /**
     * Whether the given file is accepted by this filter.
     */
    public boolean accept(File dir, String name) {
        if (format == null) {
            return true;
        }
      
        if (name.toUpperCase().endsWith(format.toUpperCase()))
            return true;
        if (format.endsWith("*"))
            return true;
        
        return false;
    }

    /**
     *	The description of this filter. For example:
     * "PROMPT workspace files (.bsw)"
     */
    public String getDescription() {

        String format2 = format.toUpperCase();
        if (format == null)
            return "PROMPT workspace files (.bsw)";
        if (format2.endsWith("JAR"))
            return "JAR (.jar)";
        if (format2.endsWith("DAT"))
            return "Data (.dat)";
        if (format2.endsWith("ARFF"))
            return "WEKA ARFF file (.arff)";
        if (format2.endsWith("TXT"))
            return "Tab delimited text format (.txt)";
        if (format2.endsWith("SER"))
            return "Serialized objects (.ser)";
        if (format2.endsWith("FASTA"))
            return "FASTA (.fasta)";
        if (format2.endsWith("XML"))
            return "Generic XML (.xml)";
        if (format2.endsWith("*"))
            return "All files (*)";
        else {
            return format2.substring(0, 1).toUpperCase() + format2.substring(1) + " (" + format2 + ")";
        }
    }


}
