/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: SaveUI.java,v 1.6 2006/03/19 11:57:55 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.ui;

import gr.zeus.ui.JMessage;

import java.awt.FileDialog;
import java.awt.Frame;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import tschmidt.utils.Saving;
import tschmidt.utils.ValidLocation;


public class SaveUI {

  private static String lastFilename;
  
  public static String load(String title, String suffix) {
   return load(title,suffix,lastFilename, true);
  }
  
  
  /**
   * @param title (Dialogtitle)
   * @param suffix (z.Bsp. '.ser')
   * @return choosen filename or null if canceled
   */
  public static String load(String title, String suffix, String lastFilenameP, boolean remember) {
    Logger logger = Logger.getLogger(SaveUI.class.getName());
    if (suffix==null) suffix="*.*";
    logger.debug("LOAD: '"+title+"' SUFFIX '"+suffix+"'");
    String filename = null;
    try {
      JFileChooser fc = new JFileChooser(lastFilenameP); 
      fc.setDialogType(JFileChooser.OPEN_DIALOG);
      fc.addChoosableFileFilter(new FileFilter(suffix));
      fc.setDialogTitle(title);
      
      int fileChooserMessage = fc.showOpenDialog(null);
      switch (fileChooserMessage) {
      case JFileChooser.APPROVE_OPTION:
        if (fc.getSelectedFile() != null)
          filename = fc.getSelectedFile().getAbsolutePath();
          if (remember) lastFilename= filename;
        break;
      case JFileChooser.CANCEL_OPTION:
        filename = null;
        break;
      }
      
      if (filename != null) {
         if (ValidLocation.isReadOk(filename))  {
            logger.debug("choosen '"+filename+"' to load from");
            return filename;
         }
         else {
           String errmsg = "Can not read from file '"+filename+"'";
           JMessage.showMessageDialog(null,errmsg,JOptionPane.ERROR_MESSAGE,JOptionPane.DEFAULT_OPTION);
           return null;
         }
      }

    } catch (Exception e1) {
      String errmsg = "Error while choosing the filename.";
      JMessage.showMessageDialog(null,errmsg,JOptionPane.ERROR_MESSAGE,JOptionPane.DEFAULT_OPTION);
    }
    return filename;
  }

  
  public static String loadDir(String title) {
      Logger logger = Logger.getLogger(SaveUI.class.getName());
      logger.debug("Choose directory: '"+title+"' ");
      String filename = null;
      try {
        JFileChooser fc = new JFileChooser(lastFilename); 
        fc.setDialogType(JFileChooser.OPEN_DIALOG);
        fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        fc.setDialogTitle(title);
        
        int fileChooserMessage = fc.showOpenDialog(null);
        switch (fileChooserMessage) {
        case JFileChooser.APPROVE_OPTION:
          if (fc.getSelectedFile() != null)
            filename = fc.getSelectedFile().getAbsolutePath();
            lastFilename= filename;
          break;
        case JFileChooser.CANCEL_OPTION:
          filename = null;
          break;
        }
        
        if (filename != null) {
           if (ValidLocation.isReadOk(filename))  {
              logger.debug("choosen '"+filename+"' to load from");
              return filename;
           }
           else {
             String errmsg = "Can not read from directory '"+filename+"'";
             JMessage.showMessageDialog(null,errmsg,JOptionPane.ERROR_MESSAGE,JOptionPane.DEFAULT_OPTION);
             return null;
           }
        }

      } catch (Exception e1) {
        String errmsg = "Error while choosing the directoy.";
        JMessage.showMessageDialog(null,errmsg,JOptionPane.ERROR_MESSAGE,JOptionPane.DEFAULT_OPTION);
      }
      return filename;
    }
  
  /**
   * @param title (Dialogtitle)
   * @param suffix (z.Bsp. '.ser')
   * @return choosen filename or null if canceled
   */
  public static String save(String title, String suffix) {
    /*
     *   if (destFilename==null) {
            JMessage.showMessageDialog(null,"Canceled saving",JOptionPane.WARNING_MESSAGE,JOptionPane.DEFAULT_OPTION);
            return;
         }
     * 
     */
    if (suffix==null) suffix="";
    Logger logger = Logger.getLogger(SaveUI.class.getName());
    logger.debug("SAVE: '"+title+"' SUFFIX '"+suffix+"'");
    String filename = null;
    try {
      JFileChooser fc = new JFileChooser(lastFilename); 
      fc.setDialogType(JFileChooser.SAVE_DIALOG);
      fc.addChoosableFileFilter(new FileFilter(suffix));
      fc.setDialogTitle(title);
      int fileChooserMessage = fc.showSaveDialog(null);
      switch (fileChooserMessage) {
      case JFileChooser.APPROVE_OPTION:
        if (fc.getSelectedFile() != null)
          filename = fc.getSelectedFile().getAbsolutePath();
          lastFilename= filename;
        break;
      case JFileChooser.CANCEL_OPTION:
        filename = null;
        break;
      }
      
      if (filename != null) {
        if (!filename.endsWith(suffix))
          filename = filename + suffix;
        lastFilename= filename;
        if (Saving.shallWeWriteFile(filename)) {
            logger.debug("choosen the filename '"+filename+"' to save to");
        } else filename=null;
      }

    } catch (Exception e1) {
      String errmsg = "Error while choosing the filename.";
      JMessage.showMessageDialog(null,errmsg,JOptionPane.ERROR_MESSAGE,JOptionPane.DEFAULT_OPTION);
    }
    return filename;
  }
  
  
  public static String lastDir;
  
  /**
   * @param title (Dialogtitle)
   * @param suffix (z.Bsp. '.ser')
   * @return choosen filename or null if canceled
   */
  public static String saveDefault(String title, String suffix, String defaultname) {
    /*
     *   if (destFilename==null) {
            JMessage.showMessageDialog(null,"Canceled saving",JOptionPane.WARNING_MESSAGE,JOptionPane.DEFAULT_OPTION);
            return;
         }
     * 
     */
    if (suffix==null) suffix="";
    Logger logger = Logger.getLogger(SaveUI.class.getName());
    logger.debug("SAVE2: '"+title+"' SUFFIX '"+suffix+"'");
    String filename = null;
    try {
        FileDialog fd = new FileDialog(new Frame(), title,    FileDialog.SAVE);
        // fd.set
         fd.setFile(defaultname);
         fd.setDirectory(lastDir);
       // fd.setLocation(50, 50);
         fd.setVisible(true);
        filename = fd.getFile();
      
        
      if (filename != null) {
        if (!filename.endsWith(suffix))
          filename = filename + suffix;
        lastDir= fd.getDirectory();
        String tmp = lastDir;
        if (! tmp.endsWith(File.separator)) tmp=tmp+File.separator;
        filename = tmp + filename;
        if (Saving.shallWeWriteFile(filename)) {
            logger.debug("choosen the filename '"+filename+"' to save to");
        } else filename=null;
      }

    } catch (Exception e1) {
      String errmsg = "Error while choosing the filename.";
      JMessage.showMessageDialog(null,errmsg,JOptionPane.ERROR_MESSAGE,JOptionPane.DEFAULT_OPTION);
    }
    return filename;
  }
  

  public String loadFile(Frame f, String title, String defDir, String suffix) {
    String filename= null;
    Logger logger = Logger.getLogger(SaveUI.class.getName());  
    try{
      FileDialog fd = new FileDialog(f, title, FileDialog.LOAD);
      fd.setFile(suffix);
      //fd.setFilenameFilter();
      fd.setDirectory(defDir);
      fd.setLocation(50, 50);
      fd.setVisible(true);
      filename=fd.getFile();
      if (filename != null) {
          if (!filename.endsWith(suffix))
            filename = filename + suffix;
          lastFilename= filename;
          if (Saving.shallWeWriteFile(filename)) {
              logger.debug("choosen the filename '"+filename+"' to save to");
          } else filename=null;
        }
  } catch (Exception e1) {
      String errmsg = "Error while choosing the filename.";
      JMessage.showMessageDialog(null,errmsg,JOptionPane.ERROR_MESSAGE,JOptionPane.DEFAULT_OPTION);
    }
      
     return filename;
      }

  

  
  
  
}
