/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: IVisualisation.java,v 1.2 2006/01/05 02:19:05 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.vis;

import java.io.IOException;

import prompt.engine.IEngineResults;
import prompt.input.IJobTemplate;



/**
 * @author Schmidt
 *
 */
public interface IVisualisation extends IJobTemplate{
    
  
    /**
     * @param dataToPlot an IEngineResult object which holds all the data that should be plotted
     * may be null, if you are plotting only hardcored non-parameterized function
     */
    public abstract void init(IEngineResults dataToPlot);
    
    
    /**
     * Saves all the data which should have been plotted. This actually refers to the EngineResult
     * which was passed when the init method was called
     * @param filename String
     * @throws IOException
     */
    public abstract void saveDataToFile(String filename) throws IOException;

    /**
     * Returns all processing and debug messages which have been created during the last 
     * run method
     * @return String
     */
    public abstract String getProcessingMessages();

    /**
     * Saves the graphic which was generated/must have been generated during the last
     * execution of the run()-method to the passed filename.<br>
     * Format: Enhanced Postscript<br>
     * @param filename String, destination filename, suffix eps should be included in the filename 
     * since no automatic suffix is appended to the filename
     */
    public abstract void saveToEps(String filename);

    
    /**
     * Saves the graphic which was generated/must have been generated during the last
     * execution of the run()-method to the passed filename.<br>
     * Format: PNG<br> 
     * @param filename String, destination filename, suffix png should be included in the filename 
     * since no automatic suffix is appended to the filename
     */    
    public abstract void saveToPng(String filename);

    /**
     * Saves the graphic which was generated/must have been generated during the last
     * execution of the run()-method to the passed filename.<br>
     * Format: Enhanced windows Metafile Format (vector format)<br> 
     * @param filename String, destination filename, suffix emf should be included in the filename 
     * since no automatic suffix is appended to the filename
     */    
    public abstract void saveToEmf(String filename);

    /**
     * Cleans up the .ps and .png graphics created by this class
     */
    public abstract void cleanUp();

    /**
     * @return temporary graphic without suffix .png or .ps
     */
   // public abstract String getFullGraphicOutputFileName();

    /**
     * Shows the graphic in a window
     */
    public abstract void showGraphic();
    
    /**
     * @param filename String, a filename which replaces the temporary plot file names (without the suffixes eps/png/emf)
     * @return Returns the command string that was passed to the plotting facility with temporary filenames replaced by the parameter filename
     */
    public String getCmdAll(String filename);
    
    /**
     * Returns all the data which should (have been) plotted. This actually returns the EngineResult
     * which was passed when the init method was called.
     * @return
     */
    public IEngineResults getData();
    
}



