/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: ViewerSaving.java,v 1.5 2006/03/19 11:57:58 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.vis;

import gr.zeus.ui.JMessage;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import prompt.ui.DataTable;
import tschmidt.utils.Saving;


public class ViewerSaving implements ActionListener {
	 
	  
	  protected static String imageDirectory;
	  protected IVisualisation visual;
	  protected String title;
	  
	  Logger logger = Logger.getLogger(this.getClass().getName());
	  boolean WINDOWS;
	  
	public ViewerSaving(IVisualisation visual2, String title) {
		this.visual= visual2;
		this.title=title;
		String os = System.getProperty("os.name");
		
		    if (os.matches(".*?Windows.*")) {
		      WINDOWS = true;
		    } else {
		      WINDOWS = false;
		    }
	}


	public void actionPerformed(ActionEvent e) {

		
		 JFileChooser fc = new JFileChooser(imageDirectory); 
		    int n = 0;
		    if (e.getActionCommand().equals("Save Data to...")) {
		      try {
		        fc.setDialogType(JFileChooser.SAVE_DIALOG);
		        if (imageDirectory != null) {
		          fc.setCurrentDirectory(new File(imageDirectory));
		        }
		        int fileChooserMessage = fc.showSaveDialog(null);
		        String filename = null;
		        switch (fileChooserMessage) {
		        case JFileChooser.APPROVE_OPTION:
		          if (fc.getSelectedFile() != null)
		            filename = fc.getSelectedFile().getAbsolutePath();
		          imageDirectory = filename.substring(0, filename.lastIndexOf(File.separator));
		          break;
		        case JFileChooser.CANCEL_OPTION:
		          filename = null;
		          break;
		        }
		        if (filename != null) {
		          if (!filename.endsWith(".txt"))
		            filename = filename + ".txt";
		          if (Saving.shallWeWriteFile(filename)) {
		              logger.debug("write data file to '"+filename+"'");
		            this.visual.saveDataToFile(filename);
		          }
		        }

		      } catch (Exception e1) {
                  logger.error(e1);
		      }
		    }

		    if (e.getActionCommand().equals("Save As EPS...")) {
		      fc.setDialogType(JFileChooser.SAVE_DIALOG);
		      if (imageDirectory != null) {
		        fc.setCurrentDirectory(new File(imageDirectory));
		      }

		      //System.getProperty("user.dir"));
		      int fileChooserMessage = fc.showSaveDialog(null);
		      String filename = null;
		      switch (fileChooserMessage) {
		      case JFileChooser.APPROVE_OPTION:
		        if (fc.getSelectedFile() != null)
		          filename = fc.getSelectedFile().getAbsolutePath();
		        imageDirectory = filename.substring(0, filename.lastIndexOf(File.separator));
		        break;
		      case JFileChooser.CANCEL_OPTION:
		        filename = null;
		        break;
		      }
		      if (filename != null) {
		        if (!filename.endsWith(".eps"))
		          filename = filename + ".eps";
		        if (Saving.shallWeWriteFile(filename))
		          visual.saveToEps(filename);

		      }
		    }
		    if (e.getActionCommand().equals("Save As EMF...")) {
		      fc.setDialogType(JFileChooser.SAVE_DIALOG);

		      int fileChooserMessage = fc.showSaveDialog(null);
		      String filename = null;
		      switch (fileChooserMessage) {
		      case JFileChooser.APPROVE_OPTION:
		        if (fc.getSelectedFile() != null)
		          filename = fc.getSelectedFile().getAbsolutePath();
		        break;
		      case JFileChooser.CANCEL_OPTION:
		        filename = null;
		        break;
		      }
		      if (filename != null) {
		        if (!filename.endsWith(".emf"))
		          filename = filename + ".emf";

		        if (Saving.shallWeWriteFile(filename))
		          visual.saveToEmf(filename);
		      }
		    }
		    if (e.getActionCommand().equals("Save As PNG...")) {
		      fc.setDialogType(JFileChooser.SAVE_DIALOG);
		 
		      int fileChooserMessage = fc.showSaveDialog(null);
		      String filename = null;
		      switch (fileChooserMessage) {
		      case JFileChooser.APPROVE_OPTION:
		        if (fc.getSelectedFile() != null)
		          filename = fc.getSelectedFile().getAbsolutePath();
		        break;
		      case JFileChooser.CANCEL_OPTION:
		        filename = null;
		        break;
		      }
		      if (filename != null) {
		        if (!filename.endsWith(".png"))
		          filename = filename + ".png";

		        if (Saving.shallWeWriteFile(filename))
		          visual.saveToPng(filename);
		      }
		    }
		    if (e.getActionCommand().equals("Save all")) {
		    
		      fc.setDialogType(JFileChooser.SAVE_DIALOG);

		      int fileChooserMessage = fc.showSaveDialog(null);
		      
		      String filename = null;
		      switch (fileChooserMessage) {
		      case JFileChooser.APPROVE_OPTION:
		        if (fc.getSelectedFile() != null)
		          filename = fc.getSelectedFile().getAbsolutePath();
		        break;
		      case JFileChooser.CANCEL_OPTION:
		        filename = null;
		        break;
		      }

		      if (filename != null) {
		        String filename2 = filename.trim().toLowerCase();
		        if (filename2.endsWith(".eps") || filename2.endsWith(".ps") || filename2.endsWith(".png") || filename2.endsWith(".txt")
		            || filename2.endsWith(".emf")) {

		         
		          n= JMessage.showMessageDialog(null, "Use\n" + filename.substring(0, filename.lastIndexOf(".")) + "\n instead of\n" + filename + "?", 
		                  JOptionPane.QUESTION_MESSAGE,JOptionPane.YES_NO_CANCEL_OPTION);
		          
		          if (n == 0) {
		            filename = filename.substring(0, filename.lastIndexOf("."));
		          }
		        }

		        if (n != 2) {

		          String[] names;
		          if (WINDOWS) {
		            names = new String[] { filename + ".eps", filename + ".png", filename + ".txt",  filename + ".R", filename + ".emf" };
		          } else {
		            names = new String[] { filename + ".eps", filename + ".png", filename + ".txt", filename + ".R" };
		          }
		          String filesCreated = "";
		          for (int i = 0; i < names.length; i++) {
		            File f = new File(names[i]);
		            if (f.exists()) {
		              filesCreated = filesCreated + names[i] + "\t (exists)\n";
		            } else {
		              filesCreated = filesCreated + names[i] + "\n";
		            }
		          }

		          
		          n= JMessage.showMessageDialog(null, "Following files will be created\n" + filesCreated, JOptionPane.INFORMATION_MESSAGE, JOptionPane.YES_NO_OPTION);
		          
		          if (n == 0) {
		            if (Saving.shallWeWriteFile(filename))
		              visual.saveToEps(names[0]);

		            if (Saving.shallWeWriteFile(names[1]))
		              visual.saveToPng(names[1]);

		            if (Saving.shallWeWriteFile(names[2])) {
		                logger.debug("write data file to '"+names[2]+"'");
		              try {
		                this.visual.saveDataToFile(names[2]);
		              } catch (IOException e1) {
		               
		                logger.error("error writing datat file to '"+names[2]+"'",e1);
		              }
		            }
		            if (Saving.shallWeWriteFile(names[3])) {
		                String filen = names[3].substring(0, names[3].lastIndexOf("."));
		              String Rcode = this.visual.getCmdAll(filen);
		              logger.debug("write R code file to '"+names[3]+"'");
		              Saving.writeStringToFile(names[3],Rcode);
		              
		            }
		            if (WINDOWS == true) {
		              if (Saving.shallWeWriteFile(names[4]))
		                visual.saveToEmf(names[4]);
		            }


		          }
		        }
		      }
		    }

		    if (e.getActionCommand().equals("Show data")) {
		      DataTable dt = new DataTable(visual.getData(), title);
		      try {
				dt.showTable();
			} catch (Exception e1) {
				
                logger.error(e1);
			}
		    }

		    if (e.getActionCommand().equals("Send to lpr")) {
		      logger.debug("print");
		      visual.saveToEps(System.getProperty("java.io.tmpdir") + File.separator + "printtmp.eps");
		      try {
		        String printCommand = "lpr " + System.getProperty("java.io.tmpdir") + File.separator + "printtmp.eps";
		        logger.debug(printCommand);
		        //Process p = 
		        	Runtime.getRuntime().exec(printCommand);
		      } catch (IOException e1) {


                    logger.error(e1);
		      }
		      File f = new File(System.getProperty("java.io.tmpdir") + File.separator + "printtmp.eps");
		      if (f.exists()) {
		        f.deleteOnExit();
		      }

		    }
		  }
}
