/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: VisualisationGnuplot.java,v 1.5 2006/03/19 11:57:58 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.vis;

import java.awt.Image;
import java.awt.Toolkit;
import java.io.BufferedWriter;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

import org.apache.log4j.Logger;

import prompt.engine.IEngineResults;
import tschmidt.ginterface.GException;
import tschmidt.ginterface.GPiper;
import tschmidt.ginterface.vis.Par;
import tschmidt.utils.StringUtils;
import de.tum.binfo.jdrm.JobTemplate;


/**
 * save speed verbessern, so wie in R-Visualisation class
 *
 */
public abstract class VisualisationGnuplot extends JobTemplate implements IVisualisationGnuplot {
  	
  public String titleWindow;
	public String acidToPlot;
	protected int dpi = 300;
	protected String fullGraphicOutputFileName;
	protected File epsFile, pngFile, emfFile;
	protected Vector epsFileData, pngFileData, emfFileData;
	protected Image image;
	//without suffix (e.g. no .png or .ps)

	protected String processingMessages; // the stuff R returns running RPiper

	protected IEngineResults data;
	Logger logger = Logger.getLogger(VisualisationGnuplot.class);

	public void saveDataToFile(String filename) 
	{
		try {
            data.saveDataToFile(filename);
        } catch (IOException e) {
           
            logger.error(e);
        }
	}

	public void init(IEngineResults resultFreq) {
		
		this.data = resultFreq;
		
	}
	
	protected boolean WINDOWS;
	 
	public VisualisationGnuplot() {
        String os = System.getProperty("os.name");
    	if (os.matches(".*?Windows.*")) {
    		WINDOWS = true;
    	} else {
    	    WINDOWS = false;
    	}
	}

	/**
	 * Returns all processing and debug messages which have been created during the last 
	 * run method
	 * @return String
	 */
	public String getProcessingMessages() {
		return processingMessages;
	}

	protected void saveGraphics() { //FIXME: whats this
	}
	public void saveToEps(String filename) {
		logger.debug("write eps file to '"+filename+"'");
		try {
			FileOutputStream fos = new FileOutputStream(filename);
			DataOutputStream dos = new DataOutputStream(fos);
			for (int i = 0; i < epsFileData.size(); i++) {
				dos.write(((Integer) epsFileData.elementAt(i)).intValue());
			}
			dos.flush();
			dos.close();
		} catch (IOException e) {
			logger.error("Couldn't write .eps file: " + e);
		}
	}

	public void saveToPng(String filename) {
		logger.debug("write png file to '"+filename+"'");
		try {
			FileOutputStream fos = new FileOutputStream(filename);
			DataOutputStream dos = new DataOutputStream(fos);
			for (int i = 0; i < pngFileData.size(); i++) {
				dos.writeByte( ((Integer) pngFileData.elementAt(i)).intValue());
			}
			dos.flush();
			dos.close();
		} catch (IOException e) {
			logger.error("Couldn't write .png file: " + e);
		}
	}

	public void saveToEmf(String filename) {
		logger.debug("write emf file to '"+filename+"'");
		try {
			FileOutputStream fos = new FileOutputStream(filename);
			DataOutputStream dos = new DataOutputStream(fos);
			for (int i = 0; i < emfFileData.size(); i++) {
				dos.writeByte(((Integer) emfFileData.elementAt(i)).intValue());
			}
			dos.flush();
			dos.close();			
		} catch (IOException e) {
			logger.error("Couldn't write .emf file: " + e);
		}
	}

	/**
	 * Cleans up the .ps and .png graphics created by this class
	 */
	public void cleanUp() {
		try {
			File f = new File(fullGraphicOutputFileName + ".eps");
			if (f.exists()) {
				f.delete();
			}
			f = new File(fullGraphicOutputFileName + ".png");
			if (f.exists()) {
				f.delete();
			}
			f = new File(fullGraphicOutputFileName + ".emf");
			if (f.exists()) {
				f.delete();
			}
			f = new File(System.getProperty("java.io.tmpdir") + File.separator + "printtmp.eps");
			if(f.exists()) {
				f.delete();
			}
		} catch (Exception e) {
		}
	}

	/**
	 * @return without suffix .png or .ps
	 */
	public String getFullGraphicOutputFileName() {
		return fullGraphicOutputFileName;
	}

	public void showGraphic() {
		if (titleWindow == null) {
			titleWindow = this.getClass().getName();
		}
		new Viewer(image, this, titleWindow);
	}

	protected Par gpar = new Par();

	
	boolean performPlottingCalled = false;
	
	private String cmdAll = null;
	private String fullDataFileName = null;
	
	 
	protected void performPlotting(String gcmds) throws GException {
		
		
		
		logger.debug("performPlotting");
		File file = null;
		try {
			file = File.createTempFile("Gnudat", null);
			BufferedWriter bw = new BufferedWriter(new FileWriter(file));
			bw.write(data.getData());
			bw.close();
		}catch (IOException z) {
			logger.error(
				"Can not write to file '" + file.toString() + "' ",z);
			
		}
		this.fullDataFileName = file.toString();

		try {
			file = File.createTempFile("GnuVis", null);
		}catch (IOException e1) {
            logger.error(e1);
		}
		this.fullGraphicOutputFileName = file.toString();
		if (WINDOWS) {
			fullDataFileName = fullDataFileName.replaceAll("\\\\", "/");
			fullGraphicOutputFileName =
				fullGraphicOutputFileName.replaceAll("\\\\", "/");
		}
		file.delete();
		//logger.debug("GVisu Check 1: "+gcmds);
		gcmds = gcmds.replaceAll("#datafile",this.fullDataFileName);
		//logger.debug("GVisu Check 2: "+gcmds);
		gcmds = getPar().toString() + gcmds;
		logger.debug(gcmds);

		String bmpCmd =
			"set terminal png; set output \""
				+ fullGraphicOutputFileName
				+ ".png\";";
		if (WINDOWS) {//Maybe some Changes?!?
			bmpCmd =
				"set terminal png; set output \""
				+ fullGraphicOutputFileName
				+ ".png\";";
		}
		
		String epsCmd = 
			 "set terminal postscript; set output \""
			+ fullGraphicOutputFileName
			+ ".eps\";";

		String emfCmd =
			"set terminal emf; set output \""
			+ fullGraphicOutputFileName
			+ ".emf\";";

		 cmdAll =
		 	      bmpCmd
				+ gcmds
				+ emfCmd
				+ gcmds
		 	    + epsCmd
				+ gcmds;
		 
		if (WINDOWS) {
			cmdAll = cmdAll + emfCmd + gcmds;
		}
		GPiper gp = new GPiper();
		try {
			this.processingMessages = gp.submitGCode(cmdAll);
		} catch (GException e) {
            logger.error(e);
		} catch (IOException e) {
            logger.error(e);
		}

		epsFileData = new Vector();
		pngFileData = new Vector();
		emfFileData = new Vector();
		int ch;
		FileInputStream fis = null;
		DataInputStream dis = null;
		try {
			 fis =
				new FileInputStream(fullGraphicOutputFileName + ".eps");
			 dis = new DataInputStream(fis);
			while ((ch = dis.read()) > -1) {
				epsFileData.addElement(new Integer(ch));
			}
			fis.close();
			File f = new File(fullGraphicOutputFileName + ".eps");
			if (f.exists()) {
				f.delete();
			}
		} catch (IOException e2) {
            logger.error(e2);
		}
		try {
			Toolkit toolkit = Toolkit.getDefaultToolkit();
			image = toolkit.getImage(fullGraphicOutputFileName + ".png");
	 		fis = new FileInputStream(fullGraphicOutputFileName + ".png");
			dis = new DataInputStream(fis);
			while ((ch = dis.read()) > -1) {
				pngFileData.addElement(new Integer(ch));
			}
			fis.close();
			File f = new File(fullGraphicOutputFileName + ".png");
			if (f.exists()) {
				f.deleteOnExit();
			}
		} catch (IOException e2) {
            logger.error(e2);
		}
		try {
			if (WINDOWS == true) {
				fis = new FileInputStream(fullGraphicOutputFileName + ".emf");
				dis = new DataInputStream(fis);
				while ((ch = dis.read()) > -1) {
					emfFileData.addElement(new Integer(ch));
				}
				fis.close();
				//logger.debug("Did emf file");
				File f = new File(fullGraphicOutputFileName + ".emf");
				if (f.exists()) {
					f.deleteOnExit();
				}

			}
		} catch (IOException e2) {
            logger.error(e2);
		}
		File f = new File(fullDataFileName);
		if (f.exists()) {
			f.delete();
		}
		logger.debug("Call the method showGraphic to view it interactiv or save the graphics to files via 'saveTo...");
		//logger.debug(cmdAll);
		performPlottingCalled=true;
	}
	/**
	 * @return Returns the plotting parameter object.
	 */
	public Par getPar() {
		return gpar;
	}
	/**
	 * Sets the plotting parameter object
	 * @param par The par to set.
	 */
	public void setPar(Par gpar) {
		this.gpar = gpar;
	}

  /**
   * @return Returns the cmdAll.
   */
  public String getCmdAll(String filename) {
    if (cmdAll==null) {
      logger.warn("You must run the 'run' method or the 'performPlotting' method first to get the R code");
      return null;
    }
    String filenameMasked = StringUtils.replaceAll(filename,"\\","\\\\");
    String cmdAll2 = cmdAll;
    cmdAll2 = StringUtils.replaceAll(cmdAll2,fullDataFileName,filenameMasked+".txt");
    
    cmdAll2= StringUtils.replaceAll(cmdAll2,fullGraphicOutputFileName, filenameMasked);

    return cmdAll2;
  }
  
  public IEngineResults getData() {
    return data;
  }
}
