/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: VisualisationR.java,v 1.5 2006/03/19 11:57:59 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.vis;

import java.awt.Image;
import java.awt.Toolkit;
import java.io.BufferedWriter;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;

import org.apache.log4j.Logger;

import prompt.engine.IEngineResults;
import tschmidt.rinterface.RException;
import tschmidt.rinterface.RPiper;
import tschmidt.rinterface.vis.Par;
import tschmidt.rinterface.vis.Title;
import tschmidt.utils.StringUtils;
import de.tum.binfo.jdrm.JobTemplate;


public abstract class VisualisationR extends JobTemplate implements IVisualisationR {
  	
  public String titleWindow;
	public String acidToPlot;
	protected int dpi = 300;
	protected String fullGraphicOutputFileName;
	protected File epsFile, pngFile, emfFile;

	
	byte[] pngFileData;
	byte[] epsFileData;
	byte[] emfFileData; 
	     
	protected Image image;
	//without suffix (e.g. no .png or .ps)

	protected String processingMessages; // the stuff R returns running RPiper

	protected IEngineResults data;
	Logger logger = Logger.getLogger(VisualisationR.class);

	public void saveDataToFile(String filename) 
	{
		try {
            data.saveDataToFile(filename);
        } catch (IOException e) {
           
            logger.error(e);
        }
	}

	public void init(IEngineResults resultFreq) {
		
		this.data = resultFreq;
		
	}
	
	protected boolean WINDOWS;
	 
	public VisualisationR() {
        String os = System.getProperty("os.name");
    	if (os.matches(".*?Windows.*")) {
    		WINDOWS = true;
    	} else {
    	    WINDOWS = false;
    	}
	}

	/**
	 * Returns all processing and debug messages which have been created during the last 
	 * run method
	 * @return String
	 */
	public String getProcessingMessages() {
		return processingMessages;
	}


	public void saveToEps(String filename) {
		logger.debug("write eps file to '"+filename+"'");
		try {
			FileOutputStream fos = new FileOutputStream(filename);
			DataOutputStream dos = new DataOutputStream(fos);
			dos.write(epsFileData);
			dos.flush();
			dos.close();
		} catch (IOException e) {
			logger.error("Couldn't write .eps file: " + e);
		}
	}

	public void saveToPng(String filename) {
		logger.debug("write png file to '"+filename+"'");
		try {
			FileOutputStream fos = new FileOutputStream(filename);
			DataOutputStream dos = new DataOutputStream(fos);
			dos.write(pngFileData);
			dos.flush();
			dos.close();
		} catch (IOException e) {
			logger.error("Couldn't write .png file: " + e);
		}
	}

	public void saveToEmf(String filename) {
		logger.debug("write emf file to '"+filename+"'");
		try {
			FileOutputStream fos = new FileOutputStream(filename);
			DataOutputStream dos = new DataOutputStream(fos);
			dos.write(emfFileData);
			dos.flush();
			dos.close();			
		} catch (IOException e) {
			logger.error("Couldn't write .emf file: " + e);
		}
	}

	/**
	 * Cleans up the .ps and .png graphics created by this class
	 */
	public void cleanUp() {
		try {
			File f = new File(fullGraphicOutputFileName + ".eps");
			if (f.exists()) {
				f.delete();
			}
			f = new File(fullGraphicOutputFileName + ".png");
			if (f.exists()) {
				f.delete();
			}
			f = new File(fullGraphicOutputFileName + ".emf");
			if (f.exists()) {
				f.delete();
			}
			f = new File(System.getProperty("java.io.tmpdir") + File.separator + "printtmp.eps");
			if(f.exists()) {
				f.delete();
			}
		} catch (Exception e) {
		}
	}

	/**
	 * @return without suffix .png or .ps
	 */
	public String getFullGraphicOutputFileName() {
		return fullGraphicOutputFileName;
	}

	public void showGraphic() {
		if (titleWindow == null) {
			titleWindow = this.getClass().getName();
		}
		 new Viewer(image, this, titleWindow);
	}

	protected Par par = new Par();
	protected Title title = new Title();
	
	 boolean performPlottingCalled = false;
	
	private String cmdAll = null;
	private String fullDataFileName = null;
	
	 
	protected void performPlotting(String rcmds) throws RException {
		rcmds= getPar().toString()+"\n"+rcmds+"\n"+getTitle().toString()+"\n";
		logger.debug(rcmds);
		
		logger.debug("performPlotting");
		File file = null;
		try {
			file = File.createTempFile("Rdat", null);
			BufferedWriter bw = new BufferedWriter(new FileWriter(file));
			bw.write(data.getData());
			bw.close();
		}catch (IOException z) {
			logger.error(
				"Can not write to file '" + file.toString() + "' ",z);
			
		}
		this.fullDataFileName = file.toString();

		try {
			file = File.createTempFile("RVis", null);
		}catch (IOException e1) {
            logger.error(e1);
		}
		this.fullGraphicOutputFileName = file.toString();
		if (WINDOWS) {
			fullDataFileName = fullDataFileName.replaceAll("\\\\", "/");
			fullGraphicOutputFileName =
				fullGraphicOutputFileName.replaceAll("\\\\", "/");
		}
		file.delete();
		String dataReadCmd;
		if (data.hasRownames()) {
			dataReadCmd =
				"data=read.table(\""
					+ fullDataFileName

					+ "\", header=TRUE, check.names=F, row.names=1, sep=\"\\t\")\n";
//			+ "\", header=TRUE, row.names=1,sep=\"\\t\")\n";

		} else {
			dataReadCmd =
				"data=read.table(\"" + fullDataFileName + "\", header=TRUE,check.names=F, sep=\"\\t\")\n";
//			"data=read.table(\"" + fullDataFileName + "\", sep=\"\\t\",header=TRUE)\n";

		}

		String bmpCmd =
			"bitmap(\""
				+ fullGraphicOutputFileName
				+ ".png\", type=\"png256\", res="
				+ dpi
				+ " )\n";
		if (WINDOWS) {
			bmpCmd =
				"png(\""
					+ fullGraphicOutputFileName
					+ ".png\", height=600, width=600)\n";
		}

		/*savePlot(filename="Rplot",
					 type=c("wmf", "png", "jpeg", "jpg", "bmp", "ps", "pdf"),
					 device=dev.cur())
		*/
		String emfCmd =
			"win.metafile(filename=\""
				+ fullGraphicOutputFileName
				+ ".emf"
				+ "\")\n";

		 cmdAll =
			dataReadCmd
				+ "postscript(\""
				+ fullGraphicOutputFileName
				+ ".eps\")\n"
				+ rcmds
				+ "dev.off()\n"
				+ bmpCmd
				+ rcmds
				+ "dev.off()\n";
		if (WINDOWS) {
			cmdAll = cmdAll + emfCmd + rcmds + "dev.off()\n";
		}
		RPiper rp = new RPiper();
		try {
			this.processingMessages = rp.submitRCode(cmdAll);
		} catch (IOException e2) {
            logger.error(e2);
		}


		//int ch;
		FileInputStream fis = null;
		DataInputStream dis = null;
		try {
			 fis =
				new FileInputStream(fullGraphicOutputFileName + ".eps");
			 dis = new DataInputStream(fis);
			 File f = new File(fullGraphicOutputFileName + ".eps");
			 epsFileData = new byte[(int) f.length()];
				dis.readFully(epsFileData);
			fis.close();
			
			if (f.exists()) {
				f.delete();
			}
		} catch (IOException e2) {
            logger.error(e2);
		}
		try {
			Toolkit toolkit = Toolkit.getDefaultToolkit();
			image = toolkit.getImage(fullGraphicOutputFileName + ".png");
	 		File f1 = new File(fullGraphicOutputFileName + ".png");
	 	
			fis = new FileInputStream(fullGraphicOutputFileName + ".png");
			dis = new DataInputStream(fis);
			pngFileData = new byte[(int) f1.length()];
			dis.readFully(pngFileData);
			fis.close();
			
			if (f1.exists()) {
				f1.deleteOnExit();
			}
		} catch (IOException e2) {
            logger.error(e2);
		}
		try {
			if (WINDOWS == true) {
				fis = new FileInputStream(fullGraphicOutputFileName + ".emf");
				dis = new DataInputStream(fis);
				File f = new File(fullGraphicOutputFileName + ".emf");
				emfFileData = new byte[(int) f.length()];
				dis.readFully(emfFileData);
				fis.close();
				//logger.debug("Did emf file");
				if (f.exists()) {
					f.deleteOnExit();
				}
			}
		} catch (IOException e2) {
            logger.error(e2);
		}
		File f = new File(fullDataFileName);
		if (f.exists()) {
			f.delete();
		}
		logger.debug("Call the method showGraphic to view it interactiv or save the graphics to files via 'saveTo...");
		//logger.debug(cmdAll);
		performPlottingCalled=true;
	}
	/**
	 * @return Returns the plotting parameter object.
	 */
	public Par getPar() {
		return par;
	}
	/**
	 * Sets the plotting parameter object
	 * @param par The par to set.
	 */
	public void setPar(Par par) {
		this.par = par;
	}
  /**
   * @return Returns the title object
   */
  public Title getTitle() {
    return title;
  }
  /**
   * @param title The title object to set.
   */
  public void setTitle(Title title) {
    this.title = title;
  }
  /**
   * @return Returns the cmdAll.
   */
  public String getCmdAll(String filename) {
    if (cmdAll==null) {
      logger.warn("You must run the 'run' method or the 'performPlotting' method first to get the R code");
      return null;
    }
    String filenameMasked = StringUtils.replaceAll(filename,"\\","\\\\");
    String cmdAll2 = cmdAll;
    cmdAll2 = StringUtils.replaceAll(cmdAll2,fullDataFileName,filenameMasked+".txt");
    
    cmdAll2= StringUtils.replaceAll(cmdAll2,fullGraphicOutputFileName, filenameMasked);

    return cmdAll2;
  }
  
  public IEngineResults getData() {
    return data;
  }
}
