/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: OneBoxPlot.java,v 1.3 2006/03/19 11:58:05 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.vis.generic;

import java.util.Iterator;
import java.util.List;

import prompt.engine.EngineResults;
import prompt.engine.IEngineResults;
import prompt.vis.VisualisationR;
import tschmidt.rinterface.RException;
import tschmidt.rinterface.RExceptionNotEnoughData;
import tschmidt.rinterface.vis.Boxplot;



public class OneBoxPlot extends VisualisationR {

    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	public Boxplot boxplot = new Boxplot();
    /**
     * @param data
     */
    public void init(IEngineResults data) {
        super.init(data);
        par.setLas(0);

        boxplot.setFirst("data[,1]");
        boxplot.setAdditional("names=c(\"" + data.getDesc1()+"\")");
    }


    public void init(List listA,  String descA) throws Exception {
        if (listA == null) {
            throw new RExceptionNotEnoughData("listA is null");
        }
    
        if (listA.size() < 10) {
            throw new RExceptionNotEnoughData("Less than 10 data values from setA");
        }


        StringBuffer str = new StringBuffer();
        str.append("VALUE\n");
        for (Iterator iter1 = listA.iterator(); iter1.hasNext();) {
            Number element = (Number) iter1.next();
            str.append(element.toString() + "\n");
        }
      
        IEngineResults testData = new EngineResults();

        testData.setResult(str.toString(), descA, "", false, this.getClass().getName());
        init(testData);
        
    }

    public void init(Number[] listA,  String descA ) throws Exception {
        if (listA == null) {
            throw new RExceptionNotEnoughData("listA is null");
        }
       
        if (listA.length < 10) {
            throw new RExceptionNotEnoughData("Less than 10 data values from setA");
        }
       

        StringBuffer str = new StringBuffer();
        str.append("VALUE\n");
        for (int i = 0; i < listA.length; i++) {
            Number element = listA[i];
            str.append( element.toString() + "\n");
        }
       
        IEngineResults testData = new EngineResults();
        testData.setResult(str.toString(), descA, "", false, this.getClass().getName());
        init(testData);

    }

    public void init(String[] listA,  String descA) throws Exception {
        if (listA == null) {
            throw new RExceptionNotEnoughData("listA is null");
        }
       
        if (listA.length < 10) {
            throw new RExceptionNotEnoughData("Less than 10 data values from setA");
        }
       

        StringBuffer str = new StringBuffer();
        str.append("VALUE\n");
        for (int i = 0; i < listA.length; i++) {
            String element = listA[i];
            str.append( element + "\n");
        }

        IEngineResults testData = new EngineResults();
        testData.setResult(str.toString(), descA, "", false, this.getClass().getName());
        init(testData);

    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return 0;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.jdrm.JobTemplate#run()
     */
    public void run() throws RException {

        String rcmds = boxplot.toString()+"\n";
        performPlotting(rcmds);

    }

}
