/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: ScatterPlot.java,v 1.4 2006/03/19 11:58:05 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.vis.generic;

import prompt.engine.EngineResults;
import prompt.engine.IEngineResults;
import prompt.vis.VisualisationR;
import tschmidt.rinterface.RException;
import tschmidt.rinterface.RExceptionNotEnoughData;
import tschmidt.rinterface.vis.Plot;

public class ScatterPlot extends VisualisationR {

    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	public Plot plot = new Plot();
    /**
     * @param data
     */
    public void init(IEngineResults data) {
        super.init(data);
        par.setLas(0);

        plot.setFirst("data$COL0, data$COL1");
        title.setXlab(data.getDesc1());
        title.setYlab(data.getDesc2());
        plot.setXlab("");
        plot.setYlab("");
        //logger.debug(data);
       // plot.setAdditional("legend=c(\"" + data.getDesc1()+ "\",\"" + data.getDesc2() + "\")");
    }

    public void init(IEngineResults data,int colNrXAxis, int colNrYAxis) {
      super.init(data);
      par.setLas(0);
      title.setXlab(data.getDesc1());
      title.setYlab(data.getDesc2());
      plot.setXlab("");
      plot.setYlab("");
     // plot.setAdditional("legend=c(\"" + data.getDesc1()+ "\",\"" + data.getDesc2() + "\")");
      setColsToPlot(colNrXAxis,colNrYAxis);
  }   
    
    /**
     * Aufzurufen nach INIT!, ueberschreibt plot.setFirst Werte
     * ERSTE SPALTE HAT DIE NUMMER 1!
     * default wird die erste Spalte auf die X-Achse und die zweite Spalte auf die Y-Achse geplottet
     * @param colNrXAxis
     * @param colNrYAxis
     */
    public void setColsToPlot(int colNrXAxis, int colNrYAxis) {
      plot.setFirst("data[,"+colNrXAxis+"],data[,"+colNrYAxis+"]");
    }

    
    public void init(int[][] values, String descA, String descB) throws Exception {
      if (values == null) {
          throw new RExceptionNotEnoughData("values is null");
      }
      if (values.length == 0) {
        throw new RExceptionNotEnoughData("values is empty");
    }
      int anzCols = values[0].length;

      StringBuffer str = new StringBuffer();
      for (int i = 0; i < anzCols; i++) {
        if (i>0) str.append("\t");
        str.append("COL"+i);
      }
      str.append("\n");
      for (int j = 0; j < values.length; j++) {
        int[] numbers = values[j];
        for (int i = 0; i < numbers.length; i++) {
          int number = numbers[i];
          if (i>0) str.append("\t");
          if (number==number) {
            str.append(number);
          } else {
            str.append("NaN");
          }
        }
        str.append("\n");
      }
      IEngineResults plotData = new EngineResults();
      plotData.setResult(str.toString(), descA, descB, false, this.getClass().getName());
      init(plotData);
      // logger.debug(testData);
      // init(testData,"data[data$DATASET==\"setA\",2] ,
      // data[data$DATASET==\"setB\",2]");
  }

    public void init(double[][] values, String descA, String descB) throws Exception {
      if (values == null) {
          throw new RExceptionNotEnoughData("values is null");
      }
      if (values.length == 0) {
        throw new RExceptionNotEnoughData("values is empty");
    }
      int anzCols = values[0].length;

      StringBuffer str = new StringBuffer();
      for (int i = 0; i < anzCols; i++) {
        if (i>0) str.append("\t");
        str.append("COL"+i);
      }
      str.append("\n");
      for (int j = 0; j < values.length; j++) {
        double[] numbers = values[j];
        for (int i = 0; i < numbers.length; i++) {
          double number = numbers[i];
          if (i>0) str.append("\t");
          if (number==number) {
            str.append(number);
          } else {
            str.append("NaN");
          }
        }
        str.append("\n");
      }
      IEngineResults plotData = new EngineResults();
      plotData.setResult(str.toString(), descA, descB, false, this.getClass().getName());
      init(plotData);
      // logger.debug(testData);
      // init(testData,"data[data$DATASET==\"setA\",2] ,
      // data[data$DATASET==\"setB\",2]");
  }
    
    
    public void init(Number[][] values, String descA, String descB) throws Exception {
        if (values == null) {
            throw new RExceptionNotEnoughData("values is null");
        }
        if (values.length == 0) {
          throw new RExceptionNotEnoughData("values is empty");
      }
        int anzCols = values[0].length;

        StringBuffer str = new StringBuffer();
        for (int i = 0; i < anzCols; i++) {
          if (i>0) str.append("\t");
          str.append("COL"+i);
        }
        str.append("\n");
        for (int j = 0; j < values.length; j++) {
          Number[] numbers = values[j];
          for (int i = 0; i < numbers.length; i++) {
            Number number = numbers[i];
            if (i>0) str.append("\t");
            if (number!=null) {
              str.append(number);
            } else {
              str.append("NaN");
            }
          }
          str.append("\n");
        }
        IEngineResults plotData = new EngineResults();
        plotData.setResult(str.toString(), descA, descB, false, this.getClass().getName());
        init(plotData);
        // logger.debug(testData);
        // init(testData,"data[data$DATASET==\"setA\",2] ,
        // data[data$DATASET==\"setB\",2]");
    }

    public void init(String[][] values, String descA, String descB) throws Exception {
      if (values == null) {
        throw new RExceptionNotEnoughData("values is null");
    }
    if (values.length == 0) {
      throw new RExceptionNotEnoughData("values is empty");
  }
    int anzCols = values[0].length;
   

    StringBuffer str = new StringBuffer();
    for (int i = 0; i < anzCols; i++) {
      if (i>0) str.append("\t");
      str.append("COL"+i);
    }
    str.append("\n");
    for (int j = 0; j < values.length; j++) {
      String[] strings = values[j];
      for (int i = 0; i < strings.length; i++) {
        String string = strings[i];
        if (i>0) str.append("\t");
        if (string!=null) {
          str.append(string);
        } else {
          str.append("NaN");
        }
      }
      str.append("\n");
    }
    IEngineResults plotData = new EngineResults();
    plotData.setResult(str.toString(), descA, descB, false, this.getClass().getName());
    init(plotData);
    // logger.debug(testData);
    // init(testData,"data[data$DATASET==\"setA\",2] ,
    // data[data$DATASET==\"setB\",2]");
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return 0;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.tum.binfo.jdrm.JobTemplate#run()
     */
    public void run() throws RException {

        String rcmds = plot.toString()+"\n";
        performPlotting(rcmds);

    }

}
