package prompt.vis.genericxml;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GradientPaint;
import java.io.IOException;
import java.text.NumberFormat;
import java.util.ArrayList;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.CategoryLabelPositions;
import org.jfree.chart.labels.AbstractCategoryItemLabelGenerator;
import org.jfree.chart.labels.CategoryItemLabelGenerator;
import org.jfree.chart.labels.ItemLabelAnchor;
import org.jfree.chart.labels.ItemLabelPosition;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.chart.renderer.category.CategoryItemRenderer;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.ui.RefineryUtilities;
import org.jfree.ui.TextAnchor;

import prompt.engine.IEngineResults;
import prompt.vis.IVisualisation;

public class CompareNumericInteractive implements IVisualisation {

	IEngineResults engineResData = null;

	static Logger logger = Logger.getLogger(CompareNumericInteractive.class);

	CategoryDataset jfreeData;

	boolean use3D = false;

	boolean showOnlySignificant = true;

	public void init(IEngineResults dataToPlot) {
		engineResData = dataToPlot;
		int ret = JOptionPane
				.showConfirmDialog(
						null,
						"<html>Show only significantly different categories (p-value less 0.05)<br>(No means show all)?</html>",
						"Significance Filter", JOptionPane.YES_NO_OPTION);
		if (ret == JOptionPane.YES_OPTION)
			showOnlySignificant = true;
		else
			showOnlySignificant = false;

		jfreeData = createDataset(engineResData);
	}

	public void saveDataToFile(String filename) throws IOException {
		engineResData.saveDataToFile(filename);
	}

	public String getProcessingMessages() {
		// Any errors that occurred can be given back here
		return "";
	}

	public void saveToEps(String filename) {
		logger.info("This method is not yet enabled");
	}

	public void saveToPng(String filename) {
		logger.info("This method is not yet enabled");
	}

	public void saveToEmf(String filename) {
		logger.info("This method is not yet enabled");
	}

	public void cleanUp() {
		logger.debug("no clean up necessary so far");
	}

	private void fillArray(String line, double[] dest) {
		String[] str = line.split("\t", line.length());
		logger.debug("values to parse " + (str.length - 1));
		for (int i = 1; i < str.length; i++) { // erste Spalte enthlt die
												// Beschreibung
			String curStr = str[i];
			int indexDest = i - 1;
			if (indexDest < dest.length) {
				double dval = Double.parseDouble(curStr);
				dest[indexDest] = dval;
			} else {
				logger.warn("more values than columns");
			}
		}
	}

	String xAxis;

	String yAxis = "relative fraction";

	String[] trySplit(String str) {
		String[] tmp = str.split("\\|", str.length());
		// for (String string : tmp) {
		// logger.debug("##"+string+"**");
		// }
		if (tmp.length == 0)
			return null;
		if (tmp.length != 2)
			return null;
		String first = tmp[0].trim();
		String second = tmp[1].trim();
		if ((first.length() > 0) && (second.length() > 0))
			return new String[] { first, second };
		return null;

	}

	 ArrayList<Double> pvalues;

	static class LabelGenerator extends AbstractCategoryItemLabelGenerator implements CategoryItemLabelGenerator {

		 /**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		ArrayList<Double> pvalues;
		
		public LabelGenerator(ArrayList pvalues) {
			super("", NumberFormat.getInstance());
			this.pvalues=pvalues;
		}

		/**
		 * Generates a label for the specified item. The label is typically a
		 * formatted version of the data value, but any text can be used.
		 * 
		 * @param dataset
		 *            the dataset (<code>null</code> not permitted).
		 * @param series
		 *            the series index (zero-based).
		 * @param category
		 *            the category index (zero-based).
		 * 
		 * @return the label (possibly <code>null</code>).
		 */
		public String generateLabel(CategoryDataset dataset, int series, int category) {

			String result = null;
			//Number value = dataset.getValue(series, category);
			//logger.debug(value+" "+series+" "+category);
			
			if (category>pvalues.size()) return null; //avoid array out of bounds if no pvals were calculated 
			double pvalue = pvalues.get(category);
			if (series==2) {
				if (pvalue!=pvalue) return null; // NaN;
				if (pvalue<0.05) result="*";
			if (pvalue<0.001) result="**";
			if (pvalue<0.0001) result="***";
			}
			return result;

		}
	}

	/**
	 * Returns a sample dataset.
	 * 
	 * @return The dataset.
	 */
	private CategoryDataset createDataset(IEngineResults eRes) {

		// parse values out of compareNumeric Result
		String all = eRes.getData();
		logger.debug("*" + all + "*");
		String[] lines = all.split("\n", all.length());
		logger.debug("lines " + lines.length);
		String[] cols = lines[0].split("\t", lines[0].length());

		ArrayList<String> catNames = new ArrayList<String>();
		for (String string : cols) {
			if ((string != null) && (string.trim().length() > 0)) {
				String curName = string.trim();
				// remove start/end " falls vorhanden
				if (curName.startsWith("\""))
					curName = curName.substring(1);
				if (curName.endsWith("\""))
					curName = curName.substring(0, curName.length() - 1);
				catNames.add(curName);
			}
		}

		final int VAL_RELA = 0;
		final int VAL_RELB = 1;
		final int VAL_REL_DIFF = 2;
	    final int VAL_PVAL = 3;
		double[][] values = new double[4][catNames.size()];
	

		fillArray(lines[2], values[VAL_RELA]);
		fillArray(lines[4], values[VAL_RELB]);
		fillArray(lines[5], values[VAL_REL_DIFF]);
		fillArray(lines[6], values[VAL_PVAL]);

		// row keys...
		String desSetA = eRes.getDesc1(); // "setA";
		String desSetB = eRes.getDesc2();

		String[] ofA = trySplit(desSetA);
		String[] ofB = trySplit(desSetB);
		if ((ofA != null) && (ofB != null)) {
			if (ofA[0].toUpperCase().equals(ofB[0].toUpperCase())) {
				xAxis = ofA[0];
				desSetA = ofA[1];
				desSetB = ofB[1];
			}

		}

		String desDiff = "Difference";

		// create the dataset...
		DefaultCategoryDataset dataset = new DefaultCategoryDataset();

		int min = values[VAL_RELA].length;
		if (min > values[VAL_RELB].length)
			min = values[VAL_RELB].length;
		if (min > values[VAL_REL_DIFF].length)
			min = values[VAL_REL_DIFF].length;
		if (min > values[VAL_PVAL].length)
			min = values[VAL_PVAL].length;

		pvalues = new ArrayList<Double>();
		
		for (int i = 0; i < min; i++) {
			double pval = values[VAL_PVAL][i];
			boolean add = true;
			if (showOnlySignificant) {
				if ((pval > 0.05) || (pval!=pval))
					add = false;
				logger.info("do not show '" + catNames.get(i) + "' as the difference here is with a p value of " + pval
						+ " not significant");
			}
			if (add) {
				pvalues.add(new Double(pval));
				double curVala = values[VAL_RELA][i];
				dataset.addValue(curVala, desSetA, catNames.get(i));
				double curValb = values[VAL_RELB][i];
				dataset.addValue(curValb, desSetB, catNames.get(i));
				double curValdiff = values[VAL_REL_DIFF][i];
				dataset.addValue(curValdiff, desDiff, catNames.get(i));
			}
		}


		
		return dataset;

	}

	public void showGraphic() {

		BarChartDemo1 demo = new BarChartDemo1(engineResData.getResultType(), jfreeData, xAxis, yAxis, pvalues);
		demo.pack();
		RefineryUtilities.centerFrameOnScreen(demo);
		demo.setVisible(true);
	}

	public String getCmdAll(String filename) {
		return "No plotting cmds as this is an interactive plot";
	}

	public IEngineResults getData() {
		return engineResData;
	}

	public void run() throws Exception {
		// TODO parse data and generate data table for JfreeChart
	}

	public class BarChartDemo1 extends JFrame {

		 /**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		ArrayList<Double> pvalues;
		/**
		 * Creates a new demo instance.
		 * 
		 * @param title
		 *            the frame title.
		 */
		public BarChartDemo1(String title, CategoryDataset dataset, String xAxis, String yAxis, ArrayList pvalues) {
			super(title);
			this.pvalues=pvalues;
			JFreeChart chart = createChart(dataset, xAxis, yAxis);
			ChartPanel chartPanel = new ChartPanel(chart, false);
			chartPanel.setPreferredSize(new Dimension(700, 420));
			setContentPane(chartPanel);
		}

		/**
		 * Creates a sample chart.
		 * 
		 * @param dataset
		 *            the dataset.
		 * 
		 * @return The chart.
		 */
		private JFreeChart createChart(CategoryDataset dataset, String xAxis, String yAxis) {
			JFreeChart chart;
			if (use3D) {
				// create the chart...
				chart = ChartFactory.createBarChart3D("", // chart title
						xAxis, // domain axis label
						yAxis, // range axis label
						dataset, // data
						PlotOrientation.VERTICAL, // orientation
						true, // include legend
						true, // tooltips?
						false // URLs?
						);
			} else {
				chart = ChartFactory.createBarChart("", // chart title
						xAxis, // domain axis label
						yAxis, // range axis label
						dataset, // data
						PlotOrientation.VERTICAL, // orientation
						true, // include legend
						true, // tooltips?
						false // URLs?
						);
			}
			// NOW DO SOME OPTIONAL CUSTOMISATION OF THE CHART...

			// set the background color for the chart...
			chart.setBackgroundPaint(Color.white);

			// get a reference to the plot for further customisation...
			CategoryPlot plot = chart.getCategoryPlot();
			plot.setBackgroundPaint(Color.lightGray);
			plot.setDomainGridlinePaint(Color.white);
			plot.setDomainGridlinesVisible(true);
			plot.setRangeGridlinePaint(Color.white);

			// set the range axis to display integers only...
			// NumberAxis rangeAxis = (NumberAxis) plot.getRangeAxis();
			// rangeAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());

			// disable bar outlines...
			BarRenderer renderer = (BarRenderer) plot.getRenderer();
			renderer.setItemMargin(0.05);
			renderer.setDrawBarOutline(true);

			// set up gradient paints for series...
			GradientPaint gp0 = new GradientPaint(0.0f, 0.0f, Color.blue, 0.0f, 0.0f, new Color(0, 0, 64));
			GradientPaint gp1 = new GradientPaint(0.0f, 0.0f, Color.green, 0.0f, 0.0f, new Color(0, 64, 0));
			GradientPaint gp2 = new GradientPaint(0.0f, 0.0f, Color.red, 0.0f, 0.0f, new Color(64, 0, 0));
			renderer.setSeriesPaint(0, gp0);
			renderer.setSeriesPaint(1, gp1);
			renderer.setSeriesPaint(2, gp2);

			CategoryAxis domainAxis = plot.getDomainAxis();
			// logger.debug(domainAxis.getCategoryMargin());
			domainAxis.setCategoryMargin(0.33);
			domainAxis.setCategoryLabelPositions(CategoryLabelPositions.createUpRotationLabelPositions(Math.PI / 6.0));
			// OPTIONAL CUSTOMISATION COMPLETED.

			
			  CategoryItemRenderer renderer2 = plot.getRenderer();
		        renderer2.setItemLabelGenerator(new LabelGenerator(pvalues));
		        renderer2.setItemLabelFont(new Font("Serif", Font.PLAIN, 14));
		        renderer2.setItemLabelsVisible(true);
		        
		        ItemLabelPosition ipos = new ItemLabelPosition(
		        		ItemLabelAnchor.OUTSIDE1, TextAnchor.BOTTOM_CENTER, 
			            TextAnchor.CENTER, 
                - Math.PI / 2);
		        
		        renderer2.setPositiveItemLabelPosition(ipos); //siehe buch s.83
		        renderer2.setNegativeItemLabelPosition(ipos);
			return chart;

		}

	}

}
