package prompt.vis.genericxml;

import gr.zeus.ui.JMessage;

import java.awt.Color;
import java.awt.Dimension;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.ui.RefineryUtilities;

import prompt.engine.IEngineResults;
import prompt.vis.IVisualisation;

public class CorrelationNumericInteractive implements IVisualisation {

	IEngineResults engineResData = null;

	Logger logger = Logger.getLogger(CompareNumericInteractive.class);

	XYDataset jfreeData;

	public void init(IEngineResults dataToPlot) {
		engineResData = dataToPlot;
		jfreeData = createDataset(engineResData);
	}

	public void saveDataToFile(String filename) throws IOException {
		engineResData.saveDataToFile(filename);
	}

	public String getProcessingMessages() {
		// Any errors that occurred can be given back here
		return "";
	}

	public void saveToEps(String filename) {
		logger.info("This method is not yet enabled");
	}

	public void saveToPng(String filename) {
		logger.info("This method is not yet enabled");
	}

	public void saveToEmf(String filename) {
		logger.info("This method is not yet enabled");
	}

	public void cleanUp() {
		logger.debug("no clean up necessary so far");
	}

	/*private void fillArray(String line, double[] dest) {
		String[] str = line.split("\t", line.length());
		logger.debug("values to parse " + (str.length - 1));
		for (int i = 1; i < str.length; i++) { // erste Spalte enthlt die
												// Beschreibung
			String curStr = str[i];
			int indexDest = i - 1;
			if (indexDest < dest.length) {
				double dval = Double.parseDouble(curStr);
				dest[indexDest] = dval;
			} else {
				logger.warn("more values than columns");
			}
		}
	} */

	String xAxis = "";

	String yAxis = "";

	String[] trySplit(String str) {
		String[] tmp = str.split(":", str.length());
		for (String string : tmp) {
			logger.debug("##" + string + "**");
		}
		if (tmp.length == 0)
			return null;
		if (tmp.length != 2)
			return null;
		String first = tmp[0].trim();
		String second = tmp[1].trim();
		if ((first.length() > 0) && (second.length() > 0))
			return new String[] { first, second };
		return null;

	}

	/**
	 * Returns a sample dataset.
	 * 
	 * @return The dataset.
	 */
	private XYDataset createDataset(IEngineResults eRes) {

		// parse values out of compareNumeric Result
		String all = eRes.getData();
		//logger.debug("*" + all + "*");
		String[] lines = all.split("\n", all.length());
		logger.debug("lines " + lines.length);
		String[] cols = lines[0].split("\t", lines[0].length());

		ArrayList<String> catNames = new ArrayList<String>();
		for (String string : cols) {
			if ((string != null) && (string.trim().length() > 0)) {
				String curName = string.trim();
				// remove start/end " falls vorhanden
				if (curName.startsWith("\""))
					curName = curName.substring(1);
				if (curName.endsWith("\""))
					curName = curName.substring(0, curName.length() - 1);
				catNames.add(curName);
			}
		}
		if (catNames.size() >= 3) {
			xAxis = catNames.get(1);
			yAxis = catNames.get(2);
		}
		if (catNames.size() == 2) {
			xAxis = catNames.get(0);
			yAxis = catNames.get(1);
		}
		if (catNames.size() < 2) {
			String errmsg = "You need at least 2 columns to plot";
			logger.fatal(errmsg);
			JMessage.showMessageDialog(null, errmsg, JOptionPane.ERROR_MESSAGE, JOptionPane.DEFAULT_OPTION);
			return null;
		}
		// wir koennten natuerlich auch die getDesc1 und 2 als Achsenbezeichner
		// nehmen...

		// create the dataset...
		XYSeriesCollection dataset = new XYSeriesCollection();

		XYSeries series1 = new XYSeries("Data");
		for (int i = 1; i < lines.length; i++) {
			String[] vals = lines[i].split("\t", lines[i].length());
			//logger.debug(vals.length);
			if (vals == null)
				continue;
			if (vals.length == 2) {
				if ((vals[0] == null) || (vals[1] == null))
					continue;
				try {
					double x = Double.parseDouble(vals[0].trim());
					double y = Double.parseDouble(vals[1].trim());
					series1.add(x,y);
				} catch (Exception e) {
					logger.debug("not a number at line "+i);
				}
			}
			if (vals.length >= 3) {
				if ((vals[1] == null) || (vals[2] == null))
					continue;
				try {
					double x = Double.parseDouble(vals[1].trim());
					double y = Double.parseDouble(vals[2].trim());
					series1.add(x,y);
				} catch (Exception e) {
					logger.debug("not a number at line "+i);
				}
			}
		}

		dataset.addSeries(series1);

		// return dataset;
		return dataset;
	}

	public void showGraphic() {

		Scatter1 demo = new Scatter1(engineResData.getResultType(), jfreeData, xAxis, yAxis);
		demo.pack();
		RefineryUtilities.centerFrameOnScreen(demo);
		demo.setVisible(true);
	}

	public String getCmdAll(String filename) {
		return "No plotting cmds as this is an interactive plot";
	}

	public IEngineResults getData() {
		return engineResData;
	}

	public void run() throws Exception {
		// TODO parse data and generate data table for JfreeChart
	}

	public class Scatter1 extends JFrame {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		/**
		 * Creates a new demo instance.
		 * 
		 * @param title
		 *            the frame title.
		 */
		public Scatter1(String title, XYDataset dataset, String xAxis, String yAxis) {
			super(title);
			JFreeChart chart = createChart(dataset, xAxis, yAxis);
			ChartPanel chartPanel = new ChartPanel(chart, false);
			chartPanel.setPreferredSize(new Dimension(700, 420));
			setContentPane(chartPanel);
		}

		/**
		 * Creates a sample chart.
		 * 
		 * @param dataset
		 *            the dataset.
		 * 
		 * @return The chart.
		 */
		private JFreeChart createChart(XYDataset dataset, String xAxis, String yAxis) {

			JFreeChart chart = ChartFactory.createScatterPlot("", xAxis, yAxis, dataset, PlotOrientation.VERTICAL,
					false, true, false);

			XYPlot plot = (XYPlot) chart.getPlot();
			plot.setNoDataMessage("NO DATA");
			XYLineAndShapeRenderer renderer = (XYLineAndShapeRenderer) plot.getRenderer();
			renderer.setSeriesOutlinePaint(0, Color.black);
			renderer.setUseOutlinePaint(true);

			NumberAxis domainAxis = (NumberAxis) plot.getDomainAxis();
			domainAxis.setAutoRangeIncludesZero(false);

			return chart;

		}

	}

}
