package prompt.vis.genericxml;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GradientPaint;
import java.io.IOException;

import javax.swing.JFrame;

import org.apache.log4j.Logger;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.CategoryLabelPositions;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.ui.RefineryUtilities;

import prompt.engine.IEngineResults;
import prompt.vis.IVisualisation;

public class FeatFrqInteractive implements IVisualisation {

	boolean use3D = false;
	
	IEngineResults engineResData = null;

	Logger logger = Logger.getLogger(FeatFrqInteractive.class);

	CategoryDataset jfreeData;

	public void init(IEngineResults dataToPlot) {
		engineResData = dataToPlot;
		jfreeData = createDataset(engineResData);
	}

	public void saveDataToFile(String filename) throws IOException {
		engineResData.saveDataToFile(filename);
	}

	public String getProcessingMessages() {
		// Any errors that occurred can be given back here
		return "";
	}

	public void saveToEps(String filename) {
		logger.info("This method is not yet enabled");
	}

	public void saveToPng(String filename) {
		logger.info("This method is not yet enabled");
	}

	public void saveToEmf(String filename) {
		logger.info("This method is not yet enabled");
	}

	public void cleanUp() {
		logger.debug("no clean up necessary so far");
	}

	String xAxis;

	String yAxis = "relative fraction";

	/**
	 * Returns a sample dataset.
	 * 
	 * @return The dataset.
	 */
	private CategoryDataset createDataset(IEngineResults eRes) {

		// parse values out of compareNumeric Result
		String all = eRes.getData();
		logger.debug("*" + all + "*");
		String[] lines = all.split("\n", all.length());
		logger.debug("lines " + lines.length);

		// row keys...
		String desSetA = eRes.getDesc1(); // "setA";
		//String desSetB = eRes.getDesc2();
		xAxis = eRes.getDesc2();
		
		// create the dataset...
		DefaultCategoryDataset dataset = new DefaultCategoryDataset();

		for (int i = 1; i < lines.length; i++) {
			if (lines[i] != null) {
				String[] row = lines[i].split("\t", lines[i].length());
				logger.debug(i+" "+row.length);
				if ((row != null) && (row.length >= 2)) {
					String curStr = row[1];
					String catName = row[0];
					if ((curStr != null) && (curStr.trim().length() > 0)) {
						double curVal = Double.parseDouble(curStr.trim());
						dataset.addValue(curVal, desSetA, catName);
						logger.debug("catName='"+catName+"'="+curVal);
					}
				}
			}
		}

		return dataset;

	}

	public void showGraphic() {
		logger.debug("use3D = "+use3D);
		BarChartDemo2 demo = new BarChartDemo2(engineResData.getResultType(), jfreeData, xAxis, yAxis);
		demo.pack();
		RefineryUtilities.centerFrameOnScreen(demo);
		demo.setVisible(true);
	}

	public String getCmdAll(String filename) {
		return "No plotting cmds as this is an interactive plot";
	}

	public IEngineResults getData() {
		return engineResData;
	}

	public void run() throws Exception {
		// TODO parse data and generate data table for JfreeChart
	}

	public class BarChartDemo2 extends JFrame {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		/**
		 * Creates a new demo instance.
		 * 
		 * @param title
		 *            the frame title.
		 */
		public BarChartDemo2(String title, CategoryDataset dataset, String xAxis, String yAxis) {
			super(title);
			JFreeChart chart = createChart(dataset, xAxis, yAxis);
			ChartPanel chartPanel = new ChartPanel(chart, false);
			chartPanel.setPreferredSize(new Dimension(700, 420));
			setContentPane(chartPanel);
		}

		/**
		 * Creates a sample chart.
		 * 
		 * @param dataset
		 *            the dataset.
		 * 
		 * @return The chart.
		 */
		private JFreeChart createChart(CategoryDataset dataset, String xAxis, String yAxis) {
			JFreeChart chart;
			if (use3D) {
			// create the chart...
				chart = ChartFactory.createBarChart3D("", // chart title
					xAxis, // domain axis label
					yAxis, // range axis label
					dataset, // data
					PlotOrientation.VERTICAL, // orientation
					true, // include legend
					true, // tooltips?
					false // URLs?
					);
			}
			else {
				chart = ChartFactory.createBarChart("", // chart title
						xAxis, // domain axis label
						yAxis, // range axis label
						dataset, // data
						PlotOrientation.VERTICAL, // orientation
						true, // include legend
						true, // tooltips?
						false // URLs?
						);	
			}

			// NOW DO SOME OPTIONAL CUSTOMISATION OF THE CHART...

			// set the background color for the chart...
			chart.setBackgroundPaint(Color.white);

			// get a reference to the plot for further customisation...
			CategoryPlot plot = chart.getCategoryPlot();
			plot.setBackgroundPaint(Color.lightGray);
			plot.setDomainGridlinePaint(Color.white);
			plot.setDomainGridlinesVisible(true);
			plot.setRangeGridlinePaint(Color.white);

			// set the range axis to display integers only...
			// NumberAxis rangeAxis = (NumberAxis) plot.getRangeAxis();
			// rangeAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());

			// disable bar outlines...
			BarRenderer renderer = (BarRenderer) plot.getRenderer();
			renderer.setItemMargin(0.05);
			renderer.setDrawBarOutline(true);

			// set up gradient paints for series...
			GradientPaint gp0 = new GradientPaint(0.0f, 0.0f, Color.blue, 0.0f, 0.0f, new Color(0, 0, 64));
			GradientPaint gp1 = new GradientPaint(0.0f, 0.0f, Color.green, 0.0f, 0.0f, new Color(0, 64, 0));
			GradientPaint gp2 = new GradientPaint(0.0f, 0.0f, Color.red, 0.0f, 0.0f, new Color(64, 0, 0));
			renderer.setSeriesPaint(0, gp0);
			renderer.setSeriesPaint(1, gp1);
			renderer.setSeriesPaint(2, gp2);

			CategoryAxis domainAxis = plot.getDomainAxis();
			// logger.debug(domainAxis.getCategoryMargin());
			domainAxis.setCategoryMargin(0.33);
			domainAxis.setCategoryLabelPositions(CategoryLabelPositions.createUpRotationLabelPositions(Math.PI / 6.0));
			// OPTIONAL CUSTOMISATION COMPLETED.

			return chart;

		}

	}

}
