/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: AaDistributionHistogram.java,v 1.5 2006/04/12 12:33:16 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.vis.seqproperties;

import prompt.engine.IEngineResults;
import prompt.ui.AaDialog;
import prompt.util.AALetter1To3;
import prompt.vis.VisualisationR;
import tschmidt.rinterface.RException;


public class AaDistributionHistogram extends VisualisationR {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	protected String acidToPlot;
	protected int setToPlot=0;
	public static final int SETA = 1;
	public static final int SETB = 2;
	
	/**
	 * @param data
	 */
	public void init(IEngineResults data) {
		super.init(data);
		par.setMar_bottom(7);
		par.setMar_left(6);
		par.setMgp_axis_title(4);
	}

	public void init(IEngineResults data, String acidToPlot)
		throws Exception {
		super.init(data);
		this.acidToPlot = acidToPlot;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {

		return 0;
	}

	/* (non-Javadoc)
	 * @see de.tum.binfo.jdrm.JobTemplate#run()
	 */
	public void run() throws RException { 
		if (acidToPlot == null) {
			String[] elements = AaDialog.getElementsFromDistribution(this.data);
			AaDialog aaDialog = new AaDialog();
			acidToPlot = aaDialog.showAaDialog(elements);
		}
		if (setToPlot==0) {
			AaDialog aaDialog = new AaDialog();
			setToPlot = aaDialog.showSelectSetDialog(data.getDesc1(),data.getDesc2());
		}


		if ((acidToPlot != null) && (setToPlot!=0)) {
						
			String ylab="number of sequences where "+acidToPlot+" occured with the given frequency";
			String xlab="Frequency of "+acidToPlot+"\\n (0% freq. not included)";
			
			String set="";String des="";
			if (setToPlot==SETA) {set="setA"; des=data.getDesc1();} else {set="setB";des=data.getDesc2();
			}
			
			
			String title ="Histogram of the frequency distribution of \\n"+(acidToPlot)+" in "+des;  //AALetter1To3.getLongAminoAcidName
			String rcmds =
				"hist(data[data$AA==\""
					+ acidToPlot
					+ "\" & data$DATASET==\""+set+"\",3],main=\""+title+"\",ylab=\""+ylab+"\",xlab=\""+xlab+"\" )\n";
			//logger.debug(rcmds);
			performPlotting(rcmds);
			

		}
	}

	/**
	 * @param string
	 */
	public void setAcidToPlot(String string) {
		acidToPlot = string;
	}

	/**
	 * @param i
	 */
	public void setSetToPlot(int i) {
		setToPlot = i;
	}

}
