/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: LengthComparePlot.java,v 1.4 2006/03/19 11:58:06 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.vis.seqproperties;

import org.apache.log4j.Logger;

import prompt.engine.IEngineResults;
import prompt.vis.VisualisationR;
import tschmidt.rinterface.RException;
import tschmidt.rinterface.vis.Barplot;

/*
 * in R: install.packages(c("gregmisc")), dann library(gplots)
 * 
 * @author schmidt
 *  
 */

public class LengthComparePlot extends VisualisationR {

 // String s = (String) JOptionPane.showInputDialog(null, "max x-Wert eingeben", "90");

  /**
	 * 
	 */
	private static final long serialVersionUID = 1L;

public Barplot barplot = new Barplot();
  
  protected String colorList;

  public LengthComparePlot() {
  }

  protected void doSettings() {
   
    
    par.setMgp_axis_title(4); //xlab und ylab weiter weg von der Plot region
    par.setMar_left(6); //links mehr platz
    par.setMar_bottom(7); //unten mehr platz
    
    par.setLab_y(9); //mehr ticks auf der y-achse
    
    title.setXlab("Length");
    title.setYlab("Fraction of the corresponding dataset");
    
    barplot.setFirst("m, cex.names=0.6, space=c(0,2), ylab=\"\",  beside=TRUE,"  
        + "legend.text=c(\"" + data.getDesc1() + "\",\"" + data.getDesc2()
        + "\",\"Difference\")");
    
    barplot.setYlim("ymin","ymax");
    
  }

  public void init(IEngineResults data) {
    super.init(data);
    doSettings();
  }

  /**
   * @param data
   * @throws Exception
   */
  public void init(IEngineResults data, String colorList) {
    super.init(data);
    this.colorList = colorList;
    doSettings();

  }

  public String color = "green";

  /*
   * (non-Javadoc)
   * 
   * @see java.lang.Object#hashCode()
   */
  public int hashCode() {

    return 0;
  }

  /*
   * (non-Javadoc)
   * 
   * @see de.tum.binfo.jdrm.JobTemplate#run()
   */
  public void run() throws RException {
    if (colorList == null) {
      colorList = "\"" + color + "\"";
    }
    Logger logger = Logger.getLogger(this.getClass());
    logger.debug("par:" + par);
    logger.debug("title:" + title);
    logger.debug("barplot:"+barplot);
    
    String rcmds = "dataPlot=data[c(2,4,5),]\n"
        + "m=as.matrix(dataPlot)\n"
        + "ymin=min(m)+( min(m)*0.1)\n"
        + "ymax=max(m)+( max(m)*0.1)\n"
        + barplot+"\n"; //box()
    //logger.debug(rcmds);
    performPlotting(rcmds);
  }
}
