/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: EC_Visualization.java,v 1.5 2006/03/19 11:58:11 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.vis.swissprot;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;
import org.jfree.chart.ChartRenderingInfo;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.entity.StandardEntityCollection;
import org.jfree.chart.labels.StandardCategoryToolTipGenerator;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.chart.urls.CategoryURLGenerator;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;

import prompt.engine.IEngineResults;
import prompt.engine.swissProt.SwissProtEngineResult;
import prompt.vis.IVisualisation;

public class EC_Visualization implements IVisualisation {

	private SwissProtEngineResult ecResult;

	private Hashtable ec;

	private int width = 800, height = 600;
	private File storage = new File(System.getProperty("user.dir") + System.getProperty( "file.separator" ) + 
								"EC" + System.getProperty( "file.separator" ));
	
	
	public void generateAllCharts(int col1, int col2){
		TreeSet[] sortedByLength = new TreeSet[]{new TreeSet(), new TreeSet(), new TreeSet(), new TreeSet()};
        Iterator keys = ec.keySet().iterator();
		String key, curStart, oldStart="root";

		TreeSet cur = null;
		while (keys.hasNext()) {
			key = keys.next().toString();
			String[] x = key.split("\\-");
			((TreeSet)sortedByLength[x.length-1]).add(key);
		}
		makeBarChart(sortedByLength[0], "root", col1, col2);//1.Stelle der EC-Nr. muss nicht weiter gefiltert werden

		htmlForCategories("root");
		Iterator curLen;
		for (int i = 1; i < sortedByLength.length; i++) {// auftrennen nach Kategorien
			curLen = sortedByLength[i].iterator();
			while (curLen.hasNext()) {
				key = curLen.next().toString();
				curStart = key.substring(0, key.lastIndexOf("-"));
				if(!curStart.equals(oldStart)){
					if(cur!=null){
						makeBarChart(cur, oldStart, col1, col2);
						htmlForCategories(oldStart);
					}
					cur = new TreeSet();
					oldStart = curStart;
				}
				cur.add(key);
			}
		}
	}

	public void makeBarChart(TreeSet keys, String name, int column1, int column2)
	{
		Iterator key = keys.iterator();
		String curKey;
		JFreeChart chart = null;
		DefaultCategoryDataset dataset = new DefaultCategoryDataset();
		for (int i = 0; i < keys.size(); i++) {
			// value, seta or setb, ecnr
			curKey = key.next().toString();
			dataset.setValue(((float[]) (ec.get(curKey)))[column1], "Set A",
					curKey);
			dataset.setValue(((float[]) (ec.get(curKey)))[column2], "Set B",
					curKey);
		}
		final CategoryAxis categoryAxis = new CategoryAxis("EC-Numbers");
		final ValueAxis valueAxis = new NumberAxis("fraction");
		final BarRenderer renderer = new BarRenderer();
		renderer.setToolTipGenerator(new StandardCategoryToolTipGenerator());
        renderer.setItemMargin(0);
		renderer.setItemURLGenerator(new CategoryURLGenerator() {
			public Object clone() throws CloneNotSupportedException {
				return null;
			}

			public String generateURL(CategoryDataset data, int ser, int cat) {
				String result = "";
				Comparable categoryKey = data.getColumnKey(cat);
				result += categoryKey.toString()+ "-" + ser + ".html";
		
				return result;
			}
		});
        final CategoryPlot plot = new CategoryPlot(dataset, categoryAxis, valueAxis, renderer);
        plot.setOrientation(PlotOrientation.HORIZONTAL);
        chart = new JFreeChart(name, JFreeChart.DEFAULT_TITLE_FONT, plot, true);
        try {
        	final ChartRenderingInfo info = new ChartRenderingInfo(new StandardEntityCollection());
            File png = new File(storage + System.getProperty("file.separator") + name + ".png");
            ChartUtilities.saveChartAsPNG(png, chart, width, height, info);            
            File html = new File(storage + System.getProperty("file.separator") + name + ".html");
            final OutputStream out = new BufferedOutputStream(new FileOutputStream(html));
            final PrintWriter writer = new PrintWriter(out);
            writer.println("<HTML>");
            writer.println("<HEAD><TITLE>" + name + "</TITLE></HEAD>");
            writer.println("<BODY>");
            ChartUtilities.writeImageMap(writer, "chart", info, false);
            writer.println("<IMG SRC=\"" + storage + System.getProperty("file.separator") + name + ".png\""
                           + "WIDTH=\"" + width + "\" HEIGHT=\"" + height + "\" BORDER=\"0\" USEMAP=\"#chart\"><br>");
            writer.println("</BODY>");
            writer.println("</HTML>");
            writer.close();

        }
        catch (IOException e) {
            Logger logger = Logger.getLogger(this.getClass());
            logger.debug("IO Error",e);
        }
	}

	public void htmlForCategories(String ec) {
		String line1, line2, top, bottom, right, newFile = "";
		Pattern pat = Pattern
				.compile("coords=\"(.*)\" title=\"\\(.*, (.*)\\).*href=\"(.*)-");
		// group1 = coords, group2 = ec, group3 = ec
		Matcher mat;
		StringTokenizer coords1, coords2;
		try 
		{
			BufferedReader in = new BufferedReader(new FileReader(storage + System.getProperty("file.separator") + ec + ".html"));
			while(in.ready())
			{
				line1 = in.readLine();
				if (line1.toLowerCase().startsWith("<area")) {
					line2 = in.readLine();
					newFile += line1 + line2;
					mat = pat.matcher(line1.toLowerCase());
					mat.find();
					coords1 = new StringTokenizer(mat.group(1), ",");
					mat = pat.matcher(line2.toLowerCase());
					mat.find();
					coords2 = new StringTokenizer(mat.group(1), ",");
					right = coords1.nextToken();
					top = coords1.nextToken(); // coordinate 2 choosen
					coords2.nextToken();
					coords2.nextToken();
					coords2.nextToken();
					bottom = coords2.nextToken(); // coordinate 4 choosen
					newFile += "<AREA SHAPE=\"RECT\" COORDS=\"0," + top + ", "
							+ right + ", " + bottom + "\" title=\""
							+ mat.group(2) + " = 0\" href=\"" + mat.group(3)
							+ ".html\">";
				} else {
					newFile += line1;
				}
			}
			in.close();
			FileWriter out = new FileWriter(storage + System.getProperty("file.separator") + ec + ".html");
			out.write(newFile);
			out.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
		}
	}
    
	public void init(IEngineResults dataToPlot) {
		this.ec = ((SwissProtEngineResult) dataToPlot).getResultTable();
		this.ecResult = (SwissProtEngineResult) dataToPlot;
		if (!this.storage.exists())
			this.storage.mkdirs();
	}

	public int getHeight() {
        return height;
    }

    public void setHeight(int height) {
        this.height = height;
    }

    public int getWidth() {
        return width;
    }

    public void setWidth(int width) {
        this.width = width;
    }

    public void saveDataToFile(String filename) throws IOException {
		// TODO Auto-generated method stub

	}

	public String getProcessingMessages() {
		// TODO Auto-generated method stub
		return null;
	}

	public void saveToEps(String filename) {
		// TODO Auto-generated method stub

	}

	public void saveToPng(String filename) {
		// TODO Auto-generated method stub

	}

	public void saveToEmf(String filename) {
		// TODO Auto-generated method stub

	}

	public void cleanUp() {
		// TODO Auto-generated method stub

	}

	public void showGraphic() {
		// TODO Auto-generated method stub

	}

	public String getCmdAll(String filename) {
		return "No external tools used !!!";
	}

	public IEngineResults getData() {
		return ecResult;
	}

	public void run() throws Exception {
		generateAllCharts(2,3);//2,3 means to plot the relRatios of each group; --> [2,3] of the float[]

	}
    
   /* public static void main(String[] args) throws Exception{
        SwissProt swissProt1 = new SwissProt();
        SwissProt swissProt2 = new SwissProt();
        swissProt1.init("/home/users/siebert/SwissProt/high.txt");
        swissProt2.init("/home/users/siebert/SwissProt/low.txt");
        swissProt1.run();
        swissProt2.run();
        
        PropertyComparison prop = new PropertyComparison();
        prop.init(swissProt1, swissProt2);
        prop.run();
        
        SwissProtEngineResult[] ss = (SwissProtEngineResult[])prop.getResultsAll();
                
        EC_Visualization test = new EC_Visualization();
        test.init(ss[0]);
        test.run();
        
        
        
    }*/

}
