/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: HierarchicalSwissProtVisualisation.java,v 1.5 2006/03/19 11:58:11 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.vis.swissprot;

import java.awt.Font;
import java.awt.geom.Rectangle2D;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.TreeMap;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JTextPane;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;

import org.jfree.chart.ChartMouseEvent;
import org.jfree.chart.ChartMouseListener;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.CategoryLabelPositions;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.entity.CategoryItemEntity;
import org.jfree.chart.entity.ChartEntity;
import org.jfree.chart.labels.StandardCategoryToolTipGenerator;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.chart.renderer.category.BarRenderer3D;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.ui.RefineryUtilities;
import org.jibble.epsgraphics.EpsGraphics2D;

import prompt.engine.EngineResults;
import prompt.engine.IEngineResults;
import prompt.engine.swissProt.SwissProtEngineResult;
import prompt.vis.IVisualisation;


public class HierarchicalSwissProtVisualisation implements IVisualisation{

    private final String[] VALUE_TYPE = {"absolute frequency", "absolute frequency", "relative frequency", "relative frequency"};
    private final ArrayList<String> resultTypes = new ArrayList(2);
    private final String[] resultDescriptions = {"EC (Enzyme Commission) numbers","OC (Organism Classification)"};
    
	private SwissProtEngineResult result;
    private Hashtable hash;
    private String setA;
    private String setB;
    private String description;
    private ChartPanel chartPanel;
    private JFrame frame = new JFrame();
    private int colSetA = 2;
    private int colSetB = 3;
    private boolean possibleToClickOnBar;
    private boolean swingCanceled = false;
    private boolean guiSetupOk =false;
    
    /*************************************************/
    private JComboBox visComboBox = new JComboBox();
	private JDialog dialog = new JDialog();
	/*************************************************/
	
	
	public HierarchicalSwissProtVisualisation(){
		resultTypes.add(0, "ec");
		resultTypes.add(1, "oc");
		this.hash=new Hashtable();
	}
	
	public void init(IEngineResults dataToPlot) {
		if(dataToPlot.getClass()==new SwissProtEngineResult("").getClass()){
        this.result = (SwissProtEngineResult) dataToPlot;
        this.hash = this.result.getResultTable();
        this.setA = (String) this.result.getDescriptions().get(0);
        this.setB = (String) this.result.getDescriptions().get(1);
        this.description = this.result.getDescription();
    }else{
    	
    	EngineResults tempResult = (EngineResults)dataToPlot;
		String data = tempResult.getData();
		try{
			BufferedReader in = new BufferedReader(new StringReader(data));
			String currentLine ;
			boolean foundHeader = false;
			while(in.ready()){
				currentLine=in.readLine();
				
					if(foundHeader){
						if(currentLine == null){break;}
						
						String[] hashInput = currentLine.split("\t");
						float[] values ={ Float.parseFloat(hashInput[1]),Float.parseFloat(hashInput[2]),Float.parseFloat(hashInput[3]),Float.parseFloat(hashInput[4]),Float.parseFloat(hashInput[5]),Float.parseFloat(hashInput[6]),Float.parseFloat(hashInput[7])};
					this.hash.put(hashInput[0],values);
						
					}
					else{
						foundHeader=true;
						}
				
				
			}ArrayList tempDescriptions = new ArrayList();
			tempDescriptions.add(0,this.setA);
			tempDescriptions.add(1, this.setB);
			this.setA=dataToPlot.getDesc1();
			this.setB=dataToPlot.getDesc2();
			this.description = resultDescriptions[resultTypes.indexOf(dataToPlot.getResultType().split(":")[1])];
			this.result = new SwissProtEngineResult(this.description, dataToPlot.toString(), this.hash,tempDescriptions, dataToPlot.getResultType());
		}
		catch(Exception e){
			e.printStackTrace();
		}
    }
		}

    public void run() throws Exception {
//    	DataTable deTable = new DataTable(result, description);
//    	deTable.showTable();
    	swingCanceled=false;
    	guiSetupOk=false;
    		
    	
        JButton okButton = new JButton("Ok");
        JButton cancelButton = new JButton("Cancel");
        StyleContext.NamedStyle centerStyle = StyleContext.getDefaultStyleContext().new NamedStyle(); 
        StyleConstants.setAlignment(centerStyle,StyleConstants.ALIGN_CENTER);              
        JTextPane description = new JTextPane();
        description.setContentType("text/html");
        description.setText("<html align=\"center\">Specify the value which shall be used for visualisation.</html>");
        description.setFont(new Font("Serif", Font.PLAIN, 12));
        description.setBounds(10, 10, 175, 55);
        dialog.setSize(200,180);
        dialog.setModal(true);
        dialog.getContentPane().setLayout(null);
        dialog.setTitle("Setup");
        visComboBox.setBounds(10, 75, 175, 30);
        visComboBox.addItem("absolute frequency");
        visComboBox.addItem("relative frequency");
        visComboBox.setSelectedIndex(1);
        dialog.getContentPane().add(visComboBox);
        dialog.getContentPane().add(description);  
        dialog.getContentPane().add(okButton);
        okButton.setBounds(10, 115, 85, 25);
            okButton.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                  int index= visComboBox.getSelectedIndex();
                  if(index==0){
                	  
                	  colSetA=0;
                	  colSetB=1;
                	  guiSetupOk=true;
                	  //chartPanel = new ChartPanel(buildChart("root"));
                  }else{
                	  
                	  if(index==1){
                		  
                		  colSetA=2;
                		  colSetB=3;
                		  guiSetupOk=true;
                		 // chartPanel = new ChartPanel(buildChart("root"));
                	  }else{
                		  
                		  swingCanceled = true;
                	  }
                  }
                  dialog.dispose();
                  //dialog.setVisible(false);
                  }
                 }
           );
    		
            dialog.getContentPane().add(cancelButton);
            cancelButton.setBounds(100, 115, 85, 25);
    		 
    		 cancelButton.addActionListener(new java.awt.event.ActionListener() {
                 public void actionPerformed(java.awt.event.ActionEvent evt) {
                	 swingCanceled=true;
                	// dialog.setVisible(false);
                	 dialog.dispose();
                
                 }
             });
    	
	   dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
	   RefineryUtilities.centerFrameOnScreen(dialog);
	   dialog.setVisible(true);
		if(!swingCanceled && guiSetupOk){
        chartPanel = new ChartPanel(buildChart("root"));
   
		}else{swingCanceled=true;
			dialog.dispose();}
//		}else{
//			JOptionPane.showMessageDialog(null,"This visualizer can not be used with this property!    Please take the other one.","Warning!",JOptionPane.OK_OPTION);
//		//	JOptionPane.showConfirmDialog(null, "This visualizer can not be used with this property!    Please take the other one.","Warning!",JOptionPane.OK_OPTION);
//			}
    	}
    
    private JFreeChart buildChart(String ec){
        TreeMap map = new TreeMap();
        Iterator iterHash = hash.keySet().iterator();
        String next;

        possibleToClickOnBar = false;
        if (ec.equals("root")){
            possibleToClickOnBar = true;
        	while (iterHash.hasNext()){
                next = iterHash.next().toString();
                if (next.split("\\.").length == 1){
                    map.put(next, hash.get(next));
                }
            }
        }
        else{
            int sites = ec.split("\\.").length;
            while (iterHash.hasNext()){
                next = iterHash.next().toString();
                if (next.split("\\.").length == (sites + 1) && next.startsWith(ec)){
                    map.put(next, hash.get(next));
                }
                if (next.split("\\.").length == (sites + 2)){
                	possibleToClickOnBar = true;
                }
            }
            if (map.isEmpty()){
            	ec = ec.replaceAll("\\.[^\\.]+$", "");
            	return buildChart(ec);
            }
        }
        
        JFreeChart chart = null;
        DefaultCategoryDataset dataset = new DefaultCategoryDataset();
        Iterator iterMap = map.keySet().iterator();
        String curKey;
        while(iterMap.hasNext()){
            curKey = iterMap.next().toString();
            dataset.setValue(((float[]) (hash.get(curKey)))[colSetA], setA, curKey);
            dataset.setValue(((float[]) (hash.get(curKey)))[colSetB], setB, curKey);
        }
        final CategoryAxis categoryAxis = new CategoryAxis(description);
        categoryAxis.setCategoryLabelPositions(CategoryLabelPositions.UP_45);
        final ValueAxis valueAxis = new NumberAxis(VALUE_TYPE[colSetA]);
       // final BarRenderer3D renderer = new BarRenderer3D();
        final BarRenderer renderer = new BarRenderer();
        renderer.setToolTipGenerator(new StandardCategoryToolTipGenerator() {
		
			/**
			 * 
			 */
			private static final long serialVersionUID = 1L;

			public String generateToolTip(CategoryDataset dataset, int row, int column) {
				String result = "";
				result += "<html>";
				result += "<b>" + dataset.getRowKey(row).toString() + "</b><br>";
				result += description.replaceFirst(" .*$","") + ": " + dataset.getColumnKey(column).toString() + "<br>";
				result += VALUE_TYPE[colSetA] + ": " + (((int) (dataset.getValue(row, column).doubleValue() * 1000)) / 1000.0) + "<br>";
				result += "<i>Click on</i><br>";
				if (possibleToClickOnBar){
					result += "&nbsp;&nbsp;bar to see " + dataset.getColumnKey(column).toString() + ".*<br>";					
				}
				if (dataset.getColumnKey(column).toString().split("\\.").length > 2){
					result += "&nbsp;&nbsp;title to see " + dataset.getColumnKey(column).toString().replaceAll("\\.[^\\.]+\\.[^\\.]+$", ".*");
				}
				else{
					if (dataset.getColumnKey(column).toString().split("\\.").length == 2){
						result += "&nbsp;&nbsp;title to see root";					
					}
				}
				result += "</html>";
				return result;
			}
		
		});
        renderer.setItemMargin(0);
        final CategoryPlot plot = new CategoryPlot(dataset, categoryAxis, valueAxis, renderer);
        plot.setOrientation(PlotOrientation.VERTICAL);
        if(!ec.equals("root")) ec = "return to " + ec;
        chart = new JFreeChart(ec, JFreeChart.DEFAULT_TITLE_FONT, plot, true);
        return chart;
    }

    public void showGraphic() {
        frame.setContentPane(chartPanel); 
        frame.pack();
        RefineryUtilities.centerFrameOnScreen(frame);
        frame.setVisible(true);
        
        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
      //  frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        chartPanel.addChartMouseListener(new ChartMouseListener() {
		
			public void chartMouseMoved(ChartMouseEvent e) {
				try{
					ChartEntity ce = chartPanel.getEntityForPoint(e.getTrigger().getX(), e.getTrigger().getY());
					if (ce.getClass() == CategoryItemEntity.class){
					}
				}
				catch(Exception exception){}
			}
		
			public void chartMouseClicked(ChartMouseEvent e) {
		    	ChartEntity ce = chartPanel.getEntityForPoint(e.getTrigger().getX(), e.getTrigger().getY());
		    	try{
			    	if (ce.getClass() == CategoryItemEntity.class){
			    		CategoryItemEntity category = (CategoryItemEntity) ce;
				    	String label = category.getDataset().getColumnKey(category.getCategoryIndex()).toString();
				    		chartPanel = new ChartPanel(buildChart(label));
				    		showGraphic();
				    }
			    	else if (ce.getClass() == ChartEntity.class){
			    		String label = chartPanel.getChart().getTitle().getText().split("return to ")[1];
			    		if (!label.equals("root")){
			    			if (!label.contains(".")){
			    				chartPanel = new ChartPanel(buildChart("root"));
			    				showGraphic();			    				
			    			}
			    			else{
			    				label = label.replaceAll("\\.[^\\.]+$", "");
			    				chartPanel = new ChartPanel(buildChart(label));
			    				showGraphic();
			    			}
			    		}
			    	}
		    	}
		    	catch(Exception exception){}
			}
		
		});
    }

    public void saveDataToFile(String destFilePath) throws IOException {
        ObjectOutputStream objOut = new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream(destFilePath)));
        objOut.writeObject(this.result);
        objOut.close();       
       }
    
    public void loadDataFromFile(String srcFilePath) throws Exception {
        ObjectInputStream objIn = new ObjectInputStream(new BufferedInputStream(new FileInputStream(srcFilePath)));
        init((SwissProtEngineResult) objIn.readObject());
        objIn.close(); 
    }
    
    public String getProcessingMessages() {
        // TODO Auto-generated method stub
        return null;
    }

    public void saveToEps(String filename) {
        // TODO Auto-generated method stub
		try {
	    	Rectangle2D area = new Rectangle2D.Double(0, 0, 800, 600);
	    	EpsGraphics2D eps = new EpsGraphics2D("demo", new File(filename), 0, 0, 800, 600);
	    	chartPanel.getChart().draw(eps, area, chartPanel.getChartRenderingInfo());
		} catch (IOException e) {
			e.printStackTrace();
		}
    }

    public void saveToPng(String filename) {
    	try {
			ChartUtilities.saveChartAsPNG(new File(filename), chartPanel.getChart(), 800, 600, chartPanel.getChartRenderingInfo());
		} catch (IOException e) {
			e.printStackTrace();
		}
    }

    public void saveToEmf(String filename) {
        // TODO Auto-generated method stub

    }

    public void cleanUp() {
        // TODO Auto-generated method stub
        
    }

    public String getCmdAll(String filename) {
        // TODO Auto-generated method stub
        return null;
    }

    public IEngineResults getData() {
        // TODO Auto-generated method stub
        return this.result;
    }
    
    public static void main(String[] args) throws Exception {
    	String feature = "EC";
//		File ser = new File("/home/s/siebertm/university/MIPS/SwissProt_" + feature + "_EngineResult.ser");
		//File ser = new File("/home/g/gormanns/SwissProt_" + feature + "_EngineResult.ser");

		HierarchicalSwissProtVisualisation vis = new HierarchicalSwissProtVisualisation();
		vis.description="OC";
//		if (true){
//	    	SwissProt swissProt1 = new SwissProt();
//	        SwissProt swissProt2 = new SwissProt();
////	        swissProt1.init("/home/s/siebertm/university/MIPS/high.txt");
////	        swissProt2.init("/home/s/siebertm/university/MIPS/low.txt");
//	        swissProt1.init("/home/g/gormanns/high.txt");
//	        swissProt2.init("/home/g/gormanns/low.txt");
//	        swissProt1.setDescription("high abundant proteins");
//	        swissProt2.setDescription("low abundant proteins");
//	        swissProt1.run();
//	        swissProt2.run();
//	        
//	        PropertyComparison prop = new PropertyComparison();
//	        prop.init(swissProt1, swissProt2);
//	        prop.run();
//	        
//	        SwissProtEngineResult[] res = (SwissProtEngineResult[]) prop.getResultsAll();
//	        int col;
//	        if (feature.equals("EC")){
//	        	col = 0;
//	        }
//	        else if (feature.equals("OC")){
//	        	col = 2;
//	        }
//	        else{
//	        	col = 0;
//	        	logger.error("feature " + feature + " cannot be used for HierarchicalSwissProtVisualization (default feature: EC)");
//	        }
//        	vis.init(res[col]);
//        	//vis.saveDataToFile(ser.getAbsolutePath());
//		}
		
		vis.run();
		//vis.showGraphic();
    }
 }
