/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: StandardSwissProtVisualisation.java,v 1.6 2006/03/19 11:58:11 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.vis.swissprot;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JTextPane;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;

import org.apache.log4j.Logger;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.CategoryLabelPositions;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.labels.StandardCategoryToolTipGenerator;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.BarRenderer3D;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.ui.RefineryUtilities;

import prompt.engine.EngineResults;
import prompt.engine.IEngineResults;
import prompt.engine.swissProt.SwissProtEngineResult;
import prompt.input.classes.swissProt.internals.Literature;
import prompt.vis.IVisualisation;


public class StandardSwissProtVisualisation implements IVisualisation{

    private final String[] VALUE_TYPE = {"absolute frequency", "absolute frequency", "relative frequency", "relative frequency"};
    private final ArrayList<String> resultTypes = new ArrayList(7);
    private final String[] resultDescriptions = {"OG (Organelle)","MEDLINE IDs","PubMed IDs","KW (Keywords)","FT (Feature Table) key names","OS (Organism Species)","DOM (Domains)"};
    
	private SwissProtEngineResult result;
    private Hashtable abstractTitle;
    private Hashtable hash;
    private String setA;
    private String setB;
    private String description;
    private ChartPanel chartPanel;
    private JFrame frame = new JFrame();
    private int colSetA = 2;
    private int colSetB = 3;
    private boolean getAbstractTitle = false;
    private boolean swingCanceled = false;
    private boolean guiSetupOk=false;
    private String chosenDomain = "IP InterPro";
    
    /*************************************************/
    private JComboBox visComboBox = new JComboBox();
	private JDialog dialog = new JDialog();
	private ArrayList domainNames = new ArrayList();
	/*************************************************/
    
    public StandardSwissProtVisualisation(){
    	
    	resultTypes.add(0, "og");
    	resultTypes.add(1, "medline");
    	resultTypes.add(2, "pubmed");
    	resultTypes.add(3, "kw");
    	resultTypes.add(4, "ft");
    	resultTypes.add(5, "os");
    	resultTypes.add(6, "dom");
    	hash = new Hashtable();
    }
    public void init(IEngineResults dataToPlot) {
   
    	if(dataToPlot.getClass()==new SwissProtEngineResult("").getClass()){
        this.result = (SwissProtEngineResult) dataToPlot;
        this.hash = this.result.getResultTable();
        this.setA = (String) this.result.getDescriptions().get(0);
        this.setB = (String) this.result.getDescriptions().get(1);
        this.description = this.result.getDescription();
        Logger logger = Logger.getLogger(this.getClass());
        logger.debug(this.description+"LIVE_LOADED");
        
		domainNames.add("IP InterPro");
		domainNames.add("PF Pfam");
		domainNames.add("PS Prosite");
		domainNames.add("PR PRINTS");
		domainNames.add("SM SMART");    
		}else{
			
			
			EngineResults tempResult = (EngineResults)dataToPlot;
			String data = tempResult.getData();
			try{
				BufferedReader in = new BufferedReader(new StringReader(data));
				String currentLine ;
				boolean foundHeader = false;
				while(in.ready()){
					currentLine=in.readLine();
					
						if(foundHeader){
							if(currentLine == null){break;}
							
							String[] hashInput = currentLine.split("\t");
							float[] values ={ Float.parseFloat(hashInput[1]),Float.parseFloat(hashInput[2]),Float.parseFloat(hashInput[3]),Float.parseFloat(hashInput[4]),Float.parseFloat(hashInput[5]),Float.parseFloat(hashInput[6]),Float.parseFloat(hashInput[7])};
						this.hash.put(hashInput[0],values);
							
						}
						else{
							foundHeader=true;
							}
					
					
				}ArrayList tempDescriptions = new ArrayList();
				tempDescriptions.add(0,this.setA);
				tempDescriptions.add(1, this.setB);
				this.setA=dataToPlot.getDesc1();
				this.setB=dataToPlot.getDesc2();
				this.description = resultDescriptions[resultTypes.indexOf(dataToPlot.getResultType().split(":")[1])];
				this.result = new SwissProtEngineResult(this.description, dataToPlot.toString(), this.hash,tempDescriptions, dataToPlot.getResultType());
			}
			catch(Exception e){
				e.printStackTrace();
			}
		}   
    }

    public void run() throws Exception {    
    	swingCanceled=false;
    	guiSetupOk=false;
    	if(!(this.description.startsWith("OC") || this.description.startsWith("EC"))){
    		JButton okButton = new JButton("Ok");
    		JButton cancelButton = new JButton("Cancel");
            StyleContext.NamedStyle centerStyle = StyleContext.getDefaultStyleContext().new NamedStyle(); 
            StyleConstants.setAlignment(centerStyle,StyleConstants.ALIGN_CENTER);              
            JTextPane description = new JTextPane();
    		description.setContentType("text/html");
            description.setText("<html align=\"center\">Specify the value which shall be used for visualisation.</html>");
    		description.setFont(new Font("Serif", Font.PLAIN, 12));
            description.setBounds(10, 10, 175, 55);
    		dialog.setSize(200,180);
    		dialog.setModal(true);
    		dialog.getContentPane().setLayout(null);
    		dialog.setTitle("Setup");
    		visComboBox.setBounds(10, 75, 175, 30);
    		visComboBox.addItem("absolute frequency");
    		visComboBox.addItem("relative frequency");
    		visComboBox.setSelectedIndex(1);
    		dialog.getContentPane().add(visComboBox);
    		dialog.getContentPane().add(description);  
    		dialog.getContentPane().add(okButton);
    		okButton.setBounds(10, 115, 85, 25);
    		okButton.addActionListener(new java.awt.event.ActionListener() {
    			public void actionPerformed(java.awt.event.ActionEvent evt) {
    				int index= visComboBox.getSelectedIndex();
    				if(index==0){                	  
    					colSetA=0;
    					colSetB=1;
    					guiSetupOk=true;
    					// chartPanel = new ChartPanel(buildChart());
    				}else{
    					
    					if(index==1){
    						
    						colSetA=2;
    						colSetB=3;
    						guiSetupOk=true;
    						//  chartPanel = new ChartPanel(buildChart());
    					}else{
    						
    						swingCanceled = true;
    					}
    				}
    				dialog.dispose();
    				//dialog.setVisible(false);
    			}
    		});    		
    		dialog.getContentPane().add(cancelButton);
    		cancelButton.setBounds(100, 115, 85, 25);
    		cancelButton.addActionListener(new java.awt.event.ActionListener() {
    			public void actionPerformed(java.awt.event.ActionEvent evt) {
    				swingCanceled=true;
    				// dialog.setVisible(false);
    				dialog.dispose();
    				
    			}
    		});
    		dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
    		RefineryUtilities.centerFrameOnScreen(dialog);
    		dialog.setVisible(true);
    		if(!swingCanceled &&guiSetupOk){
    			chartPanel = new ChartPanel(buildChart());
    			
    		}else{swingCanceled=true;
    		dialog.dispose();}
    	}
    }
    
    private JFreeChart buildChart(){
        JFreeChart chart = null;
        DefaultCategoryDataset dataset = new DefaultCategoryDataset();
        Iterator iter = hash.keySet().iterator();
        String curKey;
		if (description.replaceFirst(" .*$","").equals("MEDLINE") || description.replaceFirst(" .*$","").equals("PubMed")){
			abstractTitle = new Hashtable();
	        while(iter.hasNext()){
	            curKey = iter.next().toString();
	            dataset.setValue(((float[]) (hash.get(curKey)))[colSetA], setA, curKey);
	            dataset.setValue(((float[]) (hash.get(curKey)))[colSetB], setB, curKey);
	            try {
	            	abstractTitle.put(curKey, "Title: " + Literature.getArticleTitle(curKey) + "<br>");
	            }
	            catch (IOException e) {
	            	abstractTitle.put(curKey, "Title: not available<br>");
	            }
	        }			
			getAbstractTitle = true;
		}
		else if(description.startsWith("DOM")){
			while(iter.hasNext()){
				curKey = iter.next().toString();
				if(chosenDomain.substring(0,2).equalsIgnoreCase(curKey.trim().substring(0,2))){
					dataset.setValue(((float[]) (hash.get(curKey)))[colSetA], setA, curKey);
					dataset.setValue(((float[]) (hash.get(curKey)))[colSetB], setB, curKey);
				}				
				frame.dispose();
				frame = new JFrame();
		        frame.setLayout(new BoxLayout(frame.getContentPane(), BoxLayout.Y_AXIS));
		    	final JComboBox doms = new JComboBox();
	    		doms.setMaximumSize(new Dimension(800,20));
	    		for (int i=0; i<domainNames.size(); i++){
	    			doms.addItem(domainNames.get(i));
	    		}
	    		doms.addActionListener(new ActionListener() {			
					public void actionPerformed(ActionEvent arg0) {
						setChosenDomain((String)doms.getSelectedItem());
						chartPanel = new ChartPanel(buildChart());
						showGraphic();
					}			
				});
	    		// set selected item on top of the ComboBox (bad style)
	    		domainNames.remove(chosenDomain);
	    		domainNames.add(0, chosenDomain);
	    		frame.getContentPane().add(doms);
	    	}
		}
		else{
	        while(iter.hasNext()){
	            curKey = iter.next().toString();
	            dataset.setValue(((float[]) (hash.get(curKey)))[colSetA], setA, curKey);
	            dataset.setValue(((float[]) (hash.get(curKey)))[colSetB], setB, curKey);
	        }			
		}
        final CategoryAxis categoryAxis = new CategoryAxis(description);
        categoryAxis.setCategoryLabelPositions(CategoryLabelPositions.UP_45);
        
        final ValueAxis valueAxis = new NumberAxis(VALUE_TYPE[colSetA]);
        final BarRenderer3D renderer = new BarRenderer3D();
        renderer.setToolTipGenerator(new StandardCategoryToolTipGenerator() {
		
			/**
			 * 
			 */
			private static final long serialVersionUID = -3725482676247468558L;

			public String generateToolTip(CategoryDataset dataset, int row, int column) {
				String result = "";
				result += "<html>";
				result += "<b>" + dataset.getRowKey(row).toString() + "</b><br>";
				result += description.replaceFirst(" .*$","") + ": " + dataset.getColumnKey(column).toString() + "<br>";
				if (getAbstractTitle){
					result += abstractTitle.get(dataset.getColumnKey(column).toString());
				}
				result += VALUE_TYPE[colSetA] + ": " + (((int) (dataset.getValue(row, column).doubleValue() * 1000)) / 1000.0) + "<br>";
				result += "</html>";
				return result;
			}
		});
        renderer.setItemMargin(0);
        final CategoryPlot plot = new CategoryPlot(dataset, categoryAxis, valueAxis, renderer);
        plot.setOrientation(PlotOrientation.VERTICAL);
        chart = new JFreeChart(description, JFreeChart.DEFAULT_TITLE_FONT, plot, true);
        return chart;
    }

    public void showGraphic(){
    	frame.add(chartPanel); 
        frame.pack();
        RefineryUtilities.centerFrameOnScreen(frame);
        frame.setVisible(true);
        //frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }
    
    private void setChosenDomain(String dom){
    	this.chosenDomain = dom;
    }

    public void saveDataToFile(String destFilePath) throws IOException {
        ObjectOutputStream objOut = new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream(destFilePath)));
        objOut.writeObject(this.result);
        objOut.close();       
    }
    
    public void loadDataFromFile(String srcFilePath) throws Exception {
        ObjectInputStream objIn = new ObjectInputStream(new BufferedInputStream(new FileInputStream(srcFilePath)));
        init((SwissProtEngineResult) objIn.readObject());
        objIn.close(); 
    }
    
    public String getProcessingMessages() {
        // TODO Auto-generated method stub
        return null;
    }

    public void saveToEps(String filename) {
        // TODO Auto-generated method stub
        
    }

    public void saveToPng(String filename) {
    	try {
			ChartUtilities.saveChartAsPNG(new File(filename), chartPanel.getChart(), 800, 600, chartPanel.getChartRenderingInfo());
		} catch (IOException e) {
			e.printStackTrace();
		}
    }

    public void saveToEmf(String filename) {
        // TODO Auto-generated method stub
        
    }

    public void cleanUp() {
        // TODO Auto-generated method stub
        
    }

    public String getCmdAll(String filename) {
        // TODO Auto-generated method stub
        return null;
    }

    public IEngineResults getData() {
        return this.result;
    }
   
   /* public static void main(String[] args) throws Exception {
    	String feature = "MEDLINE";
//		File ser = new File("/home/s/siebertm/university/MIPS/SwissProt_" + feature + "_EngineResult.ser");
		File ser = new File("/home/users/siebert/SwissProt/SwissProt_" + feature + "_EngineResult.ser");

		StandardSwissProtVisualisation vis = new StandardSwissProtVisualisation();
		if (ser.exists()){
			vis.loadDataFromFile(ser.getAbsolutePath());
		}
		else{
	    	SwissProt swissProt1 = new SwissProt();
	        SwissProt swissProt2 = new SwissProt();
//	        swissProt1.init("/home/s/siebertm/university/MIPS/high.txt");
//	        swissProt2.init("/home/s/siebertm/university/MIPS/low.txt");
	        swissProt1.init("/home/users/siebert/SwissProt/high.txt");
	        swissProt2.init("/home/users/siebert/SwissProt/low.txt");
	        swissProt1.setDescription("high abundant proteins");
	        swissProt2.setDescription("low abundant proteins");
	        swissProt1.run();
	        swissProt2.run();
	        
	        PropertyComparison prop = new PropertyComparison();
	        prop.init(swissProt1, swissProt2);
	        prop.run();
	        
	        SwissProtEngineResult[] res = (SwissProtEngineResult[]) prop.getResultsAll();
	        int col;
	        if (feature.equals("OS")){
	        	col = 1;
	        }
	        else if (feature.equals("OG")){
	        	col = 3;
	        }
	        else if (feature.equals("MEDLINE")){
	        	col = 4;
	        }
	        else if (feature.equals("PubMed")){
	        	col = 5;
	        }
	        else if (feature.equals("KW")){
	        	col = 6;
	        }
	        else if (feature.equals("FT")){
	        	col = 7;
	        }
	        else{
	        	col = 6;
                Logger logger = Logger.getLogger(StandardSwissProtVisualisation.class);
	        	logger.error("feature " + feature + " cannot be used for SwissProtVisualization (default feature: KW)");
	        }
        	vis.init(res[col]);
        	vis.saveDataToFile(ser.getAbsolutePath());
		}
		
		vis.run();
		vis.showGraphic();
    }
    */
}

