/* ------------------------------------------------------------------------------- *
 * This file is part of PROMPT - a Protein Mapping and Comparison Tool             *
 * ------------------------------------------------------------------------------- *
 * Copyright (C) Thorsten Schmidt                             t.schmidt@wzw.tum.de *
 *               Technische Universitt Mnchen                                    *
 *               Wissenschaftszentrum Weihenstephan                                *
 *               Am Forum 1                                                        *
 *               85354 Freising                                                    *
 *               Germany                                                           *
 *                                                                                 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE.  See the license file for more details.                     *
 *                                                                                 *
 * ------------------------------------------------------------------------------- *
 * $Id: SwissProtVisualisation.java,v 1.4 2006/03/19 11:58:11 schmidt Exp $
 * ------------------------------------------------------------------------------- *
 * Author: Thorsten Schmidt <t.schmidt@wzw.tum.de>                                 *
 * ------------------------------------------------------------------------------- */
package prompt.vis.swissprot;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.TreeSet;

import javax.swing.JFrame;

import org.apache.log4j.Logger;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.ChartRenderingInfo;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.entity.StandardEntityCollection;
import org.jfree.chart.labels.StandardCategoryToolTipGenerator;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.BarRenderer3D;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.ui.RefineryUtilities;

import prompt.engine.IEngineResults;
import prompt.engine.swissProt.SwissProtEngineResult;
import prompt.vis.IVisualisation;

public class SwissProtVisualisation  implements IVisualisation {

	/* (non-Javadoc)
	 * @see prompt.vis.IVisualisation#init(bea.engine.IEngineResults)
	 */
    private final String[] VALUE_TYPE = {"absolute value", "absolute value", "relative value", "relative value"};
  
	private Hashtable resultTable ;
	private SwissProtEngineResult swissProtPropertyResult;
	private int width = 800, height = 600;
	private String tmpPath;
	private JFreeChart chart = null;
    private String title="";
    
	public void init(IEngineResults dataToPlot) {
	this.swissProtPropertyResult =(SwissProtEngineResult)dataToPlot;
	this.resultTable =((SwissProtEngineResult)dataToPlot).getResultTable();
	
	}

	
	
	
	public void generateAllCharts(int col1, int col2, String name) {
      

        TreeSet barPlotSet = new TreeSet();
		Iterator keys = resultTable.keySet().iterator();
		String key = "";
//		TreeSet cur = null;
		while (keys.hasNext()) {
			key = keys.next().toString();
		
			barPlotSet.add(key);
		}
		
		Iterator treeKeys = barPlotSet.iterator();
		String curKey;
		
		DefaultCategoryDataset dataset = new DefaultCategoryDataset();
		for (int i = 0; i < barPlotSet.size(); i++) {
			// value, seta or setb, ecnr
			curKey = treeKeys.next().toString();
			dataset.setValue(((float[]) (resultTable.get(curKey)))[col1], swissProtPropertyResult.getDesc1(),
					curKey);
			dataset.setValue(((float[]) (resultTable.get(curKey)))[col2],swissProtPropertyResult.getDesc2(),
					curKey);
		}
		final CategoryAxis categoryAxis = new CategoryAxis(this.swissProtPropertyResult.getDescription());
		final ValueAxis valueAxis = new NumberAxis(this.VALUE_TYPE[col1]);
		final BarRenderer3D renderer = new BarRenderer3D();
		renderer.setToolTipGenerator(new StandardCategoryToolTipGenerator());
		renderer.setItemMargin(0.05);
		final CategoryPlot plot = new CategoryPlot(dataset, categoryAxis,
				valueAxis, renderer);
		plot.setOrientation(PlotOrientation.VERTICAL);
		chart = new JFreeChart(name, JFreeChart.DEFAULT_TITLE_FONT, plot, true);

	}
	/* (non-Javadoc)
	 * @see prompt.vis.IVisualisation#saveDataToFile(java.lang.String)
	 */
	
	public void setTmpPath(String path){
		
		this.tmpPath = path;
	}
	
	public String getTmpPath(){
		
		return this.tmpPath;
		}
	


	/* (non-Javadoc)
	 * @see prompt.vis.IVisualisation#getProcessingMessages()
	 */
	public String getProcessingMessages() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see prompt.vis.IVisualisation#saveToEps(java.lang.String)
	 */
	public void saveToEps(String filename) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see prompt.vis.IVisualisation#saveToPng(java.lang.String)
	 */
	public void saveToPng(String filename) {
		// TODO Auto-generated method stub
        try {
            final ChartRenderingInfo info = new ChartRenderingInfo(
                    new StandardEntityCollection());
            File png = new File(
                    /*tmpPath//"/home/g/gormanns/"
                            +*/ filename + ".png");
          
            ChartUtilities.saveChartAsPNG(png, chart, width, height, info);
            

        } catch (IOException e) {
            Logger logger = Logger.getLogger(this.getClass());
            logger.error("IO errror",e);
        }
	}

	/* (non-Javadoc)
	 * @see prompt.vis.IVisualisation#saveToEmf(java.lang.String)
	 */
	public void saveToEmf(String filename) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see prompt.vis.IVisualisation#cleanUp()
	 */
	public void cleanUp() {
		// TODO Auto-generated method stub
	    this.chart=null;
      
	}

	/* (non-Javadoc)
	 * @see prompt.vis.IVisualisation#showGraphic()
	 */
	public void showGraphic() {
		// TODO Auto-generated method stub
        JFrame frame = new JFrame();
       
        ChartPanel chartPanel = new ChartPanel(chart); 
        
        frame.setContentPane(chartPanel); 
        frame.pack();
        RefineryUtilities.centerFrameOnScreen(frame);
        frame.setVisible(true);
	}

	/* (non-Javadoc)
	 * @see prompt.vis.IVisualisation#getCmdAll(java.lang.String)
	 */
	public String getCmdAll(String filename) {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see prompt.vis.IVisualisation#getData()
	 */
	public IEngineResults getData() {
		// TODO Auto-generated method stub
		return this.swissProtPropertyResult;
	}

	/* (non-Javadoc)
	 * @see prompt.input.IJobTemplate#run()
	 */
     public void saveDataToFile(String destFilename) throws IOException {
            
          ObjectOutputStream objOut = new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream(destFilename)));
          objOut.writeObject(this.swissProtPropertyResult);
          objOut.close();
               
               
         }
     public void loadDataFromFile(String filename) throws Exception {
               
          ObjectInputStream objIn = new ObjectInputStream(new BufferedInputStream(new FileInputStream(filename)));
          this.swissProtPropertyResult = (SwissProtEngineResult) objIn.readObject();
          objIn.close(); 
          this.resultTable = this.swissProtPropertyResult.getResultTable();
         }
    
           
	public void run() throws Exception {
	this.generateAllCharts(0, 1,this.swissProtPropertyResult.description );

	}
	
	public static void main(String[]args){
        
     
		Hashtable Test = new Hashtable();
		float[] testArray = new float[]{2.0f,1.0f};
		float[] testArray2 = new float[]{2.0f,12.0f};
		float[] testArray3 = new float[]{4.0f,1.0f};
		float[] testArray4 = new float[]{21.0f,10.0f};
		float[] testArray5 = new float[]{7.0f,11.0f};
		
		Test.put("normal", testArray);
		Test.put("2high", testArray2);
		Test.put("normal1", testArray3);
		Test.put("1high", testArray4);
		Test.put("allHigh", testArray5);
		
		SwissProtEngineResult result= new SwissProtEngineResult("property", Test);
		SwissProtVisualisation visa = new SwissProtVisualisation();
        
		visa.init(result);
		try{visa.run();}catch(Exception e){e.toString();}
        
        visa.showGraphic();
      
	}

}
